c ----------------------------
c  Version 1.2 - May 17, 2000                     MUM = MUons + Medium
c ----------------------------
c ***********************************************************************
c                       =================== CODE MUM.F ==================
c        _.o.                 
c      .^   \ ^.          The code calculates cross-sections, free paths
c    _//     !  \._         and energy losses for muon interactions and 
c  <__(______)_.^__,>          can simiulate both muons' propagation
c     ((((|   __|    (_)         through large distances in medium
c     (((C   (@,\    ()             and  muon's behaveour within
c      ((     ) _)    O             an underwater or other array
c   .-^|  (   __|     o
c .<_  \    ___/^-,  j-j               Written by I.Sokalski
c    ^-.\    )     \(_/         E-mail: sokalski@pcbai10.inr.ruhep.ru
c        ^---                       July 1999, IfH-DESY/Zeuthen
c                              November 1999 - May 2000, INR/Moscow
c ......................................................................
c
c ATTN : CERN libraries should be linked to execute this code, the 
c       following CERN codes are used: simps/dsimps, rndm, r48, hbook
c
c ATTN: Modified by PM to send all output to stderr
c       'print*,' -> 'write(0,*)'
c                       H O W    T O   U S E :
c                       ====================
c
c First of all, the subroutine INIT_MU should be executed:
c
c                        call init_mu(imed,ipn,em,vm)
c
c   INTEGER  imed = 1 -> water     
c            imed = 2 -> standard rock     
c            imed = 3 -> Antarctic ice     
c   INTEGER  ipn  = 1 -> Sigma_gamma_p for photonuclear interaction is
c                        calculated by Bezrukov_Bugaev (squared LN 
c                        dependence) (L.B.Bezrukov, E.V.Bugaev, Yad.Fiz.,
c                        V. 33, 5, 1981 (1195) (in Russian)
c            ipn.ne.1 -> It is calculated by ZEUS parametrization
c                       (J.Breitweg et al., Eur.Phys.J. C7 (1999) 609)
c   REAL*4   em (GeV) -> lower threshold for energy transfers to simulate      
c                        muon passage through a detector (below it all
c                        energy losses will be considered as "continuous"
c                        ones
c   REAL*4   vm       -> lower threshold for relative energy transfer to
c                        simulate muon propagation througth large distances
c                        in given medium (below it all energy losses will
c                        be considered as "continuous" ones
c
c   +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c        
c         ATTENTION: EM SHOULD BE IN A RANGE OF 0.01 -- 0.5 GeV
c                    VM SHOULD BE IN A RANGE OF 10^-4 - 0.2
c
c If one would like to study array responce for showers with E > E_s > 0,5 GeV 
c   it is possible by setting EM = 0.5 GeV and just ignore interactions with 
c  E_tr > E_s.  Setting EM directly to > 0.5 GeV may lead to essential errors,
c                 MUM is protected against such attemts.
c
c  Note also that decreasing VM down to minimum allowed value will lead to
c  essential increasiing of CPU. On the other hand, VM > 0.01 will lead to 
c  essential errors. The carefull investigation for "correct" value of VM
c is under progress (watch publications !), now we would recomend VM to be
c   within a range of 0.001 -- 0.01. For sure, it is quite enough to use 
c       VM = 0.01 for depths < ~ 5 - 7 km of water equivalent.
c
c   +++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++++
c
c It should be done BEFORE any other routine is called. In principle, one
c may change medium within ones code executing, i.e. call INIT_MU more than
c once.           
c                      
c N.B. Any medium consisting of any elements for muon propagation can 
c      be prepared by editing subroutine MED_CONS. 
c
c N.B.2 One can exclude catastrophic losses for knock-on electrons (all
c       ionization losses and e-diagrams for bremsstrahlung will be con-
c       tinuous losses for simulation with threshold VMIN only, simula-
c       tions with threshold EMIN will include catastrophic losses for
c       ionization, anyway). For that one should set variable NOCA in 
c       routine MED_CONS to 0.
c
c N.B.3 For investigation purposes one can multiply all dif. cross
c       sections and Bethe-Bloch formula by a factor FA which is set to be 
c       equal to 1 in basic version (subroutine MED_CONS). 
c
c After that you are welcome to use the following routines:
c                      
c                    1. MUON BREMSSTRAHLUNG
c                       -------------------
c------
c 1a/ real*4 function getlbrem(X) 
c
c gives the average free path (in cm) for muon bremsstarhlung with
c energy transfers > EMIN.  X is muon energy X (expressed in GeV) and
c should be within a range of 10 -- 10^9 GeV. E-diagrams are not taken
c into account. They are joint with ionization losses (see part for 
c knock-on electrons).
c------
c 1b/ real*4 function glbremv(X) 
c
c gives the average free path (in cm) for muon bremsstarhlung with
c energy transfers > VMIN.  X is muon energy X (expressed in GeV) and
c should be within a range of 10 -- 10^9 GeV. E-diagrams are not taken
c into account. They are joint with ionization losses (see part for 
c knock-on electrons).
c------
c 1c/ real*4 function getctbr(X)  
c
c gives the total cross-section (in cm^2) for muon bremsstarhlung with
c energy transfers > EMIN. X is muon energy X (expressed in GeV) and
c should be within a range of 10 -- 10^9 GeV. The cross-section is 
c averaged over all atoms of given medium:
c
c               sigma(E_mu) = SUM (w_i * sigma_i(E_mu)), 
c
c where w_i = N_i/N_tot, N_i - number of type i atoms in molecula, 
c N_tot - total number of atoms in molecula, sigma_i(E_mu) is a 
c cross-section for atom number i. So, it is calculated for an 
c "effective nucleus" with atomic weight  
c
c                  A_eff = SUM(N_i*A_i) / N_tot
c
c E-diagrams are not taken into account. They are joint with 
c ionization losses (see part for knock-on electrons).
c------
c 1d/ real*4 function gctbrv(X)  
c
c gives the total cross-section (in cm^2) for muon bremsstarhlung with
c energy transfers > VMIN. X is muon energy X (expressed in GeV) and
c should be within a range of 10 -- 10^9 GeV. The cross-section is 
c averaged over all atoms of given medium:
c
c               sigma(E_mu) = SUM (w_i * sigma_i(E_mu)), 
c
c where w_i = N_i/N_tot, N_i - number of type i atoms in molecula, 
c N_tot - total number of atoms in molecula, sigma_i(E_mu) is a 
c cross-section for atom number i. So, it is calculated for an 
c "effective nucleus" with atomic weight  
c
c                  A_eff = SUM(N_i*A_i) / N_tot
c
c E-diagrams are not taken into account. They are joint with 
c ionization losses (see part for knock-on electrons).
c------
c 1e/ real*4 function getcdbr(X,Y,lo)
c
c  gives the differential cross-section d_Sigma/d_v for muon brem-
c  strahlung for given energy X (in GeV, should be within a range of 10 
c  -- 10^9 GeV) and  relative energy transfer Y which should be within
c  a range of -11 < log(Y) < 0. If lo=0, natural units are used for Y,
c  otherwise (lo=1) LOG10(Y) should be used at the input. Cross-section 
c  values are expressed in cm^2 and is averaged over all atoms of given
c  medium (see (1c)). E-diagrams are not taken into account. They are 
c  joint with ionization losses (see part for knock-on electrons).
c------
c 1f/ real*4 function getdedbr(X) 
c
c  gives the energy losses (expressed in MeV/cm) for muon bremstrahlung
c  for interactions with relative energy transfers > EMIN. X is muon
c  energy expressed in GeV and should be within a range of 10 -- 10^9 
c  GeV. E-diagrams are not taken into account. They are joint with 
c  ionization losses (see part for knock-on electrons).
c------
c 1g/ real*4 function gdedbrv(X) 
c
c  gives the energy losses (expressed in MeV/cm) for muon bremstrahlung
c  for interactions with relative energy transfers > VMIN. X is muon
c  energy expressed in GeV and should be within a range of 10 -- 10^9 
c  GeV. E-diagrams are not taken into account. They are joint with 
c  ionization losses (see part for knock-on electrons).
c------
c 1h/ real*4 function gdedbrt(X) 
c
c  gives the total energy losses (expressed in MeV/cm) due to  muon 
c  bremstrahlung. X is muon energy expressed in GeV and should be
c  within a range of 10 -- 10^9 GeV. E-diagrams are not taken into 
c  account.They are joint with ionization losses (see part for knock-on 
c  electrons).
c------
c 1i/ subroutine getvbrem(X,Y,itr) 
c
c  simulates the relative energy transfer Y for muon bremsstrahlung 
c  within a range of EMIN -- 1 (if itr=0) or VMIN -- 1 (itr=1). X is 
c  muon energy expressed in GeV and should be within a range of 
c  10 GeV < X < 1000.000.000 GeV. E-diagrams are not taken into 
c  account.They are joint with ionization losses (see part for knock-
c  on-electrons).
c------
c 1j/ subroutine testbrem(lo)
c
c  performes some test with filling number of histograms and .dat files.
c  The test is short if lo=0. Otherwise it takes more time but gives more 
c  statistics. 
c                      
c                    2. E+E- PAIR PRODUCTION BY MUON
c                       ----------------------------
c------
c 2a/ real*4 function getlpair(X) 
c
c gives the average free path (in cm) for e+e- pair production with
c energy transfers > EMIN.  X is muon energy X (expressed in GeV) and
c should be within a range of 10 -- 10^9 GeV.
c------
c 2b/ real*4 function glpairv(X) 
c
c gives the average free path (in cm) for e+e- pair production with
c energy transfers > VMIN.  X is muon energy X (expressed in GeV) and
c should be within a range of 10 -- 10^9 GeV.
c------
c 2c/ real*4 function getctpa(X)  
c
c gives the total cross-section (in cm^2) for e+e- pair production with
c energy transfers > EMIN. X is muon energy X (expressed in GeV) and
c should be within a range of 10 -- 10^9 GeV. The cross-section is 
c averaged over all atoms of given medium (see 1c).
c------
c 2d/ real*4 function gctpav(X)  
c
c gives the total cross-section (in cm^2) for e+e- pair production with
c energy transfers > VMIN. X is muon energy X (expressed in GeV) and
c should be within a range of 10 -- 10^9 GeV. The cross-section is 
c averaged over all atoms of given medium (see 1c).
c------
c 2e/ real*4 function getcdp(X,Y,lo)
c
c  gives the differential cross-section d_Sigma/d_v for e+e- pair
c  productiion for given energy X (in GeV, should be within a range of 10 
c  -- 10^9 GeV) and  relative energy transfer Y which should be within a
c  range of -11 < log(Y) < 0. If lo=0, natural units are used for Y,
c  otherwise (lo=1) LOG10(Y) should be used at the input. Cross-section 
c  values are expressed in cm^2 and is averaged over all atoms of given
c  medium (see (1c)).
c------
c 2f/ real*4 function getdedpa(X) 
c
c  gives the energy losses (expressed in MeV/cm) for e+e- pair production
c  for interactions with relative energy transfers > EMIN. X is muon
c  energy expressed in GeV and should be within a range of 10 -- 10^9 
c  GeV. 
c------
c 2g/ real*4 function gdedpav(X) 
c
c  gives the energy losses (expressed in MeV/cm) for e+e- pair production
c  for interactions with relative energy transfers > VMIN. X is muon
c  energy expressed in GeV and should be within a range of 10 -- 10^9 
c  GeV. 
c------
c 2h/ real*4 function gdedpat(X) 
c
c  gives the total energy losses (expressed in MeV/cm) for e+e- pair 
c  production. X is muon energy expressed in GeV and should be within 
c  a range of 10 -- 10^9 GeV. 
c------
c 2i/ subroutine getvpa(X,Y,itr) 
c
c  simulates the relative energy transfer Y for e+e- pair production
c  within a range of EMIN -- 1 (if itr = 0) or VMIN -- 1 (otherwise). 
c  X is muon energy expressed in GeV and should be within a range of 
c  10 GeV < X < 1 EeV.
c------
c 2j/ subroutine testpair(lo)
c
c  performes some test with filling number of histograms and .dat files.
c  The test is short if lo=0. Otherwise it takes more time but gives 
c  more statistics. 
c                      
c                    3. MUON PHOTONUCLEAR INTERACTION
c                       -----------------------------
c------
c 3a/ real*4 function getlphnu(X) 
c
c gives the average free path (in cm) for photonuclear interaction with
c energy transfers > EMIN.  X is muon energy X (expressed in GeV) and
c should be within a range of 10 -- 10^9 GeV.
c------
c 3b/ real*4 function glphnuv(X) 
c
c gives the average free path (in cm) for photonuclear interaction with
c energy transfers > VMIN.  X is muon energy X (expressed in GeV) and
c should be within a range of 10 -- 10^9 GeV.
c------
c 3c/ real*4 function getctph(X)  
c
c gives the total cross-section (in cm^2) for photonuc. interaction with
c energy transfers > EMIN. X is muon energy X (expressed in GeV) and
c should be within a range of 10 -- 10^9 GeV. The cross-section is 
c averaged over all atoms of given medium (see 1c).
c------
c 3d/ real*4 function gctphv(X)  
c
c gives the total cross-section (in cm^2) for photonuc. interaction with
c energy transfers > VMIN. X is muon energy X (expressed in GeV) and
c should be within a range of 10 -- 10^9 GeV. The cross-section is 
c averaged over all atoms of given medium (see 1c).
c------
c 3e/ real*4 function getcdn(X,Y,lo)
c
c  gives the differential cross-section d_Sigma/d_v for photonuclear
c  interaction for given energy X (in GeV, should be within a range of 10 
c  -- 10^9 GeV) and  relative energy transfer Y which should be within a
c  range of -11 < log(Y) < 0. If lo=0, natural units are used for Y,
c  otherwise (lo=1) LOG10(Y) should be used at the input. Cross-section 
c  values are expressed in cm^2 and is averaged over all atoms of given
c  medium (see (1c)).
c  ATTENTION: GETCDN computes diff. cross-sections for all relative ener-
c  gy transfers > 10**(-11). In reality photonuclear cross-section is eq-
c  ual to 0 for transferred energies < 0.5 - 1 GeV. It is taken into ac-
c  count in the frame of MUM code but one should put ones attention on 
c  this fact using GETCDN for other purposes.
c------
c 3f/ real*4 function getdedph(X) 
c
c  gives the energy losses (expressed in MeV/cm) for photonuclear
c  interactions with relative energy transfers > EMIN. X is muon
c  energy expressed in GeV and should be within a range of 10 -- 10^9 
c  GeV. 
c------
c 3g/ real*4 function gdedphv(X) 
c
c  gives the energy losses (expressed in MeV/cm) for photonuclear
c  interactions with relative energy transfers > VMIN. X is muon
c  energy expressed in GeV and should be within a range of 10 -- 10^9 
c  GeV. 
c------
c 3h/ real*4 function gdedpht(X) 
c
c  gives the total energy losses (expressed in MeV/cm) for 
c  photonuclear interactions. X is muon energy expressed in GeV and 
c  should be within a range of 10 -- 10^9 GeV. 
c------
c 3i/ subroutine getvph(X,Y,itr) 
c
c  simulates the relative energy transfer Y for photonuc. interaction
c  within a range of EMIN -- 1 (if itr=0) or VMIN -- 1 (otherwise). 
c  X is muon energy expressed in GeV and should be within a range of 
c  10 GeV < X < 1 EeV.
c------
c 3j/ subroutine testphnu(lo)
c
c  performes some test with filling number of histograms and .dat files.
c  The test is relatively short if lo=0. Otherwise it takes more time 
c  but gives more statistics. 
c                      
c                    4. KNOCK-ON ELECTRONS PRODUCTIOIN
c                       ------------------------------
c------
c 4a/ real*4 function getlelec(X) 
c
c gives the average free path (in cm) for knock-on electrons production 
c (including e-diagrams for bremsstrahlung) with energy transfers > EMIN.  
c X is muon energy X (expressed in GeV) and should be within a range of 
c 10 -- 10^9 GeV.
c------
c 4b/ real*4 function glelecv(X) 
c
c gives the average free path (in cm) for knock-on electrons production 
c (including e-diagrams for bremsstrahlung) with energy transfers > VMIN.  
c X is muon energy X (expressed in GeV) and should be within a range of 
c 10 -- 10^9 GeV.
c------
c 4c/ real*4 function getctel(X)  
c
c gives the total cross-section (in cm^2) for knock-on electrons 
c production (including e-diagrams for bremsstrahlung) with energy 
c transfers > EMIN. X is muon energy X (expressed in GeV) and should be 
c within a range of 10 -- 10^9 GeV. The cross-section is  averaged over 
c all atoms of given medium (see 1c)
c------
c 4d/ real*4 function gctelv(X)  
c
c gives the total cross-section (in cm^2) for knock-on electrons 
c production (including e-diagrams for bremsstrahlung) with energy 
c transfers > VMIN. X is muon energy X (expressed in GeV) and should be 
c within a range of 10 -- 10^9 GeV. The cross-section is  averaged over 
c all atoms of given medium (see 1c)
c------
c 4e/ real*4 function getcde(X,Y)
c
c  gives the differential cross-section d_Sigma/d_v for knock-on 
c  electrons production (including e-diagrams for bremsstrahlung) for 
c  given energy X (in GeV, should be within a range of 10 -- 10^9 GeV) 
c  and  relative energy transfer Y which should be within a  range of 
c  -11 < log(Y) < 0. Cross-section values values are expressed in cm^2 
c  and is averaged over all atoms of given medium (see (1c)).
c------
c 4f/ real*4 function getdedel(X) 
c
c  gives the energy losses (expressed in MeV/cm) for knock-on electrons
c  production (including e-diagrams for bremsstrahlung) with relative 
c  energy transfers > EMIN. X is muon energy expressed in GeV and should 
c  be within a range of 10 -- 10^9 GeV. 
c------
c 4g/ real*4 function gdedelv(X) 
c
c  gives the energy losses (expressed in MeV/cm) for knock-on electrons
c  production (including e-diagrams for bremsstrahlung) with relative 
c  energy transfers > VMIN. X is muon energy expressed in GeV and should 
c  be within a range of 10 -- 10^9 GeV. 
c------
c 4h/ real*4 function gdedelbb(X) 
c
c  gives the energy losses (expressed in MeV/cm) for ionization (without
c  e-diagrams for bremsstrahlung, only Bethe-Bloch formula) X is muon 
c  energy expressed in GeV and should be within a range of 10 -- 10^9 GeV. 
c------
c 4i/ real*4 function bebl(X) 
c
c  gives the energy losses (expressed in MeV/cm) for ionization (without
c  e-diagrams for bremsstrahlung, only Bethe-Bloch formula) X is muon 
c  energy expressed in GeV and should be within a range of 0.01 -- 10^9 GeV.
c  Works more slow comparing to GDEDELBB function. 
c------
c 4j/ real*4 function edbrt(X) 
c
c  gives the energy losses (expressed in MeV/cm) due to e-diagrams for 
c  for bremsstrahlung calculated with R.Kokoulin code. X is muon energy 
c  expressed in GeV and should be within a range of 0.01 -- 10^9 GeV.
c------
c 4k/ real*4 function gdeion(X) 
c
c  gives total energy losses (expressed in MeV/cm) for knock-on 
c  electrons production (including e-diagrams for bremsstrahlung). X
c  X is muon energy expressed in GeV and should be within a range of 
c  160 MeV -- 1 EeV. 
c------
c 4l/ subroutine getvel(X,Y,itr) 
c
c  simulates the relative energy transfer Y for knock-on electrons 
c  productiion (including e-diagrams for bremsstrahlung) within a
c  range of EMIN -- 1 (itr = 0) or VMIN -- 1 (otherwise). X is muon 
c  energy expressed in GeV and should be within a range of 10 GeV < X 
c  < 1 EeV.
c------
c 4m/ subroutine testelec(lo)
c
c  performes some test with filling number of histograms and .dat files.
c  The test is short if lo=0. Otherwise it takes more time but gives more 
c  statistics. 
c                      
c                         5. OTHER SUBROUTINES
c                            -----------------
c
c------
c 5a/ subroutine oppaw
c 
c  opens HBOOK for filling histograms.
c------
c 5b/ subroutine clpaw
c
c  closes HBOOK and saves histograms.
c------
c 5c/ real*4 function cone(X) 
c
c  gives the value for "continouus" energy losses (expressed in MeV/cm) 
c  for energy transfers below EMIN.  X is muon energy expressed in GeV 
c  and should be within a range of 10 GeV -- 1 EeV. 
c------
c 5d/ real*4 function conv(X) 
c
c  gives the value for "continouus" energy losses (expressed in MeV/cm) 
c  for energy transfers below VMIN.  X is muon energy expressed in GeV 
c  and should be within a range of 10 GeV -- 1 EeV. 
c------
c 5e/ real*4 function gemleng(X) 
c
c  gives the value for muon's free path between two interactions with
c  energy transfers > EMIN (cm). X is muon energy expressed in GeV and
c  should be within a range of 10 GeV -- 1 EeV. 
c------
c 5f/ real*4 function getlarnv(X,Y) 
c
c  gives the real free path (cm) of muon without interactions with 
c  energy transfers > VMIN if muon energy at the beginning of free 
c  path is equal to Y (in GeV, should be within a range of 10 GeV 
c  -- 1 EeV) and random poisson number is X (= -ln(RNDM), where RNDM 
c  is a random number uniformly distributed on an interval 0. -- 
c  1.). It takes into account that the value for mean free path is 
c  a function of energy and is not equal to a value in a starting 
c  point L_mean(Y). The "continuous" energy losses within given
c  free path are calculated by conv(Y) routine (see above), the
c  dependance of these losses of energy is also taken into account.
c  X should be within a range 0.0001 - 1000. To be used along with 
c  function geteranv(X,Y) in enew(X,Y) subroutine. If final energy
c  after given "free segment" less than 10 GeV, the output means
c  the free path till the point whare muon energy is equal to 10 GeV
c   
c------
c 5g/ real*4 function geteranv(X,Y) 
c
c  gives the muon energy (in GeV) at the finish point of real muon 
c  free path without interactions with energy transfers > VMIN if 
c  muon energy at the beginning of free path is equal to Y (in GeV, 
c  should be within a range of 10 GeV -- 1 EeV) and random poisson 
c  number is X (= -ln(RNDM), where RNDM is a random number uniformly 
c  distributed on an interval 0. -- 1.). It takes into account that 
c  the value for mean free path is a function of energy and is not 
c  equal to a value in a starting point L_mean(Y). The "continuous" 
c  energy losses within given free path are calculated by conv(Y) 
c  routine (see above), the dependance of these losses of energy is 
c  also taken into account. X should be within a range 0.0001 - 1000. 
c  To be used along with function getlarnv(X,Y) in enew(X,Y) 
c  subroutine. If output is less than 10 GeV, it means that the
c  corresponding value of getlarnv(X,Y) with the same X,Y as an
c  input is a free path from starting point till muon energy 10 GeV. 
c------
c 5h/ real*4 function geteback(Y,X) 
c
c gives the starting muon energy (GeV) if its current energy is equal
c to Y (GeV, should be in a range of 10 GeV -- 1 EeV), the distance
c between current point and starting one is equal to X (cm, should be
c in a range of 1 -- 10^7 cm), and there were no interactions with
c relatibe energy transfers more than VMIN between starting and
c current points. To be used in enew(X,Y) subroutine.
c------
c 5i/ real*4 function enew(e,depth) 
c
c simulates the final energy of muon which started with energy E (GeV,
c should be within a range of 10 GeV - 1 EeV and has passed 
c distanse DEPTH (cm, should be in a range of 1 -- 10^7 cm) in the
c given medium. If the final energy is less than 0.16 GeV (muon
c stopped before passing DEPTH) the output is equal to 0.01 GeV.
c The procedure gives also some "history" of simulation which is kept 
c by arrays ityp(10000), eleng(10000), ener1(10000),ener2(10000) and 
c constant NUMB. All this is passed via common/VHISTORY/ and may be 
c used after the simulation for given muon is completed. 
c
c NUMB (integer*4) is equal to number of interaction which occured 
c                  during simulation for given muon
c
c ITYP(i) (integer*4) gives the type of interaction nb. i which may 
c                     be as follows:
c
c 0 - start of simulation
c 1 - muon has reached the DEPTH (which is input variable for ENEW)
c 2 - muon has stopped (its energy became less than 0.16 GeV)
c 3 - muon energy has decreased down to 10 GeV
c 4 - bremsstrahlung interactions occured
c 5 - e+e- pair has been born
c 6 - photonuclear interaction occured
c 7 - knock-on electron has been born (or bremsstrahlung interaction
c     with emmiting gamma by electron occured)
c
c ELENG(i) (real*4) - distance (cm) from the beginning of simulation
c                     till interaction nb. i
c
c ENER1(i) (real*4) - energy (GeV) before interaction nb. i ! For ityp=
c                                                           !  0,1,2,3
c ENER2(i) (real*4) - energy (GeV) aftter interaction nb. i ! it is the
c                                                           !    same
c------
c 5j/ subroutine simulde(ndepths,emu0)
c  
c The subroutine gives the muon energy at a set of depths consisting of 
c NDEPTHS levels of observation if its level at the DEPTH=0 is equal to 
c EMU0 (GeV). EMU0 should be in a range of 10 GeV - 1 EeV. The  
c subroutine uses function ENEW which gives a "muon history" during its 
c travel through a media (see comments to ENEW routine). User has to:
c
c  1. Give lines
c                  ...
c       common /frouser1/ horison(1000)
c       common /touser1/ emuon(1000) 
c                  ...
c
c    in a program unit from which SIMULDE is called. 
c
c  2. Define integer variable 0 < NDEPTHS <= 1000 in that program unit.
c
c  3. Prepare there a real*4 array HORISON(1000) which contains up to
c     1000 levels of observation. The rules for HORISON are as follows:
c 
c     - levels of observation should be expressed in cm;    
c     - they should be more than 0. and not greater than 10^7;
c     - they should follow in increasing order along array.
c
c    SIMULDE references only first NDEPTHS elements of HORISON(1000), so
c    one should take care about them, it does not a matter what will be
c    HORISON(NDEPTHS+1),HORISON(NDEPTHS+2),HORISON(NDEPTHS+3), etc.
c
c   INPUT: EMU0    - energy (GeV) at the depth = 0. !-> subroutine 
c          NDEPTHS - number of observation levels   !   parameters
c          -----------------------------------------
c          HORISON(1000) - an array containing      !-> prepared
c                          levels of observation  ! by user and passed
c                                              ! to SIMULDE via common 
c                                            !      /frouser1/             
c
c   OUTPUT: array EMUON(1000) whose first NDEPTHS elements are muon
c           energies (GeV) at given level of observation as a result
c           of MC simulation which is performed by ENEW routine.
c           Array is returned to user by common /touser1/.
c------
c 5k/  subroutine earray1(e,depth)
c  
c   Simulates the passage of muon through an array at the distance = DEPTH 
c   (cm) if initial muon energy is equal to E (GeV). Uses a model in which 
c   energy losses are divided into 2 part: "continuous" losses with energy 
c  transfers < EMIN and "catastrophic" losses with energy transfers > EMIN. 
c
c                             OUTPUT:         
c                             =======
c
c  The procedure gives some "history" of simulation which is kept by arrays
c  arrays ityp_e(20000), eleng_e(20000), ener1_e(20000),ener2_e(20000) and 
c constant NUMB_E. All this is passed via common/EHISTORY/ to program block
c from which EARRAY1 was called. This block must contain the following lines:
c
c                             ..... 
c       real*8 ener1_e(20000), ener2_e(20000)
c       real*4 eleng_e(20000),etr(20000)
c       integer ityp_e(20000),numb_e
c       common /ehistory/ ener1_e,ener2_e,eleng_e,etr,ityp_e,numb_e
c                            .....
c
c
c         DESCRIPTION OF CONSTANTS AND ARRAYS OF COMMON /EHISTORY/ :
c
c NUMB_E (integer*4) is equal to number of interaction which occured during 
c                    simulation for given muon
c
c ITYP_E(i) (integer*4) gives the type of interaction nb. i which may be as 
c                     follows:
c
c  0 - start of simulation
c  1 - muon has reached the DEPTH
c  2 - muon has stopped (its energy became less than 0.16 GeV)
c  4 - bremsstrahlung interactions occured
c  5 - e+e- pair has been born
c  6 - photonuclear interaction occured
c  7 - knock-on electron has been born (or bremsstrahlung interaction
c      with emmiting gamma by electron occured)
c
c  ELENG_E(i) (real*4) - distance (cm) from the beginning of simulation
c                        till interaction nb. i
c
c  ENER1_E(i) (real*8) - energy (GeV) before interaction nb. i ! For ityp=
c                                                              !  0,1,2,3
c  ENER2_E(i) (real*8) - energy (GeV) aftter interaction nb. i ! it is the
c                                                              !    same
c  ETR(i) (real*4) - energy transfer due to interaction nb.i (GeV), is
c                    assigned by 0. if ITYP_E < 4
c
c     DEPTH should not be more than 2 km, E should be within a range 
c                          10 GeV - 1 EeV.
c
c------
c 5l/ subroutine earray2(e,en,etr,path,jtyp)
c  
c Simulates "the next" interaction for a muon which passes an array if muon 
c   energy is equal to E (GeV). Uses a model in which  energy losses are 
c divided into 2 part: "continuous" losses with energy transfers < EMIN and 
c        "catastrophic" losses with energy transfers > EMIN. 
c
c                             INPUT:         
c                             ======
c   E which is muon energy expressed in Gev (it should be less than 1 EeV)
c
c                             OUTPUT:         
c                             =======
c
c  EN - the muon energy after the next interaction (including   !-> REAL*8
c       "continuous losses" till the point of interaction)      !
c  ETR - energy transfer (GeV) due to "next" interaction           ! REAL*4
c  PATH - distance from initial point to the next interaction (cm) ! 
c  INTEGER*4 JTYP - type of interaction which may be as follows:
c
c  2 - muon has stopped (its energy became 0.16 GeV)
c  4 - bremsstrahlung interactions occured
c  5 - e+e- pair has been born
c  6 - photonuclear interaction occured
c  7 - knock-on electron has been born (or bremsstrahlung interaction
c      with emmiting gamma by electron occured)
c
c If initial energy is less than 0.16 GeV, EN is assigned by a value of 
c                             0.01 GeV 
c------
c 5m/ subroutine bkns(empow,e1)
c
c Simulates the vertical muon spectrum at the see level according to 
c BUGAEV-KLIMUSHIN fit of NAUMOV-SINEGOVSKY spectrum. Input: empow = lower 
c cut energy for muon (lower energy for simulated spectrum), GeV. Output: 
c e1 = siimulated muon energy, GeV. EMPOW must be in a rangge of 100 GeV 
c -- 50 TeV to obtain a reasonable output.
c------
c 5n/ subroutine gais(empow,e1)
c
c Simulates the vertical muon spectrum at the see level according to 
c Gaisser spectrum. Input: empow = lower cut energy for muon (lower energy for 
c simulated spectrum), GeV. Output: e1 = siimulated muon energy, GeV. EMPOW 
c must be in a rangge of 100 GeV -- 50 TeV to obtain a reasonable output.
c------
c 5o/ subroutine specdept(depth,is,ihist,numb)
c 
c     This subroutine simulates muon vertical spectrum at the depth = DEPTH  
c   starting with Bugaev-Klimushin-Naumov-Sinegovski (IS=1) or Gaisser (IS=2)
c    vertical spectrum at see level. The DEPTH should be expressed in cm and
c    corresponds to not less than 600 m.w.e. and not more than 20000 m.w.e.
c  For 11 threshold energies (array ETH(11)) the absolute values for vertical
c   flux I(E_mu > ETH) (array FLUX(11), sm^-2 sr^-1 sec^-1) and mean emergy 
c (EMEAN(11), GeV) are computed. Arrays SRFLUX(11) and EMERR(11) contain values 
c of statistical errors (in corresponding units, at 1-sigma level) for FLUX(11) 
c    and EMEAN(11). The number of muons in depth spectrum which should be 
c simulated is given by variable NUMB. That means that simulation will continue 
c  untill NUMB muons have been detected with energy > ETH(11) at given depth. 
c   Histograms are filled if variable IHIST=1. If IHIST=0 histograms are not
c       filled (it saves ~10% of time but gives much less information).
c  Subroutine informs user by a screen output after every 0.05 * NUMB muons
c                                reached the depth.
c
c    User does not need to think about lower cut for muon energy at surface,
c  SPECDEPT computes it itself quite reliable for each given depth and medium.
c
c                            User has to:
c
c  1. Give lines
c
c                  ...
c        common /frouser2/ eth(11)
c        common /touser2/ emean(11),emerr(11),flux(11),erflux(11)
c                  ...
c
c    in a program unit from which SPECDEPT is called. 
c
c  2. Define real*4 array ETH(11) with threshold energies (in GeV) in that
c     program unit. Note that in the frame of MUM code muon is considered
c     as stopped if its energy becomes less than 0.16 GeV. In this case its
c     energy is assigned by "fake" value of 0.01 GeV (see comments to ENEW
c     routine). So, the lowest threshold should not be less than 0.01 GeV
c     It is recommended to set it to 0.02 GeV to obtain integrated flux and
c     mean energy for the whole depth spectrum E_mu > 0. ETH(i) should be in
c     a reasonable range 0.02 - 100 GeV.
c
c   INPUT: real*4 DEPTH - depth (cm) at which depth spectrum  !-> subroutine 
c                         is simulated and fluxes + mean      !   parameters
c                            energies are computed            !       
c          integer IS - type of vertical muon spectrum at sea !
c                       level (IS = 1 - Bugaev-Klimushin-Na-  !
c                       umov-Sinegovsky ; IS = 2 - Gaisser)   !
c          integer IHIST - Histograms are filled if IHIST=1   !
c                          and are not filled if IHIST=0      !
c          integer NUMB - desirable number of muons with      !
c                         energy > ETH(11) at given depth     !
c          ....................................................
c          real*4 ETH(11) - set of energy thresholds (GeV)    !-> should be
c                           above which fluxes and mean       ! prepared by
c                           energies are computed at given    ! user and pas-
c                           depth                             ! sed to SPECDEPT
c                                                             ! via common
c                                                             ! /frouser2/
c -------------------------------------------------------------
c   OUTPUT: real*4 FLUX(11) - set of integrated fluxes at     !-> returned to
c                             given depth with thresholds     ! user as a result
c                             from ETH(11), cm^-2sr^-1s^-1    ! of SPECDEPT via
c           real*4 ERFLUX(11) - set of statistical errors for !  common block
c                               FLUX(11) (in the same units,  !   /touser2/
c                               1-sigma level)                !
c           real*4 EMEAN(11) - set of mean energies for muons !
c                              with energies > ETH(xx) which  !
c                              are detected at given depth,   !
c                              GeV                            !
c           real*4 EMERR(11) - statistical errors (GeV,       !
c                              1-sigma level) for EMEAN(11)   !
c           
c ------------------------------------------------------------------------
c         LIST OF HISTOGRAMS (WHICH ARE FILLED IF IHIST=1) :
c
c           ID                                    DESCRIPTION
c     
c   INT(depth*0.0001) + 10000     Simulated muon spectrum at see level
c
c   INT(depth*0.0001) + 11000     Simulated muon spectrum at the depth=DEPTH
c
c   INT(depth*0.0001) + 11000 +   Simulated muon spectrum tyo which surface
c      + 100000*(depth*k/10)       spectrum transforms for 10 intermediate
c           where k=1,10                           depths
c
c NOTES: IF ONE CALLS SPECDEPT SEVERAL TIMES THE DIFFERENCE BETWEEN DEPTH
c        variables should exceed 10 meters, otherwise some part of histo-
c        grams will be rewritten and lost  
c
c        User should wathch oneself for ID of ones own HBOOK histograms,
c        ID should not crossed
c------
c 5p/ subroutine testcom
c
c  Produces some histograms and .dat files.  
c ***********************************************************************
c
c           =================================================
c           =     Current version has been successfully     = 
c           =                   tested at:                  =
c           =                                               =
c           =   LINUX (pcbai2/pcbai11/photon.inr.ruhep.ru)  =
c           =   HP-UX (paris.ifh.de)                        =
c           =   IRIX64 (hydra.ifh.de)                       =
c           =   SunOS (proton.jinr.ru)                      =
c           =                                               =
c           =               -   May 23, 2000  -             =                 
c           =================================================
c
****************************************************************************
**********    F U N C T I O N S   A N D   S U B R O U T I N E S   **********
****************************************************************************
****************************************************************************
**************************** COMMON SUBROUTINES : **************************
****************************************************************************
* C.1
          SUBROUTINE init_mu(imed,ipn,em,vm)
*
*  It inicializes some routines which produce the spline coeeficients for
*   further simulaions of muon energy losses by different processes. For
*    more details see comments to subroutines which referenced below in
*                            the body of routine.
*
*                                PARAMETERS:
*                                ==========
*   
*   INTEGER  imed = 1 -> water     
*            imed = 2 -> standard rock     
*            imed = 3 -> Antarctic ice     
*
*   INTEGER  ipn  = 1 -> Sigma_gamma_p for photonuclear interaction is
*                        calculated by Bezrukov_Bugaev (squared LN 
*                        dependence) (L.B.Bezrukov, E.V.Bugaev, Yad.Fiz.,
*                        V. 33, 5, 1981 (1195) (in Russian)
*            ipn.ne.1 -> It is calculated by ZEUS parametrization
*                      (J.Breitweg et al., Eur.Phys.J. C7 (1999) 609)
*  REAL*4   em (GeV) -> lower threshold for energy transfers to simulate      
*                        muon passage through a detector 
*  REAL*4   vm       -> lower threshold for relative energy transfer to
*                       simulate muon passage throuth large distances in
*                       given medium
*  
*         ATTENTION: EM SHOULD BE IN A RANGE OF 0.01 -- 0.5 GeV
*                    VM SHOULD BE IN A RANGE OF 10^-4 - 0.2
*
*  If one would like to study array responce for showers with E_s > 0,5 GeV 
*  it is possible by setting EM = 0.5 GeV and just ignore interactions with 
* E_tr > E_s.  Setting EM directly to > 0.5 GeV may lead to essential errors,
*                MUM is protected against such attemts.
*
*.........................................................................
      real*8 rvec(1)
      common /r48/ rvec
      len = 1
      write(0,*)' '
      write(0,*)'        =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-='
      write(0,*)' ' 
      write(0,*)'          Code MUM (MUons + Medium) has started.'
      write(0,*)' '
      write(0,*)'        =-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-='
      write(0,*)' ' 
      write(0,*)'          Something like 1 min. will be spent'
      write(0,*)'             for preliminary preparations.'
      write(0,*)'You will be informed as soon as MUM finishes with them...'
      write(0,*)' '
      write(0,*)'         PREPARATION OF RANDOM NUMBER GENERATOR...'
      call rm48(rvec,len)
      write(0,*)'                                                     DONE'
      write(0,*)'SUBROUTINE MED_CONS - making media, setting parameters...'
      call med_cons(imed,ipn,em,vm)
      write(0,*)'                                                     DONE'
      write(0,*)'SUBROUTINE GAMMA1 - computing bremsstrahlung: energy los-'
      write(0,*)'                    ses, cross-sections, constants for'
      write(0,*)'                    comparison function etc. ...'
      call gamma1
      write(0,*)'                                                     DONE'
      write(0,*)'SUBROUTINE PAIR1 - computing e+e- pair production: energy'
      write(0,*)'                   losses, cross-sections, comparison fu-'
      write(0,*)'                   nction and its integral etc. ...'
      call pair1 
      write(0,*)'                                                     DONE'
      write(0,*)'SUBROUTINE PHNU1 - computing photonu. interaction: energy'
      write(0,*)'                   losses, cross-sections, etc.'
      call phnu1
      write(0,*)'                                                     DONE'
      write(0,*)'SUBROUTINE ELEC1 - computing D-elect-s production: energy'
      write(0,*)'                   losses, cross-sections, etc.'
      call elec1
      write(0,*)'                                                     DONE'
      write(0,*)'SUBROUTINE ENLOS - computing continuous energy losses...'
      write(0,*)' '
      call enlos
      write(0,*)'                                                     DONE'
      write(0,*)'SUBROUTINE SPL1 - cooking 1-dimensional real*4 splines'
      write(0,*)'                  with equidestant grid...'
      call spl1
      write(0,*)'                                                     DONE'
      write(0,*)'SUBROUTINE DSPL1 - cooking 1-dimensional real*8 splines'
      write(0,*)'                   with equidestant grid...'
      call dspl1
      write(0,*)'                                                     DONE'
      write(0,*)'SUBROUTINE DSPLQ1 - cooking 1-dimensional real*8 splines'
      write(0,*)'                    with non-equidestant grid...'      
      call dsplq1
      write(0,*)'                                                     DONE'
      write(0,*)'SUBROUTINE SPL2 - cooking 2-dimensional real*4 splines'
      write(0,*)'                  with equidestant grid...'
      call spl2
      write(0,*)'                                                     DONE'
      write(0,*)'  '
      write(0,*)' APPROXIMATELY 50 PERCENTS OF PREPARATIONS HAS BEEN DONE.' 
      write(0,*)'  '
      write(0,*)'SUBROUTINES FREPATHV AND SPL2_2 - cooking different kinds'
      write(0,*)'                                  of splines to simulate'
      write(0,*)'      (THE LAST STEP !!!)         muon propagation...' 
      call frepathv
      call spl2_2
      write(0,*)'                                                     DONE'
      write(0,*)' ' 
      call prinfo(imed,ipn,em,vm)
      write(0,*)'  Preparations have been finished, starting to compute' 
      write(0,*)'                what you requested...'
      write(0,*)' ' 
      return
      end
c---------------------------------------------------------------------------
* C.1a
      subroutine prinfo(imed,ipn,em,vm)
*
*         Screen output for constants and parameters
*
      real*8 alfa,rm_e,rm_mu,r_e,z1(10),w(10),aw(10),n(10)
      real*8 a_ef,avog,ro
      real*8 z_a,ri_z,x_0,x_1,a,rm
      integer nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      common /media/ z1,w,aw,a_ef,ro,nsub
      common /med_ion/ z_a,ri_z,x_0,x_1,a,rm
      common /toprint/ n
      common /exer1/ fa
      common /exer2/ noca
      write(0,*)'========================================================='
      write(0,*)'    Constants and parameters have been set as follows:'
      write(0,*)' '
      write(0,*)'Fine structure constant  : ',alfa
      write(0,*)'Avogadro number          : ',avog
      write(0,*)'Electron mass            : ',rm_e,' MeV'
      write(0,*)'Muon mass                : ',rm_mu,' MeV'
      write(0,*)'Classical electron radii : ',r_e,' cm'
      if (imed.eq.1) write(0,*)'Medium                   :   WATER'
      if (imed.eq.2) write(0,*)'Medium                   :   STANDARD ROCK'
      if (imed.eq.3) write(0,*)'Medium                   :   ANTARCTIC ICE'
      write(0,*)'Composed of              :  ',nsub,' elements'
      write(0,*)' '
      do i=1,nsub
      nl = idint(n(i) + 2.d-1)
      nz = idint(z1(i) + 2.d-1)
      write(0,*)'    Z =',nz,',   A =',aw(i),',  ',nl,' atom(s)'
      enddo
      write(0,*)' '
      write(0,*)'Density =',ro,' g/cm3,    Ion. potential =',ri_z, ' MeV'
      write(0,*)'Density correction constants for Bethe-Bloch formula:'
      write(0,*)'X0 =',x_0,',   X1 =',x_1,',   a =',a,',   m =',rm
      write(0,*)' '
      write(0,*)'The lower cut for transfer energy to'
      write(0,*)'simulate detector responce EMIN (GeV) : ',em
      write(0,*)'(all en. losses with transfers > EMIN will be considered'
      write(0,*)'as CATASTROPHIC, the rest will be CONTINUOUS en. losses)'
      write(0,*)' '
      write(0,*)'The lower cut for relative energy transfer to simulate'
      write(0,*)'muon propagation till detector VMIN : ',vm
      write(0,*)'(all en. losses with transfers > VMIN will be considered'
      write(0,*)'as CATASTROPHIC, the rest will be CONTINUOUS en. losses)'
      if (vm.le.8.e-4) then
      write(0,*)'.......................................................'
      write(0,*)'ATTN: FOR THIS VMIN COMPUTING WILL TAKE A LONG TIME !!!'
      write(0,*)'.......................................................'
      endif
      write(0,*)' '
      if (ipn.eq.1) then
      write(0,*)'Parametrization for cross section for absorbtion of a'
      write(0,*)'real photon will be done by Bezrukov-Bugaev'
      else
      write(0,*)'Parametrization for cross section for absorbtion of a'
      write(0,*)'real photon will be done by ZEUS'
      endif
      if ((fa.lt..99999).or.(fa.gt.1.0001)) then
      write(0,*)' '
      write(0,*)'ATTN : ALL DIFF. CROSS SEC. ARE MULTIPLIED BY',fa
      endif
      if (noca.eq.0) then
      write(0,*)'  '
      write(0,*)'There are no catastrophic losses for knock-on electrons'
      write(0,*)' for simulations with threshold relative transfer VMIN'
      else
      write(0,*)'  '
      write(0,*)'Knock-on electrons are included in catastrophic losses'
      endif
      write(0,*)'========================================================='
      write(0,*)'  '
      return
      end
****************************************************************************
* C.2
       subroutine med_cons(imed,ipn,em,vm)
*
*      This subroutine sets the universal constants and prepares a medium
*                           for further computing
*
*                                PARAMETERS:
*                                ==========
*   
*   INTEGER  imed = 1 -> water     
*            imed = 2 -> standard rock     
*            imed = 3 -> Antarctic ice    
*   INTEGER  ipn  = 1 -> Sigma_gamma_p for photonuclear interaction is
*                        calculated by Bezrukov_Bugaev (squared LN 
*                        dependence) (L.B.Bezrukov, E.V.Bugaev, Yad.Fiz.,
*                        V. 33, 5, 1981 (1195) (in Russian)
*            ipn.ne.1 -> It is calculated by ZEUS parametrization
*                      (J.Breitweg et al., Eur.Phys.J. C7 (1999) 609)
*  REAL*4   em (GeV) -> lower threshold for energy transfers to simulate      
*                        muon passage through a detector 
*  REAL*4   vm       -> lower threshold for relative energy transfer to
*                       simulate muon passage throuth large distances in
*                       given medium
*
*         ATTENTION: EM SHOULD BE IN A RANGE OF 0.01 -- 2 GeV
*                    VM SHOULD BE IN A RANGE OF 10^-4 - 0.2
*
*  If one would like to study array responce for showers with E_s > 0,5 GeV 
*  it is possible by setting EM = 0.5 GeV and just ignore interactions with 
* E_tr > E_s.  Setting EM directly to > 0.5 GeV may lead to essential errors,
*                MUM is protected against such attemts.
*
*  .........................................................................
       real*8 alfa,rm_e,rm_mu,r_e,z1(10),w(10),aw(10),n(10)
       real*8 ntot,a_ef,avog,ro,zmean
       real*8 z_a,ri_z,x_0,x_1,a,rm
       integer nsub
       common /const/ alfa,rm_e,rm_mu,r_e,avog
       common /media/ z1,w,aw,a_ef,ro,nsub
       common /med_ion/ z_a,ri_z,x_0,x_1,a,rm
       common /general/ emin,vmin,emph
       common /mcef/ mcb1,mcb2,mcp1,mcp2,mcn1,mcn2,mce1,mce2
       common /zav/ zm
       common /pnsig/ ibb
       common /toprint/ n
       common /exer1/ fa
       common /exer2/ noca
c ........................................................................
        mcb1 = 0  ! 
        mcb2 = 0  !
        mcp1 = 0  ! CONSTANTS TO CALCULATE
        mcp2 = 0  !      SIMULATION
        mcn1 = 0  !      EFFICIENCY
        mcn2 = 0  !
        mce1 = 0  !
        mce2 = 0  !
c ........................................................................
c                           BASIC CONSTANTS:
c                           ***************
       avog = 6.022045d+23         !--> Avogadro number
       alfa =  7.297353053019d-3   !--> fine structure constant
       rm_e = 5.110034d-1          !--> electron mass (in MeV)
       rm_mu = 1.0565932d+2        !--> muon mass (in MeV)
       r_e = 2.8179409d-13         !--> classical electron radii (in cm)
c ........................................................................
c               THRESHOLD ENERGIE AND RELATIVE ENERGY TRANSFER:
c               ***********************************************
         emin = em      !---> threshold energy in Gev
         vmin = vm      !---> threshold relative energy transfer
         emph = 8.e-1   !---> threshold en. for photonucl. interaction, GeV  
         fa = 1.e+0     !---> factor to multiply all diff. cros-sections and
c                       !     Bethe-Bloch formula
         noca = 1       !---> if noca=0, there are no catastrophic losses
c                             for knock-on electrons             
c ........................................................................
        ibb = ipn ! if ibb=1 Sigma_gamma_p for photonuclear interaction is
c                 ! calculated by Bezrukov_Bugaev (squared LN dependence),
c                 ! otherwise it is calculated by ZEUS parametrization
c                 ! (J.Breitweg et al., Eur.Phys.J. C7 (1999) 609)
c ........................................................................
c                          MEDIUM PREPARATION:
c                          ******************
        if (imed.eq.1) then
        ro=1.d+0      ! 
        nsub=2        ! IT IS WATER. It consists of nsub=2 elements:
        n(1)=2.d+0    ! n(1) = 2 H atoms and n(2) = 1 O atom with atomic
        n(2)=1.d+0    ! weights and electric charges are equal to
        aw(1)=1.008   ! aw(1) = 1.008, aw(2) = 15.999, z1(1) = 1 and 
        aw(2)=15.999  ! z1(2) = 8, correspondingly
        z1(1)=1.d+0   ! Its density is ro = 1.0 g/qubic cm
        z1(2)=8.d+0   !
c                     !     
        z_a = 5.551d-1! Z/A                      !                    
        ri_z = 7.5d-05! Min. Ion. Potential, MeV !
        x_0 = 2.4d-1  !--------------------      !--> It is constants
        x_1 = 2.8d+0  !-- 4 constants for        !--> for Bethe-Bloch
        a = 9.1d-2    !-- dencity effect         !-->     formula 
        rm = 3.477d+0 !--------------------      !
        endif
c
        if (imed.eq.2) then
        ro=2.65d+0       ! 
        nsub=1           !
        n(1)=1.d+0       !
        aw(1)=22.        !
        z1(1)=1.1d+1     !
c                        ! ALL THE SAME FOR STANDARD ROCK  
        z_a = 5.d-1      !
        ri_z = 1.364d-04 !
        x_0 = 4.9d-2     !
        x_1 = 3.055d+0   !
        a = 8.3d-2       !
        rm = 3.412d+0    !
        endif
c
        if (imed.eq.3) then
        ro=.92d+0       ! 
        nsub=2          !
        n(1)=2.d+0      !
        n(2)=1.d+0      ! The same for Antarctic ice
        aw(1)=1.008     ! (all constants are exactly as
        aw(2)=15.999    ! for water but ro = 0.92 g/cm^3 )
        z1(1)=1.d+0     !
        z1(2)=8.d+0     !
c                       !      
        z_a = 5.551d-1  !
        ri_z = 7.5d-05  !
        x_0 = 2.4d-1    !
        x_1 = 2.8d+0    !
        a = 9.1d-2      !
        rm = 3.477d+0   !
        endif
c
        if (imed.gt.3) then
        write(0,*)'I do not know what medium do you mean, STOP me...'
        read*,lkk
        endif
c
        if ((em.lt.0.01).or.(em.gt..51)) then
        write(0,*)'Min. transf. en. EM = ',em,' GeV which is out of range'
        write(0,*)'         STOP ME....'
        read*,lkk
        endif
c
        if ((vm.lt.1.e-4).or.(vm.gt..21)) then
        write(0,*)'Min. rel. transf. en. VM = ',vm,' which is out of range'
        write(0,*)'           STOP ME....'
        read*,lkk
        endif
c
        ntot=0.d+0
        do i=1,nsub              
        ntot=ntot+n(i) !----> ntot is total number of atoms in molecula
        enddo
        do i=1,nsub
        w(i)=n(i)/ntot !-----> w(i) are relative weights of different atoms
        enddo          !       w(i) = n(i) / ntot
c
        zmean=0.d+0        
        do i=1,nsub
        zmean = zmean + (z1(i)*n(i))
        enddo
        zm = sngl(zmean/ntot) !-> mean charge of averaged atom (for Delec-s)
c
        a_ef = 0.d+0 
        do i=1,nsub         
        a_ef = a_ef + ((n(i)*aw(i))/ntot) !--> it is an effective atomic 
        enddo                             !    weight for an averaged atom
c                                         !    for which diff. and total cros-
        return                            !    csections will be computed.
        end
****************************************************************************
* C.3
         SUBROUTINE spl1
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
* SPL1 produces arrays of spline coefficients for 1-dimensional interpolatiion
*      and passes them via common blocks to corresponding subroutines.
*  .....................................................................
       common /con_los/ elosemin(17),elosvmin(17) !-> input from ENLOS
c  routine. Contain 17 values for muon "continuous energy losses" below
c  EMIN (elosemin(17), MeV/cm) and VMIN (elosvmin(17), MeV/cm) for 17
c  muon energies starting with 10 GeV and finishing with 1 EeV.
       common /eleng/ eminleng(17) !-> input from ENLOS routine. Contains
c  17 values for muon free path between 2 interactions with energy trans-
c  fers less than EMIN for 17 muon energies starting with 10 GeV and fini-
c  shing with 1 EeV.
       common /ctbr_in1/ FBC(17),FBC2(17) !-> input from GAMMA1 subroutine. 
       common /elbr_in1/ FBL(17),FBL2(17) !   Contains 17 values of mean 
c  free path (arrays FBC(17) and FBC2(17), cm) and 17 values of energy 
c  losses (arrays FBL(17) and FBL2(17), LN(MeV/cm)) for muon bremsstra-
c  hlung with energy transfers > EMIN and > VMIN for 17 muon energies 
c  starting with 10 GeV and finishing with 1000 PeV.
       common /elbr_in2/ FBL3(17) !-> input from GAMMA1 routine. Contains
c  17 values of total muon energy losses due to bremsstrahlung 
c  (log10(MeV/cm)) for muon energies 10 GeV -- 1 EeV. 
       common /ctpa_in1/ FPC(17),FPC2(17) !-> input from PAIR1 subroutine. 
       common /elpa_in1/ FPL(17),FPL2(17) !   Contains 17 values of mean 
c  free path (arrays FPC(17) and FPC2(17), LN(cm)) and 17 values of energy 
c  losses (arrays FPL(17) and FPL2(17), LN(MeV/cm)) for e+e- pair production 
c  with energy transfers > EMIN and > VMIN for 17 muon energies starting 
c  with 10 GeV and finishing with 1000 PeV.
       common /elpa_in2/ FPL3(17) !-> input from PAIR1 routine. Contains
c  17 values of total muon energy losses due to e+e- pair production 
c  (LN(MeV/cm)) for muon energies 10 GeV -- 1 EeV. 
       common /ctph_in1/ FNC(17),FNC2(65) !-> input from PHNU1 subroutine.
       common /elph_in1/ FNL(17),FNL2(65) !   Contains 17 values of mean 
c  free path (arrays FNC(17) and FNC2(17), cm) and 17 values of energy 
c  losses (arrays FNL(17), FNL2(17), LN(MeV/cm)) for muon photonuclear 
c  interactions with energy transfers > EMIN and > VMIN for 17 muon
c  energies starting with 10 GeV and finishing with 1000 PeV.
       common /elph_in2/ FNL3(17) !-> input from PHNU1 routine. Contains
c  17 values of total muon energy losses due to photonuclear interaction 
c  (log10(MeV/cm)) for muon energies 10 GeV -- 1 EeV. 
       common /ctel_in1/ FEC(17),FEC2(17) !-> input from ELEC1 subroutine.
       common /elel_in1/ FEL(17),FEL2(17) !   Contains 17 values of mean
c  free path (arrays FEC(17), FEC2(17), cm) and 17 values of energy losses
c  (arrays FEL(17), FEL2(17), log10(MeV/cm)) for knock-on-electrons produ-
c  ction with energy transfers > EMIN and >VMIN for 17 muon energies star-
c  ting with 10 GeV and finishing with 1000 PeV.
       common /elel_in2/ FELBB(17),FELBBB(17) !-> input from ELEC1 subrou-
c  tine. Contain 17 values of total energy losses for ionisation energy
c  losses (MeV/cm) computed by Bethe-Bloch formula (FELBB(17)) and  by
c  (Bethe-Bloch + e-diagram for bremsstrahlung) (FELBBB(17)).
       common /elel_in3/ FELOWTOT(101)!-> input from ELEC1 subrou-
c  tine. Contain 101 values of total energy losses for ionisation energy
c  losses (MeV/cm) computed by (Bethe-Bloch + e-diagram for bremsstrahlung)
c  for muon energies 0.14 - 14 GeV
       common /sok34/ com_pa_m(2201) !-> input from PAIR1 subroutine. Contains 
c  1601 values for comparison function to simulate energy transfers due to
c  muon pair production. It is calculated for E_mu = 1000 PeV and for 2201
c  values of relative energy transfers V = 10^(-11) -- 1. 
       common /sok1/ xmin1_c,st1_c,xmax1_c !-> values for the first values,
c  last values and steps of arguments for arrays FBC, FPC, FNC and FEC. Are
c  passed to corresponding subroutines along with spline coefficients to 
c  perform an interpolation.
       common /eminl/ xm1,s1,xma1 !-> values for the first values, last
c  values and steps of arguments for array EMINLENG. Are passed to corres-
c  ponding subroutines along with spline coefficients to perform an inter-
c  polation.
       common /sok5/ xmin1_l,st1_l,xmax1_l !-> values for the first values,
c  last values and steps of arguments for arrays FBL, FPL, FNL and FEL. Are 
c  passed to corresponding subroutines along with spline coefficients to 
c  perform an interpolation.
       common /sok1n/ xmin1_nc,st1_nc,xmax1_nc !-> values for the first
c  value, last value and step of arguments for array FNC2. Are passed
c  to subroutine GCTPHV along with splines to perform an interpolation.
       common /sok5n/ xmin1_nl,st1_nl,xmax1_nl !-> values for the first value,
c  value, last value and step of arguments for array FNL2. Are passed to 
c  to subroutine GDEDPHV along with splines to perform an interpolation.
       common /sok5et/ xmin1_lo,st1_lo,xmax1_lo !-> values for the first value,
c  value, last value and step of arguments for array FELOWTOT. Are passed to 
c  subroutine GDEDELT@ along with splines to perform an interpolation.
       common /sok55/ xmin1_p,st1_p,xmax1_p !-> values for the first value,
c  last value and step for array COM_PA_M. Are passed to subroutine COMP
c  along with splines to get values of comparison function at any relative 
c  energy transfer by interpolation.
       common /sok1_b/ CBC(19) !-> resulting array with spline coefficients
c  for mean path for muon bremsstrahlung with energy transfer > EMIN. It is
c  cooked within subroutine SPL1 and passed to subroutine GETLBREM to get
c  mean free path for any muon energy 10 GeV -- 1000 PeV by interpolation.
       common /sok5_b/ CBL(19) !-> resulting array with spline coefficients
c  for muon bremsstrahlung energy losses  with energy transfer > EMIN
c  It is cooked within subroutine SPL1 and passed to subroutine GETDEDBR to
c  get energy losses for any muon energy 10 GeV -- 1000 PeV by interpolation
       common /sok1_b2/ CBC2(19) !-> resulting array with spline coefficients
c  for mean path for muon bremsstrahlung with energy transfer > VMIN. It is
c  cooked within subroutine SPL1 and passed to subroutine GLBREMV to get
c  mean free path for any muon energy 10 GeV -- 1000 PeV by interpolation.
       common /sok5_b2/ CBL2(19) !-> resulting array with spline coefficients
c  for muon bremsstrahlung energy losses  with energy transfer > VMIN
c  It is cooked within subroutine SPL1 and passed to subroutine GDEDBRV to
c  get energy losses for any muon energy 10 GeV -- 1000 PeV by interpolation
       common /sok5_b3/ CBL3(19) !-> resulting array with spline coefficients
c  fort otal muon bremsstrahlung energy losses. It is cooked within 
c  subroutine SPL1 and passed to subroutine GDEDBRT to get energy losses 
c  for any muon energy 10 GeV -- 1000 PeV by interpolation.
       common /sok1_p/ CPC(19) !-> resulting array with spline coefficients
c  for muon pair production total cross-section with energy transfer > EMIN
c  It is cooked within subroutine SPL1 and passed to subroutine GETCTPA to
c  get cross-section values for any muon energy 10 GeV -- 1000 PeV 
c  by interpolation
       common /sok5_p/ CPL(19) !-> resulting array with spline coefficients
c  for muon energy losses for pair production with energy transfer > EMIN
c  It is cooked within subroutine SPL1 and passed to subroutine GETDEDPA to
c  get energy losses for any muon energy 10 GeV -- 1000 PeV by interpolation
       common /sok1_p2/ CPC2(19) !-> resulting array with spline coefficients
c  for mean path for E+E- pair production with energy transfer > VMIN. It is
c  cooked within subroutine SPL1 and passed to subroutine GLPAIRV to get
c  mean free path for any muon energy 10 GeV -- 1000 PeV by interpolation.
       common /sok5_p2/ CPL2(19) !-> resulting array with spline coefficients
c  for e+e- pairs production energy losses  with energy transfer > VMIN
c  It is cooked within subroutine SPL1 and passed to subroutine GDEDPAV to
c  get energy losses for any muon energy 10 GeV -- 1000 PeV by interpolation
       common /sok5_p3/ CPL3(19) !-> resulting array with spline coefficients
c  for total energy losses due to e+e- pairs production. It is cooked within 
c  subroutine SPL1 and passed to subroutine GDEDPAT to get energy losses 
c  for any muon energy 10 GeV -- 1000 PeV by interpolation.
       common /sok1_n/ CNC(19) !-> resulting array with spline coefficients
c  for muon photonuclear interaction total cross-section with energy 
c  transfers > EMIN. It is cooked within subroutine SPL1 and passed to 
c  subroutine GETCTPH to get cross-section values for any muon energy 10 GeV 
c  -- 1000 PeV by interpolation
       common /sok5_n/ CNL(19) !-> resulting array with spline coefficients
c  for muon energy losses due to photonuclear interaction with energy 
c  transfers > EMIN. It is cooked within subroutine SPL1 and passed to 
c  subroutine GETDEDPH to get energy losses for any muon energy 10 GeV -- 
c  1000 PeV by interpolation
       common /sok1_n2/ CNC2(67) !-> resulting array with spline coef-s
c  for muon photonuclear interaction total cross-section with energy 
c  transfers > VMIN. It is cooked within subroutine SPL1 and passed to 
c  subroutine GLPHNUV to get mean free path for any muon energy 10 GeV 
c  -- 1000 PeV by interpolation
       common /sok5_n2/ CNL2(67) !-> resulting array with spline coef-s
c  for muon energy losses due to photonuclear interaction with energy 
c  transfers > VMIN. It is cooked within subroutine SPL1 and passed to 
c  subroutine GDEDPHV to get energy losses for any muon energy 10 GeV -- 
c  1000 PeV by interpolation
       common /sok5_n3/ CNL3(19) !-> resulting array with spline coef-s
c  for total muon energy losses due to photonuclear interaction. It is
c  cooked within subroutine SPL1 and passed to subroutine GDEDPHT to get 
c  energy losses for any muon energy 10 GeV -- 1000 PeV by interpolation.
       common /sok1_e/ CEC(19) !-> resulting array with spline coefficients
c  for delta electrons total cross-section with energy transfers > EMIN. It 
c  is cooked within subroutine SPL1 and passed to subroutine GETCTEL to get 
c  cross-section values for any muon energy 10 GeV -- 1000 PeV by 
c  interpolation
       common /sok5_e/ CEL(19) !-> resulting array with spline coefficients
c  for muon energy losses due to delta electrons production with energy 
c  transfers > EMIN. It is cooked within subroutine SPL1 and passed to 
c  subroutine GETDEDEL to get energy losses for any muon energy 10 GeV -- 
c  1000 PeV by interpolation
       common /sok1_e2/ CEC2(19) !-> resulting array with spline coefficients
c  for delta electrons total cross-section with energy transfers > VMIN. It 
c  is cooked within subroutine SPL1 and passed to subroutine GLELECV to get 
c  mean free path for any muon energy 10 GeV -- 1000 PeV by interpolation.
       common /sok5_e2/ CEL2(19) !-> resulting array with spline coefficients
c  for muon energy losses due to delta electrons production with energy 
c  transfers > VMIN. It is cooked within subroutine SPL1 and passed to 
c  subroutine GDEDELV to get energy losses for any muon energy 10 GeV -- 
c  1000 PeV by interpolation
       common /sok5_e3/ CEBB(19) !-> resulting array with spline coefficients
c  for total muon energy losses due to ionisation calculated by Bethe-Bloch
c  formula. It is cooked within subroutine SPL1 and passed to subroutine 
c  GDEDELBB to get energy losses for any muon energy 10 GeV -- 1000 PeV by 
c  interpolation
       common /sok5_e4/ CEBBB(19) !-> resulting array with spline coef-s
c  for total muon energy losses due to ionisation calculated by Bethe-Bloch
c  formula + e-diagram for bremsstrahlung. It is cooked within subroutine SPL1 
c  and passed to subroutine GDEDELT1 to get energy losses for any muon energy 
c  10 GeV -- 1000 PeV by interpolation
       common /elem/ CLE(19) !-> resulting array with spline coefficients
c  for muon "continuous energy losses" below EMIN.
       common /elvm/ CLV(19) !-> resulting array with spline coefficients
c  for muon "continuous energy losses" below VMIN. 
       common /eminl1/ CLE1(19) !-> resulting array with spline coefficients
c  for muon free path between 2 interactions with energy transfers > EMIN.
       common /sok5_e5/ CETOT(103) !-> resulting array with spline coef-s
c  for total muon energy losses due to ionisation calculated by Bethe-Bloch
c  formula + e-diagram for bremsstrahlung. It is cooked within subroutine SPL1 
c  and passed to subroutine GDEDELT2 to get energy losses for any muon energy 
c  below 10 GeV by interpolation
       common /sok55_p/ CPC1(2203)!-> resulting array with spline coefficients
c for comparison function to simulate muon energy transfers due to pair 
c production. It is cooked within SPL1 subroutine and pased to COMP to get
c value of comparison function at any relative energy transfer by 
c interpolation.
       dimension IJ(26) !---> array which contains dimensions for input 1-di-
c  mentional arrays. Dimention of output arrays with splines should be equal
c  to (I+2), where I is dimension for corresponding input 1-dimensional array. 
       dimension xmin1(26),st1(26),xmax1(26) !-> arrays with first values, 
c  steps and last values of arguments for all input arrays (see above)
       dimension F(2201),C(2203) !-> Auxiliary arrays  
c
      data xmin1/1.e+0,1.e+0,1.e+0,1.e+0,1.e+0,1.e+0,1.e+0,1.e+0,1.e+0,   !
     +           1.e+0,1.e+0,1.e+0,1.e+0,1.e+0,1.e+0,1.e+0,1.e+0,1.e+0,   !
     +           1.e+0,1.e+0,1.e+0,1.e+0,1.e+0,-1.1e+1,-.853871964,       !
     +           1.e+0/                                                   !
      data xmax1/9.e+0,9.e+0,9.e+0,9.e+0,9.e+0,9.e+0,9.e+0,9.e+0,         !
     +           9.e+0,9.e+0,9.e+0,9.e+0,9.e+0,9.e+0,9.e+0,9.e+0,9.e+0,   !
     +           9.e+0,9.e+0,9.e+0,9.e+0,9.e+0,9.e+0,0.e+0,1.146128036,   !
     +           9.e+0/                                                   !
      data st1/5.e-1,5.e-1,5.e-1,5.e-1,5.e-1,5.e-1,5.e-1,5.e-1,           !
     +         5.e-1,5.e-1,5.e-1,5.e-1,1.25e-1,1.25e-1,5.e-1,5.e-1,       !
     +         5.e-1,5.e-1,5.e-1,5.e-1,5.e-1,5.e-1,5.e-1,5.e-3,2.e-2,     !
     +         5.e-1/                                                     !
c First values, last values and steps of argument for all input arrays 
c                             (see above)
c
      data IJ/17,17,17,17,17,17,17,17,17,17,17,17,65,65,    !-> dimensions for
     +        17,17,17,17,17,17,17,17,17,2201,101,17/       !   all input arrays 
c
      do lik=1,26  !--> A cycle along all input arrays
          N = IJ(lik)  !--> Getting dimension for given inpiut array
c
          if (lik.eq.1) then     ! GETTING FIRST VALUE,
           xmin1_c = xmin1(lik)  !      STEP AND LAST VALUE OF ARGUMENT
           st1_c = st1(lik)      !         FOR ALL INPUT ARRAYS
           xmax1_c = xmax1(lik)  !           IN ACCORING WITH THEIR NUMBERS
          endif                  !                     ( LIK )
          if (lik.eq.2) then     !  
           xmin1_l = xmin1(lik)  !
           st1_l = st1(lik)      !
           xmax1_l = xmax1(lik)  !
          endif                  !     
          if (lik.eq.13) then    !
           xmin1_nc = xmin1(lik) !
           st1_nc = st1(lik)     !
           xmax1_nc = xmax1(lik) !
          endif                  !
          if (lik.eq.14) then    !  
           xmin1_nl = xmin1(lik) !
           st1_nl = st1(lik)     !
           xmax1_nl = xmax1(lik) !
          endif                  !     
          if (lik.eq.24) then    !
           xmin1_p = xmin1(lik)  !
           st1_p = st1(lik)      !
           xmax1_p = xmax1(lik)  !
          endif                  !
          if (lik.eq.25) then    !
           xmin1_lo = xmin1(lik) !
           st1_lo = st1(lik)     !
           xmax1_lo = xmax1(lik) !
          endif                  !
          if (lik.eq.26) then    !
           xm1 = xmin1(lik)      !
           s1 = st1(lik)         !
           xma1 = xmax1(lik)     !
          endif                  !
c                                ---------------
          do jj=1,N                            !
            if (lik.eq.1) F(jj) = FBC(jj)      !
            if (lik.eq.2) F(jj) = FBL(jj)      !
            if (lik.eq.3) F(jj) = FBC2(jj)     ! Filling the auxiliary array
            if (lik.eq.4) F(jj) = FBL2(jj)     ! F with values of input array
            if (lik.eq.5) F(jj) = FBL3(jj)     ! number LIK for further
            if (lik.eq.6) F(jj) = FPC(jj)      !                processing.
            if (lik.eq.7) F(jj) = FPL(jj)      !                  
            if (lik.eq.8) F(jj) = FPC2(jj)     !
            if (lik.eq.9) F(jj) = FPL2(jj)     !                  
            if (lik.eq.10) F(jj) = FPL3(jj)    !                  
            if (lik.eq.11) F(jj) = FNC(jj)     ! 
            if (lik.eq.12) F(jj) = FNL(jj)     !
            if (lik.eq.13) F(jj) = FNC2(jj)    ! 
            if (lik.eq.14) F(jj) = FNL2(jj)    !
            if (lik.eq.15) F(jj) = FNL3(jj)    !
            if (lik.eq.16) F(jj) = FEC(jj)     ! 
            if (lik.eq.17) F(jj) = FEL(jj)     !
            if (lik.eq.18) F(jj) = FEC2(jj)    ! 
            if (lik.eq.19) F(jj) = FEL2(jj)    !
            if (lik.eq.20) F(jj) = FELBB(jj)   ! 
            if (lik.eq.21) F(jj) = FELBBB(jj)  !
            if (lik.eq.22) F(jj) = elosemin(jj)! 
            if (lik.eq.23) F(jj) = elosvmin(jj)!
            if (lik.eq.24) F(jj) = com_pa_m(jj)!
            if (lik.eq.25) F(jj) = FELOWTOT(jj)!
            if (lik.eq.26) F(jj) = eminleng(jj)!
           enddo                               !
c----------------------------------------------!
      C(1)=1.4375*F(1)-1.3750*F(2)+0.4375*F(3)  !
      C(2)=0.4375*F(1)+0.1250*F(2)-0.0625*F(3)    !
      DO 1 K=3,N                                    ! ---> Cooking splayns
1     C(K)= 0.6250*F(K-1)-0.0625*(F(K-2)+F(K))       !     and putting them into
      C(N+1)=0.4375*F(N)+0.1250*F(N-1)-0.0625*F(N-2) !     auxiliary array C
      C(N+2)=1.4375*F(N)-1.375*F(N-1)+0.4375*F(N-2)  !
c----------------------------------------------------
         mo = N+2                          !
         do jj=1,mo                        ! Spline coefficients from C are
           if (lik.eq.1) CBC(jj) = C(jj)   ! put into corresponding array N LIK
           if (lik.eq.2) CBL(jj) = C(jj)   ! which is passed to corresponding
           if (lik.eq.3) CBC2(jj) = C(jj)  ! subroutine for interpolation.
           if (lik.eq.4) CBL2(jj) = C(jj)  ! 
           if (lik.eq.5) CBL3(jj) = C(jj)  ! 
           if (lik.eq.6) CPC(jj) = C(jj)   ! 
           if (lik.eq.7) CPL(jj) = C(jj)   !
           if (lik.eq.8) CPC2(jj) = C(jj)  ! 
           if (lik.eq.9) CPL2(jj) = C(jj)  !
           if (lik.eq.10) CPL3(jj) = C(jj) !
           if (lik.eq.11) CNC(jj) = C(jj)  !
           if (lik.eq.12) CNL(jj) = C(jj)  !
           if (lik.eq.13) CNC2(jj) = C(jj) !
           if (lik.eq.14) CNL2(jj) = C(jj) !
           if (lik.eq.15) CNL3(jj) = C(jj) !
           if (lik.eq.16) CEC(jj) = C(jj)  !
           if (lik.eq.17) CEL(jj) = C(jj)  !
           if (lik.eq.18) CEC2(jj) = C(jj) !
           if (lik.eq.19) CEL2(jj) = C(jj) !
           if (lik.eq.20) CEBB(jj) = C(jj) !
           if (lik.eq.21) CEBBB(jj) = C(jj)!
           if (lik.eq.22) CLE(jj) = C(jj)  !
           if (lik.eq.23) CLV(jj) = C(jj)  !
           if (lik.eq.24) CPC1(jj) = C(jj) ! 
           if (lik.eq.25) CETOT(jj) = C(jj)! 
           if (lik.eq.26) CLE1(jj) = C(jj) ! 
         enddo                             !
      enddo
      RETURN
      END
****************************************************************************
* C.4
      SUBROUTINE dspl1
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*   DSPL1 produces arrays of REAL*8 spline coefficients for 1-dimensional 
*        interpolatiion and passes them to corresponding subroutines.
* It is absolutely the same as SPL1 subroutine but works with REAL*8 arrays
*                          both at input at output.
*  .....................................................................
      real*8 com_pa_in(1101)       !
      real*8 CP_1(1103)            !--> see comments below for commons 
      real*8 xmin_p1,st_p1,xmax_p1 !    /sok24/, /sok26/ and /sok25/
      real*8 xmin1(1),st1(1),xmax1(1) !-> arrays with first value, step and 
c last value of argument for all input arrays (see also comments to common 
c /sok25/)
      real*8 F(1101),C(1103) !-> Auxiliary arrays          
      common /sok24/ com_pa_in !-> input from PAIR1 subroutine. Contains 1101
c values for integrated comparison function to simulate energy transfers due
c to e+e- pair production. See comments to subroutine PAIR1 for more details.
      common /sok26/ CP_1 !-> resulting array with spline coefficients for 
c integrated  comparison function to simulate energy transfers due to e+e- 
c pair production. It is an output of DSPL1 subroutine which is passed then
c to C_PA_IN routine to get value of comparison function at any energy transfer 
c by interpolation.
      common /sok25/ xmin_p1,st_p1,xmax_p1 !-> values for the first value, step
c and last value of argument for input array COM_PA_IN. Are passed along with 
c output spline array CP_1 to C_PA_IN routine.
      dimension IJ(1) !-> array which contains dimensions for output 1-dimensi-
c onal arrays. Dimention of output arrays with splines should be equal to (I+2),
c where I is dimension for corresponding input 1-dimensional array. 
      data xmin1/-1.1d+1/ ! FIRST VALUES OF ARGUMENTS !    for all   
      data xmax1/0.d+0/   ! LAST VALUES OF ARGUMENTS  ! input arrays
      data st1/1.d-2/     ! STEPS OF ARGUMENTS        !  (see above)
      data IJ/1101/  !-> dimensions for all input arrays
c
      do lik=1,1  !--> A cycle along all input arrays
          N = IJ(lik) !--> Getting dimension for given input array
c
          if (lik.eq.1) then     ! GETTING FIRST VALUE, STEP AND LAST VALUE
           xmin_p1 = xmin1(lik)  ! OF ARGUMENTS FOR ALL INPUT ARRAYS IN 
           st_p1 = st1(lik)      ! ACCORDING TO THEIR NUMBERS (LIK)
           xmax_p1 = xmax1(lik)  !
          endif                  !
c       
          do jj=1,N                             ! Filling the auxiliary array F
            if (lik.eq.1) F(jj) = com_pa_in(jj) ! with values of corresponding
          enddo                                 ! input array Nb. LIK
c
      C(1)=1.4375*F(1)-1.3750*F(2)+0.4375*F(3)        !
      C(2)=0.4375*F(1)+0.1250*F(2)-0.0625*F(3)        ! Cookiing spline 
      DO 1 K=3,N                                      ! coefficients and 
1     C(K)= 0.6250*F(K-1)-0.0625*(F(K-2)+F(K))        ! putting them into
      C(N+1)=0.4375*F(N)+0.1250*F(N-1)-0.0625*F(N-2)  ! auxiliary array C
      C(N+2)=1.4375*F(N)-1.375*F(N-1)+0.4375*F(N-2)   !
c
         mo = N+2                         ! Spline coefficients from C are
         do jj=1,mo                       ! put into corresponding output
           if (lik.eq.1) CP_1(jj) = C(jj) ! array Nb. LIK whic is passe to
         enddo                            ! corresponding subroutine
c                                         ! for interpolation
      enddo
      RETURN
      END
****************************************************************************
* C.5
         SUBROUTINE spl2
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
* SPL2 produces arrays of spline coefficients for 2-dimensional interpolatiion
*      and passes them via common blocks to corresponding subroutines.
*  .....................................................................
       common /cdbr_in/ FB1(81,54),FB3(81,101),FB2(81,51) ! It is input from
c subroutine gamma1 with values of differential cross-sections for muon brem-
c strahlung for different energies (10 GeV -- 1000 PeV) and relative energy 
c transfers (10^(-11) -- 1)
       common /cdpa_in/ FP1(81,54),FP3(81,101),FP2(81,51) ! It is input from
c subroutine pair1 with values of differential cross-sections for muon pairs
c production for different energies (10 GeV -- 1000 PeV) and relative energy 
c transfers (10^(-11) -- 1)
       common /cdph_in/ FN1(81,54),FN3(81,101),FN2(81,51) ! It is input from
c subroutine phnu1 with values of differential cross-sections for muon pho-
c tonuclear inteaction for different energies (10 GeV -- 1000 PeV) and rela-
c tive energy transfers (10^(-11) -- 1)
       common /sok3/ CB1(4648) ! It is resulting arrays with spline coeffici-
       common /sok6/ CB2(4399) ! cients for 2-dimensional interpolation to
       common /sok4/ CB3(8549) ! obtain differential cross-section for muon
c bremsstrahlung. These arrays are cooked out of FB1,FB2 and FB3 arrays, res-
c pectively and are passed via commons /sok3/, /sok6/ and /sok4/ to routines
c GETCDBR1, GETCDBR2 and GETCDBR3 which perform interpolation.
       common /sok8/ CP1(4648)  ! It is resulting arrays with splayn coeffici-
       common /sok9/ CP2(4399)  ! cients for 2-dimensional interpolation to
       common /sok10/ CP3(8549) ! obtain differential cross-section for muon
c pair production. These arrays are cooked out of FP1,FP2 and FP3 arrays, res-
c pectively and are passed via commons /sok8/, /sok9/ and /sok10/ to routines
c GETCDP1, GETCDP2 and GETCDP3 which perform interpolation.
       common /mum8/ CN1(4648)  ! It is resulting arrays with splayn coeffici-
       common /mum9/ CN2(4399)  ! cients for 2-dimensional interpolation to
       common /mum10/ CN3(8549) ! obtain differential cross-section for muon
c photonuclear interaction. These arrays are cooked out of FN1,FN2 and FN3 
c arrays, respectively and are passed via commons /mum8/, /mum9/ and /mum10/ 
c to routines GETCDN1, GETCDN2 and GETCDN3 which perform interpolation.
       common /sok_2_1/ NX_1,NY_1,X0_1,SX_1,Y0_1,SY_1 ! It is commons with va- 
       common /sok_2_2/ NX_2,NY_2,X0_2,SX_2,Y0_2,SY_2 ! lues for nb. of X (1),
       common /sok_2_3/ NX_3,NY_3,X0_3,SX_3,Y0_3,SY_3 ! Y (2), initial X (3),
c                         ^    ^    ^    ^    ^    ^
c                         1    2    3    4    5    6
c                               
c step by X (4), initial Y (5) and step by Y (6) for input 2-dimensional ar-
c rays of which splines are prepared. All these values are assigned below 
c within SPL2 routine and passed to corresponding routines via commons along 
c with resulting arrays containing splines: /sok_2_1/ to getcdbr1, getcdp1
c and getcdn1 (input arrays FB1, FP1 and FN1, output arrays CB1, CP1 and 
c CN1), /sok_2_2/ to getcdbr2, getcdp2 and getcdn2 (input arrays FB2, FP2
c and FN2, output arrays CB2, CP2 and CN2), /sok_2_3/ to getcdbr3, getcdp3
c and getcdn3 (input arrays FB2, FP2 and FN2, output arrays CB2, CP2 and 
c CN2).
c
      DIMENSION D(90,110),FU(81,101),CU(8549) !------->  Auxiliary arrays
c
c     FOR BELOW EREFERENCED ARRAYS THE ORDER OF VALUES IS AS FOLLOWS:
c     
c     1 - for FB1 and CB1
c     2 - for FB2 and CB2
c     3 - for FB3 and CB3
c     4 - for FP1 and CP1
c     5 - for FP2 and CP2
c     6 - for FP3 and CP3
c     7 - for FN1 and CN1
c     8 - for FN2 and CN2
c     9 - for FN3 and CN3
c
      DIMENSION NXG(9),NYG(9),X0G(9),SXG(9),Y0G(9),SYG(9) ! -> arrays which
c  contain values for above referenced iinitial values, number of values and
c  steps (see above commons /sok_2_1/, /sok_2_2/, /sok_2_3/ etc.).
      DIMENSION IJ(9) !---> array which contains dimensions for output 1-di-
c  mentional arrays with splines. Dimention of output array should be equal
c  to (I+2)*(J+2), where I and J are dimensions for corresponding input 2-di-
c  mensional array. 
      data NXG/81,81,81,81,81,81,81,81,81/     ! See comments above for ex-
      data NYG/54,51,101,54,51,101,54,51,101/  ! planations of these DATAs
      data X0G/1.e+0,1.e+0,1.e+0,1.e+0,1.e+0,1.e+0,1.e+0,1.e+0,1.e+0/    !
      data SXG/1.e-1,1.e-1,1.e-1,1.e-1,1.e-1,1.e-1,1.e-1,1.e-1,1.e-1/    !
      data Y0G/-11.,-6.e-1,-1.e-1,-11.,-6.e-1,-1.e-1,-11.,-6.e-1,-1.e-1/ !
      data SYG/2.e-1,1.e-2,1.e-3,2.e-1,1.e-2,1.e-3,2.e-1,1.e-2,1.e-3/    !
      data IJ/4648,4399,8549,4648,4399,8549,4648,4399,8549/              !
c
      do li=1,9 ! A cycle along all input arrays
      NX = NXG(li)  ! assigns values from corresonding arrais for numbers of
      NY = NYG(li)  ! values, steps and initial values
      X0 = X0G(li)  !
      SX = SXG(li)  !
      Y0 = Y0G(li)  !
      SY = SYG(li)  !
c
       if (li.eq.1) then
         NX_1 = NX     !-> number of X values  !  For arrays FB1, FP1 and FN1,
         NY_1 = NY     !-> number of Y values  !  these values are passed to
         X0_1 = X0     !-> first value of X    !    GETCDBR1,  GETCDP1 and  
         SX_1 = SX     !-> step by X           !  GETCDN1 via common /sok_2_1/
         Y0_1 = Y0     !-> first value of Y    !
         SY_1 = SY     !-> step by Y           !
       endif
c
       if (li.eq.2) then
         NX_2 = NX    !-> number of X values  ! For arrays FB2, FP2 and FN2, 
         NY_2 = NY    !-> number of Y values  ! these values are passed to
         X0_2 = X0    !-> first value of X    !    GETCDBR2, GETCDP2  and 
         SX_2 = SX    !-> step by X           ! GETCDN2 via common /sok_2_2/
         Y0_2 = Y0    !-> first value of Y    !
         SY_2 = SY    !-> step by Y           !
       endif
c
       if (li.eq.3) then
         NX_3 = NX    !-> number of X values  !  For arrays FB3, FP3 and FN3,
         NY_3 = NY    !-> number of Y values  !   thesevalues are passed to
         X0_3 = X0    !-> first value of X    !     GETCDBR3, GETCDP3 and
         SX_3 = SX    !-> step by X           ! GETCDN3 via common  /sok_2_3/
         Y0_3 = Y0    !-> first value of Y    !
         SY_3 = SY    !-> step by Y           !
       endif
c
        do ki=1,NX                                ! Filling an auxiliary
           do kl=1,NY                             ! array FU by values 
              if (li.eq.1) FU(ki,kl) = FB1(ki,kl) ! from input array
              if (li.eq.2) FU(ki,kl) = FB2(ki,kl) ! (within a cycle by LI
              if (li.eq.3) FU(ki,kl) = FB3(ki,kl) ! along all input
              if (li.eq.4) FU(ki,kl) = FP1(ki,kl) ! arrays)
              if (li.eq.5) FU(ki,kl) = FP2(ki,kl) !
              if (li.eq.6) FU(ki,kl) = FP3(ki,kl) !
              if (li.eq.7) FU(ki,kl) = FN1(ki,kl) !
              if (li.eq.8) FU(ki,kl) = FN2(ki,kl) !
              if (li.eq.9) FU(ki,kl) = FN3(ki,kl) !
           enddo                                  !
        enddo                                     !
c-----------------------------------
      DO 1 J=1,NY                  !
      J2=J+2                       !
      DO 1 I=1,NX                  !
      I2=I+2                       !
1     D(I2,J2)=3.90625E-3*FU(I,J)  !
      J1=NY+1                      !
      J3=J2+1                      !
      J4=J3+1                      !
      DO 2 I=3,I2                  !
      A=D(I,3)                     !
      B=D(I,4)                     !---> Cooking spline coefficients
      D(I,2)=3.*(A-B)+D(I,5)       !     out of input array Nb. LI
      D(I,1)=3.*(D(I,2)-A)+B       !     and putting these splines
      A=D(I,J1)                    !     into 1-dimensional array
      B=D(I,J2)                    !                 CU
      D(I,J3)=3.*(B-A)+D(I,NY)     !
2     D(I,J4)=3.*(D(I,J3)-B)+A     !
      I1=NX+1                      !
      I3=I2+1                      !
      I4=I3+1                      !
      DO 3 J=1,J4                  !
      A=D(3,J)                     !
      B=D(4,J)                     !
      D(2,J)=3.*(A-B)+D(5,J)       !
      D(1,J)=3.*(D(2,J)-A)+B       !
      A=D(I1,J)                    !
      B=D(I2,J)                    !
      D(I3,J)=3.*(B-A)+D(NX,J)       !
3     D(I4,J)=3.*(D(I3,J)-B)+A         !
      DO 4 J=1,J2                        !
      J3=J+1                               !
      J4=J+2                                 !
      M=(J-1)*I2                               !
      DO 4 I=1,I2                                !
      I3=I+1                                       !
      I4=I+2                                         !
4     CU(M+I)=D(I,J)+D(I,J4)+D(I4,J)+D(I4,J4)+D(I3,J3)* !
     *100.-(D(I3,J)+D(I,J3)+D(I4,J3)+D(I3,J4))*10.      !
c--------------------------------------------------------
       ko = IJ(li)
       do ki=1,ko                            !
              if (li.eq.1) CB1(ki) = CU(ki)  ! Passing the values of spline
              if (li.eq.2) CB2(ki) = CU(ki)  ! coefficients form auxiliary
              if (li.eq.3) CB3(ki) = CU(ki)  ! array CU to corresponding
              if (li.eq.4) CP1(ki) = CU(ki)  ! output array Nb. LI
              if (li.eq.5) CP2(ki) = CU(ki)  !
              if (li.eq.6) CP3(ki) = CU(ki)  !
              if (li.eq.7) CN1(ki) = CU(ki)  !
              if (li.eq.8) CN2(ki) = CU(ki)  !
              if (li.eq.9) CN3(ki) = CU(ki)  !
       enddo                                 !
      enddo
      RETURN
      END
****************************************************************************
* C.6 
        SUBROUTINE enlos
*
*  The subroutine calculates the total "continous" energy losses below the
*    threshold EMIN (ENLOSEMIN(17) array) and below VMIN (ENLOSVMIN(17)).
* It also computes the mean free path between two interactions with energy
*                      transfers > EMIN (EMINLENG(17)).
*  These arrays are passed to SPL1 subroutine to cook splines for further
*                               interpolation.  
*...........................................................................
      common /elbr_in1/ elo_br1(17),elo_br2(17)  ! 
      common /elbr_in2/ elo_br3(17)              !-> From GAMMA1 
      common /elbr_in4/ elo_br4(17)              !
      common /ctbr_in1/ crt_br1(17),crt_br2(17)  ! 
c                     ---------------------------
      common /elpa_in1/ elo_pa1(17),elo_pa2(17)  !
      common /elpa_in2/ elo_pa3(17)              !-> From PAIR1
      common /elpa_in4/ elo_pa4(17)              !
      common /ctpa_in1/ crt_pa1(17),crt_pa2(17)  !
c                     ---------------------------
      common /elph_in1/ elo_ph1(17),elo_ph2(65)  !
      common /elph_in2/ elo_ph3(17)              !-> From PHNU1 
      common /elph_in4/ elo_ph4(17)              ! 
      common /ctph_in1/ crt_ph1(17),crt_ph2(65)  !
c                     ---------------------------
      common /elel_in1/ elo_el1(17),elo_el2(17)  !
      common /elel_in2/ elel_bb(17),elel_bbb(17) !-> From ELEC1
      common /ctel_in1/ crt_el1(17),crt_el2(17)  !
c                    ----------------------------
      common /con_los/ elosemin(17),elosvmin(17) !-> To SPL1
      common /eleng/ eminleng(17)                !
c                    ----------------------------
      do i=1,17
       j = (4 * i) - 3
       elosemin(i) = elo_br4(i)
       elosvmin(i) = (1.e+1**elo_br3(i)) - (1.e+1**elo_br2(i))
       elosemin(i) = elosemin(i) + elo_pa4(i)
       elosvmin(i) = elosvmin(i) + exp(elo_pa3(i)) - exp(elo_pa2(i))
       elosemin(i) = elosemin(i) + elo_ph4(i)
       elosvmin(i) = elosvmin(i) + 1.e+1**elo_ph3(i) - 1.e+1**elo_ph2(j)
       elosemin(i) = elosemin(i) + elel_bbb(i) - 1.e+1**elo_el1(i)
       elosvmin(i) = elosvmin(i) + elel_bbb(i) - 1.e+1**elo_el2(i)
       elosvmin(i) = alog(elosvmin(i)) 
       eminleng(i) = 1./crt_br1(i) + 1./exp(crt_pa1(i)) + 
     +                              1./exp(crt_ph1(i)) + 1./crt_el1(i)
       eminleng(i) = 1./eminleng(i)
      enddo
      return
      end
****************************************************************************
* C.7
             FUNCTION cone(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*   The subroutine calculates the value for total continuous muon energy 
*    losses with energy transfers below EMIN using spline coefficients 
*                     prepared by spl1 subroutine.
*
*          X is a value for muon energy expressed in GeV; XMIN,
*  XMAX and STEP are assigned with some values in subroutine SPL1 (they
*               are passed here via /sok5/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                              common /elem/
*  ......................................................................
*  ***   ATTENTION: X must be within a range of 10 GeV --- 1000 PeV   ***
*  ......................................................................
      COMMON /sok5/ XMIN,STEP,XMAX
      common /elem/ C(19)
      X1 = alog10(X)
       if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
       write(0,*)'ERROR IN FUNCTION CONE: MUON ENERGY IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      cone = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
****************************************************************************
* C.8
             FUNCTION conv(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*   The subroutine calculates the value for total continuous muon energy 
*    losses with energy transfers below VMIN using spline coefficients 
*                     prepared by spl1 subroutine.
*
*          X is a value for muon energy expressed in GeV; XMIN,
*  XMAX and STEP are assigned with some values in subroutine SPL1 (they
*               are passed here via /sok5/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                              common /elvm/
*  ......................................................................
*  ***   ATTENTION: X must be within a range of 10 GeV --- 1000 PeV   ***
*  ......................................................................
      COMMON /sok5/ XMIN,STEP,XMAX
      common /elvm/ C(19)
      X1 = alog10(X)
       if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
       write(0,*)'ERROR IN FUNCTION CONV: MUON ENERGY IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      conv = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      conv = exp(conv)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
****************************************************************************
* C.9
         FUNCTION gemleng(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*  The subroutine calculates the value for muon free path between 2 inte-
* ractions with energy transfers > EMIN using spline coefficients prepared 
*                          by spl1 subroutine.
*
*          X is a value for muon energy expressed in GeV; XMIN,
*  XMAX and STEP are assigned with some values in subroutine SPL1 (they
*               are passed here via /eminl/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                              common /eminl1/
*  ......................................................................
*  ***   ATTENTION: X must be within a range of 10 GeV --- 1000 PeV   ***
*  ......................................................................
      COMMON /eminl/ XMIN,STEP,XMAX
      common /eminl1/ C(19)
      X1 = alog10(X)
       if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
       write(0,*)'ERROR IN FUNCTION GEMLENG: MUON ENERGY IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
cccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      gemleng= (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      RETURN
cccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
****************************************************************************
* C.10
      subroutine frepathv
*
* This subroutine prepares the arrays to simulate the muon passage through
*  thick layers of matter with a model in which energy losses are divided
*   into i/ "continuous" losses with energy transfers less than VMIN and
*     ii/ "catastrophic" losses with energy transfers more than VMIN. 
*  Firstly, the energy range 10 GeV -- 1 EeV is divided in 128 subranges
*    where "continuous" lossses (calculated by CONV routine) are good 
*   aproximated linearly - dE/dx = a*E + b - and corresponding a and b  
*         for each subrange are computed (a(0:128), b(0:128)).
*  Also for each subrange values of integrals (1) and (2) are computed: 
*         
*            E0
*        INTEGRAL (dE / (dE/dx(E)["continuous"])   (1) -> leng(0:128)
*         10 GeV   
*         
*      E0
*  INTEGRAL (dE / (dE/dx(E)["continuous"] * L_mean(E))  (2) -> eta(0:128)
*    10 GeV   
*
*             where E0 is an "upper" edge of each segment:
*
*
*    leng(128),eta(128)                                leng(1),eta(1)              
*                                 .......
*         a,b(128)                                         a,b(1)
*            |                                               |
*            V                                               V
*      *-----------*-----------*    ...    *-----------*-----------*
*   e0(128)     e0(127)      e0(126)     e0(2)       e0(1)       e0(0)              
*   10 GeV        ........           1 EeV * fk^2  1 EeV * fk    1 EeV
*
*
*         e0(i): muon energy at the end of a segment Nb. i 
*                (starting from larger energy)
*
*         a(i),b(i): dE/dx(E) is pesentaed as a linear function
*                    (a(i) * E) + b(i) at segment Nb. i
*
*                 e0(i-1)
*      leng(i) = INTEGRAL (dE/(dE/dx(E))) - path which is passed by muon
*                 10 GeV                    without energy transfers > Vmin 
*                                           starting with E = 10 GeV and  
*                                           finishing with E = e0(i-1)
*
*                 e0(i-1)
*       eta(i) = INTEGRAL (dE / ( dE/dx(E) * L_mean(E) )
*                 10 GeV
*
*               All this is necessary in furter calculations to: 
*
*  - get the value of real free path between two interaction with energy 
*    transfers > VMIN taking into account that mean free path is not a
*    constant but a function of muon energy L_mean(E). So, to get the
*    energy E1 at the end of free path without energy transfers > Vmin
*    if current muon energy is equal to E0 and ETA = -ln(RNDM) where
*    RNDM is a rundom number uniformly distributed in a range 0. -- 1.
*    one need to solve the equation 
*
*                E0   
*             INTEGRAL (dE / ( dE/dx(E) * L_mean(E) ) = ETA     (3)
*                E1
*
*   instead of using L_mean(E0).
*
*  - get the muon energy at the end of free path between two interaction
*    if current muon energy is E0 and rundom number is ETA
*
*  - get the muon energy at a distance L from current point (looking back)
*    if its current energy is equal to E1 and it is known that there were
*    not catastrophic interactions at the path L.
*
*               THE OUTPUT OF SUBROUTINE IS AS FOLLOWS:
*
*   1. Arrays e0(0:128) a(0:128), b(0:128) and leng(1:128) which are used
*      just inside this routine but also are passed via COMMON /simv1/ to
*      routine GETEBACK along with constants FK, FK1, and DLNMAX which
*      are neccessary to detarmin the number subrange to which given
*      muon energy belongs. All this represents REAL*8 arrays and 
*      constants.
*
*   2. Array VPATH(111,161) with 17871 values of real free path (cm, see 
*      above) for 111 values of ETA and 161 values of E0. It is passed      
*      via common /vrand1/ to SPL2_2 routine to coock spline coefficients
*      which allow to calculate the real free path in further computing.
*      If muon energy at the end of free path is less than 10 GeV, the
*      real free path is calculated as  path till the point where muon
*      energy is equal to 10 GeV. Actually, elements of array represents
*      the values of real free path divided by ETA to make interpolation
*      to be more accurate.      
*
*   3. Array VENER(111,161) for energy at the end of real free path which
*      is determined at the same grid of initial energies and ETA as
*      VPATH(111,161) array. If energy at the end of real free path is
*      less than 10 GeV, the value of given VENER is asigned to 10 GeV.
*      Actually, the elements of vner are calculated as E1/E0 where
*      E0 is muon energy at the beginning of real free path, E1 is muon
*      energy at the end of free path. Array is passed via common 
*      /vrand2/ to SPL2_2 routine to coock spline coefficients which allow 
*      to calculate the real free path in further computing.
*
      external glbremv,glpairv,glphnuv,glelecv,conv,dsimps,gdedelt2
      real*8 e0(0:128),a(0:128),b(0:128),eta(0:128),leng(0:128)
      real*8 fk,fk1,dlnmax,slu,e,en,delta,dsimps
      real*8 low,up,step1,aux1(0:10),rest,eta_1,hd3,pat1,pat2
      real*8 ene,path
      common /vrand1/ vpath(111,161)
      common /vrand2/ vener(111,161)
      common /simv1/ fk,fk1,dlnmax,a,b,leng,e0
c
c   ---------------------------------------------------------------
c   1. Computing arrays e0(0:128), a(0:128), b(0:128), eta(0:128), 
c           leng(0:128) and constants FK, FK1 and DLNMAX
c
      fk=dexp(-(dlog(1.d+1)/1.6d+1)) !--> a coefficient to get 
      fk1 = 1.d+0 / dlog(fk)         !    e0(i)=fk*e0(i-1) and
      dlnmax = dlog(1.d+9)           !    some useful constants
c
      e0(0) = 1.d+9
        do i=1,128      
        e0(i) = e0(0) * (fk**dble(i))         !-> e0(i)  
        y2 = (conv(sngl(e0(i-1))) * 1.e-3)    !-> dE/dx (e0(i-1))
        y1 = (conv(sngl(e0(i))) * 1.e-3)      !-> dE/dx (e0(i))
        a(i) = dble((y2 - y1)) / (e0(i-1) - e0(i)) !-> a(i)
        b(i) = dble(y1) - a(i) * e0(i)             !-> b(i)
c
c                     Computing two integrals
c
c       ..................................................
c       .             e0(i-1)                            .
c       .   eta(i) = INTEGRAL(dE/(dE/dx(E) * L_mean(E))  .
c       .              e0(i)                             .
c       .                                                .
c       .                  e0(i-1)                       .
c       .       leng(i) = INTEGRAL (dE/(dE/dx(E)))       .
c       .                  e0(i)                         .
c       ..................................................
c 
c using a formula INTEGRAL [f(x) * dx] = INTEGRAL [x * f(x) * d(ln(x))] :
c
        low = dlog(e0(i))
        up = dlog(e0(i-1))
        lim1 = 10
        step1 = (up - low) / dble(lim1)
         do j=0,lim1
         ene = low + (dble(j) * step1)
         h1 = sngl(dexp(ene))
         help = (1./glbremv(h1)) + (1./glpairv(h1)) + (1./glphnuv(h1)) + 
     +           (1./glelecv(h1)) 
         aux1(j) = (1.d+0 / dble(help)) * dble(conv(h1))*1.d-3/dble(h1)
         aux1(j) = 1.d+0 / aux1(j)
         enddo
         eta(i) = dsimps(aux1,low,up,lim1)
c
         do j=0,lim1
         ene = low + (dble(j) * step1)
         h1 = sngl(dexp(ene))
         aux1(j) = dble(conv(h1)) * 1.d-3 / dble(h1)
         aux1(j) = 1.d+0 / aux1(j)
         enddo
         leng(i) = dsimps(aux1,low,up,lim1)
        enddo
c
      do i=127,1,-1
c
c              e0(i-1)
c   leng(i) = INTEGRAL (dE/(dE/dx(E))) :
c              10 GeV
c
      leng(i) = leng(i) + leng(i+1)
c
c            e0(i-1)
c  eta(i) = INTEGRAL(dE/(dE/dx(E) * L_mean(E))) :
c            10 GeV
c
      eta(i) = eta(i) + eta(i+1)
      enddo
c
c   Auxiliary arrays a(128), b(128), e0(128), eta(128), leng(128) 
c                    have been prepared.
c   ---------------------------------------------------------------
c                       2. Solving the equation (3)  
c
c  to get the final energy E1 for a set of E0 ("e" variable) and  
c                        ETA ("slu" variable):        
c
      do i=-80,30             !--> 111 values of SLU (logarithmi-
        slu = dble(i) * 5.d-2 !    cally equidestant grid with
        slu = 1.d+1**slu      !    slu_min=0.0001, slu_max=1000
        do j=180,20,-1         !-> 161 values of E (logarithmi-
          e = dble(j) * 5.d-2  !   cally equidestant grid with
          e = 1.d+1**e         !   e_min = 10 GeV, e_max = 1 EeV
            if (e.le.1.011d+1) then
            ene = 1.0000001d+1
            path = 0.d+0
            goto 444
            endif
c        vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv   
          me = idint( ( dlog(e) - dlog(1.d+9) ) / dlog(fk) ) + 1
c        ME is a number of segment which contains given energy E
c        ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
c                       Rewriting the equation (3) as
c
c                   E0         E0         E1
c                INTEGRAL = INTEGRAL - INTEGRAL = ETA  (3a)
c                   E1       10 GeV     10 GeV
c
c                                 or         
c
c                     E1         E0
c                  INTEGRAL = INTEGRAL - ETA = ETA_1   (3b)
c                   10 GeV     10 GeV
c
            if (me.lt.128) then
            eta_1 = eta(me+1) - slu
            else 
            eta_1 = (-1.d+0) * slu
            endif
        low = dlog(e0(me))
        up = dlog(e)
        lim1 = 10
        step1 = (up - low) / dble(lim1)
         do j1=0,lim1
         en = low + (dble(j1) * step1)
         h1 = sngl(dexp(en))
         help = (1./glbremv(h1)) + (1./glpairv(h1)) + (1./glphnuv(h1)) + 
     +           (1./glelecv(h1)) 
         aux1(j1) = (1.d+0 / dble(help)) * dble(conv(h1))*1.d-3/dble(h1)
         aux1(j1) = 1.d+0 / aux1(j1)
         enddo
         eta_1 = eta_1 + dsimps(aux1,low,up,lim1)
c---------------------------------
        if (eta_1.le.0.d+0) then !
        ene = 1.0000001d+1       !--> no interactions with energy transfers
        m1 = 128                 !              transfers > Vmin,
        goto 402                 !         the final energy is 10 GeV
        endif                    !
c---------------------------------
            m1 = 500
            do m=128,1,-1
              if (eta(m).ge.eta_1) then 
              m1 = m
              goto 401
              endif
            enddo
          if (m1.ge.200) then
          hd3 = dabs((eta(1) - eta_1) / eta(1))
             if (hd3.le.1.d-6) then
             m1 = 1
             eta_1 = eta(1)*9.9999999d-1
             goto 401
             endif
          write(0,*)'******** SUBROUTINE FREPATV:  ERROR !!!!! ********'
          goto 402
          endif
  401     continue
            if (m1.lt.128) then
            rest = eta_1 - eta(m1+1)
            else
            rest = eta_1
            endif
c
         ic = 0
         lim1 = 10
         ene = (e0(m1-1) + e0(m1)) * 5.d-1
         delta = e0(m1-1) - ene         
         low = dlog(e0(m1))
 3333    up = dlog(ene)
         step1 = (up - low) / dble(lim1)
         do j1=0,lim1
         en = low + (dble(j1) * step1)
         h1 = sngl(dexp(en))
         help = (1./glbremv(h1)) + (1./glpairv(h1)) + (1./glphnuv(h1)) + 
     +           (1./glelecv(h1)) 
         aux1(j1) = (1.d+0 / dble(help)) * dble(conv(h1))*1.d-3/dble(h1)
         aux1(j1) = 1.d+0 / aux1(j1)
         enddo
         pat1 = dsimps(aux1,low,up,lim1)
         delta = 5.d-1 * delta
         ic = ic + 1
         if (pat1.le.rest) then
         ene = ene + delta         
         else
         ene = ene - delta
         endif
         if (ic.eq.20) then
         goto 402
         endif
         goto 3333
 402     continue
c
c            Equation (3) has been solved., the root is ENE
c   ---------------------------------------------------------------
c            3. Computing of real free path from E0 to ENE:   
c
c                            E0
c                  PATH = INTEGRAL [ dE/(dE/dx(E)) ] =
c                            ENE
c
c          E0                          ENE
c     = INTEGRAL [ dE/(dE/dx(E)) ] - INTEGRAL [ dE/(dE/dx(E)) ]
c        10 GeV                       10 GeV 
c
      if (me.lt.128) then
        low = dlog(e0(me))
        up = dlog(e)
        lim1 = 10
        step1 = (up - low) / dble(lim1)
         do j2=0,lim1
         en = low + (dble(j2) * step1)
         h1 = sngl(dexp(en))
         aux1(j2) = dble(conv(h1)) * 1.d-3 / dble(h1)
         aux1(j2) = 1.d+0 / aux1(j2)
         enddo
         pat1 = dsimps(aux1,low,up,lim1) + leng(me+1)
      else
        low = dlog(e0(me))
        up = dlog(e)
        lim1 = 10
        step1 = (up - low) / dble(lim1)
         do j2=0,lim1
         en = low + (dble(j2) * step1)
         h1 = sngl(dexp(en))
         aux1(j2) = dble(conv(h1)) * 1.d-3 / dble(h1)
         aux1(j2) = 1.d+0 / aux1(j2)
         enddo
         pat1 = dsimps(aux1,low,up,lim1)
      endif
c
      if (m1.lt.128) then
        low = dlog(e0(m1))
        up = dlog(ene)
        lim1 = 10
        step1 = (up - low) / dble(lim1)
         do j2=0,lim1
         en = low + (dble(j2) * step1)
         h1 = sngl(dexp(en))
         aux1(j2) = dble(conv(h1)) * 1.d-3 / dble(h1)
         aux1(j2) = 1.d+0 / aux1(j2)
         enddo
         pat2 = dsimps(aux1,low,up,lim1) + leng(m1+1)
      else
        low = dlog(e0(m1))
        up = dlog(ene)
        lim1 = 10
        step1 = (up - low) / dble(lim1)
         do j2=0,lim1
         en = low + (dble(j2) * step1)
         h1 = sngl(dexp(en))
         aux1(j2) = dble(conv(h1)) * 1.d-3 / dble(h1)
         aux1(j2) = 1.d+0 / aux1(j2)
         enddo
         pat2 = dsimps(aux1,low,up,lim1)
      endif
      path = pat1 - pat2
c
c          The equation is solved, the root is PATH
c
  444 continue
      if (path.le.0.d+0) path = 1.d+0
      if (ene.le.1.00001d+1) ene = 9.999d+0 
      path = path/slu
      ene = ene / e
      vpath(i+81,j-19) = sngl(path)
      vener(i+81,j-19) = sngl(ene)
        enddo
      enddo
c
c   ALL CALCULATIONS HAVE BEEN FINISHAD, ARRAYS HAVE BEEN PREPARED    
c--------------------------------------------------------------------
      return
      end
****************************************************************************
* C.11
         SUBROUTINE spl2_2
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*     SPL2_2 produces arrays of spline coefficients for 2-dimensional 
*    interpolatiion and passes them via common blocks to corresponding 
*                             subroutines.
*  .....................................................................
      common /vrand1/ vpath(111,161) !-> input arrays from FREPATHV
      common /vrand2/ vener(111,161) !          routine
c
      common /vrand1_o/ CPA(18419)   !-> output arrays with spline coef-
      common /vrand2_o/ CEN(18419)   !   ficients to be passed to routines
c GETERNAV and GETLANRV to perform interpolation
c
       common /vpath1/ NX_1,NY_1,X0_1,SX_1,Y0_1,SY_1 !-> constants to be
       common /vpath2/ NX_2,NY_2,X0_2,SX_2,Y0_2,SY_2 !   passed to GETERANV
c and GETLANRV along with splines to perform interpolation (for explanati-
c ons see comments to SPL2 routine).
c
      DIMENSION D(130,180),FU(111,161),CU(18419) !----->  Auxiliary arrays
c
      DIMENSION NXG(2),NYG(2),X0G(2),SXG(2),Y0G(2),SYG(2) ! -> some arrays 
c which contain values for NX_1, NY_1, etc. (see comments to COMMONs 
c /vpath1/ and /vpath2/ above).
c
      DIMENSION IJ(2) !---> array which contains dimensions for output 1-di-
c  mentional arrays with splines. Dimention of output array should be equal
c  to (I+2)*(J+2), where I and J are dimensions for corresponding input 2-di-
c  mensional array. 
      data NXG/111,111/        ! See comments above for ex-
      data NYG/161,161/        ! planations of these DATAs
      data X0G/-4.,-4./        !
      data SXG/5.e-2,5.e-2/    !
      data Y0G/1.,1./          !
      data SYG/5.e-2,5.e-2/    !
      data IJ/18419,18419/     !
c
      do li=1,2     ! A cycle along all input arrays
      NX = NXG(li)  ! assigns values from corresonding arrais for numbers of
      NY = NYG(li)  ! values, steps and initial values
      X0 = X0G(li)  !
      SX = SXG(li)  !
      Y0 = Y0G(li)  !
      SY = SYG(li)  !
c
       if (li.eq.1) then
         NX_1 = NX     !-> number of X values
         NY_1 = NY     !-> number of Y values
         X0_1 = X0     !-> first value of X
         SX_1 = SX     !-> step by X
         Y0_1 = Y0     !-> first value of Y
         SY_1 = SY     !-> step by Y
       endif
c
       if (li.eq.2) then
         NX_2 = NX    !-> number of X values 
         NY_2 = NY    !-> number of Y values
         X0_2 = X0    !-> first value of X
         SX_2 = SX    !-> step by X
         Y0_2 = Y0    !-> first value of Y
         SY_2 = SY    !-> step by Y
       endif
c
        do ki=1,NX                                ! Filling an auxiliary
           do kl=1,NY                              ! array FU by values 
              if (li.eq.1) FU(ki,kl) = vpath(ki,kl) ! from input array
              if (li.eq.2) FU(ki,kl) = vener(ki,kl) ! (within a cycle by
c                                                  ! LI along all input
c                                                 ! arrays)
           enddo                                 !
        enddo                                   !
c-----------------------------------------------
      DO 1 J=1,NY                  !
      J2=J+2                       !
      DO 1 I=1,NX                  !
      I2=I+2                       !
1     D(I2,J2)=3.90625E-3*FU(I,J)  !
      J1=NY+1                      !
      J3=J2+1                      !
      J4=J3+1                      !
      DO 2 I=3,I2                  !
      A=D(I,3)                     !
      B=D(I,4)                     !---> Cooking spline coefficients
      D(I,2)=3.*(A-B)+D(I,5)       !     out of input array Nb. LI
      D(I,1)=3.*(D(I,2)-A)+B       !     and putting these splines
      A=D(I,J1)                    !     into 1-dimensional array
      B=D(I,J2)                    !                 CU
      D(I,J3)=3.*(B-A)+D(I,NY)     !
2     D(I,J4)=3.*(D(I,J3)-B)+A     !
      I1=NX+1                      !
      I3=I2+1                      !
      I4=I3+1                      !
      DO 3 J=1,J4                  !
      A=D(3,J)                     !
      B=D(4,J)                     !
      D(2,J)=3.*(A-B)+D(5,J)       !
      D(1,J)=3.*(D(2,J)-A)+B       !
      A=D(I1,J)                    !
      B=D(I2,J)                    !
      D(I3,J)=3.*(B-A)+D(NX,J)       !
3     D(I4,J)=3.*(D(I3,J)-B)+A         !
      DO 4 J=1,J2                        !
      J3=J+1                               !
      J4=J+2                                 !
      M=(J-1)*I2                               !
      DO 4 I=1,I2                                !
      I3=I+1                                       !
      I4=I+2                                         !
4     CU(M+I)=D(I,J)+D(I,J4)+D(I4,J)+D(I4,J4)+D(I3,J3)* !
     *100.-(D(I3,J)+D(I,J3)+D(I4,J3)+D(I3,J4))*10.      !
c--------------------------------------------------------
       ko = IJ(li)
       do ki=1,ko                            !
              if (li.eq.1) CPA(ki) = CU(ki)  ! Passing the values of spline
              if (li.eq.2) CEN(ki) = CU(ki)  ! coefficients form auxiliary
c                                            ! array CU to corresponding
c                                            ! output array Nb. LI
       enddo                                 !
      enddo
      RETURN
      END
****************************************************************************
* C.12
       FUNCTION getlanrv(X,Y)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*    The routine performs a spline interpolation by spline coefficients 
*   prepared by SPL2_2 routine. It determines a value for real free path
* without energy transfers > VMIN if initial muon energy is equal to Y (GeV)
*   and X = -ln(RNDM) (RNDM is a random number uniformly distributed in a
*     range of 0. -- 1.). Y should be within a range of 10 GeV - 1 EeV,
*                   X should be in a range 0.0001 - 31.62.
* The output GETLANRV(X,Y) represent the value for real free path (cm) for
*                               given X and Y.
*  ......................................................................
       common /vrand1_o/ C1(18419)
       common /vpath1/ NX,NY,X0,SX,Y0,SY
c
      X_1=alog10(X)
      if ((X_1.lt.-4.0001).or.(X_1.gt.1.5001)) then
      write(0,*)'ERROR IN FUNCTION GETLANRV: RANDOM NB. IS OUT OF RANGE'
      endif
c
      Y_1=alog10(Y)
      if ((Y_1.lt..99999).or.(Y_1.gt.9.0001)) then
      write(0,*)'ERROR IN FUNCTION GETLANRV: MUON ENERGY IS OUT OF RANGE'
      endif
c
      A3=(X_1-X0)/SX
      B3=(Y_1-Y0)/SY
      M1=INT(A3+.5)
      M2=INT(B3+.5)
      M3=NX+2
      IF(M1.LT.0.OR.M1.GE.NX.OR.M2.LT.0.OR.M2.GE.NY) THEN
      PRINT 1,X_1,M1,Y_1,M2
      ENDIF
      A3=A3-M1
      B3=B3-M2
      M1=M2*M3+M1+1
      M2=M1+M3
      M3=M2+M3
      A2 = A3*A3 +.25
      B2 = B3*B3 +.25
      A1=A2-A3
      B1=B2-B3
      A3=A2+A3
      B3=B2+B3
      A2=2.-(A2+A2)
      B2=2.-(B2+B2)
      getlanrv=(A1*C1(M1)+A2*C1(M1+1)+A3*C1(M1+2))*B1
     2      +(A1*C1(M2)+A2*C1(M2+1)+A3*C1(M2+2))*B2
     3      +(A1*C1(M3)+A2*C1(M3+1)+A3*C1(M3+2))*B3
      getlanrv = getlanrv * X
      RETURN
1     FORMAT('*MISTAKE*X=',D23.16,'MX=',I4,'Y=',D23.16,'MY=',I4)
      END
****************************************************************************
* C.13
       FUNCTION geteranv(X,Y)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*     The routine performs a spline interpolation by spline coefficients 
*  prepared by SPL2_2 routine. It determines a value for muon energy at the 
*   end of real free path without energy transfers > VMIN if initial muon
*   energy is equal to Y (GeV) and X = -ln(RNDM) (RNDM is a random number 
*  uniformly distributed in a range of 0. -- 1.). Y should be within a range 
*      of 10 GeV - 1 EeV, X should be in a range 0.0001 - 31.62.
*  The output GETERANV(X,Y) represent the relation of muon energy at the end 
*         of free path to initial energy for given X and Y.
*  ......................................................................
       common /vrand2_o/ C1(18419)
       common /vpath2/ NX,NY,X0,SX,Y0,SY
c
      X_1=alog10(X)
      if ((X_1.lt.-4.0001).or.(X_1.gt.1.5001)) then
      write(0,*)'ERROR IN FUNCTION GETERANV: RANDOM NB. IS OUT OF RANGE'
      endif
c
      Y_1=alog10(Y)
      if ((Y_1.lt..99999).or.(Y_1.gt.9.0001)) then
      write(0,*)'ERROR IN FUNCTION GETERANV: MUON ENERGY IS OUT OF RANGE'
      endif
c
      A3=(X_1-X0)/SX
      B3=(Y_1-Y0)/SY
      M1=INT(A3+.5)
      M2=INT(B3+.5)
      M3=NX+2
      IF(M1.LT.0.OR.M1.GE.NX.OR.M2.LT.0.OR.M2.GE.NY) THEN
      PRINT 1,X_1,M1,Y_1,M2
      ENDIF
      A3=A3-M1
      B3=B3-M2
      M1=M2*M3+M1+1
      M2=M1+M3
      M3=M2+M3
      A2 = A3*A3 +.25
      B2 = B3*B3 +.25
      A1=A2-A3
      B1=B2-B3
      A3=A2+A3
      B3=B2+B3
      A2=2.-(A2+A2)
      B2=2.-(B2+B2)
      geteranv=(A1*C1(M1)+A2*C1(M1+1)+A3*C1(M1+2))*B1
     2      +(A1*C1(M2)+A2*C1(M2+1)+A3*C1(M2+2))*B2
     3      +(A1*C1(M3)+A2*C1(M3+1)+A3*C1(M3+2))*B3
      RETURN
1     FORMAT('*MISTAKE*X=',D23.16,'MX=',I4,'Y=',D23.16,'MY=',I4)
      END
****************************************************************************
* C.14
      function geteback(e1,backl)
*
* The routine gives the energy ENE at the begining of muon free path without 
*   interactions with energy transfers > Vmin if its energy at the end of
*    this free path is equal to E1 (GeV) and length of free path is BACKL
*   (cm). Routine uses arrays A, B, LENG, E0, and constants FK, FK1, DLMAX
*      which were prepared during inicialization by SUBROUTINE FREEPATHV.
*                  The method is to solve an equation
*
*                    ENE   
*                  INTEGRAL (dE / ( dE/dx(E) ) = BACKL    (*)
*                     E1
*
*   BACKL should be in a range 1 cm -- 100 km, E1 should be in a range 
*    10 GeV -- 1 EeV, the output is energy expressed in GeV. If ENE is
*       found to be more than 1 EeV it is assigned by 1.00001 EeV.
* ........................................................................
      real*8 fk,fk1,dlnmax,a(0:128),b(0:128),leng(0:128),e0(0:128)
      real*8 e,ene,rest,pat1,path 
      common /simv1/ fk,fk1,dlnmax,a,b,leng,e0
c
          e = dble(e1)
          path = dble(backl)
          if (e.ge.1.d+9) then  !
          ene = 1.00001+9         !-> if E = 1 EeV than let's 
          goto 1402                !    ENE to be 1 EeV, too
          endif
c       vvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvvv   
        me = idint( ( dlog(e) - dlnmax ) * fk1 )  + 1
        if (me.gt.128) me = 128
c       ME is a number of segment which contains given energy E
c                (see comments to subroutine FREPATHV )
c       ^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^^
c                       Rewriting the equation (*) as
c
c                  ENE         ENE        E1
c                INTEGRAL = INTEGRAL - INTEGRAL = PATH
c                  E1        10 GeV     10 GeV
c
c                                 or         
c
c                     ENE        E1
c                  INTEGRAL = INTEGRAL + PATH = PAT1 (**)
c                   10 GeV     10 GeV
c
            if (me.lt.128) then
            pat1 = leng(me+1) + path
            else 
            pat1 = path
            endif
c
c                        Computing of PAT1 as
c
c                                        a(me)*E + b(me)
c       pat1 = pat1 + (1/a(me)) * ln(---------------------)
c                                     a(me)*e0(me) + b(me)
c
c      where [a * E + b] is a linear presentation of dE/dx(E): 
c
            pat1=pat1+dlog((a(me)*e+b(me))/(a(me)*e0(me)+b(me)))/a(me)
c
            m1 = 500
            do m=128,1,-1
              if (leng(m).ge.pat1) then 
              m1 = m
              goto 1401
              endif
            enddo
           if (m1.ge.200) then  ! -> ENE for given PATH is more
           ene = 1.00001d+9     !    than 1 EeV (out of reason-
           goto 1402            !     able range) - let it be
           endif                !           just 1 EeV
 1401       continue
            if (m1.lt.128) then
            rest = pat1 - leng(m1+1)
            else
            rest = pat1
            endif
c
c   The only thing to do to find a root of equation (*) is now to
c           solve the following equation for variable ENE:
c
c                             a(m1)*ENE + b(m1)
c           (1/a(m1)) * ln(---------------------) = rest
c                           a(m1)*e0(m1) + b(m1)
c
c      where [a * E + b] is a linear presentation of dE/dx(E): 
c
          ene = ((a(m1)*e0(m1)+b(m1))*dexp(rest*a(m1))-b(m1))/a(m1)
 1402     continue
          geteback = sngl(ene)
      return
      end
c
****************************************************************************
* C.15
       subroutine earray1(e,depth)
*  
*   Simulates the passage of muon through an array at the distance = DEPTH 
*   (cm) if initial muon energy is equal to E (GeV). Uses a model in which 
*   energy losses are divided into 2 part: "continuous" losses with energy 
*  transfers < EMIN and "catastrophic" losses with energy transfers > EMIN. 
*
*                             OUTPUT:         
*                             =======
*
*  The procedure gives some "history" of simulation which is kept by arrays
*  arrays ityp_e(20000), eleng_e(20000), ener1_e(20000),ener2_e(20000) and 
* constant NUMB_E. All this is passed via common/EHISTORY/ to program block
* from which EARRAY1 was called. This block must contain the following lines:
*
*                             ..... 
*       real*8 ener1_e(20000), ener2_e(20000)
*       real*4 eleng_e(20000),etr(20000)
*       integer ityp_e(20000),numb_e
*       common /ehistory/ ener1_e,ener2_e,eleng_e,etr,ityp_e,numb_e
*                            .....
*
*
*         DESCRIPTION OF CONSTANTS AND ARRAYS OF COMMON /EHISTORY/ :
*
* NUMB_E (integer*4) is equal to number of interaction which occured during 
*                    simulation for given muon
*
* ITYP_E(i) (integer*4) gives the type of interaction nb. i which may be as 
*                     follows:
*
*  0 - start of simulation
*  1 - muon has reached the DEPTH
*  2 - muon has stopped (its energy became less than 0.16 GeV)
*  4 - bremsstrahlung interactions occured
*  5 - e+e- pair has been born
*  6 - photonuclear interaction occured
*  7 - knock-on electron has been born (or bremsstrahlung interaction
*      with emmiting gamma by electron occured)
*
*  ELENG_E(i) (real*4) - distance (cm) from the beginning of simulation
*                        till interaction nb. i
*
*  ENER1_E(i) (real*8) - energy (GeV) before interaction nb. i ! For ityp=
*                                                              !  0,1,2,3
*  ENER2_E(i) (real*8) - energy (GeV) aftter interaction nb. i ! it is the
*                                                              !    same
*
*  ETR(i) (real*4) - energy transfer due to interaction nb.i (GeV), is
*                    assigned by 0. if ITYP_E < 4
*
*     DEPTH should not be more than 2 km, E should be within a range 
*                          10 GeV - 1 EeV.
* ........................................................................
       external getlpair,getlbrem,getlphnu,getlelec,simps,gdedelt2
       external gemleng
       real*8 e0,e1,e2
       real*8 ener1_e(20000), ener2_e(20000)
       real*4 eleng_e(20000),etr(20000)
       integer ityp_e(20000),numb_e
       dimension aux1(0:30)
       common /ehistory/ ener1_e,ener2_e,eleng_e,etr,ityp_e,numb_e
       parameter (itra=0)
c
c        Let's check if input is within alowwed range;
c
       if (depth.gt.2.0001e+5) then
       write(0,*)'FUNCTION EARRAY1: INPUT VALUE FOR DISTANCE ',depth,' cm'
       write(0,*)'           IS TOO LARGE, WILL NOT WORK'
       return
       endif
       if ((e.gt.1.0001e+9).or.(e.lt.10.1)) then
       write(0,*)'FUNCTION EARRAY1: INPUT VALUE FOR ENERGY ',e,' GeV'
       write(0,*)'          IS OUT OF RANGE, WILL NOT WORK'
       return
       endif
c ---------------------------------
       numb_e = 1                 !
       ityp_e(numb_e) = 0         !
       eleng_e(numb_e) = 0.       !-> Tracking simulation history
       ener1_e(numb_e) = dble(e)  !
       ener2_e(numb_e) = dble(e)  !
       etr(numb_e) = 0.           !
c----------------------------------
       pat = 0.
       e0 = dble(e)
    1  eta = -alog(rndm(5))       
       preal = eta * gemleng(sngl(e0))
       elos = cone(sngl(e0))
       e1 = e0 - dble(elos * preal * 1.d-3)
       if (e1.le.1.d+1) then       
        e1 = 1.d+1
        pat = pat + (sngl(e0 - e1)) * 1.e+3 / elos
           if (pat.ge.depth) then
           rest = pat - depth
           e1 = e1 + dble(elos * rest * 1.e-3)
c ---------------------------------
           numb_e = numb_e + 1     !
           ityp_e(numb_e) = 1      !
           eleng_e(numb_e) = depth !-> Tracking simulation history
           ener1_e(numb_e) = e1    !
           ener2_e(numb_e) = e1    !
           etr(numb_e) = 0.        !
c----------------------------------
           return
           endif
        el = sngl(e1)
        rest = depth - pat
        goto 4321         
       else
        pat = pat + preal
           if (pat.ge.depth) then
           rest = pat - depth
           e1 = e1 + dble(elos * rest * 1.e-3)
c ---------------------------------
           numb_e = numb_e + 1     !
           ityp_e(numb_e) = 1      !
           eleng_e(numb_e) = depth !-> Tracking simulation history
           ener1_e(numb_e) = e1    !
           ener2_e(numb_e) = e1    !
           etr(numb_e) = 0.        !
c----------------------------------
           return
           endif
        numb_e = numb_e + 1 !-> Tracking simulation history
        e11 = sngl(e1)             !
        pb = 1./getlbrem(e11)      !-> Computing of "weight" for
        pp = pb + 1./getlpair(e11) !   different interactions at
        pn = pp + 1./getlphnu(e11) !   current muon energy.
        pt = pn + 1./getlelec(e11) !
        ranp = rndm(8) * pt !---------> Simulation of interaction type
        if(ranp.le.pb) then          !-> type was simulated as bremsstrahlung,
         call getvbrem(e11,v,itra)   !   simulate rel. energy transfer
         ityp_e(numb_e) = 4 !-> tracking simulation history
         else 
          if(ranp.le.pp) then       !-> type was simulated as e+e- pair,
          call getvpa(e11,v,itra)   !   simulate rel. energy transfer
         ityp_e(numb_e) = 5 !-> tracking simulation history
          else
              if (ranp.le.pn) then    !-> type was simulated as photonuc.,
              call getvph(e11,v,itra) !   simulate rel. energy transfer
              ityp_e(numb_e) = 6 !-> tracking simulation history
                else
              call getvel(e11,v,itra)!-> type was simulated as knock-on
c                                    !  electron, simulate en. transfer
              ityp_e(numb_e) = 7 !-> tracking simulation history
              endif
          endif
        endif
        e2 = e1 * ( 1.d+0 - dble(v) ) ! -> The energy after interaction
c---------------------------------------
           eleng_e(numb_e) = pat       !
           ener1_e(numb_e) = e1        !-> tracking simulation history
           ener2_e(numb_e) = e2        !
           etr(numb_e) = dble(v) * e1  !
c---------------------------------------
          if (e2.le.1.d+1) then
             if (e2.le.1.6d-1) then !-> the muon stops (energy is below
                 e2 = 1.d-2         !   the Cherenkov threshold in water)
c---------------------------------------- 
                 numb_e = numb_e + 1     !
                 ityp_e(numb_e) = 2      !-> tracking simulation history
                 eleng_e(numb_e) = pat   !
                 ener1_e(numb_e) = e2    !
                 ener2_e(numb_e) = e2    !
                 etr(numb_e) = 0.        !
c----------------------------------------
                 return          !   Assigne energy = 0.01 and return...
             else
                 rest = depth - pat
                 el=sngl(e2) !-> muon has not stopped but its energy is
                 goto 4321   !   less than 10 GeV. Jump to label 4321...
             endif
          else
             e0 = e2  !-> muon enrgy is above 10 GeV. Jump to
             goto 1   !   label 1 to repeat everything once
          endif       !          again...
       endif       
c
c  Muon energy becomes less than 10 GeV. Compute the rest of its
c          travel with continuous losses only:
c
 4321   lim1 = 30
        elow = alog(1.6e-1)
        up = alog(el)
        step1 = (up - elow) / float(lim1)
         do j1=0,lim1                       
         en = elow + (float(j1) * step1)
         h1 = exp(en)
         aux1(j1) = h1 * 1.e+3 / gdedelt2(h1)
         enddo
         pat1 = simps(aux1,elow,up,lim1)
         if (pat1.lt.rest) then            ! Muon energy becomes < 0.16
         e1 = 1.d-2                        ! GeV before it reaches DEPTH
c-------------------------------------- 
         numb_e = numb_e + 1          !
         ityp_e(numb_e) = 2           !-> tracking simulation
         eleng_e(numb_e) = pat + pat1 !     history
         ener1_e(numb_e) = e1         !
         ener2_e(numb_e) = e1         !
         etr(numb_e) = 0.             !
c--------------------------------------
         return                            ! Energy = 0.01 GeV and return...
         endif
c-------------------------------------------                  
         ic = 0                            ! Iteration procedure to 
         ene = (el + 1.59999e-1) * 5.e-1   ! obtain muon energy at 
         delta = el - ene                  ! the DEPTH if his start
 3333    elow = alog(ene)                   ! energy is less than 
         step1 = (up - elow) / float(lim1)   ! 10 GeV (without "ca-
         do j1=0,lim1                         ! tastrophic" part)
         en = elow + (float(j1) * step1)      !
         h1 = exp(en)                         !
         aux1(j1) = h1 * 1.e+3 / gdedelt2(h1) !
         enddo                                !
         pat1 = simps(aux1,elow,up,lim1)    !
         delta = 5.e-1 * delta            !
         ic = ic + 1                    !
         if (pat1.eq.rest) then       !-> it seems incredible but    
         e1 = dble(ene)                    !   sometimes it occures...
c --------------------------------              !    
         numb_e = numb_e + 1     !                      !
         ityp_e(numb_e) = 1      !-> tracking simulation  !
         eleng_e(numb_e) = depth !    history            !
         ener1_e(numb_e) = e1    !                   !
         ener2_e(numb_e) = e1    !               !
         etr(numb_e) = 0.        !            !
c --------------------------------        !
         return                        !
         endif                        !
         if (pat1.le.rest) then      !
         ene = ene - delta            !
         else                           !
         ene = ene + delta               !
         endif                            !
         if (ic.eq.14) then                !
         e1 = dble(ene)                         ! -> We have reach DEPTH
c --------------------------------                  !     and finish...
         numb_e = numb_e + 1     !                      !
         ityp_e(numb_e) = 1      !-> tracking simulation  !
         eleng_e(numb_e) = depth !    history            !
         ener1_e(numb_e) = e1    !                   !
         ener2_e(numb_e) = e1    !               !
         etr(numb_e) = 0         !            !
c --------------------------------       !
         goto 9811                   !
         endif                   !
         goto 3333            !
c----------------------------
 9811 return
      end
****************************************************************************
* C.16
       subroutine earray2(e,en,etr,path,jtyp)
*  
* Simulates "the next" interaction for a muon which passes an array if muon 
*   energy is equal to E (GeV). Uses a model in which  energy losses are 
* divided into 2 part: "continuous" losses with energy transfers < EMIN and 
*        "catastrophic" losses with energy transfers > EMIN. 
*
*                             INPUT:         
*                             ======
*   E which is muon energy expressed in Gev (it should be less than 1 EeV)
*
*                             OUTPUT:         
*                             =======
*
*  EN - the muon energy after the next interaction (including  !-> REAL*8
*       "continuous losses" till the point of interaction)     !
*  ETR - energy transfer (GeV) due to "next" interaction            ! REAL*4
*  PATH - distance from initial point to the next interaction (cm)  ! 
*  INTEGER*4 JTYP - type of interaction which may be as follows:
*
*  2 - muon has stopped (its energy became 0.16 GeV)
*  4 - bremsstrahlung interactions occured
*  5 - e+e- pair has been born
*  6 - photonuclear interaction occured
*  7 - knock-on electron has been born (or bremsstrahlung interaction
*      with emmiting gamma by electron occured)
*
* If initial energy is less than 0.16 GeV, EN is assigned by a value of 
*                             0.01 GeV 
* ........................................................................
       external getlpair,getlbrem,getlphnu,getlelec,simps,gdedelt2
       external gemleng
       real*8 en,e1
       dimension aux1(0:30)
       parameter (itra=0)
c
c        Let's check if input is within alowed range;
c
       if (e.gt.1.0001e+9) then
       write(0,*)'FUNCTION EARRAY2: INPUT VALUE FOR ENERGY ',e,' GeV'
       write(0,*)'          IS OUT OF RANGE, WILL NOT WORK'
       return
       endif
       e0 = e
       if (e0.lt..16) then
       en = 1.d-2
       etr = 0.
       jtyp = 2
       path = 0.
       return
       endif
       if (e0.le.10.) then
       pat = 0.
       el = e0
       goto 4321
       endif
       eta = -alog(rndm(5))       
       preal = eta * gemleng(e0)
       elos = cone(e0)
       e1 = dble(e0) - dble(elos * preal * 1.e-3)
       if (e1.le.1.d+1) then       
        pat = (e0 - 10.) * 1.e+3 / elos
        el = 10.
        goto 4321         
       else
        e11 = sngl(e1)
        pb = 1./getlbrem(e11)      !-> Computing of "weight" for
        pp = pb + 1./getlpair(e11) !   different interactions at
        pn = pp + 1./getlphnu(e11) !   current muon energy.
        pt = pn + 1./getlelec(e11) !
        ranp = rndm(8) * pt !---------> Simulation of interaction type
        if(ranp.le.pb) then        !-> type was simulated as bremsstrahlung,
         call getvbrem(e11,v,itra)   !   simulate rel. energy transfer
         jtyp = 4
         else 
          if(ranp.le.pp) then      !-> type was simulated as e+e- pair,
          call getvpa(e11,v,itra)   !   simulate rel. energy transfer
          jtyp = 5
          else
              if (ranp.le.pn) then   !-> type was simulated as photonuc.,
              call getvph(e11,v,itra) !   simulate rel. energy transfer
              jtyp = 6
                else
              call getvel(e11,v,itra) !-> type was simulated as knock-on
              jtyp = 7
c                                    !  electron, simulate en. transfer
              endif
          endif
        endif
        etr = sngl(e1) * v  !-> transferred energy
        en = e1 * (1.d+0 - dble(v)) ! -> The energy after interaction
        path = preal
        return
       endif       
c
c  Muon energy becomes less than 10 GeV. Compute the rest of its
c          travel with continuous losses only:
c
 4321   lim1 = 30
        elow = alog(1.6e-1)
        up = alog(el)
        step1 = (up - elow) / float(lim1)
         do j1=0,lim1                       
         en1 = elow + (float(j1) * step1)
         h1 = exp(en1)
         aux1(j1) = h1 * 1.e+3 / gdedelt2(h1)
         enddo
         path = pat + simps(aux1,elow,up,lim1)
         en = 1.6d-1
         jtyp = 2
         etr = 0.
      return
      end
****************************************************************************
* C.17
       function enew(e,depth)
*  
* Simulates the energy of muon at a large depth = DEPTH (cm) if initial muon
*    energy is equal to E (GEV). Uses a model in which energy losses are
*   divided into 2 part: "continuous" losses with energy transfers < VMIN
*        and "catastrophic" losses with energy transfers > VMIN. 
*
*    Output: ENEW is simulated muon energy (GeV) at the depth DEPTH with 
*            initial energy E. If muon does not reach the DEPTH (its energy 
*            is less than 0.16 Gev) ENEW is assigned by 0.01 GeV
*
*   The procedure gives also some "history" of simulation which is kept by
*  arrays ityp(10000), eleng(10000), ener1(10000),ener2(10000) and constant
*   NUMB. All this is passed via common/VHISTORY/ and may be used after the
*                   simulation for given muon is completed. 
*
* NUMB (integer*4) is equal to number of interaction which occured during 
*                  simulation for given muon
*
* ITYP(i) (integer*4) gives the type of interaction nb. i which may be as 
*                     follows:
*
*  0 - start of simulation
*  1 - muon has reached the DEPTH (which is input variable for ENEW)
*  2 - muon has stopped (its energy became less than 0.16 GeV)
*  3 - muon energy has decreased down to 10 GeV
*  4 - bremsstrahlung interactions occured
*  5 - e+e- pair has been born
*  6 - photonuclear interaction occured
*  7 - knock-on electron has been born (or bremsstrahlung interaction
*      with emmiting gamma by electron occured)
*
*  ELENG(i) (real*4) - distance (cm) from the beginning of simulation
*                      till interaction nb. i
*
*  ENER1(i) (real*4) - energy (GeV) before interaction nb. i ! For ityp=
*                                                            !  0,1,2,3
*  ENER2(i) (real*4) - energy (GeV) aftter interaction nb. i ! it is the
*                                                            !    same
*
*     DEPTH should not be more than 100 km, E should be within a range 
*                          10 GeV - 1 EeV.
* ........................................................................
       external getlanrv,geteranv,geteback,gdedelt2
       external glpairv,glbremv,glphnuv,glelecv,simps
       real*8 rvec(1),eta,e0,pat,ddepth,e1d
       dimension aux1(0:30)
       dimension ityp(10000),eleng(10000),ener1(10000),ener2(10000)
       common /vhistory/ numb,ityp,eleng,ener1,ener2
       common /r48/ rvec
       parameter (itra=1)
       parameter (len=1)
c
c        Let's check if input is within alowwed range;
c
       if (depth.gt.1.0001e+7) then
       write(0,*)'FUNCTION ENEW: INPUT VALUE FOR DEPTH ',depth,' cm'
       write(0,*)'           IS TOO LARGE, WILL NOT WORK'
       return
       endif
       if ((e.gt.1.0001e+9).or.(e.lt.10.1)) then
       write(0,*)'FUNCTION ENEW: INPUT VALUE FOR ENERGY ',e,' GeV'
       write(0,*)'          IS OUT OF RANGE, WILL NOT WORK'
       return
       endif
c -----------------------
       numb = 1         !
       ityp(numb) = 0   !
       eleng(numb) = 0. !-> Tracking simulation history
       ener1(numb) = e  !
       ener2(numb) = e  !
c------------------------
       ddepth = dble(depth) 
       e0 = dble(e)
       pat = 0.d+0
    1  call rm48(rvec,len)    !
       eta = -dlog(rvec(1))   !---> Simulation of random number       
       if(eta.ge.3.16d+1) eta = 3.16d+1
       slu = sngl(eta)
       e0_sn = sngl(e0)
       if (slu.ge.1.e-4) then
       preal = getlanrv(slu,e0_sn)  !-> getting the real free path
       e1d = e0 * dble(geteranv(slu,e0_sn)) !-> getting muon energy 
c                                               at the end of free
c                                               path 
       else
        preal = 0.  !--> simulated free path is too small, let
        e1d = e0    !    it be equal to zero...
       endif
        e1 = sngl(e1d)
c       
       pat = pat + dble(preal)
       rest = sngl(ddepth - pat)
         if (rest.lt.0.) then   !-> muon has passed the DEPTH,
           rest = -rest           ! 
           if (rest.lt.1.) rest=1.  !  Computing its "back" energy
           if (e1.lt.10.) e1 = 10.   ! at the DEPTH and return...
           enew = geteback(e1,rest)  !
c-------------------------------
           numb = numb + 1     !
           ityp(numb) = 1      !-> Tracking
           eleng(numb) = depth ! simulation
           ener1(numb) = enew  !  history
           ener2(numb) = enew  !
c------------------------------
           return
         endif
c
         if (e1.le.1.e+1) then  !-> muon energy at the end of free 
           el = 10.             !   path is less than 10 GeV (MUM..
c-----------------------------------                             .
           numb = numb + 1         !-> tracking                .
           ityp(numb) = 3          ! simulation              .
           eleng(numb) = sngl(pat) !  history              .
           ener1(numb) = el        !                    .
           ener2(numb) = el        !                 .
c---------------------------------...................
           goto 4321            ! ..does not compute some "catastrophic"
         endif                  !   losses below 10 GeV, jump to the
c                               !   label 4321 to go the rest of DEPTH
c                               !   with "continuous" losses only...
c
       pb = 1./glbremv(e1)        !-> Computing of "weight" for different
       pp = pb + 1./glpairv(e1)   !   interactions at current muon energy.
       pn = pp + 1./glphnuv(e1)   !
       pt = pn + 1./glelecv(e1)   !
       ranp = rndm(5) * pt  ! -> Simulation of interaction type
       numb = numb + 1 !-> tracking simulation history
       if(ranp.le.pb) then        !-> type was simulated as bremsstrahlung,
       call getvbrem(e1,v,itra)   !   simulate rel. energy transfer
       ityp(numb) = 4 !-> tracking simulation history
       else 
          if(ranp.le.pp) then      !-> type was simulated as e+e- pair,
          call getvpa(e1,v,itra)   !   simulate rel. energy transfer
          ityp(numb) = 5 !-> tracking simulation history
          else
              if (ranp.le.pn) then   !-> type was simulated as photonuc.,
              call getvph(e1,v,itra) !   simulate rel. energy transfer
              ityp(numb) = 6 !-> tracking simulation history
              else
              call getvel(e1,v,itra) !-> type was simulated as knock-on
c                                    !  electron, simulate en. transfer
              ityp(numb) = 7 !-> tracking simulation history
              endif
           endif
        endif
        e2 = e1 * ( 1. - v ) ! -> The energy after interaction
c-----------------------------------
           eleng(numb) = sngl(pat) !
           ener1(numb) = e1        !-> tracking simulation history
           ener2(numb) = e2        !
c-----------------------------------
          if (e2.le.10.) then
             if (e2.le..16) then !-> the muon stops (energy is below
                 enew = 1.e-2    !   the Cherenkov threshold in water)
c---------------------------------------- 
                 numb = numb + 1         !
                 ityp(numb) = 2          !-> tracking simulation history
                 eleng(numb) = sngl(pat) !
                 ener1(numb) = enew      !
                 ener2(numb) = enew      !
c---------------------------------------
                 return          !   Assigne ENEW = 0.01 and return...
             else
                 el=e2     !-> muon has not stopped but its energy is
                 goto 4321 !   less than 10 GeV. Jump to label 4321...
             endif
          else
             e0 = dble(e2)  !-> muon enrgy is above 10 GeV. Jump to
             goto 1         !   label 1 to repeat everything once
          endif             !          again...
c
c  Muon energy becomes less than 10 GeV. Compute the rest of its
c          travel with continuous losses only:
c
 4321   lim1 = 30          
        elow = alog(1.6e-1)
        up = alog(el)
        step1 = (up - elow) / float(lim1)
         do j1=0,lim1                       
         en = elow + (float(j1) * step1)
         h1 = exp(en)
         aux1(j1) = h1 * 1.e+3 / gdedelt2(h1)
         enddo
         pat1 = simps(aux1,elow,up,lim1)
         if (pat1.lt.rest) then            ! Muon energy becomes < 0.16
         enew = 1.e-2                      ! GeV before it reaches DEPTH
c----------------------------------------------- 
         numb = numb + 1                       !
         ityp(numb) = 2                        !-> tracking simulation
         eleng(numb) = sngl(pat + dble(pat1))  !     history
         ener1(numb) = enew                    !
         ener2(numb) = enew                    !
c----------------------------------------------- 
         return                            ! ENEW = 0.01 GeV and return...
         endif
c-------------------------------------------                  
         ic = 0                            ! Iteration procedure to 
         ene = (el + 1.59999e-1) * 5.e-1   ! obtain muon energy at 
         delta = el - ene                  ! the DEPTH if his start
 3333    elow = alog(ene)                   ! energy is less than 
         step1 = (up - elow) / float(lim1)   ! 10 GeV (without "ca-
         do j1=0,lim1                         ! tastrophic" part)
         en = elow + (float(j1) * step1)      !
         h1 = exp(en)                         !
         aux1(j1) = h1 * 1.e+3 / gdedelt2(h1) !
         enddo                                !
         pat1 = simps(aux1,elow,up,lim1)    !
         delta = 5.e-1 * delta            !
         ic = ic + 1                    !
         if (pat1.eq.rest) then       !-> it seems incredible but    
         enew = ene                   !   sometimes it occures...
c ----------------------------              !    
         numb = numb + 1     !                      !
         ityp(numb) = 1      !-> tracking simulation  !
         eleng(numb) = depth !    history            !
         ener1(numb) = enew  !                   !
         ener2(numb) = enew  !               !
c ----------------------------          !
         return                        !
         endif                        !
         if (pat1.le.rest) then      !
         ene = ene - delta            !
         else                           !
         ene = ene + delta               !
         endif                            !
         if (ic.eq.14) then                !
         enew = ene                         ! -> We found ENEW
c ----------------------------                  !     and finish...
         numb = numb + 1     !                      !
         ityp(numb) = 1      !-> tracking simulation  !
         eleng(numb) = depth !    history            !
         ener1(numb) = enew  !                   !
         ener2(numb) = enew  !               !
c ----------------------------          !
         goto 9811                   !
         endif                   !
         goto 3333            !
c----------------------------
 9811 return
      end
****************************************************************************
* C.18
      subroutine simulde(ndepths,emu0)
*  
*    The subroutine gives the muon energy at a set of depths consisting of 
* NDEPTHS levels of observation if its level at the DEPTH=0 is  equal to EMU0 
*  (GeV). EMU0 should be in a range of 10 GeV - 1 EeV. The  subroutine uses 
* function ENEW which gives a "muon history" during its travel through a media 
*           (see comments to ENEW routine). User has to:
*
*  1. Give lines
*
*                  ...
*       common /frouser1/ horison(1000)
*       common /touser1/ emuon(1000) 
*                  ...
*
*    in a program unit from which SIMULDE is called. 
*
*  2. Define integer variable 0 < NDEPTHS <= 1000 in that program unit.
*
*  3. Prepare there a real*4 array HORISON(1000) which contains up to 1000 
*     levels of observation. The rules for HORISON are as follows:
* 
*     - levels of observation should be expressed in cm;    
*     - they should be more than 0. and not greater than 10^7;
*     - they should follow in increasing order along array.
*
*    SIMULDE references only first NDEPTHS elements of HORISON(1000), so
*    one should take care about them, it does not a matter what will be
*    HORISON(NDEPTHS+1),HORISON(NDEPTHS+2),HORISON(NDEPTHS+3), etc.
*
*   INPUT: EMU0    - energy (GeV) at the depth = 0. !-> subroutine 
*          NDEPTHS - number of observation levels   !   parameters
*          -----------------------------------------
*          HORISON(1000) - an array containing      !-> prepared
*                          levels of observation  ! by user and passed
*                                              ! to SIMULDE via common 
*                                            !      /frouser1/             
*
*   OUTPUT: array EMUON(1000) whose first NDEPTHS elements are muon
*           energies (GeV) at given level of observation as a result
*           of MC simulation which is performed by ENEW routine.
*           Array is returned to user by common /touser1/.
*.....................................................................
       external enew,simps,gdedelt2
       dimension ityp(10000),eleng(10000),ener1(10000),ener2(10000)
       dimension aux1(0:30)
       common /vhistory/ numb,ityp,eleng,ener1,ener2
       common /frouser1/ horison(1000)
       common /touser1/ emuon(1000) 
c
       emuon(ndepths) = enew(emu0,horison(ndepths))
       numfirst = 2
       do i=1,ndepths-1 !--> cicle along levels of observation
c---------------------------------------------
         if (eleng(numb).lt.horison(i)) then !-> muon has died before
           emuon(i) = 1.e-2                  !   given level of
           goto 1                            !    observation...
         else                                !
c---------------------------------------------
           do j=numfirst,numb  !-> cicle along muon history
c----------------------------------------------
             if (eleng(j).ge.horison(i)) then !-> we found the first event
             numfirst = j                   ! in muon history (starting with
c                                         ! less depth) whose depth is more 
c                                       ! than given level of observation
c----------------------------------------
                  if (ityp(j).ge.3) then                 !-> the nearest   
                     diffl = eleng(j) - horison(i)       ! event is either  
                        if (diffl.le.1.) then            ! brem, pn, e+e-,
                        emuon(i) = ener1(j)              ! knock-on e in-
                        goto 1                           ! teraction or
                        endif                            ! 10 GeV level
                     emuon(i) = geteback(ener1(j),diffl) !
                     goto 1                              !
                  else                                   !
c                                                        
                     if (ityp(j).eq.1) then                     !-> muon
                         if (ener1(j).ge.10.) then              ! has reached
                           diffl = eleng(j) - horison(i)        ! the maximum
                             if (diffl.le.1.) then              ! depth and
                             emuon(i) = ener1(j)                ! E_mu > 10 GeV
                             goto 1                             !
                             endif                              !
                           emuon(i) = geteback(ener1(j),diffl)  !  
                           goto 1                               !
                         else
                           goto 2  !-> the nearest event is either maximum
                         endif     !    depth with E_mu < 10 GeV or death
                     else          !    of muon (E_mu < 0.16 GeV)
                         goto 2    !
                     endif         !
                  endif            !
             else
             goto 3 !-> continue to search for the first event in muon history
             endif
    2        lim1 = 30  !-------------------------!
             rest = eleng(j) - horison(i)         !
                if (rest.le.1.) then              !
                emuon(i) = ener1(j)               !
                goto 1                            !
                endif                             !
             ic = 0                               ! Iteration procedure to 
             efin = ener1(j)                      ! obtain muon energy
             if (efin.le..15) efin = .16         !  at the level of observation
             ene = (efin + ener2(j-1)) * 5.e-1  !   if its energy at the nearest
             delta = ene - efin                !    event after level of obser-
             elow = alog(efin)                  !   vation is less than 10 GeV
 3333        up = alog(ene)                      !  
             step1 = (up - elow) / float(lim1)    !
               do j1=0,lim1                       !
                 en = elow + (float(j1) * step1)   !
                 h1 = exp(en)                        !
                 aux1(j1) = h1 * 1.e+3 / gdedelt2(h1) !
               enddo                               !
             pat1 = simps(aux1,elow,up,lim1)   !
             delta = 5.e-1 * delta         !
             ic = ic + 1                !
               if (pat1.eq.rest) then !-> It seems incredible but sometimes
               emuon(i) = ene         !      it occures ...
               goto 1                 !
               endif                  !_________
               if (pat1.le.rest) then           !
                 ene = ene + delta             !
               else                           !
                 ene = ene - delta           !
               endif                        !
               if (ic.eq.14) then          !
                 emuon(i) = ene           ! -> We found EMUON(I)
                 goto 1                  !
               endif                    !
             goto 3333                 !
    3      continue  !----------------!
           enddo
         endif 
    1  continue       
       enddo
      return
      end
****************************************************************************
* C.19
         SUBROUTINE bkns(empow,e1)
*
*   It simulates the vertical muon spectrum at the see level according
*        to BUGAEV-KLIMUSHIN fit of NAUMOV-SINEGOVSKY spectrum:
*
*    D(E) = 0.175 * ( 1/(1+(E/103)) + 0.037/(1+(E/810)) ) * E^(-2.72) 
*              ( cm^(-2) * sec^(-1) * sr^(-1) * GeV^(-1) )
*
*  Input: empow = lower cut energy for muon (lower energy for simulated 
*                 spectrum), GeV 
*  Output: e1 = siimulated muon energy, GeV 
*
*   The simulation is done by the "rejection method" (see W.H.Press et al.,
*  NUMERICAL RECEIPES (THE ART OF SCIENTIFIC COMPUTING), Cambridge University
*            press, Chapter 7, pp. 200-204) with comparison function 
*                            D(E) = 23.4 * E^(-3.72)
* ..........................................................................
*   EMPOW must be in a range of 100 GeV -- 50 TeV to obtain a reasonable 
*                                output
* ..........................................................................
       real*8 rvec(2),em,pow,e,cmp
       parameter (pow=-1.d+0/2.72d+0)
       parameter (len=2)
       em = dble(empow)
  155  call rm48(rvec,len)
       e = em * (rvec(1)**pow)
       cmp = (1.8025d+1 / (1.03d+2 + e)) + (5.24475d+0 / (8.1d+2 + e))
       cmp = (cmp * e) / (rvec(2) * 2.34d+1)
         if (cmp.ge.1.d+0) then
           e1 = sngl(e)
           return
         else
           goto 155
         endif
       return
       end
****************************************************************************
* C.20
         SUBROUTINE gais(empow,e1)
*
*   It simulates the vertical muon spectrum at the see level according
*                          to Gaisser spectrum:
*
*   D(E) = 0.14 * (1/(1+(E/104.5454)) + 0.054/(1+(E/773.7273))) * E^(-2.7) 
*              ( cm^(-2) * sec^(-1) * sr^(-1) * GeV^(-1) )
*
*  Input: empow = lower cut energy for muon (lower energy for simulated 
*                 spectrum), GeV 
*  Output: e1 = siimulated muon energy, GeV 
*
*   The simulation is done by the "rejection method" (see W.H.Press et al.,
*  NUMERICAL RECEIPES (THE ART OF SCIENTIFIC COMPUTING), Cambridge University
*            press, Chapter 7, pp. 200-204) with comparison function 
*                            D(E) = 21. * E^(-3.7)
* ..........................................................................
*   EMPOW must be in a range of 100 GeV -- 50 TeV to obtain a reasonable 
*                                output
* ..........................................................................
       real*8 rvec(2),em,pow,e,cmp
       parameter (pow=-1.d+0/2.7d+0)
       parameter (len=2)
       em = dble(empow)
  155  call rm48(rvec,len)
       e = em * (rvec(1)**pow)
       cmp = (1.4636356d+1 / (1.045454d+2 + e)) + (5.841818388d+0 / 
     +       (7.727273d+2 + e))
       cmp = (cmp * e) / (rvec(2) * 2.1d+1)
         if (cmp.ge.1.d+0) then
           e1 = sngl(e)
           return
         else
           goto 155
         endif
       return
       end
****************************************************************************
* C.21
        subroutine specdept(depth,is,ihist,numb)
* 
*     This subroutine simulates muon vertical spectrum at the depth = DEPTH  
*   starting with Bugaev-Klimushin-Naumov-Sinegovski (IS=1) or Gaisser (IS=2)
*    vertical spectrum at see level. The DEPTH should be expressed in cm and
*    corresponds to not less than 600 m.w.e. and not more than 20000 m.w.e.
*  For 11 threshold energies (array ETH(11)) the absolute values for vertical
*   flux I(E_mu > ETH) (array FLUX(11), sm^-2 sr^-1 sec^-1) and mean emergy 
* (EMEAN(11), GeV) are computed. Arrays SRFLUX(11) and EMERR(11) contain values 
* of statistical errors (in corresponding units, at 1-sigma level) for FLUX(11) 
*    and EMEAN(11). The number of muons in depth spectrum which should be 
* simulated is given by variable NUMB. That means that simulation will continue 
*  untill NUMB muons have been detected with energy > ETH(11) at given depth. 
*   Histograms are filled if variable IHIST=1. If IHIST=0 histograms are not
*       filled (it saves ~10% of time but gives much less information).
*  Subroutine informs user by a screen output after every 0.05 * NUMB muons
*                                reached the depth.
*
*    User does not need to think about lower cut for muon energy at surface,
*  SPECDEPT computes it itself quite reliable for each given depth and medium.
*
*                            User has to:
*
*  1. Give lines
*
*                  ...
*        common /frouser2/ eth(11)
*        common /touser2/ emean(11),emerr(11),flux(11),erflux(11)
*                  ...
*
*    in a program unit from which SPECDEPT is called. 
*
*  2. Define real*4 array ETH(11) with threshold energies (in GeV) in that
*     program unit. Note that in the frame of MUM code muon is considered
*     as stopped if its energy becomes less than 0.16 GeV. In this case its
*     energy is assigned by "fake" value of 0.01 GeV (see comments to ENEW
*     routine). So, the lowest threshold should not be less than 0.01 GeV
*     It is recommended to set it to 0.02 GeV to obtain integrated flux and
*     mean energy for the whole depth spectrum E_mu > 0. ETH(i) should be in
*     a reasonable range 0.02 - 100 GeV.
*
*   INPUT: real*4 DEPTH - depth (cm) at which depth spectrum  !-> subroutine 
*                         is simulated and fluxes + mean      !   parameters
*                            energies are computed            !       
*          integer IS - type of vertical muon spectrum at sea !
*                       level (IS = 1 - Bugaev-Klimushin-Na-  !
*                       umov-Sinegovsky ; IS = 2 - Gaisser)   !
*          integer IHIST - Histograms are filled if IHIST=1   !
*                          and are not filled if IHIST=0      !
*          integer NUMB - desirable number of muons with      !
*                         energy > ETH(11) at given depth     !
*          ....................................................
*          real*4 ETH(11) - set of energy thresholds (GeV)    !-> should be
*                           above which fluxes and mean       ! prepared by
*                           energies are computed at given    ! user and pas-
*                           depth                             ! sed to SPECDEPT
*                                                             ! via common
*                                                             ! /frouser2/
* -------------------------------------------------------------
*   OUTPUT: real*4 FLUX(11) - set of integrated fluxes at     !-> returned to
*                             given depth with thresholds     ! user as a result
*                             from ETH(11), cm^-2sr^-1s^-1    ! of SPECDEPT via
*           real*4 ERFLUX(11) - set of statistical errors for !  common block
*                               FLUX(11) (in the same units,  !   /touser2/
*                               1-sigma level)                !
*           real*4 EMEAN(11) - set of mean energies for muons !
*                              with energies > ETH(xx) which  !
*                              are detected at given depth,   !
*                              GeV                            !
*           real*4 EMERR(11) - statistical errors (GeV,       !
*                              1-sigma level) for EMEAN(11)   !
*           
* ------------------------------------------------------------------------
*         LIST OF HISTOGRAMS (WHICH ARE FILLED IF IHIST=1) :
*
*           ID                                    DESCRIPTION
*     
*   INT(depth*0.0001) + 10000     Simulated muon spectrum at see level
*
*   INT(depth*0.0001) + 11000     Simulated muon spectrum at the depth=DEPTH
*
*   INT(depth*0.0001) + 11000 +   Simulated muon spectrum tyo which surface
*      + 100000*(depth*k/10)       spectrum transforms for 10 intermediate
*           where k=1,10                           depths
*
* NOTES: IF ONE CALLS SPECDEPT SEVERAL TIMES THE DIFFERENCE BETWEEN DEPTH
*        variables should exceed 10 meters, otherwise some part of histo-
*        grams will be rewritten and lost  
*
*        User should wathch oneself for ID of ones own HBOOK histograms,
*        ID should not crossed
*........................................................................
        external enew,dsimps
        parameter (ndepths=10)
        real*8 z1(10),w(10),aw(10),a_ef,ro,dsimps
        real*8 ecu(11),ecusq(11),aux(0:10000),up,dow,e,e1,step,whole
        integer nsub
        character *6 txt1
        character *6 txt3
        character *100 txt2
        dimension nflux(11),id3(ndepths)
        common /media/ z1,w,aw,a_ef,ro,nsub
        common /frouser1/ horison(1000)
        common /touser1/ emuon(1000) 
        common /frouser2/ eth(11)
        common /touser2/ emean(11),emerr(11),flux(11),erflux(11)
c-------------------------------------------------------------------
c     Let's check the depth, it should be within a reasonable range:
c
        dch = depth * sngl(ro) * .01
        if (dch.lt.6.e+2) then
        write(0,*)'SPECDEPT: DEPTH is too little (',dch,' m w.e.),'
        write(0,*)'will not work since results will be unreasonable...'
        return
        endif
        if (dch.gt.2.01e+4) then
        write(0,*)'SPECDEPT: DEPTH is too large (',dch,' m w.e.),'
        write(0,*)'will not work since results will be unreasonable...'
        return
        endif
        write(0,*)' '
        write(0,*)' SUBROUTINE SPECDEPT STARTS :'
        write(0,*)' '
c--------------------------------------------------------------------
c        Preparation of HORISON array for subroutine SIMULDE
c 
        difho = depth / float(ndepths)
        do i=1,ndepths
        horison(i) = float(i) * difho
        enddo
c--------------------------------------------------------------------
c Iteration to compute "lower cut energy" in muon spectrum from which 
c     simulation will be performed for given depth (EN variable):
c
        ic = 0
        enlog = 6.
        delta = enlog
    1   en = 10.**enlog       
          if (ic.eq.12) goto 2
          do i=1,1000
          echeck = enew(en,depth)
            if (echeck.ge.1.3e-2) then
            delta = delta * .5
            enlog = enlog - delta
            ic = ic +1 
            goto 1
            endif
          enddo
          delta = delta * .5
          enlog = enlog + delta
          ic = ic +1 
          goto 1
    2   en = .65 * en     
c----------------------------------------------------------------------
c Booking histograms for surface and depth spectra if IHIST > 0 :
c
         if (ihist.gt.0) then
         k = int((depth * 1.e-2) + .4)
         write(txt1,'(i6)') k
         txt2 = 'Diff. muon spectrum at see level to simulate spectrum  
     + at the depth '//txt1//' meters'
         id1 = int((depth * 1.e-4) + .4) + 10000
         CALL HBOOK1(id1,txt2,100,-.9,7.,0.)
         txt2 = 'Diff. muon spectrum at  the depth '//txt1//' meters'
         id2 = int((depth * 1.e-4) + .4) + 11000
         CALL HBOOK1(id2,txt2,100,-.9,7.,0.)
c
            do i=1,ndepths
            id3(i) = i*100000 + id2
            k1 = int((horison(i) * 1.e-2) + .4)
            write(txt3,'(i6)') k1
            txt2='Interim spectrum at the depth '//txt3//' m for muons g
     + oing down to '//txt1//' m'
            CALL HBOOK1(id3(i),txt2,100,-.9,7.,0.)
            enddo
c
         endif
c----------------------------------------------------------------------
c  Calculation of integral vertical muon flux at see level with
c                  EN as a threshold energy:
c
       lim = 10000
       if (is.eq.1) then
c
c  BUGAEV - KLIMUSHIN - NAUMOV - SINEGOVSKY muon spectrum at see level:
c
       dow = dlog(dble(en))
       up = dlog(1.d+9)
       step = 1.d-4 * (up - dow)
       do i=0,lim
       e = dow + (step * dble(i))
       e1 = dexp(e)
       aux(i) = (1.8025d+1 / (1.03d+2+e1)) + (5.24475d+0 / (8.1d+2+e1)) 
       aux(i) = (e1**(-2.72d+0)) * aux(i) * e1
       enddo
       whole = dsimps(aux,dow,up,lim)
       else
c
c             GAISSER muon spectrum at see level:
c
       dow = dlog(dble(en))
       up = dlog(1.d+9)
       step = 1.d-4 * (up - dow)
       do i=0,lim
       e = dow + (step * dble(i))
       e1 = dexp(e)
       aux(i) = (1.463636d+1 / (1.045454d+2+e1)) + (5.8418184d+0 / 
     +          (7.727273d+2+e1)) 
       aux(i) = (e1**(-2.7)) * aux(i) * e1
       enddo
       whole = dsimps(aux,dow,up,lim)
       endif
       surint = sngl(whole)
c-----------------------------------------------------------------------
c           GETTING SPECTRA FOR 11 THRESHOLD ENERGY (ETH(11))
c 
c     EMEAN(i) - the mean energies for Ith threshold
c     EMERR(i) - Stst. error (1 sigma) for EMEAN(i)
c     NFLUX(i) - number of muons detected above threshold ETH(i) 
c     FLUX(i) - integral muon fllux at given depth with threshold ETH(i)
c     ERFLUX(i) - statistical error (1 sigma) for FLUX(i)
c 
          do i=1,11
          nflux(i) = 0
          ecu(i) = 0.d+0
          ecusq(i) = 0.d+0
          enddo
c
        som1 = ( float(numb) * .05 ) + .5
        ind = int(som1)
        ic = 0
        ic1 = 0
    3   if (is.eq.1) then
        call bkns(en,er)        
        ic = ic + 1
        else
        call gais(en,er)
        ic = ic + 1
        endif
              if (ihist.gt.0) then       ! Filling histogram
              erlo = alog10(er)          ! for see-level spectrum
              call hfill(id1,erlo,0.,1.) ! if IHIST > 0
              endif                      !
        call simulde(ndepths,er)
        ed = emuon(ndepths)
              if (ihist.gt.0) then                 ! Filling histograms
                 do i=1,ndepths                    ! for spectrum at
                   if (emuon(i).gt..02) then       ! intermediate depths
                   erlo1 = alog10(emuon(i))        ! if IHIST > 0
                   call hfill(id3(i),erlo1,0.,1.)  !
                   endif                           !
                 enddo                             !
              endif                                !
              if (ed.gt..02) then        !
              if (ihist.gt.0) then       ! Filling histogram
              erlo = alog10(ed)          ! for depth spectrum
              call hfill(id2,erlo,0.,1.) ! if IHIST > 0
              endif                      !
              endif                      !
          do i=1,11
             if (ed.ge.eth(i)) then 
               nflux(i) = nflux(i) + 1
               ecu(i) = ecu(i) + dble(ed)
               ecusq(i) = ecusq(i) + (dble(ed) * dble(ed))
             endif
          enddo
        if (ed.ge.eth(11)) ic1=ic1+1
         if (ic1.eq.ind) then
         ic1 = 0
         write(0,*)nflux(11),' (out of ',numb,') muons have reached the dep
     +th...'
         endif
        if (nflux(11).lt.numb) goto 3         
        do i=1,11
        emean(i) = sngl( ecu(i) / dble(nflux(i)) )
        ecu(i) = (dble(nflux(i)) * ecusq(i)) - (ecu(i) * ecu(i)) 
        ecu(i) = ecu(i) / (dble(nflux(i)) * (dble(nflux(i)) - 1.d+0))
        emerr(i) = sngl(DSQRT(ecu(i)/dble(nflux(i))))  
        flux(i) = (float(nflux(i)) / float(ic)) * surint
        erflux(i) = (sqrt(float(nflux(i))) / float(nflux(i))) * flux(i)
        enddo
         return
         end
****************************************************************************
* C.22
        subroutine testcom
*
*   Performes some test of routines for common usage with filling number of 
*                     histograms and .dat files.
*
      external gdedelt2,filco1,filco2,filco3,filco4,getlanrv,geteranv
      external filco5,filco6,filco7,filco8,filco9,filco10,filco11
      external filco12,filco13,enew,filco14,filco15,filco16,filco6a
      external glbremv,glpairv,glphnuv,glelecv,conv,simps,geteback
      external filco6b,filco 7a,filco8a,filco9a,filco10a
      dimension aux1(0:1500)
      dimension aux(0:10000)
      character *100 txt2
      common /tofil13/ whole
      common /tofil16/ whole1
      write(0,*)' '
      write(0,*)'             TESTING THE COMMON PART:'
      write(0,*)' '
c
c---------------------------------------------------------------------------
c  1. Files with energy losses due t0 ionization foe muon energies < 10 GeV
c---------------------------------------------------------------------------
       write(0,*)'  '
       write(0,*)'Computing and recording en. losses for E_mu < 10 GeV...'
       write(0,*)'  '
       open(11,file='en_low.dat',status='new',form='formatted')
       open(12,file='enlos_low.dat',status='new',form='formatted')
       do i=1,46
       en=float(i)
       en = 1.e+1**(alog10(.16)-.04+(en*.04))
       elo = gdedelt2(en)
       write (11,234) en
       write (12,234) elo
       enddo
c  EN_LOW.DAT - 46 values of muon energy for which energy losses are computed
c               (GeV).
c  ENLOS_LOW.DAT - 46 values for ionization energy losses (Bethe-Bloch + e-
c                  diagrams for bremsstrahlung) computed with splines 
c                  (GDEDELT2 routine).
      close (12)
      close (11)
c---------------------------------------------------------------------------
c   2. QUALITY OF CALCULATION CONTINUOUS ENERGY LOSSES AND FREE PATHS 
c                         BELOW EMIN AND VMIN
c---------------------------------------------------------------------------
       write(0,*)'Testing continuous energy losses...'
       write(0,*)'  '
      txt2 = 'Comm, CONE values'
          CALL HBFUN1(4903,txt2,97,.9583334,9.04165,filco1)
      txt2 = 'Comm, CONV values'
          CALL HBFUN1(4901,txt2,97,.9583334,9.04165,filco3)
      txt2 = 'Comm, rel. err. CONV - energy losses (percents)'
          CALL HBFUN1(4902,txt2,97,.9583334,9.04165,filco2)
      txt2 = 'Comm, rel. err. CONE - energy losses (percents)'
          CALL HBFUN1(4904,txt2,97,.9583334,9.04165,filco4)
      txt2 = 'Comm, mean free path above EMIN, cm'
          CALL HBFUN1(4905,txt2,97,.9583334,9.04165,filco5)
      txt2 = 'Comm, mean free path above VMIN, cm'
          CALL HBFUN1(4906,txt2,97,.9583334,9.04165,filco6)
      txt2 = 'Comm, diff.1 (p-s) btw. tot. en. los. computed by 2 ways'
          CALL HBFUN1(4987,txt2,97,.9583334,9.04165,filco6a)
      txt2 = 'Comm, diff.2 (p-s) btw. tot. en. los. computed by 2 ways'
          CALL HBFUN1(4988,txt2,97,.9583334,9.04165,filco6b)
c---------------------------------------------------------------------------
c                   3. TEST FOR SUBROUTINE GETLANRV  
c---------------------------------------------------------------------------
       write(0,*)'Testing subroutine GETLANRV...'
       write(0,*)'  '
      txt2 = 'Comm, FREE PATH(E)/GETLANRV(E), (ETA = 0.5)'
          CALL HBFUN1(4907,txt2,296,1.04,9.,filco7)
      txt2 = 'Comm, FREE PATH(E)/GETLANRV(E), (ETA = 1.)'
          CALL HBFUN1(4908,txt2,296,1.04,9.,filco8)
      txt2 = 'Comm, FREE PATH(E)/GETLANRV(E), (ETA = 2.)'
          CALL HBFUN1(4909,txt2,296,1.04,9.,filco9)
      txt2 = 'Comm, FREE PATH(E)/GETLANRV(E), (ETA = 3.)'
          CALL HBFUN1(4910,txt2,296,1.04,9.,filco10)
      txt2 = 'Comm, noncorrect E.losses/correct E.losses, (ETA = 0.5)'
          CALL HBFUN1(4107,txt2,296,1.04,9.,filco7a)
      txt2 = 'Comm, noncorrect E.losses/correct E.losses, (ETA = 1.)'
          CALL HBFUN1(4108,txt2,296,1.04,9.,filco8a)
      txt2 = 'Comm, noncorrect E.losses/correct E.losses, (ETA = 2.)'
          CALL HBFUN1(4109,txt2,296,1.04,9.,filco9a)
      txt2 = 'Comm, noncorrect E.losses/correct E.losses, (ETA = 3.)'
          CALL HBFUN1(4110,txt2,296,1.04,9.,filco10a)
c---------------------------------------------------------------------------
c                4. TEST FOR MUON SPECTRUM SIMULATION  
c---------------------------------------------------------------------------
       write(0,*)'Testing muon spectrum simulation...'
       write(0,*)'  '
c      
c             BUGAEV - KLIMUSHIIN - NAUMOV - SINEGOVSKY:
c   
      txt2 = 'NAUMOV,SINEGOVSKY,BUGAEV,KLIMUSHIN VERTICAL MUON SPECTRUM'
          CALL HBFUN1(4911,txt2,97,.9583334,9.04165,filco11)
      txt2 = 'RELATION BETWEEN NSBK MUON SPECTRUM and COMP. FUNCTION'
          CALL HBFUN1(4912,txt2,97,.9583334,9.04165,filco12)
c
       lim = 10000
       dow = alog(1.e+1**2.48)
       up = alog(1.e+1**6.)
       step = 1.e-4 * (up - dow)
       do i=0,lim
       e = dow + (step * float(i))
       e1 = exp(e)
       aux(i) = (18.025 / (103. + e1)) + (5.24475 / (810. + e1)) 
       aux(i) = (e1**(-2.72)) * aux(i) * e1
       enddo
       whole = simps(aux,dow,up,lim)
       txt2='BKNS verical surface muon spectrum (prediced, rel. units)' 
          CALL HBFUN1(4981,txt2,100,2.48,6.,filco13)
c
       txt2='BKNS verical surface muon spectrum (MC, rel. units)' 
       CALL HBOOK1(4980,txt2,100,2.48,6.,0.)
       txt2='BKNS verical muon spectrum at 1 km w.e..(MC, rel. units)' 
       CALL HBOOK1(4982,txt2,100,-2.,5.,0.)
       em = 301.995172
       j = 0
       do i=1,20000000
       j = j + 1
       call bkns(em,e)      
           if (j.eq.100) then
             j = 0
             e2 = enew(e,1.e+5)
             e2 = alog10(e2)
             call hfill(4982,e2,0.,1.) 
           endif
       e1 = alog10(e)
       call hfill(4980,e1,0.,1.) 
       enddo
c      
c                             GAISSER:
c   
      txt2 = 'GAISSER VERTICAL MUON SPECTRUM'
          CALL HBFUN1(4913,txt2,97,.9583334,9.04165,filco14)
      txt2 = 'RELATION BETWEEN GAISSER MUON SPECTRUM and COMP. FUNCTION'
          CALL HBFUN1(4914,txt2,97,.9583334,9.04165,filco15)
c
       lim = 10000
       dow = alog(1.e+1**2.48)
       up = alog(1.e+1**6.)
       step = 1.e-4 * (up - dow)
       do i=0,lim
       e = dow + (step * float(i))
       e1 = exp(e)
       aux(i) = (14.63636 / (104.5454+e1)) + (5.8418184 / (772.7273+e1)) 
       aux(i) = (e1**(-2.7)) * aux(i) * e1
       enddo
       whole1 = simps(aux,dow,up,lim)
       txt2='GAIS verical surface muon spectrum (prediced, rel. units)' 
          CALL HBFUN1(4991,txt2,100,2.48,6.,filco16)
c
       txt2='GAIS verical surface muon spectrum (MC, rel. units)' 
       CALL HBOOK1(4990,txt2,100,2.48,6.,0.)
       txt2='GAIS verical muon spectrum at 1 km w.e..(MC, rel. units)' 
       CALL HBOOK1(4992,txt2,100,-2.,5.,0.)
       em = 301.995172
       j = 0
       do i=1,20000000
       j = j + 1
       call gais(em,e)      
           if (j.eq.100) then
             j = 0
             e2 = enew(e,1.e+5)
             e2 = alog10(e2)
             call hfill(4992,e2,0.,1.) 
           endif
       e1 = alog10(e)
       call hfill(4990,e1,0.,1.) 
       enddo
c
c---------------------------------------------------------------------------
c       5. QUALITY OF SIMULATION FOR MUON WHIICH PASSES THE MEDIUM
c---------------------------------------------------------------------------
       write(0,*)'Testing quality slu - path - energy splines...'
       write(0,*)'  '
c
         txt2='Difference GETLANRV-exactly calculated path (cm) for slu
     + less than 3. vs. Emu'
          CALL HBOOK2(4955,txt2,100,1.,9.,100,-17.,+8.,0.)
         txt2='Difference GETLANRV-exactly calculated path (cm) for slu
     + more than 3. vs. Emu'
          CALL HBOOK2(4956,txt2,100,1.,9.,100,-350.,+50.,0.)
         txt2='Err.(percents) GETLANRV-exactly calculated path (cm) for
     + slu more than 3. vs. Emu'
          CALL HBOOK2(4957,txt2,100,1.,9.,100,-.4,+.1,0.)
         txt2='Err.(percents) GETLANRV-exactly calculated path (cm) for
     + slu less than 3. vs. Emu'
          CALL HBOOK2(4958,txt2,100,1.,9.,100,-85.,+35.,0.)
         txt2='Err.(per-s) btw. E and GETEBACK(GETERANV(slu,E),GETLANRV(
     +slu,E)) vs. energy'
          CALL HBOOK2(4960,txt2,100,1.,9.,100,-.2,+.2,0.)
         txt2='Err.(per-s) btw. E and GETEBACK(GETERANV(eta,E),GETLANRV(
     +eta,E)) vs. energy'
          CALL HBOOK2(4965,txt2,100,1.,9.,100,-.15,+.15,0.)
c      
      do i=-80,29                         !--> 110 values of SLU (logarithmi-
        slu = 2.5e-2 + (float(i) * 5.e-2) !    cally equidestant grid with
        slu = 1.e+1**slu                  !    slu_min=0.0001, slu_max=30
        do j=179,20,-1                     !-> 160 values of E (logarithmi-
          e = 2.5e-2 + (float(j) * 5.e-2)  !   cally equidestant grid with
          e = 1.e+1**e                     !   e_min = 10 GeV, e_max = 1 EeV
c           
       preal = getlanrv(slu,e)
       e1 = e * geteranv(slu,e)
       e3 = geteback(e1,preal)
       e3 = 1.e+2 * ( ( e - e3 ) / e )
       elog = alog10(e) 
       call hfill(4960,elog,e3,1.)
c
           if (e1.gt.10.) then
           elow = alog(e1)
           else
           elow = alog(10.)
           endif
        up = alog(e)
        lim1 = 1000
        step1 = (up - elow) / float(lim1)
         do j1=0,lim1
         en = elow + (float(j1) * step1)
         h1 = exp(en)
         help = (1./glbremv(h1)) + (1./glpairv(h1)) + (1./glphnuv(h1)) + 
     +           (1./glelecv(h1)) 
         aux1(j1) = (1.e+0 / help) * conv(h1) * 1.e-3/ h1
         aux1(j1) = 1.e+0 / aux1(j1)
         enddo
         eta_1 = simps(aux1,elow,up,lim1)
c
         do j1=0,lim1
         en = elow + (float(j1) * step1)
         h1 = exp(en)
         aux1(j1) = conv(h1) * 1.e-3/ h1
         aux1(j1) = 1.e+0 / aux1(j1)
         enddo
         path_1 = simps(aux1,elow,up,lim1)
c
        if (eta_1.le.1.e-4) eta_1 = 1.e-4
        preal1 = getlanrv(eta_1,e)
        e4 = e * geteranv(eta_1,e)
        e3 = geteback(e4,preal1)
        e3 = 1.e+2 * ( ( e - e3 ) / e )
        elog = alog10(e) 
        call hfill(4965,elog,e3,1.)

c 
         err1 = (path_1 - preal)
         err = ( (path_1 - preal) / path_1) * 1.e+2
         if (slu.le.3.) call hfill(4955,elog,err1,1.)
         if (slu.ge.3.) call hfill(4956,elog,err1,1.)
         if (slu.ge.3.) call hfill(4957,elog,err,1.)
         if (slu.le.3.) call hfill(4958,elog,err,1.)
c
        enddo
      enddo
c
        txt2='Err.(percents) between iteratively computed back energies'
        CALL HBOOK2(4970,txt2,100,1.,9.,100,-.0025,.0025,0.)
c
c We compute "back" energy with GETBACK, than compute preciely the
c free path between "back" and current energy and then compute "back"
c energy once again with GETBACK and newly computed free path as a
c input. The difference between two GETBACK output is plotted.  
c
      do i=20,180          !-> 161 muon energies E: logarithmically
      e = float(i) * 5.e-2 !  equidestant grid from 10 GeV to 1 EeV
      e = 1.e+1**e         ! 
        do j=0,140              !-> 141 values for PATH: logarithm.
        path = float(j) * 5.e-2 !           equidestant grid
        path = 1.e+1**path      !          from 1 cm to 100 km
c
        e1 = geteback(e,path)
          if (e1.ge.1.e+9) then
           up = alog(1.e+9)
           else
           up = alog(e1)
           endif
        elow = alog(e)
        lim1 = 1000
        step1 = (up - elow) / float(lim1)
c
         do j1=0,lim1
         en = elow + (float(j1) * step1)
         h1 = exp(en)
           if (h1.gt.1.e+9) then
             h2 = (1.e+9 - h1) * 1.e-9
             if (h2.le.1.e-6) h1 = 1.e+9
           endif
         aux1(j1) = h1 * 1.e+3 / conv(h1)
         enddo
         path_1 = simps(aux1,elow,up,lim1)
         if (e1.ge.1.e+9) then
         e4 = 1.0001e+9
         e3 = geteback(e,path_1)
         else
         e4 = geteback(e,path_1)
         e3 = e1
         endif
         err = 1.e+2 * ( (e4 - e3) / e4 )
         elog = alog10(e)
         call hfill(4970,elog,err,1.)
        enddo
      enddo
c---------------------------------------------------------------------------
  234 format (e13.7) 
      return
      end
c===========================================================================
c C.22a
c                AN AUXILIARY FUNCTION FOR TESTCOM SUBROUTINE
c
      function filco1(x)
      external cone
      x1 = 1.e+1**x
      filco1 = cone(x1)
      return
      end
c===========================================================================
c C.22b
c                AN AUXILIARY FUNCTION FOR TESTCOM SUBROUTINE
c
      function filco2(x)
      external conv,getcdbr,getcdp,getcdn,getcde,gdeion,simps
      parameter (lo=0)
      parameter (lim = 1000)
      integer nsub
      real*8 z1(10),w(10),aw(10),a_ef,ro,alfa
      real*8 rm_e,rm_mu,r_e,avog
      common /general/ emin,vmin,emph
      common /media/ z1,w,aw,a_ef,ro,nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      common /exer2/ noca
      dimension aux(0:lim)
c
      x1 = 1.e+1**x
      filco2 = conv(x1)
c
      dummy = 0.
      vlow = 1.e-3 / x1
      if (vlow.le.1.e-11) vlow = 1.e-11
      vup = alog(vmin)
      vlow = alog(vlow)
      step1 = (vup - vlow) / float(lim)
      if (vup.le.vlow) goto 1
      do i=0,lim
      v = step1 * float(i) + vlow      
      v1 = exp(v)      
      aux(i) = getcdbr(x1,v1,lo) * v1 * v1      
      enddo
      dummy = dummy+simps(aux,vlow,vup,lim)*sngl(avog*ro/a_ef)*x1*1.e+3
c
 1    vlow = 2.046e-3/x1
      if (vlow.le.1.e-11) vlow = 1.e-11
      vup = alog(vmin)
      vlow = alog(vlow)
      step1 = (vup - vlow) / float(lim)
      if (vup.le.vlow) goto 2
      do i=0,lim
      v = step1 * float(i) + vlow     
      v1 = exp(v)      
      aux(i) = getcdp(x1,v1,lo) * v1 * v1     
      enddo
      dummy = dummy+simps(aux,vlow,vup,lim)*sngl(avog*ro/a_ef)*x1*1.e+3
c
 2    vlow = vmin
      vup = sngl((rm_mu * rm_mu * 1.d-3) / (2.d+0 * rm_e))
      vup = 1. / (1.e+0 + (vup / x1))
      vup = alog(vup)
      vlow = alog(vlow)
      step1 = (vup - vlow) / float(lim)
      if (vup.le.vlow) then
      aa = 0.
      goto 3
      endif
      do i=0,lim
      v = step1 * float(i) + vlow     
      v1 = exp(v)      
      aux(i) = getcde(x1,v1) * v1 * v1     
      enddo
      aa = simps(aux,vlow,vup,lim)*sngl(avog*ro/a_ef)*x1*1.e+3
 3    aa = gdeion(x1) - aa
      if (noca.eq.0) aa = gdeion(x1)
      dummy = dummy + aa
c
      vlow = emph/x1
      if (vlow.ge.vmin) goto 4
      vup = alog(vmin)
      vlow = alog(vlow)
      step1 = (vup - vlow) / float(lim)
      do i=0,lim
      v = step1 * float(i) + vlow     
      v1 = exp(v)      
      aux(i) = getcdn(x1,v1,lo) * v1 * v1     
      enddo
      dummy = dummy+simps(aux,vlow,vup,lim)*sngl(avog*ro/a_ef)*x1*1.e+3
c
 4    filco2 = 1.e+2 * ( (dummy - filco2) / filco2 ) !---> rel.error (percents)
      return
      end
c===========================================================================
c C.22c
c                AN AUXILIARY FUNCTION FOR TESTCOM SUBROUTINE
c
      function filco4(x)
      external cone,getcdbr,getcdp,getcdn,getcde,gdeion,simps
      parameter (lo=0)
      parameter (lim = 1000)
      real*8 z1(10),w(10),aw(10),a_ef,ro,alfa
      real*8 rm_e,rm_mu,r_e,avog
      integer nsub
      common /general/ emin,vmin,emph
      common /media/ z1,w,aw,a_ef,ro,nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      dimension aux(0:lim)
c
      x1 = 1.e+1**x
      filco4 = cone(x1)
c
      dummy = 0.
      vlow = 1.e-3 / x1
      if (vlow.le.1.e-11) vlow = 1.e-11
      vup = alog(emin/x1)
      vlow = alog(vlow)
      step1 = (vup - vlow) / float(lim)
      if (vup.le.vlow) goto 1
      do i=0,lim
      v = step1 * float(i) + vlow      
      v1 = exp(v)      
      aux(i) = getcdbr(x1,v1,lo) * v1 * v1      
      enddo
      dummy = dummy+simps(aux,vlow,vup,lim)*sngl(avog*ro/a_ef)*x1*1.e+3
c
 1    vlow = 2.2e-3/x1
      if (vlow.le.1.e-11) vlow = 1.e-11
      vup = alog(emin/x1)
      vlow = alog(vlow)
      step1 = (vup - vlow) / float(lim)
      if (vup.le.vlow) goto 2
      do i=0,lim
      v = step1 * float(i) + vlow     
      v1 = exp(v)      
      aux(i) = getcdp(x1,v1,lo) * v1 * v1     
      enddo
      dummy = dummy+simps(aux,vlow,vup,lim)*sngl(avog*ro/a_ef)*x1*1.e+3
c
 2    vlow = emin/x1
      vup = sngl((rm_mu * rm_mu * 1.e-3) / (2.d+0 * rm_e))
      vup = 1. / (1.e+0 + (vup / x1))
      vup = alog(vup)
      vlow = alog(vlow)
      step1 = (vup - vlow) / float(lim)
      if (vup.le.vlow) then
      aa = 0.
      goto 3
      endif
      do i=0,lim
      v = step1 * float(i) + vlow     
      v1 = exp(v)      
      aux(i) = getcde(x1,v1) * v1 * v1     
      enddo
      aa = simps(aux,vlow,vup,lim)*sngl(avog*ro/a_ef)*x1*1.e+3
 3    aa = gdeion(x1) - aa
      dummy = dummy + aa
c
      vlow = emph/x1
      t = emin/x1
      if (vlow.ge.t) goto 4
      vup = alog(t)
      vlow = alog(vlow)
      step1 = (vup - vlow) / float(lim)
      do i=0,lim
      v = step1 * float(i) + vlow     
      v1 = exp(v)      
      aux(i) = getcdn(x1,v1,lo) * v1 * v1     
      enddo
      dummy = dummy+simps(aux,vlow,vup,lim)*sngl(avog*ro/a_ef)*x1*1.e+3
c
 4    filco4 = 1.e+2 * ( (dummy - filco4) / filco4 ) !---> rel.error (percents)
      return
      end
c===========================================================================
c C.22d
c                AN AUXILIARY FUNCTION FOR TESTCOM SUBROUTINE
c
      function filco3(x)
      external conv
      x1 = 1.e+1**x
      filco3 = conv(x1)
      return
      end
c===========================================================================
c C.22e
c                AN AUXILIARY FUNCTION FOR TESTCOM SUBROUTINE
c
      function filco5(x)
      external gemleng
      x1 = 1.e+1**x
      filco5 = gemleng(x1)
      return
      end
c===========================================================================
c C.22f
c                AN AUXILIARY FUNCTION FOR TESTCOM SUBROUTINE
c
      function filco6(x)
      external glbremv,glpairv,glphnuv,glelecv
      x1 = 1.e+1**x
      filco6 = 1./glbremv(x1) + 1./glpairv(x1) + 1./glphnuv(x1) + 
     +         1./glelecv(x1)
      filco6 = 1./filco6
      return
      end
c===========================================================================
c C.22g
c                AN AUXILIARY FUNCTION FOR TESTCOM SUBROUTINE
c
      function filco6a(x)
      external gdedbrv,gdedpav,gdedphv,gdedelv,gdedbrt,gdedpat,gdedpht
      external gdeion,conv
      x1 = 1.e+1**x
      dum1 = gdedbrt(x1) + gdedpat(x1) + gdedpht(x1) + gdeion(x1)
      dum2 = conv(x1)+gdedbrv(x1)+gdedpav(x1)+gdedphv(x1)+gdedelv(x1)
      filco6a = (1.e+2 * (dum1 - dum2) ) / dum1
      return
      end
c===========================================================================
c C.22h
c                AN AUXILIARY FUNCTION FOR TESTCOM SUBROUTINE
c
      function filco6b(x)
      external getdedbr,getdedpa,getdedph,getdedel,gdedbrt,gdedpat
      external gdeion,cone,gdedpht
      x1 = 1.e+1**x
      dum1 = gdedbrt(x1) + gdedpat(x1) + gdedpht(x1) + gdeion(x1)
      dum2=cone(x1)+getdedbr(x1)+getdedpa(x1)+getdedph(x1)+getdedel(x1)
      filco6b = (1.e+2 * (dum1 - dum2) ) / dum1
      return
      end
c===========================================================================
c C.22i
c                AN AUXILIARY FUNCTION FOR TESTCOM SUBROUTINE
c
      function filco7(x)
      external glbremv,glpairv,glphnuv,glelecv,getlanrv,conv,simps
      parameter (lim = 1600)
      dimension aux(0:lim)
      eta = .5
      x1 = 1.e+1**x
      filco7 = 1./glbremv(x1) + 1./glpairv(x1) + 1./glphnuv(x1) + 
     +         1./glelecv(x1)
      filco7 = eta / filco7
c
      dow = alog(10.)
      up = alog(x1)
      step = (up - dow) / float(lim)
      do i=0,lim
      e = dow + (float(i) * step)
      e = exp(e)
      aux(i) = e / (conv(e) * 1.e-3)
      enddo
      chd = simps(aux,dow,up,lim)
      if (chd.lt.filco7) then
      filco7 = chd
      endif
c
      filco7 = getlanrv(eta,x1) / filco7
      filco7 = 1. / filco7
      return
      end
c===========================================================================
c C.22j
c                AN AUXILIARY FUNCTION FOR TESTCOM SUBROUTINE
c
      function filco7a(x)
      external glbremv,glpairv,glphnuv,glelecv,getlanrv,conv,simps
      external gdedbrv,gdedpav,gdedphv,gdedelv,gdedbrt,gdedpat,gdedpht
      external gdeion
      parameter (lim = 1600)
      dimension aux(0:lim)
      eta = .5
      x1 = 1.e+1**x
      filco7a = 1./glbremv(x1) + 1./glpairv(x1) + 1./glphnuv(x1) + 
     +         1./glelecv(x1)
      filco7a = eta / filco7a
c
      dow = alog(10.)
      up = alog(x1)
      step = (up - dow) / float(lim)
      do i=0,lim
      e = dow + (float(i) * step)
      e = exp(e)
      aux(i) = e / (conv(e) * 1.e-3)
      enddo
      chd = simps(aux,dow,up,lim)
      if (chd.lt.filco7a) then
      filco7a = chd
      endif
c
      filco7a = getlanrv(eta,x1) / filco7a
      filco7a = 1. / filco7a
      ev = gdedbrv(x1) + gdedpav(x1) + gdedphv(x1) + gdedelv(x1)
      et = gdedbrt(x1) + gdedpat(x1) + gdedpht(x1) + gdeion(x1)
      filco7a = ( ( (1./filco7a) * ev) + conv(x1)) / et
      return
      end
c===========================================================================
c C.22k
c                AN AUXILIARY FUNCTION FOR TESTCOM SUBROUTINE
c
      function filco10(x)
      external glbremv,glpairv,glphnuv,glelecv,getlanrv,conv,simps
      parameter (lim = 1600)
      dimension aux(0:lim)
      eta = 3.
      x1 = 1.e+1**x
      filco10 = 1./glbremv(x1) + 1./glpairv(x1) + 1./glphnuv(x1) + 
     +         1./glelecv(x1)
      filco10 = eta / filco10
c
      dow = alog(10.)
      up = alog(x1)
      step = (up - dow) / float(lim)
      do i=0,lim
      e = dow + (float(i) * step)
      e = exp(e)
      aux(i) = e / (conv(e) * 1.e-3)
      enddo
      chd = simps(aux,dow,up,lim)
      tmp = getlanrv(eta,x1)
      if (chd.lt.filco10) then
      filco10 = chd
      endif
c
      filco10 = getlanrv(eta,x1) / filco10
      filco10 = 1. / filco10
      return
      end
c===========================================================================
c C.22l
c                AN AUXILIARY FUNCTION FOR TESTCOM SUBROUTINE
c
      function filco10a(x)
      external glbremv,glpairv,glphnuv,glelecv,getlanrv,conv,simps
      external gdedbrv,gdedpav,gdedphv,gdedelv,gdedbrt,gdedpat,gdedpht
      external gdeion
      parameter (lim = 1600)
      dimension aux(0:lim)
      eta = 3.
      x1 = 1.e+1**x
      filco10a = 1./glbremv(x1) + 1./glpairv(x1) + 1./glphnuv(x1) + 
     +         1./glelecv(x1)
      filco10a = eta / filco10a
c
      dow = alog(10.)
      up = alog(x1)
      step = (up - dow) / float(lim)
      do i=0,lim
      e = dow + (float(i) * step)
      e = exp(e)
      aux(i) = e / (conv(e) * 1.e-3)
      enddo
      chd = simps(aux,dow,up,lim)
      tmp = getlanrv(eta,x1)
      if (chd.lt.filco10a) then
      filco10a = chd
      endif
c
      filco10a = getlanrv(eta,x1) / filco10a
      filco10a = 1. / filco10a
      ev = gdedbrv(x1) + gdedpav(x1) + gdedphv(x1) + gdedelv(x1)
      et = gdedbrt(x1) + gdedpat(x1) + gdedpht(x1) + gdeion(x1)
      filco10a = ( ( (1./filco10a) * ev) + conv(x1)) / et
      return
      end
c===========================================================================
c C.22m
c                AN AUXILIARY FUNCTION FOR TESTCOM SUBROUTINE
c
      function filco8(x)
      external glbremv,glpairv,glphnuv,glelecv,getlanrv,conv,simps
      parameter (lim = 1600)
      dimension aux(0:lim)
      eta = 1.
      x1 = 1.e+1**x
      filco8 = 1./glbremv(x1) + 1./glpairv(x1) + 1./glphnuv(x1) + 
     +         1./glelecv(x1)
      filco8 = eta / filco8
c
      dow = alog(10.)
      up = alog(x1)
      step = (up - dow) / float(lim)
      do i=0,lim
      e = dow + (float(i) * step)
      e = exp(e)
      aux(i) = e / (conv(e) * 1.e-3)
      enddo
      chd = simps(aux,dow,up,lim)
      tmp = getlanrv(eta,x1)
      if (chd.lt.filco8) then
      filco8 = chd
      endif
c
      filco8 = getlanrv(eta,x1) / filco8
      filco8 = 1. / filco8
      return
      end
c===========================================================================
c C.22n
c                AN AUXILIARY FUNCTION FOR TESTCOM SUBROUTINE
c
      function filco8a(x)
      external glbremv,glpairv,glphnuv,glelecv,getlanrv,conv,simps
      external gdedbrv,gdedpav,gdedphv,gdedelv,gdedbrt,gdedpat,gdedpht
      external gdeion
      parameter (lim = 1600)
      dimension aux(0:lim)
      eta = 1.
      x1 = 1.e+1**x
      filco8a = 1./glbremv(x1) + 1./glpairv(x1) + 1./glphnuv(x1) + 
     +         1./glelecv(x1)
      filco8a = eta / filco8a
c
      dow = alog(10.)
      up = alog(x1)
      step = (up - dow) / float(lim)
      do i=0,lim
      e = dow + (float(i) * step)
      e = exp(e)
      aux(i) = e / (conv(e) * 1.e-3)
      enddo
      chd = simps(aux,dow,up,lim)
      tmp = getlanrv(eta,x1)
      if (chd.lt.filco8a) then
      filco8a = chd
      endif
c
      filco8a = getlanrv(eta,x1) / filco8a
      filco8a = 1. / filco8a
      ev = gdedbrv(x1) + gdedpav(x1) + gdedphv(x1) + gdedelv(x1)
      et = gdedbrt(x1) + gdedpat(x1) + gdedpht(x1) + gdeion(x1)
      filco8a = ( ( (1./filco8a) * ev) + conv(x1)) / et
      return
      end
c===========================================================================
c C.22o
c                AN AUXILIARY FUNCTION FOR TESTCOM SUBROUTINE
c
      function filco9(x)
      external glbremv,glpairv,glphnuv,glelecv,getlanrv,conv,simps
      parameter (lim = 1600)
      dimension aux(0:lim)
      eta = 2.
      x1 = 1.e+1**x
      filco9 = 1./glbremv(x1) + 1./glpairv(x1) + 1./glphnuv(x1) + 
     +         1./glelecv(x1)
      filco9 = eta / filco9
c
      dow = alog(10.)
      up = alog(x1)
      step = (up - dow) / float(lim)
      do i=0,lim
      e = dow + (float(i) * step)
      e = exp(e)
      aux(i) = e / (conv(e) * 1.e-3)
      enddo
      chd = simps(aux,dow,up,lim)
      tmp = getlanrv(eta,x1)
      if (chd.lt.filco9) then
      filco9 = chd
      endif
c
      filco9 = getlanrv(eta,x1) / filco9
      filco9 = 1./filco9
      return
      end
c===========================================================================
c C.22p
c                AN AUXILIARY FUNCTION FOR TESTCOM SUBROUTINE
c
      function filco9a(x)
      external glbremv,glpairv,glphnuv,glelecv,getlanrv,conv,simps
      external gdedbrv,gdedpav,gdedphv,gdedelv,gdedbrt,gdedpat,gdedpht
      external gdeion
      parameter (lim = 1600)
      dimension aux(0:lim)
      eta = 2.
      x1 = 1.e+1**x
      filco9a = 1./glbremv(x1) + 1./glpairv(x1) + 1./glphnuv(x1) + 
     +         1./glelecv(x1)
      filco9a = eta / filco9a
c
      dow = alog(10.)
      up = alog(x1)
      step = (up - dow) / float(lim)
      do i=0,lim
      e = dow + (float(i) * step)
      e = exp(e)
      aux(i) = e / (conv(e) * 1.e-3)
      enddo
      chd = simps(aux,dow,up,lim)
      tmp = getlanrv(eta,x1)
      if (chd.lt.filco9a) then
      filco9a = chd
      endif
c
      filco9a = getlanrv(eta,x1) / filco9a
      filco9a = 1./filco9a
      ev = gdedbrv(x1) + gdedpav(x1) + gdedphv(x1) + gdedelv(x1)
      et = gdedbrt(x1) + gdedpat(x1) + gdedpht(x1) + gdeion(x1)
      filco9a = ( ( (1./filco9a) * ev) + conv(x1)) / et
      return
      end
c===========================================================================
c C.22q
c                AN AUXILIARY FUNCTION FOR TESTCOM SUBROUTINE
c
      function filco11(x)
      real*8 e,out,h1,h2
      e = dble(x)
      e = 1.d+1**e
      out = .175d+0 * (e**(-2.72d+0))
      h1 = 1.d+0 / (1.d+0 + (e / 1.03d+2)) 
      h2 = 3.7d-2 / (1.d+0 + (e / 8.1d+2))
      out = out * (h1 + h2)
      filco11 = sngl(out) 
      return
      end
c===========================================================================
c C.22r
c                AN AUXILIARY FUNCTION FOR TESTCOM SUBROUTINE
c
      function filco12(x)
      external filco11
      real*8 e,out,h1,h2,out1
      e = dble(x)
      e = 1.d+1**e
      out = .175d+0 * (e**(-2.72d+0))
      h1 = 1.d+0 / (1.d+0 + (e / 1.03d+2)) 
      h2 = 3.7d-2 / (1.d+0 + (e / 8.1d+2))
      out = out * (h1 + h2)
      out1 = 2.34d+1 * e**(-3.72d+0)
      filco12 = sngl((out1 - out) / out)
      return
      end
c===========================================================================
c C.22s
c                AN AUXILIARY FUNCTION FOR TESTCOM SUBROUTINE
c
      function filco13(x)
      external simps
      parameter (lim=100)
      dimension aux(0:100)
      common /tofil13/ whole
      dow = alog(1.e+1**(x - .0176))
      up = alog(1.e+1**(x + .0176)) 
      step = .01 * (up - dow) 
      do i=0,100
      e = dow + (step * float(i))
      e1 = exp(e)
      aux(i) = (18.025 / (103. + e1)) + (5.24475 / (810. + e1)) 
      aux(i) = (e1**(-2.72)) * aux(i) * e1
      enddo
      filco13 = simps(aux,dow,up,lim)
      filco13 = (filco13 / whole) * 2.e+7
      return
      end
c===========================================================================
c C.22t
c                AN AUXILIARY FUNCTION FOR TESTCOM SUBROUTINE
c
      function filco14(x)
      real*8 e,out,h1,h2
      e = dble(x)
      e = 1.d+1**e
      out = .14d+0 * (e**(-2.7d+0))
      h1 = 1.d+0 / (1.d+0 + (e / 1.045454d+2)) 
      h2 = 5.4d-2 / (1.d+0 + (e / 7.727273d+2))
      out = out * (h1 + h2)
      filco14 = sngl(out) 
      return
      end
c===========================================================================
c C.22u
c                AN AUXILIARY FUNCTION FOR TESTCOM SUBROUTINE
c
      function filco15(x)
      external filco11
      real*8 e,out,h1,h2,out1
      e = dble(x)
      e = 1.d+1**e
      out = .14d+0 * (e**(-2.7d+0))
      h1 = 1.d+0 / (1.d+0 + (e / 1.045454d+2)) 
      h2 = 5.4d-2 / (1.d+0 + (e / 7.727273d+2))
      out = out * (h1 + h2)
      out1 = 2.1d+1 * e**(-3.7d+0)
      filco15 = sngl((out1 - out) / out)
      return
      end
c===========================================================================
c C.22v
c                AN AUXILIARY FUNCTION FOR TESTCOM SUBROUTINE
c
      function filco16(x)
      external simps
      parameter (lim=100)
      dimension aux(0:100)
      common /tofil16/ whole
      dow = alog(1.e+1**(x - .0176))
      up = alog(1.e+1**(x + .0176)) 
      step = .01 * (up - dow) 
      do i=0,100
      e = dow + (step * float(i))
      e1 = exp(e)
      aux(i) = (14.63636/(104.5454 + e1)) + (5.84182/(772.7273 + e1)) 
      aux(i) = (e1**(-2.7)) * aux(i) * e1
      enddo
      filco16 = simps(aux,dow,up,lim)
      filco16 = (filco16 / whole) * 2.e+7
      return
      end
****************************************************************************
************************ BREMSSTRAHLUNG SUBROUTINES : **********************
****************************************************************************
* B.1
*      THIS REAL*8 FUNCTION CALCULATES DIFFERENTIAL CROS-SECTION FOR MUON
*     BREMSSTRAHLUNG IN ACCORDING TO FORMULAE TAKEN FRON E.V.BUGAEV AT AL,
*                  E-PREPRINT hep-ph/9803488 v2 Jun 1998)
*
*                                INPUT:
*                                =====
*
*  real*8 z     : electric charge of nucley
*  real*8 en    : muon energy (GeV)
*  real*8 rnu   : relative energy transfer = (E_transferred / E_mu)
*
*                                OUTPUT:
*                                ======
*
*  real*8 brem : d_sigma/d_v (sq. cm) for nucleus with given Z, muon energy 
*                                       and relative energy transfer
*  .........................................................................
       real*8 function brem(z,en,rnu)
       external fu1,fu2  ! auxiliary real*8 functions (see below B.2 and B.3)
       real*8 fu1,fu2
       real*8 z,en,rnu,rnu1
       real*8 alfa,rm_e,rm_mu,r_e,fact,a1,a2,del1,del2,x1,x2,ulim,avog
       real*8 eln1,eln2,psi10,psi20,psi1,psi2,z13,z23,qc,dze,q_min,h1
       common /exer1/ fa
       common /const/ alfa,rm_e,rm_mu,r_e,avog ! universal constants which are 
c                                              ! provided by subroutine 
c                                              ! MED_CONS and are passed here 
c                                              ! by common /const/
c  .........................................................................
c       SOME CONSTANTS FOR BREMSSTRAHLUNG FORMULAE (see hep-ph/9803488):
c       ****************************************************************
      fact=((4.d+0*z*z*r_e*r_e*rm_e*rm_e)/(rm_mu*rm_mu))*alfa
      rnu1=1.d+0-rnu
      z13=z**(1.d+1/3.d+1)
      z23=z13*z13
      qc=(1.9d+0*rm_mu)/z13
      dze=sqrt(1.d+0+((4.d+0*rm_mu*rm_mu)/(qc*qc)))
      a1 = 1.117d+2/(z13*rm_e)
      a2 = 7.242d+2/(z23*rm_e)
      del1=dlog(rm_mu/qc)+((dze/2.d+0)*dlog((dze+1.d+0)/(dze-1.d+0)))
      del2=dlog(rm_mu/qc)+((2.d+0*rm_mu*rm_mu)/(qc*qc))
      del2 = del2+((dze/4.d+0)*(3.d+0-(dze*dze))*dlog((dze+1.d+0)/(dze-1
     *.d+0)))
      q_min=((rnu/rnu1)*rm_mu*rm_mu*5.d-1)/(en*1.d+3)
      x1 = a1 * q_min
      x2 = a2 * q_min
c  .........................................................................
c    ULIM represents the upper limit for energy transfer via bremsstrahlung 
c    for nucleus with given electric charge Z, muon energy EN and  relative 
c                                  transfer RNU:
c
        ulim=((z)**(1.d+0/3.d+0))*1.236540953d+0*(rm_mu/(en*1.d+3))
        ulim = 1.d+0 - ulim
c  .........................................................................
        if (ulim.lt.rnu) then   !  if input energy transfer is greater than
        k1=0                    !  upper limit the diff. cross-section = 0.
        else                    !  Else it is calculated by fornmulae for
        k1=1                    !  bremstrahlung.
        endif                   !
        h1=dble(k1)             !
c  .........................................................................
c      CALCULATION OF DIFFERENTIAL CROSS-SECTION FOR MUON BREMSSTRAHLUNG:
        eln1=dlog((rm_mu*rm_mu*a1*a1)/(1.d+0+x1*x1))
        eln2=dlog((rm_mu*rm_mu*a2*a2)/(1.d+0+x2*x2))
        psi10 = 5.d-1 * (1.+eln1) - fu2(x1)
        psi10 = psi10 + (1.d+0/z)*(5.d-1*(1.d+0+eln2) - fu2(x2))
        psi20=(1.d+1/3.d+1) + 5.d-1*eln1 + fu1(x1)
        psi20=psi20+(1.d+0/z)*((1.d+0/3.d+0)+5.d-1*eln2+fu1(x2))
        psi1=psi10-del1
        psi2=psi20-del2
        brem=(fact/rnu)*((rnu1*rnu1+1.d+0)*psi1-(2.d+0/3.d+0)*rnu1*psi2)
        brem = brem * h1 * dble(fa)
        return
        end
****************************************************************************
* B.2
*                   REAL*8 function fu1 calculates the term             
*        2 * x^2 * (1 - x * ARCTAN (1/x) + (3/4) * LN ((x^2)/(1+x^2)))  
*  in formulae for muon bremsstrahlung. It is referenced by subroutine BREM 
*                       and reference FUNCTION fu2.
*  .........................................................................
       real*8 function fu1(c)
       external fu2
       real*8 c,fu2
      if (c.gt.1.d+3) then
        fu1 = -0.8333333333333d+0
      else
        if (c.lt.1.d-5) then
         ik=0
         fu1=dble(ik)
        else
         fu1=2.d+0*c*c*(1.d+0-fu2(c)+(7.5d-1)*dlog((c*c)/(1.d+0+(c*c)))) 
        endif
      endif
      return
      end
****************************************************************************
* B.3
*           REAL*8 function fu2 calculates the term x * arctan (1/x)
*  in formulae for muon bremsstrahlung. It is referenced by function FU1
*  .........................................................................
       real*8 function fu2(d)
       real*8 d
       if (d.gt.5.d+4) then
          ik = 1
          fu2 = dble(ik)
       else
           if (d.lt.1.d-8) then
             fu2 = d * 3.14159265359d+0 * 5.d-1
           else
             fu2 = d * datan(1.d+0/d)
           endif
       endif
       return
       end
**************************************************************************
* B.4
*       THIS FUNCTION CALCULATES DIFFERENTIAL CROS-SECTION FOR MUON
*        BREMSSTRAHLUNG IN GIVEN MEDIA USING REAL*8 FUNCTION BREM  
*
*                                  INPUT:
*                                  =====
*
*  real*4 ene    : muon energy (GeV)
*  real*4 v      : relative energy transfer
*
*                                  OUTPUT:
*                                  ======
*
*  real*4 brem_tot   : d_sigma/d_v (sq. cm), for given media, muon energy and 
*                      relative energy transfer. Values of diff. cross-section 
*                      are averaged over all nuclei which given media consists 
*                      of:
*
*                      d_s/d_v = SUM (w_i * d_s/d_v_i), where w_i = N_i/N_tot
*                      N_i - number of type i atoms in molecula
*                      N_tot - total number of atoms in molecula
*
*                      So, it is calculated for an "effective nucleus" with
*                      atomic weight A_eff = SUM(N_i*A_i) / N_tot
*  .........................................................................
       function brem_tot(ene,v)
       external brem
       real*8 z1(10),w(10),aw(10),a_ef,en,rnu,h1,brem,ro
       integer nsub
       common /media/ z1,w,aw,a_ef,ro,nsub ! this common determines media com-
c                                     ! position: AW (atomic weights), A_EF
c                                   ! (effective atomic weight), Z1 (charges), 
c                                 ! W(relative weights), NSUB (number of media 
c                               ! components), ro (cm/qubic cm) - media densi-
c                             ! ty.  All these  are prepared by subroutine
c                           ! MED_CONS and passed here via commom /media/.
c
       if (v.gt..999995) v = .999995
       en = dble(ene)
       rnu = dble(v)
       h1 = w(1) * brem(z1(1),en,rnu)
         if (nsub.ge.2) then
           do l=2,nsub
             h1 = h1 + (w(l) * brem(z1(l),en,rnu))
           enddo
         endif
       brem_tot = sngl(h1)
       return
       end
****************************************************************************
* B.5
       subroutine gamma1
*
*                      The subroutine calculates:
*
*                                 [1]
*                           
*  the differential cross-sections d_Sigma/d_v for muon bremsstrahlung using 
*            FUNCTION BREM_TOT. The result is put in three arrays
*      fcd1(81,54), fcd2(81,101) and fcd3(81,51) which are passed to 
* subroutines SPL2 via common block /cdbr_in/. The reason to use just 3 arrays
*     is that the region of energy transfers is divided into 3 regions: 
*      10^(-11) - 10^(-0.6), 10^(-0.6) - 10^(-0.1) and 10^(-0.1) - 1 
*     for which different grid steps are accepted (0.2, 0.01 and 0.001, 
*           correspondingly) to obtain accuracy better than 0.001.
*    Dif. cross-sections are computed for 81 values of muon energy starting 
*  with E_mu = 10 GeV and finishing with E_mu = 1000 Pev with logarithmically
*                          equidistant griid.     
*  SPL2 cooks spline coefficients out of these 3 arrays which are used in
*                          further computing.
*
*                                 [2]
*
*      Energy losses and mean free path  for muon bremsstrahlung with
*   energy transfers which exceed EMIN, VMIN and 0 and with en. transfers 
*    < EMIN - arrays elo_br1(17), crt_br1(17); elo_br2(17), crt_br2(17); 
*   elo_br3(17); elo_br4(17), correspondingly, which are computed for 17 
*   values of muon energy starting with E_mu = 10 GeV and finishing with 
*   E_mu = 1000 Pev with logarithmically equidistant griid. These arrays 
*  are passed to subroutines SPL1 and ENLOS via common blocks /elbr_in1/, 
*       /elbr_in2/, /elbr_in4/ and /ctbr_in1/ where they are used 
*    to prepare spline coefficients for further computing and calculate 
*           non-stochastical energy losses below EMIN and VMIN.
*
*                                 [3]
*
*    A value of CF which is a factor in a comparison function CF/V (V is
* relative energy transfer) which is used in subroutuine GETVBREM to simulate
*               energy transfers due to bremsstrahlung. 
*       CF is passed to subroutine getvbrem via common /br_ref/
*  .....................................................................
       external brem_tot,dsimps
       real*8 um,ene,u(10)
       real*8 aux1(0:2000),aux2(0:2000),h1,h2,dsimps
       real*8 alfa,rm_e,rm_mu,r_e,avog,z1(10),w(10),aw(10),a_ef,ro
       integer nsub
       common /const/ alfa,rm_e,rm_mu,r_e,avog ! from initial subroutine
       common /media/ z1,w,aw,a_ef,ro,nsub     !       MED_CONS
       common /general/ emin,vmin,emph         !
       common /cdbr_in/ fcd1(81,54),fcd2(81,101),fcd3(81,51) ! To SPL2
       common /ctbr_in1/ crt_br1(17),crt_br2(17) ! To SPL1 and ENLOS
       common /elbr_in1/ elo_br1(17),elo_br2(17) ! To SPL1 and ENLOS
       common /elbr_in2/ elo_br3(17)  ! to SPL1 and ENLOS
       common /elbr_in4/ elo_br4(17)  ! to ENLOS
       common /br_ref/ cf ! To GETVBREM
       common /help_1/ aux1,aux2
c   ....................................................................
c      1.PREPARATION OF ARRAYS WITH DIFF. CROSS-SEECTIONS FOR MUON
c                               BREMSSTRAHLUNG:
c
c                !       .......................... 
      do j=1,81  !--->   j CYCLE BY ENERGIES starts
c                !       .......................... 
        en = float(j)
        en = 1.e+1**(9.e-1 + (1.e-1 * en))
c                     !         ..................................
      do k=1,206      !--->     k CYCLE BY ENERGY TRANSFERS starts
c                     !         ..................................
      if (k.eq.206) then
        kk = 0
        fcd2(j,k-105) = float(kk)
        goto 975
      endif
c
      if (k.le.54) then
         k01 = k - 1
         rnu = 1.e+1**(float(k01) * 2.e-1 - 1.1e+1)
      else
         if (k.le.105) then
           k01 = k - 55
           rnu = 1.e+1**(float(k01) * 1.e-2 - 6.e-1)
         else
           k01 = k - 106
           rnu = 1.e+1**(float(k01) * 1.e-3 - 1.e-1)
         endif
      endif
c
      cr_dif=brem_tot(en,rnu)
      if (k.le.54) fcd1(j,k) = cr_dif
      if ((k.gt.54).AND.(k.lt.106)) fcd3(j,k-54) = cr_dif
      if (k.ge.106) fcd2(j,k-105) = cr_dif
c
 975  continue
c              !     ..................................... 
      enddo    !----> k CYCLE BY ENERGY TRANSFERS finishes 
c              !     ..................................... 
      enddo    !----> j CYCLE BY ENERGIES finishes 
c              !     .................................... 
c   ....................................................................
c    2.PREPARATION OF ARRAYS WITH ENERGY LOSSES AND TOTAL CROSS-SECTIONS
c                         FOR MUON BREMSSTRAHLUNG:
c
c                    a) Energy transfers > EMIN :
      lim = 2000
      do i=1,17
      en = float(i)
      en = (1.e+1)**(5.e-1 + (5.e-1 * en))
      vmi = alog(emin / en) !------------> The lower limit for integration
c
        um = 0.d+0
        do k=1,nsub
        ene=dble(en)
        u(k)=((z1(k))**(1.d+0/3.d+0))*1.236540953d+0*(rm_mu/(ene*1.d+3))
        u(k) = 1.d+0 - u(k)
        um = dmax1(u(k),um)
        enddo
        vma=sngl(um)  !----------------> The upper limit for integration
        vma=alog(vma) !
        vst = (vma - vmi) / float(lim) !-> step for a rel. en. transfers grid
c
      do j=0,lim
      rnu = vmi + vst*(float(j))
      rnu = exp(rnu)
      aux1(j) = dble(brem_tot(en,rnu) * rnu)       !-> array to be integrated 
c                                                  !   to get mean free path
      aux2(j) = aux1(j) * dble(rnu)                !-> array to be integrated 
      enddo                                        !   for getting en. losses
c
      h1 = dble(vmi)
      h2 = dble(vma)
      crt_br1(i) = dsimps(aux1,h1,h2,lim) 
      crt_br1(i) = sngl(a_ef / (avog * ro))/crt_br1(i) !-> array with 17 values 
c                                                      !   of free path
      elo_br1(i) = sngl( dsimps(aux2,h1,h2,lim) * avog * ro / a_ef )
      elo_br1(i) = elo_br1(i) * en * 1.e+3 
      elo_br1(i) = alog10(elo_br1(i)) !-> array with 17 values of en. losses
      enddo
c
c                    b) Energy transfers > VMIN :
      do i=1,17
      en = float(i)
      en = (1.e+1)**(5.e-1 + (5.e-1 * en))
      vmi = alog(vmin)          !------------> The lower limit for integration
c
        um = 0.d+0
        do k=1,nsub
        ene=dble(en)
        u(k)=((z1(k))**(1.d+0/3.d+0))*1.236540953d+0*(rm_mu/(ene*1.d+3))
        u(k) = 1.d+0 - u(k)
        um = dmax1(u(k),um)
        enddo
        vma=sngl(um)  !----------------> The upper limit for integration
        vma=alog(vma) !
        vst = (vma - vmi) / float(lim) !-> step for a rel. en. transfers grid
c
      do j=0,lim
      rnu = vmi + vst*(float(j))
      rnu = exp(rnu)
      aux1(j) = dble(brem_tot(en,rnu) * rnu)       !-> array to be integrated 
c                                                  !   to get mean free path
      aux2(j) = aux1(j) * dble(rnu)                !-> array to be integrated 
      enddo                                        !   for getting en. losses
c
      h1 = dble(vmi)
      h2 = dble(vma)
      crt_br2(i) = dsimps(aux1,h1,h2,lim) 
      crt_br2(i) = sngl(a_ef / (avog * ro))/crt_br2(i) !-> array with 17 values 
c                                                      !   of free path
      elo_br2(i) = sngl( dsimps(aux2,h1,h2,lim) * avog * ro / a_ef )
      elo_br2(i) = elo_br2(i) * en * 1.e+3 
      elo_br2(i) = alog10(elo_br2(i)) !-> array with 17 values of en. losses
      enddo
c
c                    c) Energy transfers > 0 :
      do i=1,17
      en = float(i)
      en = (1.e+1)**(5.e-1 + (5.e-1 * en))
      vmi = alog(1.e-3/en)      !------------> The lower limit for integration
c
        um = 0.d+0
        do k=1,nsub
        ene=dble(en)
        u(k)=((z1(k))**(1.d+0/3.d+0))*1.236540953d+0*(rm_mu/(ene*1.d+3))
        u(k) = 1.d+0 - u(k)
        um = dmax1(u(k),um)
        enddo
        vma=sngl(um)  !----------------> The upper limit for integration
        vma=alog(vma) !
        vst = (vma - vmi) / float(lim) !-> step for a rel. en. transfers grid
c
      do j=0,lim
      rnu = vmi + vst*(float(j))
      rnu = exp(rnu)
      aux2(j) = dble(brem_tot(en,rnu) * rnu * rnu) !-> array to be integrated 
c                                                  !   to get en. losses
      enddo
c
      h1 = dble(vmi)
      h2 = dble(vma)
      elo_br3(i) = sngl( dsimps(aux2,h1,h2,lim) * avog * ro / a_ef )
      elo_br3(i) = elo_br3(i) * en * 1.e+3 
      elo_br3(i) = alog10(elo_br3(i)) !-> array with 17 values of en. losses
      enddo
c                    c) Energy transfers < EMIN
      do i=1,17
      en = float(i)
      en = (1.e+1)**(5.e-1 + (5.e-1 * en))
      vmi = alog(1.e-3/en)      !------------> The lower limit for integration
      vma = alog(emin/en)   !----------------> The upper limit for integration
      vst = (vma - vmi) / float(lim) !----> step for a rel. en. transfers grid
c
      do j=0,lim
      rnu = vmi + vst*(float(j))
      rnu = exp(rnu)
      aux2(j) = dble(brem_tot(en,rnu) * rnu * rnu) !-> array to be integrated 
c                                                  !   to get en. losses
      enddo
c
      h1 = dble(vmi)
      h2 = dble(vma)
      elo_br4(i) = sngl( dsimps(aux2,h1,h2,lim) * avog * ro / a_ef )
      elo_br4(i) = elo_br4(i) * en * 1.e+3 !-> array with 17 values
      enddo                                !    of en. losses
c   ....................................................................
c                      3. COMPUTING OF CF FACTOR:
      en = 1.e+9
      rnu = emin / en
      cf = brem_tot(en,rnu)
      cf = cf * rnu * 1.02 
c
      return
      end
****************************************************************************
* B.6
      FUNCTION getlbrem(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*    Function GETLBREM calculates the muon's free path for bremmstrahlung
*        with energy treansfers > EMIN using spline coefficients
*                        prepared by SPL1 routine.
*          X is a value for muon energy expressed in GeV; XMIN,
*   XMAX and STEP are assigned with some values in subroutine SPL1 (they
*              are passed here via /sok1/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                              common /sok1_b/
* ..........................................................................
*        X MUST BE WITHIN THE RANGE OF 10 -- 1000,000,000 GeV
* ..........................................................................
*
      COMMON /sok1/ XMIN,STEP,XMAX
      common /sok1_b/ C(19)
      X1 = alog10(X)
       if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
       write(0,*)'ERROR IN FUNCTION GETLBREM: MUON ENERGY IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      getlbrem = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
****************************************************************************
* B.7
      FUNCTION glbremv(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*    Function GLBREMV calculates the muon's free path for bremmstrahlung
*        with energy treansfers > VMIN using spline coefficients
*                        prepared by SPL1 routine.
*          X is a value for muon energy expressed in GeV; XMIN,
*   XMAX and STEP are assigned with some values in subroutine SPL1 (they
*              are passed here via /sok1/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                              common /sok1_b2/
* ..........................................................................
*        X MUST BE WITHIN THE RANGE OF 10 -- 1000,000,000 GeV
* ..........................................................................
*
      COMMON /sok1/ XMIN,STEP,XMAX
      common /sok1_b2/ C(19)
      X1 = alog10(X)
       if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
       write(0,*)'ERROR IN FUNCTION GLBREMV: MUON ENERGY IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      glbremv = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
****************************************************************************
* B.8
       function getctbr(u)
*
*      The subroutine calculates the value for total muon bremsstrahlung
*              crossection with energy transfers above EMIN.
*   The resulting cross-section is averaged over all atoms of given media:
*
*                      sigma(E_mu) = SUM (w_i * sigma_i(E_mu)), 
*
*      where w_i = N_i/N_tot, N_i - number of type i atoms in molecula, 
* N_tot - total number of atoms in molecula, sigma_i(E_mu) is a cross-section
*                               for atom number i.
*     So, it is calculated for an "effective nucleus" with atomic weight 
*                         A_eff = SUM(N_i*A_i) / N_tot
*
*     It uses some media parameters prepared by subroutine MED_CONS
*                      (commons /const/ and /media/)
*                Input: u = muon energy (GeV) 
*                Output: getctbr = total cross-section (cm)
* ..........................................................................
*        U MUST BE WITHIN THE RANGE OF 10 -- 1000,000,000 GeV
* ..........................................................................
      external getlbrem
      real*8 z1(10),w(10),aw(10),a_ef,alfa,rm_e,rm_mu,r_e,avog,ro
      integer nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      common /media/ z1,w,aw,a_ef,ro,nsub
      if ((u.gt.1.0001e+9).or.(u.lt.9.9999e+0)) then   
      write(0,*)'ERROR IN FUNCTION GETCTBR: MUON ENERGY IS OUT OF RANGE'
      endif
      aef = sngl(a_ef / (avog * ro))
      getctbr = aef / getlbrem(u)
      return
      end
************************************************************************
* B.9
       function gctbrv(u)
*
*      The subroutine calculates the value for total muon bremsstrahlung
*              crossection with energy transfers above VMIN.
*   The resulting cross-section is averaged over all atoms of given media:
*
*                      sigma(E_mu) = SUM (w_i * sigma_i(E_mu)), 
*
*      where w_i = N_i/N_tot, N_i - number of type i atoms in molecula, 
* N_tot - total number of atoms in molecula, sigma_i(E_mu) is a cross-section
*                               for atom number i.
*     So, it is calculated for an "effective nucleus" with atomic weight 
*                         A_eff = SUM(N_i*A_i) / N_tot
*
*     It uses some media parameters prepared by subroutine MED_CONS
*                      (commons /const/ and /media/)
*                Input: u = muon energy (GeV) 
*                Output: gctbrv = total cross-section (cm)
* ..........................................................................
*        U MUST BE WITHIN THE RANGE OF 10 -- 1000,000,000 GeV
* ..........................................................................
      external glbremv
      real*8 z1(10),w(10),aw(10),a_ef,alfa,rm_e,rm_mu,r_e,avog,ro
      integer nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      common /media/ z1,w,aw,a_ef,ro,nsub
      if ((u.gt.1.0001e+9).or.(u.lt.9.9999e+0)) then   
      write(0,*)'ERROR IN FUNCTION GCTBRV: MUON ENERGY IS OUT OF RANGE'
      endif
      aef = sngl(a_ef / (avog * ro))
      gctbrv = aef / glbremv(u)
      return
      end
***************************************************************************
* B.10
         FUNCTION getdedbr(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*       The subroutine calculates the value for muon energy losses for 
*        bremsstrahlung with energy transfers above EMIN using spline 
*                coefficients prepared by spl1 subroutine.
*
*          X is a value for muon energy expressed in GeV; XMIN,
*  XMAX and STEP are assigned with some values in subroutine SPL1 (they
*               are passed here via /sok5/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                              common /sok5_b/
*  ......................................................................
*  ***   ATTENTION: X must be within a range of 10 GeV --- 1000 PeV   ***
*  ......................................................................
      COMMON /sok5/ XMIN,STEP,XMAX
      common /sok5_b/ C(19)
      X1 = alog10(X)
       if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
       write(0,*)'ERROR IN FUNCTION GETDEDBR: MUON ENERGY IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      getdedbr = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      getdedbr = (1.e+1)**(getdedbr)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
****************************************************************************
* B.11
      FUNCTION gdedbrv(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*       The subroutine calculates the value for muon energy losses for 
*        bremsstrahlung with energy transfers above VMIN using spline 
*                coefficients prepared by spl1 subroutine.
*
*          X is a value for muon energy expressed in GeV; XMIN,
*  XMAX and STEP are assigned with some values in subroutine SPL1 (they
*               are passed here via /sok5/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                              common /sok5_b2/
*  ......................................................................
*  ***   ATTENTION: X must be within a range of 10 GeV --- 1000 PeV   ***
*  ......................................................................
      COMMON /sok5/ XMIN,STEP,XMAX
      common /sok5_b2/ C(19)
      X1 = alog10(X)
       if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
       write(0,*)'ERROR IN FUNCTION GDEDBRV: MUON ENERGY IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      gdedbrv = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      gdedbrv = (1.e+1)**(gdedbrv)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
****************************************************************************
* B.12
         FUNCTION gdedbrt(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*   The subroutine calculates the value for total muon energy losses due to 
*    bremsstrahlung using spline coefficients prepared by spl1 subroutine.
*
*          X is a value for muon energy expressed in GeV; XMIN,
*  XMAX and STEP are assigned with some values in subroutine SPL1 (they
*               are passed here via /sok5/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                              common /sok5_b3/
*  ......................................................................
*  ***   ATTENTION: X must be within a range of 10 GeV --- 1000 PeV   ***
*  ......................................................................
      COMMON /sok5/ XMIN,STEP,XMAX
      common /sok5_b3/ C(19)
      X1 = alog10(X)
       if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
       write(0,*)'ERROR IN FUNCTION GDEDBRT: MUON ENERGY IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      gdedbrt = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      gdedbrt = (1.e+1)**(gdedbrt)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
****************************************************************************
* B.13
       FUNCTION getcdbr(X,Y,loga)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*  The subroutine calculates the value for differential muon bremsstrahlung
*   crossection d_Sigma/d_v using functions getcdbr1, getcdbr2, getcdbr3 or 
*  brem_tot depending on value of energy transfer Y. For definition of cross-
*  section for muon bremsstrahlung in given media see comments to subroutine 
*                                 BREM_TOT.
* X and Y are values for muon energy expressed in GeV and for relative energy 
*  transfer, respectively. If LOGA=0, then Y is expressed in natural units, 
*           otherwise the input for Y is presented as LOG10(Y).
*  .....................................................................
*        ATTENTION: X must be within a range of 10 GeV -- 1000 PeV
*                   Y must be within a range of 10**(-11) -- 10**(0)
*  ......................................................................
       external getcdbr1,getcdbr2,getcdbr3,brem_tot
       if (loga.eq.0) then
          if (Y.le..251188643) then
            getcdbr = getcdbr1(X,Y,loga)
          else
            if (Y.le..794328234) then
               getcdbr = getcdbr2(X,Y,loga)
            else
               if (X.le.1.e+2) then
               Z = 0.97
               else 
               Z = 0.992
               endif
               if (Y.le.Z) then
               getcdbr = getcdbr3(X,Y,loga)
               else
               getcdbr = brem_tot(X,Y)
               endif
            endif
          endif
       else
          if (Y.le.-6.e-1) then
            getcdbr = getcdbr1(X,Y,loga)
          else
            if (Y.le.-1.e-1) then
               getcdbr = getcdbr2(X,Y,loga)
            else
               if (X.le.1.e+2) then
               Z = -1.3228265e-2
               else 
               Z = -3.4883278e-3
               endif
               if (Y.le.Z) then 
               getcdbr = getcdbr3(X,Y,loga)
               else
               Y1 = (1.e+1)**Y
               getcdbr = brem_tot(X,Y1)
               endif
            endif
          endif
       endif
      return
      end
****************************************************************************
* B.14
       FUNCTION getcdbr1(X,Y,loga)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*  The subroutine calculates the value for differential muon bremsstrahlung
* crossection d_Sigma/d_v using spline coefficients prepared by SPL2 routine.
* For definition of cross-section for muon bremsstrahlung in given media see 
*                    comments to subroutine BREM_TOT.
*  .....................................................................
*   X and Y in body program is a value for muon energy expressed in GeV and
*    for relative energy transfer, respectively. NX, NY, X0, SX, Y0 and SY
*   are assigned with some values in subroutine SPL2 (they are passed here
*   via /sok_2_1/ common block. Array with spline coefficients C1(4648) is 
*            cooked by SPL2 and passed here via common /sok3/
*            If LOGA=0, then Y is expressed in natural units, 
*           otherwise the input for Y is presented as LOG10(Y).
*  ......................................................................
*        ATTENTION: X must be within a range of 10 GeV -- 1000 PeV
*                   Y must be within a range of 10**(-11) -- 10**(-0.6)
*  ......................................................................
       common /sok3/ C1(4648)
       common /sok_2_1/ NX,NY,X0,SX,Y0,SY
c
      X_1=alog10(X)
c
      if ((X_1.lt..99999e+0).or.(X_1.gt.9.0001e+0)) then
      write(0,*)'ERROR IN FUNCTION GETCDBR1: MUON ENERGY IS OUT OF RANGE'
      endif
c
      if (loga.eq.0) then    
      Y_1=alog10(Y)
      else
      Y_1=Y
      endif
c
      if ((Y_1.lt.-11.001).or.(Y_1.gt.-5.9999e-1)) then
      write(0,*)'ERROR IN FUNCTION GETCDBR1: EN. TRANSFER IS OUT OF RANGE'
      endif
c
      A3=(X_1-X0)/SX
      B3=(Y_1-Y0)/SY
      M1=INT(A3+.5)
      M2=INT(B3+.5)
      M3=NX+2
ccc      IF(M1.LT.0.OR.M1.GE.NX.OR.M2.LT.0.OR.M2.GE.NY) THEN
ccc      PRINT 1,X_1,M1,Y_1,M2
ccc      ENDIF
      A3=A3-M1
      B3=B3-M2
      M1=M2*M3+M1+1
      M2=M1+M3
      M3=M2+M3
      A2 = A3*A3 +.25
      B2 = B3*B3 +.25
      A1=A2-A3
      B1=B2-B3
      A3=A2+A3
      B3=B2+B3
      A2=2.-(A2+A2)
      B2=2.-(B2+B2)
      getcdbr1=(A1*C1(M1)+A2*C1(M1+1)+A3*C1(M1+2))*B1
     2      +(A1*C1(M2)+A2*C1(M2+1)+A3*C1(M2+2))*B2
     3      +(A1*C1(M3)+A2*C1(M3+1)+A3*C1(M3+2))*B3
      if (getcdbr1.lt.0.e+0) getcdbr1 = 0.e+0
      RETURN
ccc1     FORMAT('*MISTAKE*X=',D23.16,'MX=',I4,'Y=',D23.16,'MY=',I4)
      END
****************************************************************************
* B.15
       FUNCTION getcdbr2(X,Y,loga)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*  The subroutine calculates the value for differential muon bremsstrahlung
* crossection d_Sigma/d_v using spline coefficients prepared by SPL2 routine.
* For definition of cross-section for muon bremsstrahlung in given media see 
*                    comments to subroutine BREM_TOT.
*  .....................................................................
*   X and Y in body program is a value for muon energy expressed in GeV and
*    for relative energy transfer, respectively. NX, NY, X0, SX, Y0 and SY
*   are assigned with some values in subroutine SPL2 (they are passed here
*   via /sok_2_2/ common block. Array with spline coefficients C1(4399) is 
*            cooked by SPL2 and passed here via common /sok6/
*            If LOGA=0, then Y is expressed in natural units, 
*           otherwise the input for Y is presented as LOG10(Y).
*  ......................................................................
*        ATTENTION: X must be within a range of 10 GeV -- 1000 PeV
*                   Y must be within a range of 10**(-0.6) -- 10**(-0.1)
*  ......................................................................
       common /sok6/ C1(4399)
       common /sok_2_2/ NX,NY,X0,SX,Y0,SY
c
      X_1=alog10(X)
      if ((X_1.lt..99999e+0).or.(X_1.gt.9.000001e+0)) then
      write(0,*)'ERROR IN FUNCTION GETCDBR2: MUON ENERGY IS OUT OF RANGE'
      endif
c
      if (loga.eq.0) then    
      Y_1=alog10(Y)
      else
      Y_1=Y
      endif
c
      if ((Y_1.lt.-.60001).or.(Y_1.gt.-.99999e-1)) then
      write(0,*)'ERROR IN FUNCTION GETCDBR2: EN. TRANSFer IS OUT OF RANGE'
      endif
c
      A3=(X_1-X0)/SX
      B3=(Y_1-Y0)/SY
      M1=INT(A3+.5)
      M2=INT(B3+.5)
      M3=NX+2
ccc      IF(M1.LT.0.OR.M1.GE.NX.OR.M2.LT.0.OR.M2.GE.NY) THEN
ccc      PRINT 1,X_1,M1,Y_1,M2
ccc      ENDIF
      A3=A3-M1
      B3=B3-M2
      M1=M2*M3+M1+1
      M2=M1+M3
      M3=M2+M3
      A2=A3*A3+.25
      B2=B3*B3+.25
      A1=A2-A3
      B1=B2-B3
      A3=A2+A3
      B3=B2+B3
      A2=2.-(A2+A2)
      B2=2.-(B2+B2)
      getcdbr2=(A1*C1(M1)+A2*C1(M1+1)+A3*C1(M1+2))*B1
     2      +(A1*C1(M2)+A2*C1(M2+1)+A3*C1(M2+2))*B2
     3      +(A1*C1(M3)+A2*C1(M3+1)+A3*C1(M3+2))*B3
      if (getcdbr2.lt.0.e+0) getcdbr2 = 0.e+0
      RETURN
ccc1     FORMAT('*MISTAKE*X=',D23.16,'MX=',I4,'Y=',D23.16,'MY=',I4)
      END
****************************************************************************
* B.16
       FUNCTION getcdbr3(X,Y,loga)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*  The subroutine calculates the value for differential muon bremsstrahlung
* crossection d_Sigma/d_v using spline coefficients prepared by SPL2 routine.
* For definition of cross-section for muon bremsstrahlung in given media see 
*                    comments to subroutine BREM_TOT.
*  .....................................................................
*   X and Y in body program is a value for muon energy expressed in GeV and
*    for relative energy transfer, respectively. NX, NY, X0, SX, Y0 and SY
*   are assigned with some values in subroutine SPL2 (they are passed here
*   via /sok_2_3/ common block. Array with spline coefficients C2(8549) is 
*            cooked by SPL2 and passed here via common /sok4/
*            If LOGA=0, then Y is expressed in natural units, 
*           otherwise the input for Y is presented as LOG10(Y).
*  ......................................................................
*        ATTENTION: X must be within a range of 10 GeV -- 1000 PeV
*                   Y must be within a range of 10**(-0.1) -- 1.0
*  ......................................................................
       common /sok4/ C2(8549)
       common /sok_2_3/ NX,NY,X0,SX,Y0,SY
c
      X_1=alog10(X)
      if ((X_1.lt..99999e+0).or.(X_1.gt.9.000001e+0)) then
      write(0,*)'ERROR IN FUNCTION GETCDBR3: MUON ENERGY IS OUT OF RANGE'
      endif
c
      if (loga.eq.0) then    
      Y_1=alog10(Y)
      else
      Y_1=Y
      endif
c
      if ((Y_1.lt.-1.00001e-1).or.(Y_1.gt.1.e-10)) then
      write(0,*)'ERROR IN FUNCTION GETCDBR3: EN. TRANSFER IS OUT OF RANGE'
      endif
c
      A3=(X_1-X0)/SX
      B3=(Y_1-Y0)/SY
      M1=INT(A3+.5)
      M2=INT(B3+.5)
      M3=NX+2
ccc      IF(M1.LT.0.OR.M1.GE.NX.OR.M2.LT.0.OR.M2.GE.NY) THEN
ccc      PRINT 1,X_1,M1,Y_1,M2
ccc      ENDIF
      A3=A3-M1
      B3=B3-M2
      M1=M2*M3+M1+1
      M2=M1+M3
      M3=M2+M3
      A2=A3*A3+.25
      B2=B3*B3+.25
      A1=A2-A3
      B1=B2-B3
      A3=A2+A3
      B3=B2+B3
      A2=2.-(A2+A2)
      B2=2.-(B2+B2)
      getcdbr3=(A1*C2(M1)+A2*C2(M1+1)+A3*C2(M1+2))*B1
     2      +(A1*C2(M2)+A2*C2(M2+1)+A3*C2(M2+2))*B2
     3      +(A1*C2(M3)+A2*C2(M3+1)+A3*C2(M3+2))*B3
      if (getcdbr3.lt.0.e+0) getcdbr3 = 0.e+0
      RETURN
ccc1     FORMAT('*MISTAKE*X=',D23.16,'MX=',I4,'Y=',D23.16,'MY=',I4)
      END
****************************************************************************
* B.17
*
      SUBROUTINE getvbrem(emw,vbr,itr)
*
*   It simulates the relative enegy transfers for muon's bremsstrahlung with
*    energy transfers > EMIN (itr=0) or VMIN (itr=1) using function GETCDBR. 
*                Input: emw = muon energy expressed in GeV, 
*                Output: vbr = relative energy transfer. 
*
*   The simulation is done by the "rejection method" (see W.H.Press et al.,
*  NUMERICAL RECEIPES (THE ART OF SCIENTIFIC COMPUTING), Cambridge University
*            press, Chapter 7, pp. 200-204) with comparison function 
*                              f(v) = cf / v
* where cf is calculated by GAMMA1 routine and passed here via common /br_ref/
* ..........................................................................
*            EMW MUST BE WITHIN THE RANGE OF 10 -- 1000,000,000 GeV
* ..........................................................................
      external getcdbr
      parameter (lo=1)
      common /general/ emin,vmin,emph
      common /mcef/ mcb1,mcb2,mcp1,mcp2,mcn1,mcn2,mce1,mce2
      common /br_ref/ cf
      mcb1 = mcb1 + 1
         if (itr.eq.0) then
          algemin = alog10(emin/emw)
         else
          algemin = alog10(vmin)
         endif
  155 ax = algemin * rndm(5)   
      mcb2 = mcb2 + 1
      y1 = getcdbr(emw,ax,lo)
      vbr = exp(-2.3025851e+0 * ax)
      y2 = cf * vbr
      yc = y2 * rndm(8)
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc!    
           if (y2.lt.y1) then                             !    THIS PART  MAY BE 
           write(0,*)'* GETVBREM ERROR * f(v) < d_Sigma/d_v' !--> REMOVED AFTER ALL
           endif                                          !    TESTS  ARE PASSED
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccc!
        if (y1.ge.yc) then
        vbr = 1.e+0 / vbr
        else
        goto 155
        endif
      return
      end
****************************************************************************
* B.18
         subroutine testbrem(lukum)
*
*  Performes some test for all routines which relate e+e- pair production
*              with filling number of histograms and .dat files.
*  The test is short if lukum=0. Otherwise it takes more time but gives 
*                            more statistics. 
*
      external brem_tot,getcdbr,getdedbr,getctbr,getlbrem
      external gdedbrv,gdedbrt,gctbrv,glbremv
      external filbr1,filbr2,filbr3,filbr1a,filbr4,filbr5,filbr6
      dimension ene(25)
      real*8 esum,esumsq,eterm
      character *10 txt1
      character *100 txt2
      character *1 txt3 
      character *11 txt4 
      common /general/ emin,vmin,emph
      common /br1/ NA,enpa
      common /mcef/ mcb1,mcb2,mcp1,mcp2,mcn1,mcn2,mce1,mce2
      data ene / 1.e+1,2.e+1,5.e+1,1.e+2,2.e+2,5.e+2,1.e+3,2.e+3,5.e+3,
     +           1.e+4,2.e+4,5.e+4,1.e+5,2.e+5,5.e+5,1.e+6,2.e+6,5.e+6,
     +           1.e+7,2.e+7,5.e+7,1.e+8,2.e+8,5.e+8,1.e+9             /
c
      write(0,*)' '
      write(0,*)'                TESTING THE PART FOR BREMSSTRAHLUNG:'
      write(0,*)' '
         if (lukum.eq.0) then
           NA = 500000
         else
           NA = 5000000
         endif
c---------------------------------------------------------------------------
c       1. DIFFERENCES BETWEEN GETCDBR AND BREM_TOT (PRECISION OF SPLINES)
c---------------------------------------------------------------------------
      write(0,*)'Computing precision of splines for dif. cross-sections:'
      write(0,*)'  '
c
      itmp = 1
      do i=1,25
         k = int(ene(i))
         write(txt1,'(i10)') k
c
         if (i.eq.itmp) then
         itmp = itmp + 3
         j = 3000 + i
         txt2='Err.(percents) between BREMTOT and GETCDBR (lo=1) for '//
     +txt1//' GeV muon vs. V'
          CALL HBOOK2(j,txt2,100,-1.1e+1,0.e+0,100,-.48,+.02,0.)
         j = 3200 + i
         txt2='Err.(percents) between GETCDBR (lo=1) and GETCDBR (lo=0)
     +for '//txt1//' GeV muon vs. V'
          CALL HBOOK2(j,txt2,100,-1.1e+1,0.e+0,100,-.00003,+.00003,0.)
         j = 4000 + i
         txt2='Err.(percents) between BREMTOT and GETCDBR (lo=1) for '//
     +txt1//' GeV muon vs. V'
          CALL HBOOK2(j,txt2,100,.95,1.,100,-.02,+.14,0.)
         j = 4200 + i
         txt2='Err.(percents) between GETCDBR (lo=1) and GETCDBR (lo=0)
     +for '//txt1//' GeV muon vs. V'
          CALL HBOOK2(j,txt2,100,.95,1.,100,-5.e-6,+5.e-6,0.)
          endif
c
        enddo
c
         txt2='Err.(percents) between BREMTOT and GETCDBR (lo=1) for all
     + muon energies vs. V'
          CALL HBOOK2(3000,txt2,100,-1.1e+1,0.e+0,100,-.48,+.02,0.)
         txt2='Err.(percents) between BREMTOT and GETCDBR (lo=0) for all
     + muon energies vs. V'
          CALL HBOOK2(3100,txt2,100,-1.1e+1,0.e+0,100,-.48,+.02,0.)
         txt2='Err.(percents) between GETCDBR (lo=1) and GETCDBR (lo=0)
     +for all muon energies vs. V'
        CALL HBOOK2(3200,txt2,100,-1.1e+1,0.e+0,100,-.00003,+.00003,0.)
         txt2='Err.(percents) between BREMTOT and GETCDBR (lo=1) for all
     + muon energies vs. V'
          CALL HBOOK2(4000,txt2,100,.95,1.,100,-.02,+.14,0.)
         txt2='Err.(percents) between BREMTOT and GETCDBR (lo=0) for all
     + muon energies vs. V'
          CALL HBOOK2(4100,txt2,100,.95,1.,100,-.02,+.14,0.)
         txt2='Err.(percents) between GETCDBR (lo=1) and GETCDBR (lo=0)
     +forr all muon energies vs. V'
          CALL HBOOK2(4200,txt2,100,.95,1.,100,-5.e-6,+5.e-6,0.)
c
      l0 = 0
      l1 = 1
      itmp=1
      do i=1,25
         e = ene(i)
         vmin1 = emin/e
         vmin_0 = amin1(vmin,vmin1)
         write(0,*)'Energy Nb.',i,' of 25 is processed (',e,' GeV ) ...'
         do j=11000,200,-1
           v1 = -float(j) * 1.e-3
           v = 1.e+1**v1
           if (v.lt.vmin_0) goto 1001
           dc_r = brem_tot(e,v) 
           dc_s_n = getcdbr(e,v,l0)
           dc_s_l = getcdbr(e,v1,l1)
           p1 = 1.e+2 * ((dc_s_n - dc_r)/dc_r)
           p2 = 1.e+2 * ((dc_s_l - dc_r)/dc_r)
           p3 = ((2.e+2 * (dc_s_n - dc_s_l)) / (dc_s_n + dc_s_l))
           if (i.ne.itmp) goto 9801
           k3000 = 3000 + i
           CALL HFILL(k3000,v1,p2,1.)
 9801      CALL HFILL(3000,v1,p2,1.)
           CALL HFILL(3100,v1,p1,1.)
           if (i.ne.itmp) goto 9802
           k3200 = 3200 + i
 9802      CALL HFILL(k3200,v1,p3,1.)
           CALL HFILL(3200,v1,p3,1.)
 1001      continue  
         enddo
c
         do j=95000,99999
           v = float(j) * 1.e-5
           v1 = alog10(v)
           dc_r = brem_tot(e,v) 
           dc_s_n = getcdbr(e,v,l0)
           dc_s_l = getcdbr(e,v1,l1)
c
           if (dc_r.le.0.) then
           if (dc_s_n.le.1.e-37) then
           p1 = 0. 
           else
           p1=-0.14
           write(0,*)'ERROR DURING BREM TEST (HIST 41xx)'
           endif
           else
           p1 = 1.e+2 * ((dc_s_n - dc_r)/dc_r)
           endif
c
           if (dc_r.le.0.) then
           if (dc_s_l.le.1.e-37) then
           p2 = 0. 
           else
           p2=-0.14
           write(0,*)'ERROR DURING BREM TEST (HIST 40xx)'
           endif
           else
           p2 = 1.e+2 * ((dc_s_l - dc_r)/dc_r)
           endif
c
           gopa = dc_s_n + dc_s_l
           if (gopa.le.0.) then
           p3 = 0.
           else
           p3 = ((2.e+2 * (dc_s_n - dc_s_l)) / (dc_s_n + dc_s_l))
           endif
c
           if (i.ne.itmp) goto 9811
           k4000 = 4000 + i
           CALL HFILL(k4000,v,p2,1.)
 9811      CALL HFILL(4000,v,p2,1.)
           CALL HFILL(4100,v,p1,1.)
           if (i.ne.itmp) goto 9812
           k4200 = 4200 + i
           CALL HFILL(k4200,v,p3,1.)
 9812      CALL HFILL(4200,v,p3,1.)
         enddo
         if (i.eq.itmp) itmp = itmp + 3
      enddo
c---------------------------------------------------------------------------
c                2. QUALITY OF ENERGY TRANSFER SIMULATION
c---------------------------------------------------------------------------
c
c                             a) ABOVE EMIN
      write(0,*)'  '
      write(0,*)'Testing GETVBREM (en. transfers simulation) with EMIN ...'
      write(0,*)'  '
c
        do i=1,25
          k = int(ene(i))
          write(txt1,'(i10)') k
          txt2 = 'Brem, MC En. transfers distribution for '//txt1//' Gev
     + muon (EMIN)'
          i1 = i + 3500
          CALL HBOOK1(i1,txt2,100,-11.,0.,0.)
        enddo
c
       open(11,file='cross_br.dat',status='new',form='formatted')
       open(12,file='leng_br.dat',status='new',form='formatted')
       open(13,file='loss_br.dat',status='new',form='formatted')
       open(14,file='loss_br1.dat',status='new',form='formatted')
       open(15,file='en_br.dat',status='new',form='formatted')
       open(16,file='er_lo_br.dat',status='new',form='formatted')
       open(17,file='efmc_br.dat',status='new',form='formatted')
c  EN_BR.DAT - 25 values of muon energy for which all other values are computed.
c  CROSS_BR.DAT - 25 values of total cross-section for bremsstrahlung with 
c                 energy transfers > emin (Sq. cm).
c  LENG_BR.DAT - 25 values for averaged free path between two bremsstrahlung
c                interactions with energy transfer > emin (cm). 
c  LOSS_BR.DAT - 25 values for energy losses due to bremsstrahlung with energy
c                transfers > emin (MeV/cm) computed with splines (GETDEDBR 
c                routine).
c  LOSS_BR1.DAT - 25 values for energy losses for bremsstrahlung with energy
c                 transfers > emin (MeV/cm) simulated by MC (GETVBREM routine)
c                 with 500000 interactions for each muon energy (lukum = 0) or
c                 5000000 interactions (lukum > 0).
c ER_LO_BR.DAT - 25 values of statistical error for values in LOSS_BR1.DAT on 
c                1-sigma level (MeV/cm).
c EFMC_BR.DAT - 25 values of MC efficiency = 
c                              (number of simulation / number of call getvbrem)
      itr = 0
      do i=1,25
       write(0,*)'Energy Nb.',i,' of 25 is processed (',ene(i),' GeV ) ...'
       i1 = i + 3500
       elos = getdedbr(ene(i))
       tcros = getctbr(ene(i)) 
       rr = getlbrem(ene(i)) 
       write (13,234) elos
       write (11,234) tcros
       write (15,234) ene(i)
       write (12,234) rr
       esum = 0.d+0
       esumsq = 0.d+0
         mcb1 = 0
         mcb2 = 0
         do j=1,NA
           call getvbrem(ene(i),v,itr)
            v2 = alog10(v)
            eterm1 = ene(i) * v * 1.e+3
            eterm = dble(eterm1)
            esum = esum + eterm
            esumsq = esumsq + (eterm * eterm)
           CALL HFILL(i1,v2,0.,1.)
         enddo  
       efb = float(mcb2) / float(mcb1)
       mcb1 = 0
       mcb2 = 0
       write (17,234) efb
       esum1 = sngl(esum)
       esumsq1 = sngl(esumsq)
       RNA = float(NA)
       err=((RNA*esumsq1) - (esum1*esum1))/(RNA*RNA*(RNA - 1.))
       err = sqrt(err) / rr
       esum1 = (esum1 / rr) / RNA
       write (14,234) esum1
       write (16,234) err
       write(0,*)'En.los.=',esum1,' +-',err,' (should be ',elos,' ) MeV/cm'
          k = int(ene(i))
          write(txt1,'(i10)') k
          txt2 = 'Brem, En. transfers distribution for '//txt1//' Gev mu
     +on (EMIIN)'
          i1 = i + 3600
          enpa = ene(i)
          CALL HBFUN1(i1,txt2,100,-1.1e+1,0.e+0,filbr1)
      enddo
       close (17)
       close (16)
       close (15)
       close (14)
       close (13)
       close (12)
       close (11)
c                             b) ABOVE VMIN
      write(0,*)'  '
      write(0,*)'Testing GETVBREM (en. transfers simulation) with VMIN ...'
      write(0,*)'  '
c
        do i=1,25
          k = int(ene(i))
          write(txt1,'(i10)') k
          txt2 = 'Brem, MC En. transfers distribution for '//txt1//' Gev
     + muon (VMIN)'
          i1 = i + 3800
          CALL HBOOK1(i1,txt2,100,-11.,0.,0.)
        enddo
c
       open(11,file='cross_br_v.dat',status='new',form='formatted')
       open(12,file='leng_br_v.dat',status='new',form='formatted')
       open(13,file='loss_br_v.dat',status='new',form='formatted')
       open(15,file='loss_br_t.dat',status='new',form='formatted')
       open(14,file='loss_br1_v.dat',status='new',form='formatted')
       open(16,file='er_lo_br_v.dat',status='new',form='formatted')
       open(17,file='efmc_br_v.dat',status='new',form='formatted')
c  CROSS_BR_V.DAT - 25 values of total cross-section for bremsstrahlung with 
c                   energy transfers > vmin (Sq. cm).
c  LENG_BR_V.DAT - 25 values for averaged free path between two bremsstrahlung
c                  interactions with energy transfer > vmin (cm). 
c  LOSS_BR_V.DAT - 25 values for energy losses due to bremsstrahlung with energy
c                  transfers > vmin (MeV/cm) computed with splines (GDEDBRV 
c                  routine).
c  LOSS_BR_T.DAT - 25 values for total energy losses due to bremsstrahlung
c                  (MeV/cm) computed with splines (GDEDBRT routine).
c  LOSS_BR1_V.DAT - 25 values for energy losses for bremsstrahlung with energy
c                 transfers > vmin (MeV/cm) simulated by MC (GETVBREM routine)
c                 with 500000 interactions for each muon energy (lukum = 0) or
c                 5000000 interactions (lukum > 0).
c ER_LO_BR_V.DAT - 25 values of statistical error for values in LOSS_BR1_V.DAT 
c                  on 1-sigma level (MeV/cm).
c EFMC_BR_V.DAT - 25 values of MC efficiency = 
c                              (number of simulation / number of call getvbrem)
      itr = 1
      do i=1,25
       write(0,*)'Energy Nb.',i,' of 25 is processed (',ene(i),' GeV ) ...'
       i1 = i + 3800
       elos = gdedbrv(ene(i))
       elost = gdedbrt(ene(i))
       tcros = gctbrv(ene(i)) 
       rr = glbremv(ene(i)) 
       write (13,234) elos
       write (15,234) elost
       write (11,234) tcros
       write (12,234) rr
       esum = 0.d+0
       esumsq = 0.d+0
         mcb1 = 0
         mcb2 = 0
         do j=1,NA
           call getvbrem(ene(i),v,itr)
            v2 = alog10(v)
            eterm1 = ene(i) * v * 1.e+3
            eterm = dble(eterm1)
            esum = esum + eterm
            esumsq = esumsq + (eterm * eterm)
           CALL HFILL(i1,v2,0.,1.)
         enddo  
       efb = float(mcb2) / float(mcb1)
       mcb1 = 0
       mcb2 = 0
       write (17,234) efb
       esum1 = sngl(esum)
       esumsq1 = sngl(esumsq)
       RNA = float(NA)
       err=((RNA*esumsq1) - (esum1*esum1))/(RNA*RNA*(RNA - 1.))
       err = sqrt(err) / rr
       esum1 = (esum1 / rr) / RNA
       write (14,234) esum1
       write (16,234) err
       write(0,*)'En.los.=',esum1,' +-',err,' (should be ',elos,' ) MeV/cm'
          k = int(ene(i))
          write(txt1,'(i10)') k
          txt2 = 'Brem, En. transfers distribution for '//txt1//' Gev mu
     +on (VMIN)'
          i1 = i + 3900
          enpa = ene(i)
          CALL HBFUN1(i1,txt2,100,-1.1e+1,0.e+0,filbr1a)
      enddo
       close (17)
       close (16)
       close (15)
       close (14)
       close (13)
       close (12)
       close (11)
c---------------------------------------------------------------------------
c     3. Files with differential cros-sections for e+e- pair productioin
c---------------------------------------------------------------------------
c 9 files (br_cr_1.dat, ... , br_cr_9.dat) are produced with 1101 values in
c    each. These values are equal to differential cross-section for muon  
c   bremsstrahlung for log10(v) = -11.00, -10.99, -10.98, ... , -0.01, 0.
c Figure "X" in file name br_cr_X.dat corresponds to muon energy for which 
c            cross-section is computed (E_mu = 10.**X GeV).
       write(0,*)'  '
       write(0,*)'Computing dif. crossect. for a set of muon energies ...'
       write(0,*)'  '
       ilk = 0
       do i=1,25,3
       ilk = ilk + 1
       write(txt3,'(i1)') ilk
       txt4='br_cr_'//txt3//'.dat'
       open(11,file=txt4,status='new',form='formatted')
          do j=1100,0,-1
             v1 = -float(j) * 1.e-2
             v1 = 1.e+1 ** v1
             some = brem_tot(ene(i),v1)
             write (11,234) some
          enddo
      close (11)
      enddo 
c---------------------------------------------------------------------------
c  4. DIFFERENCES BETWEEN GETLBREM, GETDEDBR, GLBREMV, GDEDBRV, GDEDBRT AND 
c            REAL VALUES OF AVERAGED FREE PATH AND ENERGY LOSSES 
c                          (PRECISION OF SPLINES)
c---------------------------------------------------------------------------
       write(0,*)'Testing quality of 1-dim. splines ...'
       write(0,*)'  '
      txt2 = 'Brem, rel. err. GETLBREM - mean free path (percents)'
          CALL HBFUN1(3701,txt2,97,.9583334,9.04165,filbr2)
      txt2 = 'Brem, rel. err. GETDEDBR - energy losses (percents)'
          CALL HBFUN1(3702,txt2,97,.9583334,9.04165,filbr3)
      txt2 = 'Brem, rel. err. GLBREMV - mean free path (percents)'
          CALL HBFUN1(3703,txt2,97,.9583334,9.04165,filbr4)
      txt2 = 'Brem, rel. err. GDEDBRV - energy losses (percents)'
          CALL HBFUN1(3704,txt2,97,.9583334,9.04165,filbr5)
      txt2 = 'Brem, rel. err. GDEDBRT - energy losses (percents)'
          CALL HBFUN1(3705,txt2,97,.9583334,9.04165,filbr6)
c---------------------------------------------------------------------------
  234  format (e13.7) 
      return
      end
c===========================================================================
c B.18a
c                AN AUXILIARY FUNCTION FOR TESTBREM SUBROUTINE
c
      function filbr1(x)
      external getctbr,brem_tot,simps
      common /br1/ NA,enpa
      common /general/ emin,vmin,emph
      dimension aux(0:20)
      lim = 20
      vtr=emin/enpa
        op1 = x - .055
        op2 = x + .055
        ope1 = 2.302585093 * op1
        ope2 = 2.302585093 * op2
        op2_0 = 1.0e+1**(op2)
          if (op2_0.le.vtr) then
          filbr1 = 0.
          goto 422
          else
            do j=0,lim
              rnu = 1.e+1**(op1 + ( (.11/float(lim)) * (float(j))))
              aux(j) = brem_tot(enpa,rnu) * rnu
            enddo
          filbr1 = simps(aux,ope1,ope2,lim) 
          filbr1 = (filbr1 / getctbr(enpa)) * float(NA)
          goto 422
          endif
 422  return
      end
c===========================================================================
c B.18a1
c                AN AUXILIARY FUNCTION FOR TESTBREM SUBROUTINE
c
      function filbr1a(x)
      external gctbrv,brem_tot,simps
      common /br1/ NA,enpa
      common /general/ emin,vmin,emph
      dimension aux(0:20)
      lim = 20
      vtr=vmin
        op1 = x - .055
        op2 = x + .055
        ope1 = 2.302585093 * op1
        ope2 = 2.302585093 * op2
        op2_0 = 1.0e+1**(op2)
          if (op2_0.le.vtr) then
          filbr1a = 0.
          goto 422
          else
            do j=0,lim
              rnu = 1.e+1**(op1 + ( (.11/float(lim)) * (float(j))))
              aux(j) = brem_tot(enpa,rnu) * rnu
            enddo
          filbr1a = simps(aux,ope1,ope2,lim) 
          filbr1a = (filbr1a / gctbrv(enpa)) * float(NA)
          goto 422
          endif
 422  return
      end
c===========================================================================
c B.18b
c                AN AUXILIARY FUNCTION FOR TESTBREM SUBROUTINE
c
      function filbr2(x)
      external getlbrem,brem_tot,simps
      integer nsub
      real*8 um,ene,u(10),z1(10),w(10),aw(10),a_ef,ro,alfa
      real*8 rm_e,rm_mu,r_e,avog
      common /general/ emin,vmin,emph
      common /media/ z1,w,aw,a_ef,ro,nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      dimension aux(0:2000)
      x1 = 1.e+1**x
      lim=2000
      vmi = alog(emin / x1) !------------> The lower limit for integration
      um = 0.d+0
        do k=1,nsub
        ene=dble(x1)
        u(k)=((z1(k))**(1.d+0/3.d+0))*1.236540953d+0*(rm_mu/(ene*1.d+3))
        u(k) = 1.d+0 - u(k)
        um = dmax1(u(k),um)
        enddo
      vma=sngl(um)  !----------------> The upper limit for integration
      vma=alog(vma) !
      vst = (vma - vmi) / float(lim) !-> step for a rel. en. transfers grid
        do j=0,lim
        rnu = exp(vmi + vst*(float(j)))
        aux(j) = brem_tot(x1,rnu) * rnu       !-> array to be integrated 
c                                             !   for getting free path
        enddo     
c
      filbr2 = simps(aux,vmi,vma,lim)
      filbr2 = sngl(a_ef / (avog * ro)) / filbr2
      dummy = getlbrem(x1)
      filbr2 = 1.e+2 * ( (dummy - filbr2) / filbr2 ) !-> rel.error (percents)
      return
      end
c===========================================================================
c B.18c
c                AN AUXILIARY FUNCTION FOR TESTBREM SUBROUTINE
c
      function filbr3(x)
      external getdedbr,brem_tot,simps
      real*8 um,ene,u(10),z1(10),w(10),aw(10),a_ef,ro,alfa
      real*8 rm_e,rm_mu,r_e,avog
      integer nsub
      common /general/ emin,vmin,emph
      common /media/ z1,w,aw,a_ef,ro,nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      dimension aux(0:2000)
      x1 = 1.e+1**x
      lim=2000
      vmi = alog(emin / x1) !------------> The lower limit for integration       
      um = 0.d+0
        do k=1,nsub
        ene=dble(x1)
        u(k)=((z1(k))**(1.d+0/3.d+0))*1.236540953d+0*(rm_mu/(ene*1.d+3))
        u(k) = 1.d+0 - u(k)
        um = dmax1(u(k),um)
        enddo
      vma=sngl(um)  !----------------> The upper limit for integration
      vma=alog(vma) !
      vst = (vma - vmi) / float(lim) !-> step for a rel. en. transfers grid
       do j=0,lim
       rnu = exp(vmi + vst*(float(j)))
       aux(j) = brem_tot(x1,rnu) * rnu * rnu    !-> array to be integrated 
c                                               !  for getting energy losses
       enddo     
c
      filbr3 = simps(aux,vmi,vma,lim) * sngl(avog*ro/a_ef) * x1 * 1.e+3
      dummy = getdedbr(x1)
      filbr3 = 1.e+2 * ( (dummy - filbr3) / filbr3 ) !--> rel.error (percents)
      return
      end
c===========================================================================
c B.18d
c                AN AUXILIARY FUNCTION FOR TESTBREM SUBROUTINE
c
      function filbr4(x)
      external glbremv,brem_tot,simps
      real*8 um,ene,u(10),z1(10),w(10),aw(10),a_ef,ro,alfa
      real*8 rm_e,rm_mu,r_e,avog
      integer nsub
      common /general/ emin,vmin,emph
      common /media/ z1,w,aw,a_ef,ro,nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      dimension aux(0:2000)
      x1 = 1.e+1**x
      lim=2000
      vmi = alog(vmin)  !------------> The lower limit for integration
      um = 0.d+0
        do k=1,nsub
        ene=dble(x1)
        u(k)=((z1(k))**(1.d+0/3.d+0))*1.236540953d+0*(rm_mu/(ene*1.d+3))
        u(k) = 1.d+0 - u(k)
        um = dmax1(u(k),um)
        enddo
      vma=sngl(um)  !----------------> The upper limit for integration
      vma=alog(vma) !
      vst = (vma - vmi) / float(lim) !-> step for a rel. en. transfers grid
        do j=0,lim
        rnu = exp(vmi + vst*(float(j)))
        aux(j) = brem_tot(x1,rnu) * rnu       !-> array to be integrated 
c                                             !   for getting free path
        enddo     
c
      filbr4 = simps(aux,vmi,vma,lim)
      filbr4 = sngl(a_ef / (avog * ro)) / filbr4
      dummy = glbremv(x1)
      filbr4 = 1.e+2 * ( (dummy - filbr4) / filbr4 ) !-> rel.error (percents)
      return
      end
c===========================================================================
c B.18e
c                AN AUXILIARY FUNCTION FOR TESTBREM SUBROUTINE
c
      function filbr5(x)
      external gdedbrv,brem_tot,simps
      real*8 um,ene,u(10),z1(10),w(10),aw(10),a_ef,ro,alfa
      real*8 rm_e,rm_mu,r_e,avog
      integer nsub
      common /general/ emin,vmin,emph
      common /media/ z1,w,aw,a_ef,ro,nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      dimension aux(0:2000)
      x1 = 1.e+1**x
      lim=2000
      vmi = alog(vmin)   !------------> The lower limit for integration       
      um = 0.d+0
        do k=1,nsub
        ene=dble(x1)
        u(k)=((z1(k))**(1.d+0/3.d+0))*1.236540953d+0*(rm_mu/(ene*1.d+3))
        u(k) = 1.d+0 - u(k)
        um = dmax1(u(k),um)
        enddo
      vma=sngl(um)  !----------------> The upper limit for integration
      vma=alog(vma) !
      vst = (vma - vmi) / float(lim) !-> step for a rel. en. transfers grid
       do j=0,lim
       rnu = exp(vmi + vst*(float(j)))
       aux(j) = brem_tot(x1,rnu) * rnu * rnu    !-> array to be integrated 
c                                               !  for getting energy losses
       enddo     
c
      filbr5 = simps(aux,vmi,vma,lim) * sngl(avog*ro/a_ef) * x1 * 1.e+3
      dummy = gdedbrv(x1)
      filbr5 = 1.e+2 * ( (dummy - filbr5) / filbr5 ) !--> rel.error (percents)
      return
      end
c===========================================================================
c B.18f
c                AN AUXILIARY FUNCTION FOR TESTBREM SUBROUTINE
c
      function filbr6(x)
      external gdedbrt,brem_tot,simps
      real*8 um,ene,u(10),z1(10),w(10),aw(10),a_ef,ro,alfa
      real*8 rm_e,rm_mu,r_e,avog
      integer nsub
      common /general/ emin,vmin,emph
      common /media/ z1,w,aw,a_ef,ro,nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      dimension aux(0:2000)
      x1 = 1.e+1**x
      lim=2000
      vmi = alog(1.e-3/x1)   !------------> The lower limit for integration       
      um = 0.d+0
        do k=1,nsub
        ene=dble(x1)
        u(k)=((z1(k))**(1.d+0/3.d+0))*1.236540953d+0*(rm_mu/(ene*1.d+3))
        u(k) = 1.d+0 - u(k)
        um = dmax1(u(k),um)
        enddo
      vma=sngl(um)  !----------------> The upper limit for integration
      vma=alog(vma) !
      vst = (vma - vmi) / float(lim) !-> step for a rel. en. transfers grid
       do j=0,lim
       rnu = exp(vmi + vst*(float(j)))
       aux(j) = brem_tot(x1,rnu) * rnu * rnu    !-> array to be integrated 
c                                               !  for getting energy losses
       enddo     
c
      filbr6 = simps(aux,vmi,vma,lim) * sngl(avog*ro/a_ef) * x1 * 1.e+3
      dummy = gdedbrt(x1)
      filbr6 = 1.e+2 * ( (dummy - filbr6) / filbr6 ) !--> rel.error (percents)
      return
      end
****************************************************************************
*********************** PAIR PRODUCTION SUBROUTINES : **********************
****************************************************************************
* P.1
*      This routine was kindly provided by R.P.Kokoulin in November,1999
*  It was only a little bit changed (search for string "sokalski" to locate
*     2 places where it was done). Firstly, some correction in according 
*   to Yadernaya Fizika N11 (1999) (Kokoulin et al) was iinserted. Secondly,
*   the units for output value were changed. In original version the differen-
*  tial cross-section for direct muon e+e- pair production was calculated for 
*   element with given electric charge Z, atomic weight A, energy transfer EP
*  (GeV) and muon energy Tkin (Gev) in units cm^2/(g*GeV). In current version 
*   the output value is multiplied by (A/N_a) * E_mu, so it represents diff. 
*   cross-section for atom d_sigma/d_v where v is relative energy transfer.
*
*                                INPUT:
*                                =====
*
*  real*4 Z      : electric charge of nucley
*  real*4 A      : atomic weights of nucley
*  real*4 Tkin   : Muon kinetic energy (GeV)
*  real*4 EP     : Energy transfer due to e+e- pair production (GeV) 
*
*                                OUTPUT:
*                                ======
*
*  real*4 CRP_G4 : d_sigma/d_v (sq. cm) for nucleus with given Z, A, muon 
*                                       energy and energy transfer
*
*                     BELOW IS CODE FROM R.P.KOKOULIN:
*                     ===============================
*
c ************************************************************************
C ***	Cross section for electron pair production by fast muon         **
C ***	By R.P.Kokoulin, December 1997                                  **
C ***	Formulae from Kokoulin & Petrukhin 1971, Hobart, Eqs.(1,8,9,10) **
c ************************************************************************
	function CRP_G4 (Z,A,Tkin,EP)
	parameter	(ame=0.51099907e-3)	!!!	GeV
	parameter	(amu=0.105658389)	!!!	GeV
	parameter	(re=2.81794092e-13)	!!!	cm
	parameter	(avno=6.022137e23)
	parameter	(pi=3.14159265)
	parameter	(alpha=1./137.036)
	parameter	(rmass=amu/ame)		!!!	"207"
	parameter	(coeff=4./(3.*pi)*(alpha*re)**2*avno) !!! cm^2
	parameter	(sqrte=1.64872127)	!!!	sqrt(2.71828...)
	parameter	(c3=3.*sqrte*amu/4.)	!!!	for limits
	parameter	(c7=4.*ame)		!!!	-"-
	parameter	(c8=6.*amu**2)		!!!	-"-
      DIMENSION XGI(8),WGI(8)			!!!	Gauss, N=8
      DATA XGI /.0199,.1017,.2372,.4083,.5917,.7628,.8983,.9801/
      DATA WGI /.0506,.1112,.1569,.1813,.1813,.1569,.1112,.0506/
	data	bbbtf,bbbh /183.,202.4/		!!!	for the moment...
	data	g1tf,g2tf /1.95e-5,5.3e-5/
	data	g1h,g2h   / 4.4e-5,4.8e-5/
        common /exer1/ fa
c
		E=tkin+amu
	z13=z**0.3333333			!!!
	e1=e-ep
	crp_g4=0.
	if (e1.le.c3*z13) return	!!! ep > max
	alf=c7/ep			!!! 4m/ep
	a3=1.-alf
	if (a3.le.0.) return		!!! ep < min
C***		zeta calculation
	if (z.le.1.5) then	!!! special case of hidrogen
		bbb=bbbh
		g1=g1h
		g2=g2h
	else
		bbb=bbbtf
		g1=g1tf
		g2=g2tf
	end if
cccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
c  INSERTED BY SOKALSKI TO TAKE INTO ACCOUNT DIFFERENT BBB for
c  DIFFERENT NUCLEI (S.P.KELNER, R.P.KOKOULIN, A.A.PETRUKHIN,
c           PREPRINT MPhEI 003-98, IN RUSSIAN)
c
       ztmp = z + 1.e-2
       itmp = int(ztmp)
       if (itmp.eq.1) bbb=202.4
       if (itmp.eq.2) bbb=151.9
       if (itmp.eq.3) bbb=159.8
       if (itmp.eq.4) bbb=172.3
       if (itmp.eq.5) bbb=177.8
       if (itmp.eq.6) bbb=178.3
       if (itmp.eq.7) bbb=176.5
       if (itmp.eq.8) bbb=173.3
       if (itmp.eq.9) bbb=169.9
       if (itmp.eq.10) bbb=165.7
       if (itmp.eq.11) bbb=165.7
       if (itmp.eq.12) bbb=167.0
       if (itmp.eq.13) bbb=169.0
       if (itmp.eq.14) bbb=170.7
       if (itmp.eq.15) bbb=172.1
       if (itmp.eq.16) bbb=173.3
       if (itmp.eq.17) bbb=174.2
       if (itmp.eq.18) bbb=174.7
       if (itmp.eq.19) bbb=176.6
       if (itmp.eq.20) bbb=178.5
       if (itmp.eq.21) bbb=179.2
       if (itmp.eq.22) bbb=179.8
       if (itmp.eq.23) bbb=179.9
       if (itmp.eq.26) bbb=179.0
       if (itmp.eq.29) bbb=176.5
       if (itmp.eq.32) bbb=176.8
       if (itmp.eq.35) bbb=177.6
       if (itmp.eq.42) bbb=180.7
       if (itmp.eq.50) bbb=182.7
       if (itmp.eq.53) bbb=183.9
       if (itmp.eq.74) bbb=183.4
       if (itmp.eq.82) bbb=184.1
       if (itmp.eq.92) bbb=186.6
cccccccccccccccccccccccccccccccccccccccccccccccccc
	zeta1=0.073*alog(e/(amu+g1*z13**2*e))-0.26
		if (zeta1.gt.0.) then
	zeta2=0.058*alog(e/(amu+g2*z13   *e))-0.14
	zeta=zeta1/zeta2
		else
	zeta=0.
		end if
	z2=z*(z+zeta)				!!!
	screen0=2.*ame*sqrte*bbb/(z13*ep) !!! be careful with "ame"
	a0=e*e1
	a1=ep*ep/a0			!!!	2*beta
	bet=0.5*a1			!!!	beta
	xi0=0.25*rmass*rmass*a1		!!!	xi0
	del=c8/a0			!!!	6mu^2/EE'
	tmn=alog((alf+2.*del*a3)/(1.+(1.-del)*sqrt(a3))) !!! log(1-rmax)
	sum=0.
	do i=1,8		!!!	integration
	a4=exp(tmn*xgi(i))	!!!	1-r
	a5=a4*(2.-a4)		!!!	1-r2
	a6=1.-a5		!!!	r2
	a7=1.+a6		!!!	1+r2
	a9=3.+a6		!!!	3+r2
	xi=xi0*a5
	xii=1./xi
	xi1=1.+xi
	screen=screen0*xi1/a5
	yeu=5.-a6+4.*bet*a7
	yed=2.*(1.+3.*bet)*alog(3.+xii)-a6-a1*(2.-a6)
	ye1=1.+yeu/yed
	ale=alog(bbb/z13*sqrt(xi1*ye1)/(1.+screen*ye1))
	cre=0.5*alog(1.+(1.5/rmass*z13)**2*xi1*ye1)
		if (xi.le.1e3) then	!!!
	be=((2.+a6)*(1.+bet)+xi*a9)*alog(1.+xii)+(a5-bet)/xi1-a9
		else
	be=(3.-a6+a1*a7)/(2.*xi) !!!-(6.-5.*a6+3.*bet*a6)/(6.*xi*xi)
		end if
	fe=amax1(0.,(ale-cre)*be)
	ymu=4.+a6+3.*bet*a7
	ymd=a7*(1.5+a1)*alog(3.+xi)+1.-1.5*a6
	ym1=1.+ymu/ymd
	alm_crm=alog(bbb*rmass/(1.5*z13*z13*(1.+screen*ym1)))
		if (xi.ge.1e-3) then	!!!
	a10=(1.+a1)*a5		!!!	(1+2b)(1-r2)
	bm=(a7*(1.+1.5*bet)-a10*xii)*alog(xi1)+xi*(a5-bet)/xi1+a10
		else
	bm=(5.-a6+bet*a9)*(xi/2.) !!!-(11.-5.*a6+.5*bet*(5.+a6))*(xi*xi/6.)
		end if
	fm=amax1(0.,(alm_crm)*bm)
	sum=sum+a4*(fe+fm/rmass**2)*wgi(i)
	end do
c	crp_g4=-tmn*sum*(z2/a)*coeff*e1/(e*ep) ! Zakommentirovano
c                                              ! Sokalskim 7.12.99 
	crp_g4=fa*(-tmn*sum*z2*coeff*e1/ep)/avno  ! Vstavleno Sokalskim
c                                              ! (dobavlen mnozhitel
c                                              ! (A / N_a) * E ) 
	return
	end
****************************************************************************
* P.2
*
* This routine simply adopts CRP_G4 routine to the form which is uniformed 
*     with corresponding functions of MUM code for other types of muon
*                        interactions (see, e.g., BREM).
*
*    THIS REAL*8 FUNCTION CALCULATES DIFFERENTIAL CROS-SECTION FOR MUON
*   E+E- PAIR PRODUCTION IN ACCORDING TO R.P.KOKOULIN ET AL. (see CRP_G4)
*
*                                INPUT:
*                                =====
*
*  real*8 z     : electric charge of nucley
*  real*8 en    : muon energy (GeV)
*  real*8 rnu   : relative energy transfer = (E_trnsferred / E_mu)
*
*                                OUTPUT:
*                                ======
*
*  real*8 pair   : d_sigma/d_v (sq. cm) for nucleus with given Z, muon energy 
*                                       and relative energy transfer
*
*   PUT YOUR ATTENTION THAT ATOMIC WEIGHT IS A DUMMY ARGUMENT SINCE CRP_G4
*                WAS SLIGHTLY CHANGED (SEE COMMENTS THERE)
*  .........................................................................
       real*8 function pair(z,en,rnu)
       external CRP_G4
       real*8 z,en,rnu
       z0 = sngl(z)
       A = 10.
       Tkin = sngl(en - 1.05658389d-1) 
       ep = sngl(rnu * en)
       pa = CRP_G4(z0,A,Tkin,ep)
       pair = dble(pa)
       return
       end
****************************************************************************
* P.3
*       THIS FUNCTION CALCULATES DIFFERENTIAL CROS-SECTION FOR MUON
*        PAIR PRODUCTION IN GIVEN MEDIA USING REAL*8 FUNCTION PAIR  
*
*                                  INPUT:
*                                  =====
*
*  real*4 ene    : muon energy (GeV)
*  real*4 v      : relative energy transfer
*
*                                  OUTPUT:
*                                  ======
*
*  real*4 pair_tot   : d_sigma/d_v (sq. cm), for given media, muon energy and 
*                      relative energy transfer. Values of diff. cross-section 
*                      are averaged over all nuclei which given media consists 
*                      of:
*
*                      d_s/d_v = SUM (w_i * d_s/d_v_i), where w_i = N_i/N_tot
*                      N_i - number of type i atoms in molecula
*                      N_tot - total number of atoms in molecula
*
*                      So, it is calculated for an "effective nucleus" with
*                      atomic weight A_eff = SUM(N_i*A_i) / N_tot
*  .........................................................................
       function pair_tot(ene,v)
       external pair
       real*8 z1(10),w(10),aw(10),a_ef,en,rnu,h1,pair,ro
       integer nsub
       common /media/ z1,w,aw,a_ef,ro,nsub ! this common determines media com-
c                                     ! position: AW (atomic weights), A_EF
c                                   ! (effective atomic weight), Z1 (charges), 
c                                 ! W(relative weights), NSUB (number of media 
c                                ! components), ro (cm/qubic cm) - media densi-
c                                ! ty.  All these  are prepared by subroutine
c                               ! MED_CONS and passed here via commom /media/.
c
       if (v.gt..999995) v = .999995
       en = dble(ene)
       rnu = dble(v)
       h1 = w(1) * pair(z1(1),en,rnu)
         if (nsub.ge.2) then
           do l=2,nsub
             h1 = h1 + (w(l) * pair(z1(l),en,rnu))
           enddo
         endif
       pair_tot = sngl(h1)
       return
       end
****************************************************************************
* P.4
       subroutine pair1
*
*                      The subroutine calculates:
*
*                                   [1]
*                           
* The differential cross-sections d_Sigma/d_v for e+e- production by muon which
*  passes a medi using FUNCTION PAIR_TOT. The result is put in three arrays
*       fcd1(81,54), fcd2(81,101) and fcd3(81,51) which are passed to 
* subroutines SPL2 via common block /cdpa_in/. The reason to use just 3 arrays
*     is that the region of energy transfers is divided into 3 regions: 
*      10^(-11) - 10^(-0.6), 10^(-0.6) - 10^(-0.1) and 10^(-0.1) - 1 
*     for which different grid steps are accepted (0.2, 0.01 and 0.001, 
*           correspondingly) to obtain accuracy better than 0.001.
*    Dif. cross-sections are computed for 81 values of muon energy starting 
*  with E_mu = 10 GeV and finishing with E_mu = 1000 Pev with logarithmically
*                          equidistant griid.     
*  SPL2 cooks spline coefficients out of these 3 arrays which are used in
*                          further computing.
*
*                                   [2]
*
*    Energy losses and mean free path for e+e- production by muon which
*    passes a media with energy transfers which exceed EMIN, VMIN and 0
*   and with en. transfers below EMIN - arrays elo_pa1(17), crt_pa1(17); 
*           elo_pa2(17), crt_pa2(17); elo_pa3(17); elo_pa4(17),
*      correspondingly, which are computed for 17 values of muon energy 
*    starting  with E_mu = 10 GeV and finishing with E_mu = 1000 Pev with 
*  logarithmically equidistant grid. These arrays are passed to subroutines
*   SPL1 and ENLOS via common blocks /elpa_in1/, /elpa_in2/, /elpa_in4/ and 
*  /ctpa_in1/ where they are used to prepare spline coefficients for further 
*  computing and calculate non-stochastical energy losses below EMIN and VMIN.
*
*                                   [3]
*
* A comparison function COM_PA_M which is used in subroutine GETVPA to simulate
*   energy transfers due to e+e- pair production. COM_PA_M is computed as a 
*   differential cross-section for e+e- pair production by muon with energy 
*  1000 PeV. The values of diff. cross-section are multiplied by factor 1.03
* to make sure that comparison function is exced the diff. cross-sectiions for 
* any muon energy in a range of 10GeV - 1000 PeV and relative energy transfers 
*                     in a range of (emin/1000 PeV) -- 1): 
*          COMPARISON FUNCTION = 1.03 * [d_sigma / d_v (1000 PeV)]
*   Values of computed comparison function are put into array COM_PA_M(2201) 
*   which is determined on a logarithmically equidistant grid with values of 
*            relative energy transfers in a range of 10^(-11) -- 1.
*     This array is passed to subroutine SPL1 to cook spline coefficients
* which are used in further comuting to get value of comparison function at any
*             value of relative energy transfer by interpolation.
*
*                                   [4]
*
*     Array COM_PA_IN with values of integrated comparison function:
*          COM_PA_IN (v1) = INT( v = 10^(-11) -- v1 ) (COM_PA_M)
*  which is used in further computing to simulate energy transfers due to
* e+e- pair production. COM_PA_IN is determined on a logarithmically equidistant
*      grid of relative energy transfers in a range of 10^(-11) -- 1.
*  This REAL*8 array is passed to subroutine DSPL1 to cook spline coefficients
*   which are used in further comuting to get value of integrated comparison 
*    function at any value of relative energy transfer by interpolation.
*  Also a value of TOT_PA = COM_PA_IN (v1=1.0) is computed and passed to
*         subroutine GETVPA which simulate energy transfers.
*  .....................................................................
       external pair_tot,dsimps
       real*8 ai,bi,h1,h2,um,ene,u(10),dsimps
       real*8 aux1(0:2000),aux2(0:2000)
       real*8 alfa,rm_e,rm_mu,r_e,avog,z1(10),w(10),aw(10),a_ef,ro
       real*8 com1_pa(0:2200),com_pa_in(1101),com_p_h(0:2),tot_pa
       integer nsub
       dimension com_pa_m(2201)
       common /help_1/ aux1,aux2
       common /const/ alfa,rm_e,rm_mu,r_e,avog ! from initial subroutine
       common /media/ z1,w,aw,a_ef,ro,nsub     !       MED_CONS
       common /general/ emin,vmin,emph         !       
       common /cdpa_in/ fcd1(81,54),fcd2(81,101),fcd3(81,51) ! to SPL2
       common /ctpa_in1/ crt_pa1(17),crt_pa2(17) ! To SPL1 and ENLOS
       common /elpa_in1/ elo_pa1(17),elo_pa2(17) ! To SPL1 and ENLOS
       common /elpa_in2/ elo_pa3(17)   !-> to SPL1 and ENLOS
       common /elpa_in4/ elo_pa4(17)   !-> to ENLOS
       common /sok34/ com_pa_m       ! 
       common /sok24/ com_pa_in  ! To DSPL1 
       common /sok33/ tot_pa  ! To GETVPA
       common /fac_pa/ fac ! To COMP
       fac = 1.055
c ....................................................................
c      1.PREPARATION OF ARRAYS WITH DIFF. CROSS-SEECTIONS FOR MUON
c                           E+E- PAIR PRODUCTION:
c
c                !       .......................... 
      do j=1,81  !--->   j CYCLE BY ENERGIES starts
c                !       .......................... 
        en = float(j)
        en = 1.e+1**(9.e-1 + (1.e-1 * en))
c                     !         .................................. 
      do k=1,206      !--->     k CYCLE BY ENERGY TRANSFERS starts
c                     !         ..................................
      if (k.eq.206) then
        fcd2(j,k-105) = -37.0
        goto 975
      endif
c
      if (k.le.54) then
         k01 = k - 1
         rnu = 1.e+1**(float(k01) * 2.e-1 - 1.1e+1)
      else
         if (k.le.105) then
           k01 = k - 55
           rnu = 1.e+1**(float(k01) * 1.e-2 - 6.e-1)
         else
           k01 = k - 106
           rnu = 1.e+1**(float(k01) * 1.e-3 - 1.e-1)
         endif
      endif
c
      cr_dif=pair_tot(en,rnu)
      if (cr_dif.le.1.e-37) cr_dif = 1.e-37
      cr_dif=alog(cr_dif)
c
      if (k.le.54) fcd1(j,k) = cr_dif
      if ((k.gt.54).AND.(k.lt.106)) fcd3(j,k-54) = cr_dif
      if (k.ge.106) fcd2(j,k-105) = cr_dif
c
 975  continue
c              !     .................................... 
      enddo    !---> k CYCLE BY ENERGY TRANSFERS finishes 
c              !     .................................... 
      enddo    !----> j CYCLE BY ENERGIES finishes 
c              !     .................................... 
c   ....................................................................
c      2.PREPARATION OF ARRAYS WITH ENERGY LOSSES AND MEAN FREE PATH
c                         FOR MUON BREMSSTRAHLUNG:
c
c                    a) Energy transfers > EMIN :
      lim = 2000
      do i=1,17
      en = float(i)
      en = (1.e+1)**(5.e-1 + (5.e-1 * en))
      vmi = alog(emin / en) !------------> The lower limit for integration
        um = 0.d+0
        do k=1,nsub
        ene=dble(en)
        u(k)=((z1(k))**(1.d+0/3.d+0))*1.236540953d+0*(rm_mu/(ene*1.d+3))
        u(k) = 1.d+0 - u(k)
        um = dmax1(u(k),um)
        enddo
        vma=sngl(um)  !----------------> The upper limit for integration
        vma=alog(vma) !
        vst = (vma - vmi) / float(lim) !-> step for a rel. en. transfers grid
      do j=0,lim
      rnu = vmi + vst*(float(j))
      rnu = exp(rnu)
      aux1(j) = dble(pair_tot(en,rnu) * rnu)       !-> array to be integrated 
c                                                  !   to get mean free path
      aux2(j) = aux1(j) * dble(rnu)                !-> array to be integrated 
      enddo                                        !   for getting en. losses
c
      h1 = dble(vmi)
      h2 = dble(vma)
      crt_pa1(i) = dsimps(aux1,h1,h2,lim)               !-> array with 17 values
      crt_pa1(i) = sngl(a_ef / (avog * ro))/crt_pa1(i)  !   of free path
      crt_pa1(i) = alog(crt_pa1(i))
      elo_pa1(i) = sngl( dsimps(aux2,h1,h2,lim) * avog * ro / a_ef )
      elo_pa1(i) = elo_pa1(i) * en * 1.e+3 
      elo_pa1(i) = alog(elo_pa1(i)) !-> array with 17 values of en. losses
      enddo
c                    b) Energy transfers > VMIN :
      do i=1,17
      en = float(i)
      en = (1.e+1)**(5.e-1 + (5.e-1 * en))
      vmi = alog(vmin) !------------> The lower limit for integration
        um = 0.d+0
        do k=1,nsub
        ene=dble(en)
        u(k)=((z1(k))**(1.d+0/3.d+0))*1.236540953d+0*(rm_mu/(ene*1.d+3))
        u(k) = 1.d+0 - u(k)
        um = dmax1(u(k),um)
        enddo
        vma=sngl(um)  !----------------> The upper limit for integration
        vma=alog(vma) !
        vst = (vma - vmi) / float(lim) !-> step for a rel. en. transfers grid
c
      do j=0,lim
      rnu = vmi + vst*(float(j))
      rnu = exp(rnu)
      aux1(j) = dble(pair_tot(en,rnu) * rnu)       !-> array to be integrated 
c                                                  !   to get mean free path
      aux2(j) = aux1(j) * dble(rnu)                !-> array to be integrated 
      enddo                                        !   for getting en. losses
c
      h1 = dble(vmi)
      h2 = dble(vma)
      crt_pa2(i) = dsimps(aux1,h1,h2,lim)               !-> array with 17 values
      crt_pa2(i) = sngl(a_ef / (avog * ro))/crt_pa2(i)  !   of free path
      crt_pa2(i) = alog(crt_pa2(i))
      elo_pa2(i) = sngl( dsimps(aux2,h1,h2,lim) * avog * ro / a_ef )
      elo_pa2(i) = elo_pa2(i) * en * 1.e+3 
      elo_pa2(i) = alog(elo_pa2(i)) !-> array with 17 values of en. losses
      enddo
c                    c) Energy transfers > 0 :
      do i=1,17
      en = float(i)
      en = (1.e+1)**(5.e-1 + (5.e-1 * en))
      vmi = alog(2.046e-3/en) !------------> The lower limit for integration
c
        um = 0.d+0
        do k=1,nsub
        ene=dble(en)
        u(k)=((z1(k))**(1.d+0/3.d+0))*1.236540953d+0*(rm_mu/(ene*1.d+3))
        u(k) = 1.d+0 - u(k)
        um = dmax1(u(k),um)
        enddo
        vma=sngl(um)  !----------------> The upper limit for integration
        vma=alog(vma) !
        vst = (vma - vmi) / float(lim) !-> step for a rel. en. transfers grid
      do j=0,lim
      rnu = vmi + vst*(float(j))
      rnu = exp(rnu)
      aux2(j) = dble(pair_tot(en,rnu) * rnu * rnu) !-> array to be integrated 
      enddo                                        !   for getting en. losses
c
      h1 = dble(vmi)
      h2 = dble(vma)
      elo_pa3(i) = sngl( dsimps(aux2,h1,h2,lim) * avog * ro / a_ef )
      elo_pa3(i) = elo_pa3(i) * en * 1.e+3 
      elo_pa3(i) = alog(elo_pa3(i)) !-> array with 17 values of en. losses
      enddo
c                    d) Energy transfers < EMIN :
      do i=1,17
      en = float(i)
      en = (1.e+1)**(5.e-1 + (5.e-1 * en))
      vmi = alog(2.046e-3/en) !------------> The lower limit for integration
      vma = alog(emin/en)  !----------------> The upper limit for integration
      vst = (vma - vmi) / float(lim) !---> step for a rel. en. transfers grid
      do j=0,lim
      rnu = vmi + vst*(float(j))
      rnu = exp(rnu)
      aux2(j) = dble(pair_tot(en,rnu) * rnu * rnu) !-> array to be integrated 
      enddo                                        !   for getting en. losses
c
      h1 = dble(vmi)
      h2 = dble(vma)
      elo_pa4(i) = sngl( dsimps(aux2,h1,h2,lim) * avog * ro / a_ef )
      elo_pa4(i) = elo_pa4(i) * en * 1.e+3 !-> array with 17 values  
      enddo                                !   of en. losses
c   ....................................................................
c   3. PREPARATION OF ARRAY COM_PA_M WITH VALUES OF COMPARISON FUNCTION:
         en = 1.e+9
           do i=1,2201
              i1 = i - 1
              rnu = 1.e+1**(float(i1) * 5.e-3 - 1.1e+1)
              com_pa_m(i) = fac * pair_tot(en,rnu)
              com1_pa(i-1) = dble(com_pa_m(i) * rnu)
           enddo
           do i=1,2201
              if (com_pa_m(i).le.0.d+0) com_pa_m(i) = 1.0001d-37 
              com_pa_m(i)=alog(com_pa_m(i))
           enddo
c   ....................................................................
c       4. PREPARATION OF ARRAY COM_PA_IN WITH VALUES OF INTEGRATED 
c                 COMPARISON FUNCTION AND VALUE OF TOT_PA:
        com_pa_in(1) = 0.d+0 
        h1 = 0.d+0
        m = 2
          do i=2,1101
             i1 = 2 * i
               com_p_h(0) = com1_pa(i1-4)
               com_p_h(1) = com1_pa(i1-3)
               com_p_h(2) = com1_pa(i1-2)
             ai = -((dble(1102 - i)) * 1.d-2)
             bi = ai + 1.d-2
             ai = 2.302585093 * ai
             bi = 2.302585093 * bi
             h2 = dsimps(com_p_h,ai,bi,m)
             h1 = h1 + h2
             com_pa_in(i) = h1
          enddo
        tot_pa = com_pa_in(1101)
       return
       end
****************************************************************************
* P.5
      FUNCTION getlpair(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*  Function GETLPAIR calculates the muon's free path for e+e- pair production
*   with energy treansfers > EMIN using spline coefficients prepared by spl1 
*                                subroutine.
*          X is a value for muon energy expressed in GeV; XMIN,
*   XMAX and STEP are assigned with some values in subroutine SPL1 (they
*              are passed here via /sok1/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                              common /sok1_p/
* ..........................................................................
*        X MUST BE WITHIN THE RANGE OF 10 -- 1000,000,000 GeV
* ..........................................................................
      COMMON /sok1/ XMIN,STEP,XMAX
      common /sok1_p/ C(19)
      X1 = alog10(X)
       if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
       write(0,*)'ERROR IN FUNCTION GETLPAIR: MUON ENERGY IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      getlpair = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      getlpair = exp(getlpair)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
****************************************************************************
* P.6
      FUNCTION glpairv(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*  Function GLPAIRV calculates the muon's free path for e+e- pair production
*   with energy treansfers > VMIN using spline coefficients prepared by spl1 
*                                subroutine.
*          X is a value for muon energy expressed in GeV; XMIN,
*   XMAX and STEP are assigned with some values in subroutine SPL1 (they
*              are passed here via /sok1/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                              common /sok1_p2/
* ..........................................................................
*        X MUST BE WITHIN THE RANGE OF 10 -- 1000,000,000 GeV
* ..........................................................................
      COMMON /sok1/ XMIN,STEP,XMAX
      common /sok1_p2/ C(19)
      X1 = alog10(X)
       if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
       write(0,*)'ERROR IN FUNCTION GLPAIRV: MUON ENERGY IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      glpairv = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      glpairv = exp(glpairv)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
****************************************************************************
* P.7
       function getctpa(u)
*
*      The subroutine calculates the value for total cross-section for e+e-
*    pair production with energy transfers above EMIN using getlpair routine.
*     It also uses some media parameters prepared by subroutine MED_CONS
*                       (commons /const/ and /media/)
*   The resulting cross-section is averaged over all atoms of given media:
*
*                      sigma(E_mu) = SUM (w_i * sigma_i(E_mu)), 
*
*      where w_i = N_i/N_tot, N_i - number of type i atoms in molecula, 
* N_tot - total number of atoms in molecula, sigma_i(E_mu) is a cross-section
*                               for atom number i.
*     So, it is calculated for an "effective nucleus" with atomic weight 
*                         A_eff = SUM(N_i*A_i) / N_tot
*
*           U is a value for muon energy expressed in GeV
*  ......................................................................
*  ***   ATTENTION: U must be within a range of 10 GeV -- 1000 PeV   ***
*  ......................................................................
       external getlpair
       real*8 z1(10),w(10),aw(10),a_ef,alfa,rm_e,rm_mu,r_e,avog,ro
       integer nsub
       common /const/ alfa,rm_e,rm_mu,r_e,avog
       common /media/ z1,w,aw,a_ef,ro,nsub
      if ((u.gt.1.0001e+9).or.(u.lt.9.9999e+0)) then   
      write(0,*)'ERROR IN FUNCTION GETCTPA: MUON ENERGY IS OUT OF RANGE'
      endif
      aef = sngl(a_ef / (avog * ro))
      getctpa = aef / getlpair(u)
      return
      end
*************************************************************************
* P.8
       function gctpav(u)
*
*      The subroutine calculates the value for total cross-section for e+e-
*    pair production with energy transfers above VMIN using glpairv routine.
*     It also uses some media parameters prepared by subroutine MED_CONS
*                       (commons /const/ and /media/)
*   The resulting cross-section is averaged over all atoms of given media:
*
*                      sigma(E_mu) = SUM (w_i * sigma_i(E_mu)), 
*
*      where w_i = N_i/N_tot, N_i - number of type i atoms in molecula, 
* N_tot - total number of atoms in molecula, sigma_i(E_mu) is a cross-section
*                               for atom number i.
*     So, it is calculated for an "effective nucleus" with atomic weight 
*                         A_eff = SUM(N_i*A_i) / N_tot
*
*           U is a value for muon energy expressed in GeV
*  ......................................................................
*  ***   ATTENTION: U must be within a range of 10 GeV -- 1000 PeV   ***
*  ......................................................................
       external glpairv
       real*8 z1(10),w(10),aw(10),a_ef,alfa,rm_e,rm_mu,r_e,avog,ro
       integer nsub
       common /const/ alfa,rm_e,rm_mu,r_e,avog
       common /media/ z1,w,aw,a_ef,ro,nsub
      if ((u.gt.1.0001e+9).or.(u.lt.9.9999e+0)) then   
      write(0,*)'ERROR IN FUNCTION GCTPAV: MUON ENERGY IS OUT OF RANGE'
      endif
      aef = sngl(a_ef / (avog * ro))
      gctpav = aef / glpairv(u)
      return
      end
************************************************************************
* P.9
       FUNCTION getdedpa(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*       The subroutine calculates the value for muon energy losses for 
*     e+e- pair production with energy transfers above EMIN using spline 
*                coefficients prepared by spl1 subroutine.
*
*          X is a value for muon energy expressed in GeV; XMIN,
*  XMAX and STEP are assigned with some values in subroutine SPL1 (they
*               are passed here via /sok5/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                              common /sok5_p/
*  ......................................................................
*  ***   ATTENTION: X must be within a range of 10 GeV --- 1000 PeV   ***
*  ......................................................................
      COMMON /sok5/ XMIN,STEP,XMAX
      common /sok5_p/ C(19)
      X1 = alog10(X)
       if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
       write(0,*)'ERROR IN FUNCTION GETDEDPA: MUON ENERGY IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      getdedpa = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      getdedpa = exp(getdedpa)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
****************************************************************************
* P.10
      FUNCTION gdedpav(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*       The subroutine calculates the value for muon energy losses for 
*     e+e- pair production with energy transfers above VMIN using spline 
*                coefficients prepared by spl1 subroutine.
*
*          X is a value for muon energy expressed in GeV; XMIN,
*  XMAX and STEP are assigned with some values in subroutine SPL1 (they
*               are passed here via /sok5/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                              common /sok5_p2/
*  ......................................................................
*  ***   ATTENTION: X must be within a range of 10 GeV --- 1000 PeV   ***
*  ......................................................................
      COMMON /sok5/ XMIN,STEP,XMAX
      common /sok5_p2/ C(19)
      X1 = alog10(X)
       if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
       write(0,*)'ERROR IN FUNCTION GDEDPAV: MUON ENERGY IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      gdedpav = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      gdedpav = exp(gdedpav)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
****************************************************************************
* P.11
       FUNCTION gdedpat(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*  The subroutine calculates the value for total muon energy losses due to
*     e+e- pair production using spline coefficients prepared by spl1 
*                                   subroutine.
*
*          X is a value for muon energy expressed in GeV; XMIN,
*  XMAX and STEP are assigned with some values in subroutine SPL1 (they
*               are passed here via /sok5/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                              common /sok5_p3/
*  ......................................................................
*  ***   ATTENTION: X must be within a range of 10 GeV --- 1000 PeV   ***
*  ......................................................................
      COMMON /sok5/ XMIN,STEP,XMAX
      common /sok5_p3/ C(19)
      X1 = alog10(X)
       if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
       write(0,*)'ERROR IN FUNCTION GDEDPAT: MUON ENERGY IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      gdedpat = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      gdedpat = exp(gdedpat)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
****************************************************************************
* P.12
        FUNCTION getcdp(X,Y,lo)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
* The subroutine calculates the value for differential cross-sectionm for muon
* muon pair production d_Sigma/d_v using functions getcdp1, getcdp2, getcdp3 or 
*  pair_tot depending on value of energy transfer Y. For definition of cross-
*     section for muon e+e- pair production in given media see comments to 
*                            subroutine PAIR_TOT.
* X and Y are values for muon energy expressed in GeV and for relative energy 
*   transfer, respectively. If LO=0, then Y is expressed in natural units, 
*           otherwise the input for Y is presented as LOG10(Y).
*  .....................................................................
*        ATTENTION: X must be within a range of 10 GeV -- 1000 PeV
*                   Y must be within a range of 10**(-11) -- 10**(0)
*  ......................................................................
       external getcdp1,getcdp2,getcdp3,pair_tot
       if (lo.eq.0) then
          if (Y.le..251188643) then
                getcdp = getcdp1(X,Y,lo)
               getcdp = exp(getcdp)
          else
               if (Y.le..794328234) then
                  getcdp = getcdp2(X,Y,lo)
                  getcdp = exp(getcdp)
               else
                 if (Y.le..965) then
                    getcdp = getcdp3(X,Y,lo)
                    getcdp = exp(getcdp)
                 else
                    getcdp = pair_tot(X,Y)
                 endif
               endif
          endif
       else
          if (Y.le.-6.e-1) then
                getcdp = getcdp1(X,Y,lo)
                getcdp = exp(getcdp)
          else
                if (Y.le.-1.e-1) then
                   getcdp = getcdp2(X,Y,lo)
                   getcdp = exp(getcdp)
                else
                   if (Y.le.-1.547272686e-2) then 
                     getcdp = getcdp3(X,Y,lo)
                     getcdp = exp(getcdp)
                   else
                     Y1 = (1.e+1)**Y
                     getcdp = pair_tot(X,Y1)
                   endif
                endif
          endif
       endif
      if (getcdp.lt.0.e+0) getcdp = 0.e+0
      return
      end
****************************************************************************
* P.13
        FUNCTION getcdp1(X,Y,loga)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*   The subroutine calculates the value for differential cross-section for muon
* pair production d_Sigma/d_v using spline coefficients prepared by SPL2 routine.
*  For definition of cross-section for e+e- pair production in given media see 
*                    comments to subroutine PAIR_TOT.
*  .....................................................................
*   X and Y in body program is a value for muon energy expressed in GeV and
*    for relative energy transfer, respectively. NX, NY, X0, SX, Y0 and SY
*   are assigned with some values in subroutine SPL2 (they are passed here
*   via /sok_2_1/ common block. Array with spline coefficients C1(4648) is 
*            cooked by SPL2 and passed here via common /sok8/
*            If LOGA=0, then Y is expressed in natural units, 
*           otherwise the input for Y is presented as LOG10(Y).
*                  Output is presented as LN (d_sigma/d_v)
*  ......................................................................
*        ATTENTION: X must be within a range of 10 GeV -- 1000 PeV
*                   Y must be within a range of 10**(-11) -- 10**(-0.6)
*  ......................................................................
      common /sok8/ C1(4648)
      common /sok_2_1/ NX,NY,X0,SX,Y0,SY
      X_1=alog10(X)
      if ((X_1.lt..99999e+0).or.(X_1.gt.9.0001e+0)) then
      write(0,*)'ERROR IN FUNCTION GETCDP1: MUON ENERGY IS OUT OF RANGE'
      endif
c
      if (loga.eq.0) then    
      Y_1=alog10(Y)
      else
      Y_1=Y
      endif
      if ((Y_1.lt.-11.001).or.(Y_1.gt.-5.9999e-1)) then
      write(0,*)'ERROR IN FUNCTION GETCDP1: EN. TRANSFER IS OUT OF RANGE'
      endif
c
      A3=(X_1-X0)/SX
      B3=(Y_1-Y0)/SY
      M1=INT(A3+.5)
      M2=INT(B3+.5)
      M3=NX+2
ccc      IF(M1.LT.0.OR.M1.GE.NX.OR.M2.LT.0.OR.M2.GE.NY) THEN
ccc      PRINT 1,X_1,M1,Y_1,M2
ccc      ENDIF
      A3=A3-M1
      B3=B3-M2
      M1=M2*M3+M1+1
      M2=M1+M3
      M3=M2+M3
      A2=A3*A3+.25
      B2=B3*B3+.25
      A1=A2-A3
      B1=B2-B3
      A3=A2+A3
      B3=B2+B3
      A2=2.-(A2+A2)
      B2=2.-(B2+B2)
      getcdp1=(A1*C1(M1)+A2*C1(M1+1)+A3*C1(M1+2))*B1
     2      +(A1*C1(M2)+A2*C1(M2+1)+A3*C1(M2+2))*B2
     3      +(A1*C1(M3)+A2*C1(M3+1)+A3*C1(M3+2))*B3
      RETURN
ccc1     FORMAT('GETCDP1: *MISTAKE*X= ',D23.16,' MX= ',I4,' Y= ',D23.16,' M
ccc     *Y= ',I4)
      END
****************************************************************************
* P.14
       FUNCTION getcdp2(X,Y,loga)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*   The subroutine calculates the value for differential cross-section for muon
* pair production d_Sigma/d_v using spline coefficients prepared by SPL2 routine.
*  For definition of cross-section for e+e- pair production in given media see 
*                    comments to subroutine PAIR_TOT.
*  .....................................................................
*   X and Y in body program is a value for muon energy expressed in GeV and
*    for relative energy transfer, respectively. NX, NY, X0, SX, Y0 and SY
*   are assigned with some values in subroutine SPL2 (they are passed here
*   via /sok_2_2/ common block. Array with spline coefficients C1(4399) is 
*            cooked by SPL2 and passed here via common /sok9/
*            If LOGA=0, then Y is expressed in natural units, 
*           otherwise the input for Y is presented as LOG10(Y).
*                  Output is presented as LN (d_sigma/d_v)
*  ......................................................................
*        ATTENTION: X must be within a range of 10 GeV -- 1000 PeV
*                   Y must be within a range of 10**(-0.6) -- 10**(-0.1)
*  ......................................................................
       common /sok9/ C1(4399)
       common /sok_2_2/ NX,NY,X0,SX,Y0,SY
      X_1=alog10(X)
      if ((X_1.lt..99999e+0).or.(X_1.gt.9.0001e+0)) then
      write(0,*)'ERROR IN FUNCTION GETCDP2: MUON ENERGY IS OUT OF RANGE'
      endif
      if (loga.eq.0) then    
      Y_1=alog10(Y)
      else
      Y_1=Y
      endif
      if ((Y_1.lt.-.60001).or.(Y_1.gt.-.99999e-1)) then
      write(0,*)'ERROR IN FUNCTION GETCDP2: EN. TRANSFer IS OUT OF RANGE'
      endif
      A3=(X_1-X0)/SX
      B3=(Y_1-Y0)/SY
      M1=INT(A3+.5)
      M2=INT(B3+.5)
      M3=NX+2
ccc      IF(M1.LT.0.OR.M1.GE.NX.OR.M2.LT.0.OR.M2.GE.NY) THEN
ccc      PRINT 1,X_1,M1,Y_1,M2
ccc      ENDIF
      A3=A3-M1
      B3=B3-M2
      M1=M2*M3+M1+1
      M2=M1+M3
      M3=M2+M3
      A2=A3*A3+.25
      B2=B3*B3+.25
      A1=A2-A3
      B1=B2-B3
      A3=A2+A3
      B3=B2+B3
      A2=2.-(A2+A2)
      B2=2.-(B2+B2)
      getcdp2=(A1*C1(M1)+A2*C1(M1+1)+A3*C1(M1+2))*B1
     2      +(A1*C1(M2)+A2*C1(M2+1)+A3*C1(M2+2))*B2
     3      +(A1*C1(M3)+A2*C1(M3+1)+A3*C1(M3+2))*B3
      RETURN
ccc1     FORMAT('GCDP2:*MISTAKE*X=',D23.16,'MX=',I4,'Y=',D23.16,'MY=',I4)
      END
****************************************************************************
* P.15
       FUNCTION getcdp3(X,Y,loga)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*   The subroutine calculates the value for differential cross-section for muon
* pair production d_Sigma/d_v using spline coefficients prepared by SPL2 routine.
*  For definition of cross-section for e+e- pair production in given media see 
*                    comments to subroutine PAIR_TOT.
*  .....................................................................
*   X and Y in body program is a value for muon energy expressed in GeV and
*    for relative energy transfer, respectively. NX, NY, X0, SX, Y0 and SY
*   are assigned with some values in subroutine SPL2 (they are passed here
*   via /sok_2_3/ common block. Array with spline coefficients C2(8549) is 
*            cooked by SPL2 and passed here via common /sok10/
*            If LOGA=0, then Y is expressed in natural units, 
*           otherwise the input for Y is presented as LOG10(Y).
*                  Output is presented as LN (d_sigma/d_v)
*  ......................................................................
*        ATTENTION: X must be within a range of 10 GeV -- 1000 PeV
*                   Y must be within a range of 10**(-0.1) -- 1
*  ......................................................................
       common /sok10/ C2(8549)
       common /sok_2_3/ NX,NY,X0,SX,Y0,SY
      X_1=alog10(X)
      if ((X_1.lt..99999e+0).or.(X_1.gt.9.0001e+0)) then
      write(0,*)'ERROR IN FUNCTION GETCDP3: MUON ENERGY IS OUT OF RANGE'
      endif
      if (loga.eq.0) then    
      Y_1=alog10(Y)
      else
      Y_1=Y
      endif
      if ((Y_1.lt.-1.0001e-1).or.(Y_1.gt.1.e-6)) then
      write(0,*)'ERROR IN FUNCTION GETCDP3: EN. TRANSFER IS OUT OF RANGE'
      endif
      A3=(X_1-X0)/SX
      B3=(Y_1-Y0)/SY
      M1=INT(A3+.5)
      M2=INT(B3+.5)
      M3=NX+2
ccc      IF(M1.LT.0.OR.M1.GE.NX.OR.M2.LT.0.OR.M2.GE.NY) THEN
ccc      PRINT 1,X_1,M1,Y_1,M2
ccc      ENDIF
      A3=A3-M1
      B3=B3-M2
      M1=M2*M3+M1+1
      M2=M1+M3
      M3=M2+M3
      A2=A3*A3+.25
      B2=B3*B3+.25
      A1=A2-A3
      B1=B2-B3
      A3=A2+A3
      B3=B2+B3
      A2=2.-(A2+A2)
      B2=2.-(B2+B2)
      getcdp3=(A1*C2(M1)+A2*C2(M1+1)+A3*C2(M1+2))*B1
     2      +(A1*C2(M2)+A2*C2(M2+1)+A3*C2(M2+2))*B2
     3      +(A1*C2(M3)+A2*C2(M3+1)+A3*C2(M3+2))*B3
      RETURN
ccc1     FORMAT('*MISTAKE*X=',D23.16,'MX=',I4,'Y=',D23.16,'MY=',I4)
      END
****************************************************************************
* P.16
      FUNCTION comp(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*       The subroutine calculates the values for comparison function
*  to simulate energy transfers due to e+e- pair production by muon in media.
*  The input values for comparison function are calculated by subroutine PAIR1.
*  Then subroutine SPL1 cooks an array with splines which is passed here to COMP
*    by common /sok55_p/. COMP gives the value of comparison function basing 
*   on these splines for any relative energy transfers which are in a range of 
*                                10^(-11) -- 1.
*           Input: LOG10(V) where V is relative energy transfer
*           Output: Value for comparison function
*  ......................................................................
*  Comparison function is calculated in subroutine PAIR1 as d_sigma/d_v
*        for e+e- pair production for muon energy E_mu = 1000 PeV 
*   Splines are prepared by SPL1 for LN of comp. function values, so here 
*  in COMP one takes EXP from final value obtained by spline interpolations.
*  ......................................................................
*         ***   ATTENTION: X must be within a range of -11 -- 0 ***
*  ......................................................................
      external pair_tot
      common /sok55/ XMIN,STEP,XMAX
      common /sok55_p/ C(2203)
      common /fac_pa/ fac
      parameter (en = 1.e+9) 
      X1 = X
      if (X1.lt.-1.5472686e-2) then
       if ((X1.lt.-11.001e+0).or.(X1.gt.1.e-5)) then
       write(0,*)'ERROR IN FUNCTION COMP: ENERGY transfer IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      comp = exp((Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3))
      else
      rnu = 1.e+1**X1
      comp = fac * pair_tot(en,rnu)
      endif
      RETURN
ccc1     FORMAT('*MISTAKE* X1=',D23.16,'  XMIN=',D23.16,'  XMAX=',D23.16)
      END
****************************************************************************
* P.17
      real*8 FUNCTION c_pa_in(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*   The subroutine calculates the value of integrated comparison function which 
*  is necessary for simulation of energy transfers due to e+e- pairs production.
*           using spline coefficients prepared by dspl1 subroutine.
*
*      X is a value for relative energy transfer; XMIN, XMAX and STEP are 
*   assigned with some values in subroutine DSPL1 (they are passed here via 
*                           /sok25/ common block).
*    Array with spalin coefficients C(1103) is passed here from DSPL1 via
*                               common /sok26/
*  ......................................................................
*     ***   ATTENTION: X must be within a range of 10^(-11) -- 1   ***
*  ......................................................................
      real*8 XMIN,STEP,XMAX
      real*8 C(1103)
      real*8 X,X1,Y,Z
      COMMON /sok25/ XMIN,STEP,XMAX
      common /sok26/ C
       X1 = dlog10(X)
       if ((X1.lt.-11.0001d+0).or.(X1.gt.1.d-6)) then
       write(0,*)'ERROR IN FUNCTION C_PA_IN: TRANSFER IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = IDINT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      c_pa_in = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
****************************************************************************
* P.18
        SUBROUTINE DSPLQ1
*
*    It is based on SUBROUITINE SPLINE (W.H.Press et al., Numerical Recopes, 
*  Chapter 3, section 3.3) which is tuned to NATURAL CUBIC SPLINE and adopted 
*                           for purpose of MUM.
*
*   The routine prepares cubic spline coefficients (array Y2) to get value of
*  LOG10(relative energy transfer) by interpolation with a value of integrated
*   comparison function (X array) as an argument. It is necessary to simulate
*            relative energy transfers due to e+e- pair production.
* 
*   BOTH INPUT ARRAYS (Y and X) and output array Y2 are of real*8 precision.
*  ......................................................................
      PARAMETER (N=1101)
      real*8 x(N),y(N),Y2(N),U(N),P,SIG,QN,UN
      common /sok24/ X     !-> input array with values of integrated comparison
c                              function (is passed from PAIR1 routine)
      common /pa_in_s2/ Y  !-> array with rel.energy transfers (to DSPLIN1)
      common /pa_in_s3/ Y2 !-> output array with spline coefficients (to DSPLIN1)
      do i=1,N                              ! Preparation of array Y(1101):
        Y(i) = -1.1d+1 + dble(i-1) * 1.d-2  !   log10(rel. en. transfer)
      enddo   !-----------------------------!       from -11.0 to 0. 
      Y(N) = 0.d+0                          !------ with step 0.01
      Y2(1)=0.d+0 
      U(1)=0.d+0
      DO 11 I=2,N-1 !------------------------------------------!
        SIG=(X(I)-X(I-1))/(X(I+1)-X(I-1))                      !  Cooking of
        P=SIG*Y2(I-1)+2.d+0                                    !    spline 
        Y2(I)=(SIG-1.d+0)/P                                    ! coefficients
        U(I)=(6.d+0*((Y(I+1)-Y(I))/(X(I+1)-X(I))-(Y(I)-Y(I-1)) ! and putting
     *      /(X(I)-X(I-1)))/(X(I+1)-X(I-1))-SIG*U(I-1))/P      !  them into
11    CONTINUE                                                 !   output
c                                                              !  Y2(1101)
      QN=0.d+0                                                 !  array to
      UN=0.d+0                                                 !  be passed 
c                                                              ! to routine
      Y2(N)=(UN-QN*U(N-1))/(QN*Y2(N-1)+1.d+0)                  !  DSPLIN1
      DO 12 K=N-1,1,-1                                         !
        Y2(K)=Y2(K)*Y2(K+1)+U(K)                               !
12    CONTINUE  !----------------------------------------------!
      RETURN
      END
***********************************************************************
* P.19
      real*8 FUNCTION DSPLIN1(X1)
*
*   It is based on SUBROUITINE SPLINT (W.H.Press et al., Numerical Recopes, 
*       Chapter 3, section 3.3) which is adopted for purpose of MUM.
*
*    The routine performes an interpolation using splines prepared by DSPLQ1.
*      The value of integrated comparison function for simulation of energy 
*   transfers due to e+e- pair production  (see comments to routines PAIR1 and 
*    DSPLQ1) X1 is an input, the value of LOG10(relative energy transfer) (Y1, 
*            DSPLIN1) is an output to be used in routine GETVPA.
* 
*   BOTH INPUT AND OUTPUT VARIABLES (X1 and DSPLIN1) are of real*8 precision.
c............................................................................
      PARAMETER (N=1101)
      real*8 X1,Y1
      real*8 XA(N),YA(N),Y2A(N),X,Y,H,A,B
      common /sok24/ XA !--------------------> values of integrated comparison 
c                                                   function (from PAIR1) 
      common /pa_in_s2/ YA !-> array with log10(rel. en. transf.) (from DSPLQ1) 
      common /pa_in_s3/ Y2A !-> array with cubic splines from DSPLQ1
      X=X1
      KLO=1                                                  
      KHI=N
1     IF (KHI-KLO.GT.1) THEN
        K=(KHI+KLO)/2
        IF(XA(K).GT.X)THEN
          KHI=K
        ELSE
          KLO=K
        ENDIF
      GOTO 1
      ENDIF
      H=XA(KHI)-XA(KLO)
      IF (H.EQ.0.d+0) PAUSE 'Bad XA input at function DSPLIN1...'
      A=(XA(KHI)-X)/H
      B=(X-XA(KLO))/H
      Y=A*YA(KLO)+B*YA(KHI)+
     *      ((A*A*A-A)*Y2A(KLO)+(B*B*B-B)*Y2A(KHI))*(H*H)/6.d+0
      Y1 = Y
      DSPLIN1 = Y1
                      if ((Y1.ge.1.d-10).OR.(Y1.lt.-1.1001d+1)) then
      write(0,*)'DSPLIN1 ERR: IN =',X1,'(MAX=',XA(N),'), OUT=',Y1
                       endif
      RETURN
      END
****************************************************************************
* P.20
      SUBROUTINE getvpa(emw,vbr,itr)
*
* It simulates the relative enegy transfers for e+e- pair production by muon
*       with energy transfers > EMIN (if itr=0) or > VMIN (otherwise).
*                Input: emw = muon energy expressed in GeV, 
*               Output: vbr = relative energy transfer. 
*
*   The simulation is done by the "rejection method" (see W.H.Press et al.,
*  NUMERICAL RECEIPES (THE ART OF SCIENTIFIC COMPUTING), Cambridge University
*   press, Chapter 7, pp. 200-204). Comparison function is computed within 
*  routine PAIR1 as a differential cross-section for e+e- pair production by 
* muon with energy 1000 PeV. The values of diff. cross-section are multiplied 
*   by factor 1.03 to make sure that comparison function is exced the diff. 
*   cross-sectiions for any muon energy in a range of 10GeV - 1000 PeV and 
*      relative energy transfers in a range of (emin/1000 PeV) -- 1): 
*          COMPARISON FUNCTION = 1.03 * [d_sigma / d_v (1000 PeV)]
* ..........................................................................
*            EMW MUST BE WITHIN THE RANGE OF 10 -- 1000,000 GeV
* ..........................................................................
       external c_pa_in,dsplin1,getcdp,comp
       parameter (lo=1)
       parameter (len=1)
       real*8 c_pa_in,tot_pa,ai1,dsplin1,ax,algemin,arn
       common /general/ emin,vmin,emph
       common/sok33/ tot_pa
       common /mcef/ mcb1,mcb2,mcp1,mcp2,mcn1,mcn2,mce1,mce2
        mcp1 = mcp1 + 1
          if (itr.eq.0) then
          algemin = dble(emin/emw)
          else 
          algemin = dble(amax1(vmin,2.044e-3/emw))
          endif
        ai1 = c_pa_in(algemin)
        algemin = tot_pa - ai1
  155   arn = rndm(5)
        mcp2 = mcp2 + 1
        ax = (algemin * arn) + ai1
          if (ax.gt.tot_pa) ax = tot_pa
        ax = dsplin1(ax)
          if (ax.ge.-1.d-5) ax = -1.d-5
        ay = sngl(ax)
c 
        if (vmin.le.8.e-4) then
        az = (1.e+1)**ay
        echeck = az * emw
        if (echeck.ge.6.5e-3) then
        y1 = getcdp(emw,ay,lo)
        else
        y1 = pair_tot(emw,az)
        endif
        else
        y1 = getcdp(emw,ay,lo)
        endif
c
        vbr = sngl(ax)
        y2 = comp(vbr)        
        arn = rndm(8)
        yc = y2 * arn
cccccccccccccccccccccccccccccccccccccccccccccccccccccccc!    
           if (y2.lt.y1) then                           !    THIS PART  MAY BE 
           write(0,*)'* GETVPA ERROR * f(v) < d_Sigma/d_v' !--> REMOVED AFTER ALL
           endif                                        !    TESTS  ARE PASSED
cccccccccccccccccccccccccccccccccccccccccccccccccccccccc!
        if (y1.ge.yc) then
        vbr = (1.e+1)**ay
        else
        goto 155
        endif
      return
      end
****************************************************************************
* P.21
         subroutine testpair(lukum)
*
*  Performes some test for all routines which relate e+e- pair production
*              with filling number of histograms and .dat files.
*  The test is short if lukum=0. Otherwise it takes ~3.2 hours but gives 
*                            more statistics. 
*...........................................................................
      external pair_tot,getcdp,c_pa_in,DSPLIN1,getdedpa,getctpa,getlpair
      external gdedpav,gdedpat,gctpav,glpairv
      external filpa1,filpa1a,filpa2,filpa3,filpa4,filpa5,filpa6
      dimension ene(25)
      real*8 c_pa_in,DSPLIN1
      real*8 v_doub,vr_doub,v1_doub,v1r_doub,c_doub,esum,esumsq,eterm
      character *10 txt1
      character *100 txt2
      character *1 txt3 
      character *11 txt4 
      common /general/ emin,vmin,emph
      common /pa1/ NA,enpa
      common /mcef/ mcb1,mcb2,mcp1,mcp2,mcn1,mcn2,mce1,mce2
      data ene / 1.e+1,2.e+1,5.e+1,1.e+2,2.e+2,5.e+2,1.e+3,2.e+3,5.e+3,
     +           1.e+4,2.e+4,5.e+4,1.e+5,2.e+5,5.e+5,1.e+6,2.e+6,5.e+6,
     +           1.e+7,2.e+7,5.e+7,1.e+8,2.e+8,5.e+8,1.e+9             /
      write(0,*)' '
      write(0,*)'         TESTING THE PART FOR E+E- PAIR PRODUCTION:'
      write(0,*)' '
         if (lukum.eq.0) then
           NA = 500000
         else
           NA = 10000000
         endif
c---------------------------------------------------------------------------
c       1. DIFFERENCES BETWEEN GETCDP AND PAIR_TOT (PRECISION OF SPLINES)
c---------------------------------------------------------------------------
      write(0,*)'Computing precision of splines for dif. cross-sections:'
      write(0,*)'  '
      itmp = 1
      do i=1,25
         k = int(ene(i))
         write(txt1,'(i10)') k
         if (i.eq.itmp) then
         itmp = itmp + 3
         j = 1000 + i
         txt2='Err.(percents) between PAIR-TOT and GETCDP (lo=1) for '//
     +txt1//' GeV muon vs. V'
          CALL HBOOK2(j,txt2,100,-1.1e+1,0.e+0,100,-.15,+.05,0.)
         j = 1200 + i
         txt2='Err.(percents) between GETCDP (lo=1) and GETCDP (lo=0) fo
     +r '//txt1//' GeV muon vs. V'
          CALL HBOOK2(j,txt2,100,-1.1e+1,0.e+0,100,-.001,+.001,0.)
         j = 2000 + i
         txt2='Err.(percents) between PAIR-TOT and GETCDP (lo=1) for '//
     +txt1//' GeV muon vs. V'
          CALL HBOOK2(j,txt2,100,.95,1.,100,-.05,+.25,0.)
         j = 2200 + i
         txt2='Err.(percents) between GETCDP (lo=1) and GETCDP (lo=0) fo
     +r '//txt1//' GeV muon vs. V'
          CALL HBOOK2(j,txt2,100,.95,1.,100,-.0003,+.0007,0.)
          endif
        enddo
c
         txt2='Err.(percents) between PAIR-TOT and GETCDP (lo=1) for all
     + muon energies vs. V'
          CALL HBOOK2(1000,txt2,100,-1.1e+1,0.e+0,100,-.15,+.05,0.)
         txt2='Err.(percents) between PAIR-TOT and GETCDP (lo=0) for all
     + muon energies vs. V'
          CALL HBOOK2(1100,txt2,100,-1.1e+1,0.e+0,100,-.15,+.05,0.)
         txt2='Err.(percents) between GETCDP (lo=1) and GETCDP (lo=0) fo
     +r all muon energies vs. V'
          CALL HBOOK2(1200,txt2,100,-1.1e+1,0.e+0,100,-.001,+.001,0.)
         txt2='Err.(percents) between PAIR-TOT and GETCDP (lo=1) for all
     + muon energies vs. V'
          CALL HBOOK2(2000,txt2,100,.95,1.,100,-.05,+.25,0.)
         txt2='Err.(percents) between PAIR_TOT and GETCDP (lo=0) for all
     + muon energies vs. V'
          CALL HBOOK2(2100,txt2,100,.95,1.,100,-.05,+.25,0.)
         txt2='Err.(percents) between GETCDP (lo=1) and GETCDP (lo=0) fo
     +r all muon energies vs. V'
          CALL HBOOK2(2200,txt2,100,.95,1.,100,-.0006,+.0014,0.)
c
      l0 = 0
      l1 = 1
      itmp=1
      do i=1,25
         e = ene(i)
         vmin1 = emin/e
         vmin_0 = amin1(vmin,vmin1)
         write(0,*)'Energy Nb.',i,' of 25 is processed (',e,' GeV ) ...'
         do j=11000,200,-1
           v1 = -float(j) * 1.e-3
           v = 1.e+1**v1
           if (v.lt.vmin_0) goto 1001
           dc_r = pair_tot(e,v) 
           dc_s_n = getcdp(e,v,l0)
           dc_s_l = getcdp(e,v1,l1)
           p1 = 1.e+2 * ((dc_s_n - dc_r)/dc_r)
           p2 = 1.e+2 * ((dc_s_l - dc_r)/dc_r)
           p3 = ((2.e+2 * (dc_s_n - dc_s_l)) / (dc_s_n + dc_s_l))
           if (i.ne.itmp) goto 9801
           k1000 = 1000 + i
           CALL HFILL(k1000,v1,p2,1.)
 9801      CALL HFILL(1000,v1,p2,1.)
           CALL HFILL(1100,v1,p1,1.)
           if (i.ne.itmp) goto 9802
           k1200 = 1200 + i
 9802      CALL HFILL(k1200,v1,p3,1.)
           CALL HFILL(1200,v1,p3,1.)
 1001      continue
         enddo
c
         do j=95000,99999
           v = float(j) * 1.e-5
           v1 = alog10(v)
           dc_r = pair_tot(e,v) 
           dc_s_n = getcdp(e,v,l0)
           dc_s_l = getcdp(e,v1,l1)
           if (dc_r.le.0.) then
           if (dc_s_n.le.1.e-37) then
           p1 = 0. 
           else
           p1=-0.14
           write(0,*)'ERROR DURING PAIR TEST (HIST 21xx)'
           endif
           else
           p1 = 1.e+2 * ((dc_s_n - dc_r)/dc_r)
           endif
c
           if (dc_r.le.0.) then
           if (dc_s_l.le.1.e-37) then
           p2 = 0. 
           else
           p2=-0.14
           write(0,*)'ERROR DURING PAIR TEST (HIST 20xx)'
           endif
           else
           p2 = 1.e+2 * ((dc_s_l - dc_r)/dc_r)
           endif
c
           gopa = dc_s_n + dc_s_l
           if (gopa.le.0.) then
           p3 = 0.
           else
           p3 = ((2.e+2 * (dc_s_n - dc_s_l)) / (dc_s_n + dc_s_l))
           endif
c
           if (i.ne.itmp) goto 9811
           k2000 = 2000 + i
           CALL HFILL(k2000,v,p2,1.)
 9811      CALL HFILL(2000,v,p2,1.)
           CALL HFILL(2100,v,p1,1.)
           if (i.ne.itmp) goto 9812
           k2200 = 2200 + i
           CALL HFILL(k2200,v,p3,1.)
 9812      CALL HFILL(2200,v,p3,1.)
         enddo
         if (i.eq.itmp) itmp = itmp + 3
      enddo
c---------------------------------------------------------------------------
c     2. ERRORS IN DIRECT AND INVERSE COMPUTING OF INTEGRATED COMPARISON 
c            FUNCTION (DIFFERENCES BETWEEN V and DSPLIN!(C_PA_IN(V))
c---------------------------------------------------------------------------
      write(0,*)'  '
      write(0,*)'Testing routines for integrated comparison function...'
      write(0,*)'  '
      txt2='Error (percents) between V and V1=DSPLIN1(C-PA-IN(V)) vs. V'
          CALL HBOOK2(1300,txt2,100,-11.,0.,100,-.0025,+.0015,0.)
      txt2='Error (percents) between V and V1=DSPLIN1(C-PA-IN(V)) vs. V'
          CALL HBOOK2(1301,txt2,100,.9,1.,100,-.01,+.01,0.)
         do j=110000,460,-1
           v1 = -float(j) * 1.e-4
           v1_doub = dble(v1)
           v_doub = 1.d+1**v1_doub
           c_doub = c_pa_in(v_doub)
           v1r_doub = DSPLIN1(c_doub)
           vr_doub = 1.d+1**(v1r_doub)
           h1 = SNGL(v_doub)
           h1_1 = SNGL(v1_doub)
           h2 = SNGL(vr_doub)
           a1 = 1.e+2 * ((h2 - h1) / h1)
           CALL HFILL(1300,h1_1,a1,1.)
         enddo
         do j=900000,999999
           v = float(j) * 1.e-6
           v_doub = dble(v)
           v1_doub = dlog10(v_doub)
           c_doub = c_pa_in(v_doub)
           v1r_doub = DSPLIN1(c_doub)
           vr_doub = 1.d+1**(v1r_doub)
           h2 = SNGL(vr_doub)
           a1 = 1.e+2 * ((h2 - v) / v)
           CALL HFILL(1301,v,a1,1.)
         enddo
c---------------------------------------------------------------------------
c                3. QUALITY OF ENERGY TRANSFER SIMULATION
c---------------------------------------------------------------------------
c                         a) ABOVE EMIN
c
      write(0,*)'Testing GETVPA (en. transfer simulation) with EMIN ...'
      write(0,*)'  '
        itr = 0
        do i=1,25
          k = int(ene(i))
          write(txt1,'(i10)') k
          txt2 = 'Pairs, MC En. transfers distribution for '//txt1//'Gev
     + muon (EMIN)'
          i1 = i + 1500
          CALL HBOOK1(i1,txt2,100,-11.,0.,0.)
        enddo
       open(11,file='cross_pa.dat',status='new',form='formatted')
       open(12,file='leng_pa.dat',status='new',form='formatted')
       open(13,file='loss_pa.dat',status='new',form='formatted')
       open(14,file='loss_pa1.dat',status='new',form='formatted')
       open(15,file='en_pa.dat',status='new',form='formatted')
       open(16,file='er_lo_pa.dat',status='new',form='formatted')
       open(17,file='efmc_pa.dat',status='new',form='formatted')
c
c  EN_PA.DAT - 25 values of muon energy for which all other values are computed.
c  CROSS_PA.DAT - 25 values of total cross-section for e+e- pair production with 
c                 energy transfers > emin (Sq. cm).
c  LENG_PA.DAT - 25 values for averaged free path between production of two e+e- 
c                pairs with energy transfer > emin (cm). 
c  LOSS_PA.DAT - 25 values for energy losses for e+e- pair production with energy
c                transfers > emin (MeV/cm) computed with splines (GETDEDPA 
c                routine).
c  LOSS_PA1.DAT - 25 values for energy losses for e+e- pair production with energy
c                 transfers > emin (MeV/cm) simulated by MC (GETVPA routine) with 
c                 500000 interactions for each muon energy (lukum = 0) or 10000000
c                 interactions (lukum > 0).
c ER_LO_PA.DAT - 25 values of statistical error for values in LOSS_PA1.DAT on 
c                1-sigma level (MeV/cm).
c EFMC_PA.DAT - 25 values of MC efficiency = 
c                              (number of simulation / number of call getvpa)
      do i=1,25
       write(0,*)'Energy Nb.',i,' of 25 is processed (',ene(i),' GeV ) ...'
       i1 = i + 1500
       elos = getdedpa(ene(i))
       tcros = getctpa(ene(i)) 
       rr = getlpair(ene(i)) 
       write (13,234) elos
       write (11,234) tcros
       write (15,234) ene(i)
       write (12,234) rr
       esum = 0.d+0
       esumsq = 0.d+0
         mcp1 = 0
         mcp2 = 0
         do j=1,NA
           call getvpa(ene(i),v,itr)
            v2 = alog10(v)
            eterm1 = ene(i) * v * 1.e+3
            eterm = dble(eterm1)
            esum = esum + eterm
            esumsq = esumsq + (eterm * eterm)
           CALL HFILL(i1,v2,0.,1.)
         enddo  
       efp = float(mcp2)/float(mcp1)
       mcp1 = 0
       mcp2 = 0
       write (17,234) efp
       esum1 = sngl(esum)
       esumsq1 = sngl(esumsq)
       RNA = float(NA)
       err=((RNA*esumsq1) - (esum1*esum1))/(RNA*RNA*(RNA - 1.))
       err = sqrt(err) / rr
       esum1 = (esum1 / rr) / RNA
       write (14,234) esum1
       write (16,234) err
       write(0,*)'En.los.=',esum1,' +-',err,' (should be ',elos,' ) MeV/cm'
          k = int(ene(i))
          write(txt1,'(i10)') k
          txt2 = 'Pairs, En. transfers distribution for '//txt1//' Gev
     + muon'
          i1 = i + 1600
          enpa = ene(i)
          CALL HBFUN1(i1,txt2,100,-1.1e+1,0.e+0,filpa1)
      enddo
       close (17)
       close (16)
       close (15)
       close (14)
       close (13)
       close (12)
       close (11)
c                           b) ABOVE VMIN
      write(0,*)'  '
      write(0,*)'Testing GETVPA (en. transfer simulation) with VMIN...'
      write(0,*)'  '
        itr = 1
        do i=1,25
          k = int(ene(i))
          write(txt1,'(i10)') k
          txt2 = 'Pairs, MC En. transfers distribution for '//txt1//'Gev
     + muon (VMIN)'
          i1 = i + 1800
          CALL HBOOK1(i1,txt2,100,-11.,0.,0.)
        enddo
       open(11,file='cross_pa_v.dat',status='new',form='formatted')
       open(12,file='leng_pa_v.dat',status='new',form='formatted')
       open(13,file='loss_pa_v.dat',status='new',form='formatted')
       open(14,file='loss_pa1_v.dat',status='new',form='formatted')
       open(15,file='loss_pa_t.dat',status='new',form='formatted')
       open(16,file='er_lo_pa_v.dat',status='new',form='formatted')
       open(17,file='efmc_pa_v.dat',status='new',form='formatted')
c  CROSS_PA_V.DAT - 25 values of total cross-section for e+e- pair production with 
c                   energy transfers > vmin (Sq. cm).
c  LENG_PA_V.DAT - 25 values for averaged free path between production of two e+e- 
c                  pairs with energy transfer > vmin (cm). 
c  LOSS_PA_V.DAT - 25 values for energy losses for e+e- pair production with energy
c                  transfers > vmin (MeV/cm) computed with splines (GDEDPAV 
c                  routine).
c  LOSS_PA_T.DAT - 25 values for total energy losses due to e+e- pair production 
c                  (MeV/cm) computed with splines (GDEDPAT routine).
c  LOSS_PA1_V.DAT - 25 values for energy losses for e+e- pair production with energy
c                   transfers > vmin (MeV/cm) simulated by MC (GETVPA routine) with 
c                   500000 interactions for each muon energy (lukum = 0) or 10000000
c                   interactions (lukum > 0).
c ER_LO_PA_V.DAT - 25 values of statistical error for values in LOSS_PA1_V.DAT on 
c                  1-sigma level (MeV/cm).
c EFMC_PA_V.DAT - 25 values of MC efficiency = 
c                              (number of simulation / number of call getvpa)
      do i=1,25
       write(0,*)'Energy Nb.',i,' of 25 is processed (',ene(i),' GeV ) ...'
       i1 = i + 1800
       elos = gdedpav(ene(i))
       elost = gdedpat(ene(i))
       tcros = gctpav(ene(i)) 
       rr = glpairv(ene(i)) 
       write (13,234) elos
       write (11,234) tcros
       write (15,234) elost
       write (12,234) rr
       esum = 0.d+0
       esumsq = 0.d+0
         mcp1 = 0
         mcp2 = 0
         do j=1,NA
           call getvpa(ene(i),v,itr)
            v2 = alog10(v)
            eterm1 = ene(i) * v * 1.e+3
            eterm = dble(eterm1)
            esum = esum + eterm
            esumsq = esumsq + (eterm * eterm)
           CALL HFILL(i1,v2,0.,1.)
         enddo  
       efp = float(mcp2)/float(mcp1)
       mcp1 = 0
       mcp2 = 0
       write (17,234) efp
       esum1 = sngl(esum)
       esumsq1 = sngl(esumsq)
       RNA = float(NA)
       err=((RNA*esumsq1) - (esum1*esum1))/(RNA*RNA*(RNA - 1.))
       err = sqrt(err) / rr
       esum1 = (esum1 / rr) / RNA
       write (14,234) esum1
       write (16,234) err
       write(0,*)'En.los.=',esum1,' +-',err,' (should be ',elos,' ) MeV/cm'
          k = int(ene(i))
          write(txt1,'(i10)') k
          txt2 = 'Pairs, En. transfers distribution for '//txt1//' Gev
     + muon'
          i1 = i + 1900
          enpa = ene(i)
          CALL HBFUN1(i1,txt2,100,-1.1e+1,0.e+0,filpa1a)
      enddo
       close (17)
       close (16)
       close (15)
       close (14)
       close (13)
       close (12)
       close (11)
c---------------------------------------------------------------------------
c     4. Files with differential cros-sections for e+e- pair productioin
c---------------------------------------------------------------------------
c 9 files (pa_cr_1.dat, ... , pa_cr_9.dat) are produced with 1101 values in
c each. These values are equal to differential cross-section of e+e- pair 
c   production for log10(v) = -11.00, -10.99, -10.98, ... , -0.01, 0.
c Figure "X" in file name pa_cr_X.dat corresponds to muon energy for which 
c            cross-section is computed (E_mu = 10.**X GeV).
c
       write(0,*)'  '
       write(0,*)'Computing dif. crossect. for a set of muon energies ...'
       write(0,*)'  '
       ilk = 0
       do i=1,25,3
       ilk = ilk + 1
       write(txt3,'(i1)') ilk
       txt4='pa_cr_'//txt3//'.dat'
       open(11,file=txt4,status='new',form='formatted')
          do j=1100,0,-1
             v1 = -float(j) * 1.e-2
             v1 = 1.e+1 ** v1
             some = pair_tot(ene(i),v1)
             write (11,234) some
          enddo
      close (11)
      enddo 
c---------------------------------------------------------------------------
c  5. DIFFERENCES BETWEEN GETLPAIR, GETDEDPA, GLPAIRV, GDEDPAV, GDEDPAT and 
c     REAL VALUES OF AVERAGED FREE PATH AND ENERGY LOSSES (PRECISION OF 
c     SPLINES)
c---------------------------------------------------------------------------
       write(0,*)'Testing quality of 1-dim. splines...'
       write(0,*)'  '
      txt2 = 'Pairs, rel. err. GETLPAIR - mean free path (percents)'
          CALL HBFUN1(1701,txt2,97,.9583334,9.04165,filpa2)
      txt2 = 'Pairs, rel. err. GETDEDPA - energy losses (percents)'
          CALL HBFUN1(1702,txt2,97,.9583334,9.04165,filpa3)
      txt2 = 'Pairs, rel. err. GLPAIRV - mean free path (percents)'
          CALL HBFUN1(1703,txt2,97,.9583334,9.04165,filpa4)
      txt2 = 'Pairs, rel. err. GDEDPAV - energy losses (percents)'
          CALL HBFUN1(1704,txt2,97,.9583334,9.04165,filpa5)
      txt2 = 'Pairs, rel. err. GDEDPAT - energy losses (percents)'
          CALL HBFUN1(1705,txt2,97,.9583334,9.04165,filpa6)
c---------------------------------------------------------------------------
  234  format (e13.7) 
       return
      end
c===========================================================================
c P.21a
c                AN AUXILIARY FUNCTION FOR TESTPAIR SUBROUTINE
c
      function filpa1(x)
      external getctpa,pair_tot,simps
      common /pa1/ NA,enpa
      common /general/ emin,vmin,emph
      dimension aux(0:20)
      lim = 20
      vtr=emin/enpa
        op1 = x - .055
        op2 = x + .055
        ope1 = 2.302585093 * op1
        ope2 = 2.302585093 * op2
        op2_0 = 1.0e+1**(op2)
          if (op2_0.le.vtr) then
          filpa1 = 0.
          goto 422
          else
            do j=0,lim
              rnu = op1 + ( (.11/float(lim)) * (float(j))) 
              rnu0 = 1.e+1**(rnu)
              aux(j) = pair_tot(enpa,rnu0) * rnu0
            enddo
          filpa1 = simps(aux,ope1,ope2,lim) 
          filpa1 = (filpa1 / getctpa(enpa)) * float(NA)
          goto 422
          endif
 422  return
      end
c===========================================================================
c P.21b
c                AN AUXILIARY FUNCTION FOR TESTPAIR SUBROUTINE
c
      function filpa1a(x)
      external gctpav,pair_tot,simps
      common /pa1/ NA,enpa
      common /general/ emin,vmin,emph
      dimension aux(0:20)
      lim = 20
      vtr=vmin
        op1 = x - .055
        op2 = x + .055
        ope1 = 2.302585093 * op1
        ope2 = 2.302585093 * op2
        op2_0 = 1.0e+1**(op2)
          if (op2_0.le.vtr) then
          filpa1a = 0.
          goto 422
          else
            do j=0,lim
              rnu = 1.e+1**(op1 + ( (.11/float(lim)) * (float(j)))) 
              aux(j) = pair_tot(enpa,rnu) * rnu
            enddo
          filpa1a = simps(aux,ope1,ope2,lim) 
          filpa1a = (filpa1a / gctpav(enpa)) * float(NA)
          goto 422
          endif
 422  return
      end
c===========================================================================
c B.21c
c                AN AUXILIARY FUNCTION FOR TESTBREM SUBROUTINE
c
      function filpa2(x)
      external getlpair,pair_tot,simps
      real*8 um,ene,u(10),z1(10),w(10),aw(10),a_ef,ro,alfa
      real*8 rm_e,rm_mu,r_e,avog
      integer nsub
      common /general/ emin,vmin,emph
      common /media/ z1,w,aw,a_ef,ro,nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      dimension aux(0:2000)
      x1 = 1.e+1**x
      lim=2000
      vmi = alog(emin / x1) !------------> The lower limit for integration
      um = 0.d+0
        do k=1,nsub
        ene=dble(x1)
        u(k)=((z1(k))**(1.d+0/3.d+0))*1.236540953d+0*(rm_mu/(ene*1.d+3))
        u(k) = 1.d+0 - u(k)
        um = dmax1(u(k),um)
        enddo
      vma=sngl(um)  !----------------> The upper limit for integration
      vma=alog(vma) !
      vst = (vma - vmi) / float(lim) !-> step for a rel. en. transfers grid
        do j=0,lim
        rnu = exp(vmi + vst*(float(j)))
        aux(j) = pair_tot(x1,rnu) * rnu       !-> array to be integrated 
c                                            !   for getting free path
        enddo     
      filpa2 = simps(aux,vmi,vma,lim)
      filpa2 = sngl(a_ef / (avog * ro)) / filpa2
      dummy = getlpair(x1)
      filpa2 = 1.e+2 * ( (dummy - filpa2) / filpa2 ) !-> rel.error (percents)
      return
      end
c===========================================================================
c P.21d
c                AN AUXILIARY FUNCTION FOR TESTPAIR SUBROUTINE
c
      function filpa3(x)
      external getdedpa,pair_tot,simps
      real*8 um,ene,u(10),z1(10),w(10),aw(10),a_ef,ro,alfa
      real*8 rm_e,rm_mu,r_e,avog
      integer nsub
      common /general/ emin,vmin,emph
      common /media/ z1,w,aw,a_ef,ro,nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      dimension aux(0:2000)
      x1 = 1.e+1**x
      lim=2000
      vmi = alog(emin / x1) !------------> The lower limit for integration
      um = 0.d+0
        do k=1,nsub
        ene=dble(x1)
        u(k)=((z1(k))**(1.d+0/3.d+0))*1.236540953d+0*(rm_mu/(ene*1.d+3))
        u(k) = 1.d+0 - u(k)
        um = dmax1(u(k),um)
        enddo
      vma=sngl(um)  !----------------> The upper limit for integration
      vma=alog(vma) !
      vst = (vma - vmi) / float(lim) !-> step for a rel. en. transfers grid
        do j=0,lim
        rnu = exp(vmi + vst*(float(j)))
        aux(j) = pair_tot(x1,rnu) * rnu * rnu    !-> array to be integrated 
c                                               !  for getting energy losses
        enddo     
      filpa3 = simps(aux,vmi,vma,lim) * sngl(avog*ro/a_ef) * x1 * 1.e+3
      dummy = getdedpa(x1)
      filpa3 = 1.e+2 * ( (dummy - filpa3) / filpa3 ) !--> rel.error (percents)
      return
      end
c===========================================================================
c B.21e
c                AN AUXILIARY FUNCTION FOR TESTPAIR SUBROUTINE
c
      function filpa4(x)
      external glpairv,pair_tot,simps
      real*8 um,ene,u(10),z1(10),w(10),aw(10),a_ef,ro,alfa
      real*8 rm_e,rm_mu,r_e,avog
      integer nsub
      common /general/ emin,vmin,emph
      common /media/ z1,w,aw,a_ef,ro,nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      dimension aux(0:2000)
      x1 = 1.e+1**x
      lim=2000
      vmi = alog(vmin) !------------> The lower limit for integration
      um = 0.d+0
        do k=1,nsub
        ene=dble(x1)
        u(k)=((z1(k))**(1.d+0/3.d+0))*1.236540953d+0*(rm_mu/(ene*1.d+3))
        u(k) = 1.d+0 - u(k)
        um = dmax1(u(k),um)
        enddo
      vma=sngl(um)  !----------------> The upper limit for integration
      vma=alog(vma) !
      vst = (vma - vmi) / float(lim) !-> step for a rel. en. transfers grid
        do j=0,lim
        rnu = exp(vmi + vst*(float(j)))
        aux(j) = pair_tot(x1,rnu) * rnu       !-> array to be integrated 
c                                            !   for getting free path
        enddo     
      filpa4 = simps(aux,vmi,vma,lim)
      filpa4 = sngl(a_ef / (avog * ro)) / filpa4
      dummy = glpairv(x1)
      filpa4 = 1.e+2 * ( (dummy - filpa4) / filpa4 ) !-> rel.error (percents)
      return
      end
c===========================================================================
c P.21f
c                AN AUXILIARY FUNCTION FOR TESTPAIR SUBROUTINE
c
      function filpa5(x)
      external gdedpav,pair_tot,simps
      real*8 um,ene,u(10),z1(10),w(10),aw(10),a_ef,ro,alfa
      real*8 rm_e,rm_mu,r_e,avog
      integer nsub
      common /general/ emin,vmin,emph
      common /media/ z1,w,aw,a_ef,ro,nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      dimension aux(0:2000)
      x1 = 1.e+1**x
      lim=2000
      vmi = alog(vmin) !------------> The lower limit for integration
      um = 0.d+0
        do k=1,nsub
        ene=dble(x1)
        u(k)=((z1(k))**(1.d+0/3.d+0))*1.236540953d+0*(rm_mu/(ene*1.d+3))
        u(k) = 1.d+0 - u(k)
        um = dmax1(u(k),um)
        enddo
      vma=sngl(um)  !----------------> The upper limit for integration
      vma=alog(vma) !
      vst = (vma - vmi) / float(lim) !-> step for a rel. en. transfers grid
        do j=0,lim
        rnu = exp(vmi + vst*(float(j)))
        aux(j) = pair_tot(x1,rnu) * rnu * rnu    !-> array to be integrated 
c                                               !  for getting energy losses
        enddo     
      filpa5 = simps(aux,vmi,vma,lim) * sngl(avog*ro/a_ef) * x1 * 1.e+3
      dummy = gdedpav(x1)
      filpa5 = 1.e+2 * ( (dummy - filpa5) / filpa5 ) !--> rel.error (percents)
      return
      end
c===========================================================================
c P.21g
c                AN AUXILIARY FUNCTION FOR TESTPAIR SUBROUTINE
c
      function filpa6(x)
      external gdedpat,pair_tot,simps
      real*8 um,ene,u(10),z1(10),w(10),aw(10),a_ef,ro,alfa
      real*8 rm_e,rm_mu,r_e,avog
      integer nsub
      common /general/ emin,vmin,emph
      common /media/ z1,w,aw,a_ef,ro,nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      dimension aux(0:2000)
      x1 = 1.e+1**x
      lim=2000
      vmi = alog(2.046e-3/x1) !------------> The lower limit for integration
      um = 0.d+0
        do k=1,nsub
        ene=dble(x1)
        u(k)=((z1(k))**(1.d+0/3.d+0))*1.236540953d+0*(rm_mu/(ene*1.d+3))
        u(k) = 1.d+0 - u(k)
        um = dmax1(u(k),um)
        enddo
      vma=sngl(um)  !----------------> The upper limit for integration
      vma=alog(vma) !
      vst = (vma - vmi) / float(lim) !-> step for a rel. en. transfers grid
        do j=0,lim
        rnu = exp(vmi + vst*(float(j)))
        aux(j) = pair_tot(x1,rnu) * rnu * rnu    !-> array to be integrated 
c                                                !  for getting energy losses
        enddo     
      filpa6 = simps(aux,vmi,vma,lim) * sngl(avog*ro/a_ef) * x1 * 1.e+3
      dummy = gdedpat(x1)
      filpa6 = 1.e+2 * ( (dummy - filpa6) / filpa6 ) !--> rel.error (percents)
      return
      end
****************************************************************************
************************* PHOTONUCLEAR SUBROUTINES : ***********************
****************************************************************************
* N.1
*     THIS REAL*8 FUNCTION CALCULATES DIFFERENTIAL CROS-SECTION FOR MUON
*       PHOTONUCLEAR INTERACTION ACCORDING TO EITHER BUGAEV-BEZRUKOV OR 
*                           ZEUS PARAMETRIZATION
*
*                                  INPUT:
*                                  =====
*
*  real*8 z     : electric charge of nucley
*  real*8 en    : muon energy (GeV)
*  real*8 rnu   : relative energy transfer = (E_transferred / E_mu)
*  real*8 a     : atomic weight of nucleus
*
*                                 OUTPUT:
*                                 ======
*
*  real*8 phnu   : d_sigma/d_v (sq. cm) for nucleus with given Z, muon
*                  energy, relative energy transfer and A
*  .........................................................................
      real*8 function phnu(z,en,rnu,a)
      real*8 z,en,rnu,a,rnu1
      real*8 alfa,rm_e,rm_mu,r_e,fact,avog,m1,m2,mn,s,sigma,t,hv,zet
      real*8 te1,te2,te3,g,etr
      common /pnsig/ ibb  ! From MED_CONS, indicates Sigma_gamma_p
      common /exer1/ fa
      common /const/ alfa,rm_e,rm_mu,r_e,avog ! universal constants which are
c                                             ! provided by subroutine
c                                             ! MED_CONS and are passed here
c                                             ! by common /const/
      parameter (m1 = 5.4d-1)  ! GeV squared
      parameter (m2 = 1.8d+0)  ! GeV squared
      parameter (mn = .939d+0) ! nucleon mass (Gev) = (Mp + Mn) / 2
      parameter (fact = 2.903524525d-4) ! fact = alfa / (8 * pi)
      rnu1 = 1.d+0 - rnu
      etr = rnu * en
      s = etr * 2.d+0 * mn
       if (ibb.eq.1) then
        sigma = 1.143d+2 + 1.647d+0 * ((dlog(2.213d-2 * etr))**2.d+0)
c       it is Sigma_gamma_p parametrization from Bezrukov-Bugaev      
       else
        sigma = (6.35d+1 * (s**9.7d-2)) + (1.45d+2 / (s**5.d-1))
c       it is Sigma_gamm_p parametrization from ZEUS (J.Breitweg 
c       et al., hep-ex/9809005, Eur.Phys.J., C7 (1999) 609)
       endif
      sigma = sigma * 1.d-30
      t = (rm_mu * rm_mu * 1.d-6 * rnu * rnu) / rnu1
      hv = 1.d+0 - (2.d+0 / rnu) + (2.d+0 / (rnu * rnu))
      zet = 2.82d-3 * sigma * 1.d+30 * (a**(1.d+0 / 3.d+0))
      te1 = hv * dlog(1.d+0 + (m2/t))
      if(t.gt.1.d-7) then
      te2 = 1.d+0 - ((.25d+0 * m2/t) * dlog(1.d+0 + (t/m2)))
      te2 = te2 * 2.d+0 * rm_mu * rm_mu * 1.d-6 / t
      else
      te2 = (1.5d+0 / t) * rm_mu * rm_mu * 1.d-6
      endif
      te3 = (dlog(1.d+0 + (m1/t)) - (m1/(m1 + t))) * hv
      te3=te3-((2.d+0*rm_mu*rm_mu*1.d-6/t)*(1.d+0-((.25d+0*m1)/(m1+t))))
      g = ( (1.d+0 + zet) * dexp(-zet) ) - 1.d+0
      g = ( ( g / (zet * zet) ) + 5.d-1 ) * ( 9.d+0 / zet )
      if(z.lt.1.5d+0) g = 3.d+0  !---------------> Especial case for hydrogen
      phnu = te1 - te2 + (g * te3)
      phnu = phnu * fact * rnu * sigma * a * dble(fa)
      if (phnu.lt.0.d+0) phnu=0.d+0
      return
      end
****************************************************************************
* N.2
*        THIS FUNCTION CALCULATES DIFFERENTIAL CROSS-SECTION FOR MUON
*     PHOTONUCLEAR INTERACTION IN GIVEN MEDIA USING REAL*8 FUNCTION PHNU  
*
*                                  INPUT:
*                                  =====
*
*  real*4 ene    : muon energy (GeV)
*  real*4 v      : relative energy transfer
*
*                                  OUTPUT:
*                                  ======
*
*  real*4 phnu_tot   : d_sigma/d_v (sq. cm), for given media, muon energy and 
*                      relative energy transfer. Values of diff. cross-section 
*                      are averaged over all nuclei which given media consists 
*                      of:
*
*                      d_s/d_v = SUM (w_i * d_s/d_v_i), where w_i = N_i/N_tot
*                      N_i - number of type i atoms in molecula
*                      N_tot - total number of atoms in molecula
*
*                      So, it is calculated for an "effective nucleus" with
*                      atomic weight A_eff = SUM(N_i*A_i) / N_tot
*  .........................................................................
       function phnu_tot(ene,v)
       external phnu
       real*8 z1(10),w(10),aw(10),a_ef,en,rnu,h1,phnu,ro
       integer nsub
       common /media/ z1,w,aw,a_ef,ro,nsub ! this common determines media com-
c                                     ! position: AW (atomic weights), A_EF
c                                   ! (effective atomic weight), Z1 (charges), 
c                                 ! W(relative weights), NSUB (number of media 
c                                ! components), ro (cm/qubic cm) - media densi-
c                                ! ty.  All these  are prepared by subroutine
c                               ! MED_CONS and passed here via commom /media/.
c
       if (v.gt..999998) v = .999998
       en = dble(ene)
       rnu = dble(v)
           h1 = w(1) * phnu(z1(1),en,rnu,aw(1))
         if (nsub.ge.2) then
           do l=2,nsub
             h1 = h1 + (w(l) * phnu(z1(l),en,rnu,aw(l)))
           enddo
         endif
       phnu_tot = sngl(h1)
       return
       end
****************************************************************************
* N.3
       subroutine phnu1
*                         The subroutine calculates:
*
*                                    [1]
*                           
* the differential cross-sections d_Sigma/d_v for muon photonuclear interaction 
*        using FUNCTION PHNU_TOT. The result is put in three arrays
*      fcd1(81,54), fcd2(81,101) and fcd3(81,51) which are passed to 
* subroutines SPL2 via common block /cdph_in/. The reason to use just 3 arrays
*     is that the region of energy transfers is divided into 3 regions: 
*      10^(-11) - 10^(-0.6), 10^(-0.6) - 10^(-0.1) and 10^(-0.1) - 1 
*     for which different grid steps are accepted (0.2, 0.01 and 0.001, 
*           correspondingly) to obtain accuracy better than 0.001.
*    Dif. cross-sections are computed for 81 values of muon energy starting 
*  with E_mu = 10 GeV and finishing with E_mu = 1000 Pev with logarithmically
*                          equidistant griid.     
*  SPL2 cooks spline coefficients out of these 3 arrays which are used in
*                          further computing.
*
*                                    [2]
*
*  Energy losses and mean free path for muon photonuclear interaction with
*   energy transfers which exceed EMIN, VMIN and 0 and en. transfers below 
*     EMIN - arrays elo_ph1(17), crt_ph1(17); elo_ph2(65), crt_ph2(65); 
*  elo_ph3(17); elo_ph3(17), correspondingly, which are computed for 17 or 
*  65 values of muon energy starting  with E_mu = 10 GeV and finishing with 
* E_mu = 1 Eev with logarithmically equidistant grid. These arrays are passed 
* to subroutine SPL1 via common blocks /elph_in1/, /elph_in2/, /elph_in4/ and 
*  /ctph_in1/ where they are used to prepare spline coefficients for further 
*                                 computing.
*
*                                    [3]
*
*      A values for variables AT and BT which are neccessary to compute a 
*  comparison function AT / (V**(BT)) (V is relative energy transfer) which is 
*  used in subroutuine GETVPH to simulate energy transfers due to photonuclear 
*                             interaction. 
*    Also several constants composed of AT and BT (us2, us2_1,us3, us4) are 
*     computed to be passed along with AT and BT to subroutine getvph via 
*                             common /ph_ref/.
*     If comparison function is found during simulation to be less than diff.
*  cross-section calculated by splines, it is increased without a stop. If total
*          increase exceed 50%, a message appears on the screen.
*  ..................................................................... 
       external phnu_tot,dsimps
       real*8 aux1(0:2000),aux2(0:2000),h1,h2,dsimps
       real*8 alfa,rm_e,rm_mu,r_e,avog,z1(10),w(10),aw(10),a_ef,ro
       integer nsub
       common /const/ alfa,rm_e,rm_mu,r_e,avog ! from initial subroutine
       common /media/ z1,w,aw,a_ef,ro,nsub     !       MED_CONS
       common /general/ emin,vmin,emph         !
       common /cdph_in/ fcd1(81,54),fcd2(81,101),fcd3(81,51) ! To SPL2
       common /ctph_in1/ crt_ph1(17),crt_ph2(65) ! To SPL1 and ENLOS
       common /elph_in1/ elo_ph1(17),elo_ph2(65) ! To SPL1 and ENLOS
       common /elph_in2/ elo_ph3(17)   ! To SPL1 and ENLOS
       common /elph_in4/ elo_ph4(17)   ! To ENLOS
       common /ph_ref/ at,bt,us2,us2_1,us3,us4,at0  ! To GETVPH
       common /pnsig/ ibb  ! From MED_CONS, indicates Sigma_gamma_p
       common /help_1/ aux1,aux2
c   ....................................................................
c      1.PREPARATION OF ARRAYS WITH DIFF. CROSS-SEECTIONS FOR MUON
c                         PHOTONUCLEAR INTERACTION:
c
c                !       .......................... 
      do j=1,81  !--->   j CYCLE BY ENERGIES starts
c                !       .......................... 
        en = float(j)
        en = 1.e+1**(9.e-1 + (1.e-1 * en))
c                     !         ..................................
      do k=1,206      !--->     k CYCLE BY ENERGY TRANSFERS starts
c                     !         ..................................
      if (k.eq.206) then
        rnu = 1.e+0
        fcd2(j,101) = phnu_tot(en,rnu)
        if (fcd2(j,101).le.1.e-37) fcd2(j,101) = 1.e-37
        fcd2(j,101) = alog(fcd2(j,101))
        goto 975
      endif
c
      if (k.le.54) then
         k01 = k - 1
         rnu = 1.e+1**(float(k01) * 2.e-1 - 1.1e+1)
      else
         if (k.le.105) then
           k01 = k - 55
           rnu = 1.e+1**(float(k01) * 1.e-2 - 6.e-1)
         else
           k01 = k - 106
           rnu = 1.e+1**(float(k01) * 1.e-3 - 1.e-1)
         endif
      endif
c
      cr_dif=phnu_tot(en,rnu)
      if (cr_dif.le.1.e-37) cr_dif = 1.e-37
      cr_dif=alog(cr_dif)
c
      if (k.le.54) fcd1(j,k) = cr_dif
      if ((k.gt.54).AND.(k.lt.106)) fcd3(j,k-54) = cr_dif
      if (k.ge.106) fcd2(j,k-105) = cr_dif
c
 975  continue
c              !     ..................................... 
      enddo    !----> k CYCLE BY ENERGY TRANSFERS finishes 
c              !     ..................................... 
      enddo    !----> j CYCLE BY ENERGIES finishes 
c              !     .................................... 
c   ....................................................................
c      2.PREPARATION OF ARRAYS WITH ENERGY LOSSES AND MEAN FREE PATH
c                     FOR MUON PHOTONUCLEAR INTERACTION:
c
c                       a) Energy transfers > EMIN :
      lim = 2000
      do i=1,17
      en = float(i)
      en = (1.e+1)**(5.e-1 + (5.e-1 * en))
      vmi1 = alog(emin / en)
      vmi2 = alog(emph / en)
      vmi = amax1(vmi1,vmi2) !--------------> The lower limit for integration
      vma = 0.e+0            !--------------> The upper limit for integration
      vst = (vma - vmi) / float(lim) !------> Step for a rel. en. transfers grid
c
      do j=0,lim
      rnu = vmi + vst*(float(j))
      rnu = exp(rnu)
      aux1(j) = dble(phnu_tot(en,rnu) * rnu)       !-> array to be integrated 
c                                                  !   to get mean free path
      aux2(j) = aux1(j) * dble(rnu)                !-> array to be integrated 
      enddo                                        !   for getting en. losses
c
      h1 = dble(vmi)
      h2 = dble(vma)
      crt_ph1(i) = dsimps(aux1,h1,h2,lim)              !-> array with 17 values
      crt_ph1(i) = sngl(a_ef / (avog * ro))/crt_ph1(i) !   of mean free path       
      crt_ph1(i) = alog(crt_ph1(i))                    !
      elo_ph1(i) = sngl( dsimps(aux2,h1,h2,lim) * avog * ro / a_ef )
      elo_ph1(i) = elo_ph1(i) * en * 1.e+3 
      elo_ph1(i) = alog10(elo_ph1(i)) !-> array with 17 values of en. losses
      enddo
c                       b) Energy transfers > VMIN :
      do i=1,65
      en = float(i)
      en = (1.e+1)**(8.75e-1 + (1.25e-1 * en))
      vmi1 = alog(vmin)
      vmi2 = alog(emph / en)
      vmi = amax1(vmi1,vmi2) !--------------> The lower limit for integration
      vma = 0.e+0            !--------------> The upper limit for integration
      vst = (vma - vmi) / float(lim) !------> Step for a rel. en. transfers grid
c
      do j=0,lim
      rnu = vmi + vst*(float(j))
      rnu = exp(rnu)
      aux1(j) = dble(phnu_tot(en,rnu) * rnu)       !-> array to be integrated 
c                                                  !   to get mean free path
      aux2(j) = aux1(j) * dble(rnu)                !-> array to be integrated 
      enddo                                        !   for getting en. losses
c
      h1 = dble(vmi)
      h2 = dble(vma)
      crt_ph2(i) = dsimps(aux1,h1,h2,lim)              !-> array with 17 values
      crt_ph2(i) = sngl(a_ef / (avog * ro))/crt_ph2(i) !   of mean free path       
      elo_ph2(i) = sngl( dsimps(aux2,h1,h2,lim) * avog * ro / a_ef )
      elo_ph2(i) = elo_ph2(i) * en * 1.e+3 
      elo_ph2(i) = alog10(elo_ph2(i)) !-> array with 17 values of en. losses
      enddo
c                       c) Energy transfers > 0 :
      do i=1,17
      en = float(i)
      en = (1.e+1)**(5.e-1 + (5.e-1 * en))
      vmi = alog(emph / en)  !--------------> The lower limit for integration
      vma = 0.e+0            !--------------> The upper limit for integration
      vst = (vma - vmi) / float(lim) !------> Step for a rel. en. transfers grid
c
      do j=0,lim
      rnu = vmi + vst*(float(j))
      rnu = exp(rnu)
      aux2(j) = dble(phnu_tot(en,rnu) * rnu * rnu) !-> array to be integrated 
c                                                  !   to get en. losses
      enddo
c
      h1 = dble(vmi)
      h2 = dble(vma)
      elo_ph3(i) = sngl( dsimps(aux2,h1,h2,lim) * avog * ro / a_ef )
      elo_ph3(i) = elo_ph3(i) * en * 1.e+3 
      elo_ph3(i) = alog10(elo_ph3(i)) !-> array with 17 values of en. losses
      enddo
c                       d) Energy transfers < EMIN :
      do i=1,17
      en = float(i)
      en = (1.e+1)**(5.e-1 + (5.e-1 * en))
      vmi = alog(emph / en)  !--------------> The lower limit for integration
      vma = alog(emin / en)            !--------------> The upper limit for integration
      vst = (vma - vmi) / float(lim) !------> Step for a rel. en. transfers grid
      if (vma.le.vmi) then
      elo_ph4(i) = 0.
      goto 1234
      endif
c
      do j=0,lim
      rnu = vmi + vst*(float(j))
      rnu = exp(rnu)
      aux2(j) = dble(phnu_tot(en,rnu) * rnu * rnu) !-> array to be integrated 
c                                                  !   to get en. losses
      enddo
c
      h1 = dble(vmi)
      h2 = dble(vma)
      elo_ph4(i) = sngl( dsimps(aux2,h1,h2,lim) * avog * ro / a_ef )
      elo_ph4(i) = elo_ph4(i) * en * 1.e+3 !-> array with 17 values
 1234 continue                             !   of en. losses 
      enddo
c   ....................................................................
c          3. COMPUTING OF CONSTANTS FOR COMPARISON FUNCTION:
c
       if(ibb.eq.1) then
         en1 = 1.e+9
         en2 = 10.
         v1 = 1.78e-2
         v2 = 1.e-10
         bt = alog10(phnu_tot(en2,v2)) - alog10(phnu_tot(en1,v1))
         bt = bt / (alog10(v1/v2))
       else
         en1 = 1000.
         en2 = 100.
         v1 = 2.e-4
         v2 = 2.e-3
         bt = alog10(phnu_tot(en1,v1)) - alog10(phnu_tot(en2,v2))
       endif
      at = 1.02 * (v2**bt) * phnu_tot(en2,v2) 
      at0 = at
      us2 = at / (1.-bt)
      us2_1 = 1. / us2
      us3 = .434294481 / (1. - bt)
      us4 = 2.302585093 * (-bt)
      return
      end
****************************************************************************
* N.4
            FUNCTION getlphnu(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*    Function GETLPHNU calculates the muon's free path for photonuclear 
*    interaction with energy transfers > EMIN using spline coefficients
*                       prepared by spl1 subroutine.
*
*          X is a value for muon energy expressed in GeV; XMIN,
*   XMAX and STEP are assigned with some values in subroutine SPL1 (they
*              are passed here via /sok1/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                              common /sok1_n/
*  ......................................................................
*  ***   ATTENTION: X must be within a range of 10 GeV -- 1000 PeV   ***
*  ......................................................................
      COMMON /sok1/ XMIN,STEP,XMAX
      common /sok1_n/ C(19)
      X1 = alog10(X)
      if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
      write(0,*)'ERROR IN FUNCTION GETLPHNU: MUON ENERGY IS OUT OF RANGE'
      endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      getlphnu = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      getlphnu = exp(getlphnu)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
****************************************************************************
* N.5
            FUNCTION glphnuv(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*    Function GLPHNUV calculates the muon's free path for photonuclear 
*    interaction with energy transfers > VMIN using spline coefficients
*                       prepared by spl1 subroutine.
*
*          X is a value for muon energy expressed in GeV; XMIN,
*   XMAX and STEP are assigned with some values in subroutine SPL1 (they
*              are passed here via /sok1n/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                              common /sok1_n2/
*  ......................................................................
*  ***   ATTENTION: X must be within a range of 10 GeV -- 1000 PeV   ***
*  ......................................................................
      COMMON /sok1n/ XMIN,STEP,XMAX
      common /sok1_n2/ C(67)
      X1 = alog10(X)
      if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
      write(0,*)'ERROR IN FUNCTION GLPHNUV: MUON ENERGY IS OUT OF RANGE'
      endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      glphnuv = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
****************************************************************************
* N.6
         function getctph(u)
*
*     The subroutine calculates the value for total cross-section for muon 
*      photonuclear interaction with energy transfers > EMIN using getphnu
*     subroutine. It also uses some media parameters prepared by subroutine 
*                      MED_CONS (commons /const/ and /media/)
*   The resulting cross-section is averaged over all atoms of given media:
*
*                      sigma(E_mu) = SUM (w_i * sigma_i(E_mu)), 
*
*       where w_i = N_i/N_tot, N_i - number of type i atoms in molecula, 
* N_tot - total number of atoms in molecula, sigma_i(E_mu) is a cross-section
*                               for atom number i.
*     So, it is calculated for an "effective nucleus" with atomic weight 
*                         A_eff = SUM(N_i*A_i) / N_tot
*
*                 Input: u = muon energy (GeV) 
*                 Output: getctph = total cross-section (sq. cm)
* ..........................................................................
*        U MUST BE WITHIN THE RANGE OF 10 -- 1000,000,000 GeV
* ..........................................................................
      external getlphnu
      real*8 z1(10),w(10),aw(10),a_ef,alfa,rm_e,rm_mu,r_e,avog,ro
      integer nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      common /media/ z1,w,aw,a_ef,ro,nsub
      if ((u.gt.1.0001e+9).or.(u.lt.9.9999e+0)) then   
      write(0,*)'ERROR IN FUNCTION GETCTPH: MUON ENERGY IS OUT OF RANGE'
      endif
      aef = sngl(a_ef / (avog * ro))
      getctph = aef / getlphnu(u)
      return
      end
****************************************************************************
* N.7
         function gctphv(u)
*
*     The subroutine calculates the value for total cross-section for muon 
*      photonuclear interaction with energy transfers > VMIN using getphnu
*     subroutine. It also uses some media parameters prepared by subroutine 
*                      MED_CONS (commons /const/ and /media/)
*   The resulting cross-section is averaged over all atoms of given media:
*
*                      sigma(E_mu) = SUM (w_i * sigma_i(E_mu)), 
*
*       where w_i = N_i/N_tot, N_i - number of type i atoms in molecula, 
* N_tot - total number of atoms in molecula, sigma_i(E_mu) is a cross-section
*                               for atom number i.
*     So, it is calculated for an "effective nucleus" with atomic weight 
*                         A_eff = SUM(N_i*A_i) / N_tot
*
*                 Input: u = muon energy (GeV) 
*                 Output: getctph = total cross-section (sq. cm)
* ..........................................................................
*        U MUST BE WITHIN THE RANGE OF 10 -- 1000,000,000 GeV
* ..........................................................................
      external glphnuv
      real*8 z1(10),w(10),aw(10),a_ef,alfa,rm_e,rm_mu,r_e,avog,ro
      integer nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      common /media/ z1,w,aw,a_ef,ro,nsub
      if ((u.gt.1.0001e+9).or.(u.lt.9.9999e+0)) then   
      write(0,*)'ERROR IN FUNCTION GCTPHV: MUON ENERGY IS OUT OF RANGE'
      endif
      aef = sngl(a_ef / (avog * ro))
      gctphv = aef / glphnuv(u)
      return
      end
************************************************************************
* N.8
              FUNCTION getdedph(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*       The subroutine calculates the value for muon energy losses for 
*  photonuclear interaction with energy transfers above EMIN using spline 
*                coefficients prepared by spl1 subroutine.
*
*          X is a value for muon energy expressed in GeV; XMIN,
*  XMAX and STEP are assigned with some values in subroutine SPL1 (they
*               are passed here via /sok5/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                              common /sok5_n/
*  ......................................................................
*  ***   ATTENTION: X must be within a range of 10 GeV --- 1000 PeV   ***
*  ......................................................................
      COMMON /sok5/ XMIN,STEP,XMAX
      common /sok5_n/ C(19)
      X1 = alog10(X)
       if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
       write(0,*)'ERROR IN FUNCTION GETDEDPH: MUON ENERGY IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      getdedph = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      getdedph = (1.e+1)**(getdedph)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
************************************************************************
* N.9
              FUNCTION gdedphv(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*       The subroutine calculates the value for muon energy losses for 
*  photonuclear interaction with energy transfers above VMIN using spline 
*                coefficients prepared by spl1 subroutine.
*
*          X is a value for muon energy expressed in GeV; XMIN,
*  XMAX and STEP are assigned with some values in subroutine SPL1 (they
*               are passed here via /sok5n/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                              common /sok5_n2/
*  ......................................................................
*  ***   ATTENTION: X must be within a range of 10 GeV --- 1000 PeV   ***
*  ......................................................................
      COMMON /sok5n/ XMIN,STEP,XMAX
      common /sok5_n2/ C(67)
      X1 = alog10(X)
       if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
       write(0,*)'ERROR IN FUNCTION GDEDPHV: MUON ENERGY IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      gdedphv = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      gdedphv = (1.e+1)**(gdedphv)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
************************************************************************
* N.10
              FUNCTION gdedpht(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
* The subroutine calculates the value for total muon energy losses due to 
*  photonuclear interaction using spline coefficients prepared by spl1 
*                                subroutine.
*
*          X is a value for muon energy expressed in GeV; XMIN,
*  XMAX and STEP are assigned with some values in subroutine SPL1 (they
*               are passed here via /sok5/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                              common /sok5_n3/
*  ......................................................................
*  ***   ATTENTION: X must be within a range of 10 GeV --- 1000 PeV   ***
*  ......................................................................
      COMMON /sok5/ XMIN,STEP,XMAX
      common /sok5_n3/ C(19)
      X1 = alog10(X)
       if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
       write(0,*)'ERROR IN FUNCTION GDEDPHT: MUON ENERGY IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      gdedpht = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      gdedpht = (1.e+1)**(gdedpht)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
****************************************************************************
* N.11
       FUNCTION getcdn(X,Y,lo)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
* The subroutine calculates the value for differential cross-section for muon
*    photonuclear interaction d_Sigma/d_v using functions getcdn1, getcdn2, 
*  getcdn3 or phnu_tot depending on value of energy transfer Y. For definition 
* of crosssection for muon e+e- pair production in given media see comments to 
*                            subroutine PHNU_TOT.
* X and Y are values for muon energy expressed in GeV and for relative energy 
*   transfer, respectively. If LO=0, then Y is expressed in natural units, 
*           otherwise the input for Y is presented as LOG10(Y).
*  .....................................................................
*        ATTENTION: X must be within a range of 10 GeV -- 1000 PeV
*                   Y must be within a range of 10**(-11) -- 10**(0)
*  ......................................................................
*    ATTENTION: GETCDN computes diff. cross-sections for all relative 
*  energy transfers > 10**(-11). In reality photonuclear cross-section
*   is equal to 0 for transferred energies < 0.5 - 1 GeV. It is taken
*    into account in the frame of MUM code but one should put ones 
*      attention on this fact using GETCDN for other purposes.
*  ......................................................................
       external getcdn1,getcdn2,getcdn3,phnu_tot
       if (lo.eq.0) then
          if (Y.le..251188643) then
                getcdn = getcdn1(X,Y,lo)
               getcdn = exp(getcdn)
          else
               if (Y.le..794328234) then
                  getcdn = getcdn2(X,Y,lo)
                  getcdn = exp(getcdn)
               else
                 if (Y.le..965) then
                    getcdn = getcdn3(X,Y,lo)
                    getcdn = exp(getcdn)
                 else
                    getcdn = phnu_tot(X,Y)
                 endif
               endif
          endif
       else
          if (Y.le.-6.e-1) then
                getcdn = getcdn1(X,Y,lo)
                getcdn = exp(getcdn)
          else
                if (Y.le.-1.e-1) then
                   getcdn = getcdn2(X,Y,lo)
                   getcdn = exp(getcdn)
                else
                   if (Y.le.-1.54727e-2) then 
                     getcdn = getcdn3(X,Y,lo)
                     getcdn = exp(getcdn)
                   else
                     Y1 = (1.e+1)**Y
                     getcdn = phnu_tot(X,Y1)
                   endif
                endif
          endif
       endif
      if (getcdn.lt.0.e+0) getcdn = 0.e+0
      return
      end
****************************************************************************
* N.12
       FUNCTION getcdn1(X,Y,loga)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*   The subroutine calculates the value for differential cross-section for muon
*   photonuclear interaction d_Sigma/d_v using spline coefficients prepared by 
*    SPL2 routine. For definition of cross-section for e+e- pair production in 
*               given media see comments to subroutine PHNU_TOT.
*  .....................................................................
*   X and Y in body program is a value for muon energy expressed in GeV and
*    for relative energy transfer, respectively. NX, NY, X0, SX, Y0 and SY
*   are assigned with some values in subroutine SPL2 (they are passed here
*   via /sok_2_1/ common block. Array with spline coefficients C1(4648) is 
*            cooked by SPL2 and passed here via common /mum8/
*            If LOGA=0, then Y is expressed in natural units, 
*           otherwise the input for Y is presented as LOG10(Y).
*                  Output is presented as LN (d_sigma/d_v)
*  ......................................................................
*        ATTENTION: X must be within a range of 10 GeV -- 1000 PeV
*                   Y must be within a range of 10**(-11) -- 10**(-0.6)
*  ......................................................................
       common /mum8/ C1(4648)
       common /sok_2_1/ NX,NY,X0,SX,Y0,SY
      X_1=alog10(X)
      if ((X_1.lt..99999e+0).or.(X_1.gt.9.0001e+0)) then
      write(0,*)'ERROR IN FUNCTION GETCDN1: MUON ENERGY IS OUT OF RANGE'
      endif
      if (loga.eq.0) then    
      Y_1=alog10(Y)
      else
      Y_1=Y
      endif
      if ((Y_1.lt.-11.001).or.(Y_1.gt.-5.9999e-1)) then
      write(0,*)'ERROR IN FUNCTION GETCDN1: EN. TRANSFER IS OUT OF RANGE'
      endif
      A3=(X_1-X0)/SX
      B3=(Y_1-Y0)/SY
      M1=INT(A3+.5)
      M2=INT(B3+.5)
      M3=NX+2
ccc      IF(M1.LT.0.OR.M1.GE.NX.OR.M2.LT.0.OR.M2.GE.NY) THEN
ccc      PRINT 1,X_1,M1,Y_1,M2
ccc      ENDIF
      A3=A3-M1
      B3=B3-M2
      M1=M2*M3+M1+1
      M2=M1+M3
      M3=M2+M3
      A2=A3*A3+.25
      B2=B3*B3+.25
      A1=A2-A3
      B1=B2-B3
      A3=A2+A3
      B3=B2+B3
      A2=2.-(A2+A2)
      B2=2.-(B2+B2)
      getcdn1=(A1*C1(M1)+A2*C1(M1+1)+A3*C1(M1+2))*B1
     2      +(A1*C1(M2)+A2*C1(M2+1)+A3*C1(M2+2))*B2
     3      +(A1*C1(M3)+A2*C1(M3+1)+A3*C1(M3+2))*B3
      RETURN
ccc1     FORMAT('GETCDN1: *MISTAKE*X= ',D23.16,' MX= ',I4,' Y= ',D23.16,' M
ccc     *Y= ',I4)
      END
****************************************************************************
* N.13
       FUNCTION getcdn2(X,Y,loga)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*   The subroutine calculates the value for differential cross-section for muon
*   photonuclear interaction d_Sigma/d_v using spline coefficients prepared by 
*    SPL2 routine. For definition of cross-section for e+e- pair production 
*              in given media see comments to subroutine PHNU_TOT.
*  .....................................................................
*   X and Y in body program is a value for muon energy expressed in GeV and
*    for relative energy transfer, respectively. NX, NY, X0, SX, Y0 and SY
*   are assigned with some values in subroutine SPL2 (they are passed here
*   via /sok_2_2/ common block. Array with spline coefficients C1(4399) is 
*            cooked by SPL2 and passed here via common /mum9/
*            If LOGA=0, then Y is expressed in natural units, 
*           otherwise the input for Y is presented as LOG10(Y).
*                  Output is presented as LN (d_sigma/d_v)
*  ......................................................................
*        ATTENTION: X must be within a range of 10 GeV -- 1000 PeV
*                   Y must be within a range of 10**(-0.6) -- 10**(-0.1)
*  ......................................................................
       common /mum9/ C1(4399)
       common /sok_2_2/ NX,NY,X0,SX,Y0,SY
      X_1=alog10(X)
      if ((X_1.lt..99999e+0).or.(X_1.gt.9.0001e+0)) then
      write(0,*)'ERROR IN FUNCTION GETCDN2: MUON ENERGY IS OUT OF RANGE'
      endif
      if (loga.eq.0) then    
      Y_1=alog10(Y)
      else
      Y_1=Y
      endif
      if ((Y_1.lt.-.60001).or.(Y_1.gt.-.99999e-1)) then
      write(0,*)'ERROR IN FUNCTION GETCDN2: EN. TRANSFer IS OUT OF RANGE'
      endif
      A3=(X_1-X0)/SX
      B3=(Y_1-Y0)/SY
      M1=INT(A3+.5)
      M2=INT(B3+.5)
      M3=NX+2
ccc      IF(M1.LT.0.OR.M1.GE.NX.OR.M2.LT.0.OR.M2.GE.NY) THEN
ccc      PRINT 1,X_1,M1,Y_1,M2
ccc      ENDIF
      A3=A3-M1
      B3=B3-M2
      M1=M2*M3+M1+1
      M2=M1+M3
      M3=M2+M3
      A2=A3*A3+.25
      B2=B3*B3+.25
      A1=A2-A3
      B1=B2-B3
      A3=A2+A3
      B3=B2+B3
      A2=2.-(A2+A2)
      B2=2.-(B2+B2)
      getcdn2=(A1*C1(M1)+A2*C1(M1+1)+A3*C1(M1+2))*B1
     2      +(A1*C1(M2)+A2*C1(M2+1)+A3*C1(M2+2))*B2
     3      +(A1*C1(M3)+A2*C1(M3+1)+A3*C1(M3+2))*B3
      RETURN
ccc1     FORMAT('GCDN2:*MISTAKE*X=',D23.16,'MX=',I4,'Y=',D23.16,'MY=',I4)
      END
****************************************************************************
* N.14
       FUNCTION getcdn3(X,Y,loga)
*
*  THE ROUTINE USES A PART OF splains.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*   The subroutine calculates the value for differential cross-section for muon
*   photonuclear interaction d_Sigma/d_v using spline coefficients prepared by 
*    SPL2 routine. For definition of cross-section for e+e- pair production 
*              in given media see comments to subroutine PHNU_TOT.
*  .....................................................................
*   X and Y in body program is a value for muon energy expressed in GeV and
*    for relative energy transfer, respectively. NX, NY, X0, SX, Y0 and SY
*   are assigned with some values in subroutine SPL2 (they are passed here
*   via /sok_2_3/ common block. Array with spline coefficients C2(8549) is 
*            cooked by SPL2 and passed here via common /mum10/
*            If LOGA=0, then Y is expressed in natural units, 
*           otherwise the input for Y is presented as LOG10(Y).
*                  Output is presented as LN (d_sigma/d_v)
*  ......................................................................
*        ATTENTION: X must be within a range of 10 GeV -- 1000 PeV
*                   Y must be within a range of 10**(-0.1) -- 1
*  ......................................................................
       common /mum10/ C2(8549)
       common /sok_2_3/ NX,NY,X0,SX,Y0,SY
      X_1=alog10(X)
      if ((X_1.lt..99999e+0).or.(X_1.gt.9.0001e+0)) then
      write(0,*)'ERROR IN FUNCTION GETCDN3: MUON ENERGY IS OUT OF RANGE'
      endif
      if (loga.eq.0) then    
      Y_1=alog10(Y)
      else
      Y_1=Y
      endif
      if ((Y_1.lt.-1.0001e-1).or.(Y_1.gt.1.e-6)) then
      write(0,*)'ERROR IN FUNCTION GETCDN3: EN. TRANSFER IS OUT OF RANGE'
      endif
      A3=(X_1-X0)/SX
      B3=(Y_1-Y0)/SY
      M1=INT(A3+.5)
      M2=INT(B3+.5)
      M3=NX+2
ccc      IF(M1.LT.0.OR.M1.GE.NX.OR.M2.LT.0.OR.M2.GE.NY) THEN
ccc      PRINT 1,X_1,M1,Y_1,M2
ccc      ENDIF
      A3=A3-M1
      B3=B3-M2
      M1=M2*M3+M1+1
      M2=M1+M3
      M3=M2+M3
      A2=A3*A3+.25
      B2=B3*B3+.25
      A1=A2-A3
      B1=B2-B3
      A3=A2+A3
      B3=B2+B3
      A2=2.-(A2+A2)
      B2=2.-(B2+B2)
      getcdn3=(A1*C2(M1)+A2*C2(M1+1)+A3*C2(M1+2))*B1
     2      +(A1*C2(M2)+A2*C2(M2+1)+A3*C2(M2+2))*B2
     3      +(A1*C2(M3)+A2*C2(M3+1)+A3*C2(M3+2))*B3
      RETURN
ccc1     FORMAT('*MISTAKE*X=',D23.16,'MX=',I4,'Y=',D23.16,'MY=',I4)
      END
****************************************************************************
* N.15
          SUBROUTINE getvph(emw,vbr,itr)
*
*      It simulates the relative enegy transfers for muon's photonuclear
* interaction with energy transfers > EMIN (if itr=0) or > VMIN (otherwise)
*                 Input: emw = muon energy expressed in GeV, 
*                 Output: vbr = relative energy transfer. 
*
*   The simulation is done by the "rejection method" (see W.H.Press et al.,
*  NUMERICAL RECEIPES (THE ART OF SCIENTIFIC COMPUTING), Cambridge University
*            press, Chapter 7, pp. 200-204) with comparison function 
*                              f(v) = at / v**bt
*  where at and bt are constants  calculated by PHNU1 routine and passed here 
*                             via common /ph_ref/
* ..........................................................................
*            EMW MUST BE WITHIN THE RANGE OF 10 -- 1000,000,000 GeV
* ..........................................................................
      external getcdn
      parameter (lo=1)
      common /general/ emin,vmin,emph
      common /mcef/ mcb1,mcb2,mcp1,mcp2,mcn1,mcn2,mce1,mce2
      common /ph_ref/ at,bt,us2,us2_1,us3,us4,at0
      mcn1 = mcn1 + 1
        if (itr.eq.0) then
          vtr=amax1((emin/emw),(emph/emw))
        else
          vtr=amax1(vmin,(emph/emw))
        endif
      us1 = vtr**(1. - bt)
      algemin = us2 * (1. - us1)
  155 ax = algemin * rndm(5)   
      vbr = (alog(us1 + (ax * us2_1))) * us3
      mcn2 = mcn2 + 1
      y1 = getcdn(emw,vbr,lo)
      y2 = at * exp(vbr * us4)
      yc = y2 * rndm(8)
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
           if (y2.lt.y1) then
           fnew = y1 / y2
           at = at * (fnew)
           fnew1 = at / at0
           us2 = at / (1.-bt)
           us2_1 = 1. / us2
           us3 = .434294481 / (1. - bt)
           us4 = 2.302585093 * (-bt)
       write(0,*)'***** COMPARISON FUNCTION FOR PH.NUC. SIMULATION: *****'
           write(0,*)'Variable  AT has been increased with factor',fnew
           write(0,*)'Now it differs from init. value with factor',fnew1
       write(0,*)'   No reasons to trouble, it is for information only'
             if (fnew1.gt.1.5) then
       write(0,*)'************ ERROR ************ f(v) < d_Sigma/d_v ****'
       write(0,*)'**** VARIABLE  AT  HAS BEEN INCREASED TOO MUCH !!! ****'
             endif
       write(0,*)'*******************************************************'
           endif
ccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccccc
        if (y1.ge.yc) then
        vbr = 1.e+1**vbr
        else
        goto 155
        endif
      return
      end
****************************************************************************
* N.16
              subroutine testphnu(lukum)
*
*  Performes some test for all routines which relate photonuclear interaction
*              with filling number of histograms and .dat files.
*  The test is short if lukum=0. Otherwise it takes more time but gives more 
*                                statistics. 
*...........................................................................
      external phnu_tot,getcdn,getdedph,getctph,getlphnu
      external gdedphv,gdedpht,gctphv,glphnuv
      external filph1,filph1a,filph2,filph3,filph4,filph5,filph6
      dimension ene(25)
      real*8 esum,esumsq,eterm
      character *10 txt1
      character *100 txt2
      character *1 txt3 
      character *11 txt4 
      common /general/ emin,vmin,emph
      common /ph1/ NA,enpa
      common /mcef/ mcb1,mcb2,mcp1,mcp2,mcn1,mcn2,mce1,mce2
      data ene / 1.e+1,2.e+1,5.e+1,1.e+2,2.e+2,5.e+2,1.e+3,2.e+3,5.e+3,
     +           1.e+4,2.e+4,5.e+4,1.e+5,2.e+5,5.e+5,1.e+6,2.e+6,5.e+6,
     +           1.e+7,2.e+7,5.e+7,1.e+8,2.e+8,5.e+8,1.e+9             /
c
      write(0,*)' '
      write(0,*)'      TESTING THE PART FOR PHOTONUCLEAR INTERACTION:'
      write(0,*)' '
         if (lukum.eq.0) then
           NA = 500000
         else
           NA = 9000000
         endif
c---------------------------------------------------------------------------
c     1. DIFFERENCES BETWEEN GETCDN AND PHNU_TOT (PRECISION OF SPLINES)
c---------------------------------------------------------------------------
      write(0,*)'Computing precision of splines for dif. cross-sections:'
      write(0,*)'  '
c
      itmp = 1
      do i=1,25
         k = int(ene(i))
         write(txt1,'(i10)') k
         if (i.eq.itmp) then
         itmp = itmp + 3
         j = 5000 + i
         txt2='Err.(percents) between PHNUTOT and GETCDN (lo=1) for '//
     +txt1//' GeV muon vs. V'
          CALL HBOOK2(j,txt2,100,-1.1e+1,0.e+0,100,-.065,+.065,0.)
         j = 5200 + i
         txt2='Err.(percents) between GETCDN (lo=1) and GETCDN (lo=0)
     +for '//txt1//' GeV muon vs. V'
          CALL HBOOK2(j,txt2,100,-1.1e+1,0.e+0,100,-1.e-6,+1.e-6,0.)
         j = 6000 + i
         txt2='Err.(percents) between PHNUTOT and GETCDN (lo=1) for '//
     +txt1//' GeV muon vs. V'
          CALL HBOOK2(j,txt2,100,.95,1.,100,-.002,+.002,0.)
         j = 6200 + i
         txt2='Err.(percents) between GETCDN (lo=1) and GETCDN (lo=0)
     +for '//txt1//' GeV muon vs. V'
          CALL HBOOK2(j,txt2,100,.95,1.,100,-5.e-8,+5.e-8,0.)
          endif
c
        enddo
c
         txt2='Err.(percents) between PHNUTOT and GETCDN (lo=1) for all
     + muon energies vs. V'
          CALL HBOOK2(5000,txt2,100,-1.1e+1,0.e+0,100,-.065,+.065,0.)
         txt2='Err.(percents) between PHNUTOT and GETCDN (lo=0) for all
     + muon energies vs. V'
          CALL HBOOK2(5100,txt2,100,-1.1e+1,0.e+0,100,-.065,+.065,0.)
         txt2='Err.(percents) between GETCDN (lo=1) and GETCDN (lo=0)
     +for all muon energies vs. V'
        CALL HBOOK2(5200,txt2,100,-1.1e+1,0.e+0,100,-1.e-6,+1.e-6,0.)
         txt2='Err.(percents) between PHNUTOT and GETCDN (lo=1) for all
     + muon energies vs. V'
          CALL HBOOK2(6000,txt2,100,.95,1.,100,-.002,+.002,0.)
         txt2='Err.(percents) between PHNUTOT and GETCDN (lo=0) for all
     + muon energies vs. V'
          CALL HBOOK2(6100,txt2,100,.95,1.,100,-.002,+.002,0.)
         txt2='Err.(percents) between GETCDN (lo=1) and GETCDN (lo=0)
     +for all muon energies vs. V'
          CALL HBOOK2(6200,txt2,100,.95,1.,100,-5.e-8,+5.e-8,0.)
c
      l0 = 0
      l1 = 1
      itmp=1
      do i=1,25
         e = ene(i)
         vmin1 = emin/e
         vmin_0 = amin1(vmin,vmin1)
         write(0,*)'Energy Nb.',i,' of 25 is processed (',e,' GeV ) ...'
         do j=11000,200,-1
           v1 = -float(j) * 1.e-3
           v = 1.e+1**v1
           if (v.lt.vmin_0) goto 1001
           dc_r = phnu_tot(e,v) 
           dc_s_n = getcdn(e,v,l0)
           dc_s_l = getcdn(e,v1,l1)
           p1 = 1.e+2 * ((dc_s_n - dc_r)/dc_r)
           p2 = 1.e+2 * ((dc_s_l - dc_r)/dc_r)
           p3 = ((2.e+2 * (dc_s_n - dc_s_l)) / (dc_s_n + dc_s_l))
           if (i.ne.itmp) goto 9801
           k5000 = 5000 + i
           CALL HFILL(k5000,v1,p2,1.)
 9801      CALL HFILL(5000,v1,p2,1.)
           CALL HFILL(5100,v1,p1,1.)
           if (i.ne.itmp) goto 9802
           k5200 = 5200 + i
 9802      CALL HFILL(k5200,v1,p3,1.)
           CALL HFILL(5200,v1,p3,1.)
 1001      continue
         enddo
c
         do j=95000,99999
           v = float(j) * 1.e-5
           v1 = alog10(v)
           dc_r = phnu_tot(e,v) 
           dc_s_n = getcdn(e,v,l0)
           dc_s_l = getcdn(e,v1,l1)
c
           if (dc_r.le.0.) then
           if (dc_s_n.le.1.e-37) then
           p1 = 0. 
           else
           p1=-0.14
           write(0,*)'ERROR DURING PHNU TEST (HIST 61xx)'
           endif
           else
           p1 = 1.e+2 * ((dc_s_n - dc_r)/dc_r)
           endif
c
           if (dc_r.le.0.) then
           if (dc_s_l.le.1.e-37) then
           p2 = 0. 
           else
           p2=-0.14
           write(0,*)'ERROR DURING PHNU TEST (HIST 60xx)'
           endif
           else
           p2 = 1.e+2 * ((dc_s_l - dc_r)/dc_r)
           endif
c
           gopa = dc_s_n + dc_s_l
           if (gopa.le.0.) then
           p3 = 0.
           else
           p3 = ((2.e+2 * (dc_s_n - dc_s_l)) / (dc_s_n + dc_s_l))
           endif
c
           if (i.ne.itmp) goto 9811
           k6000 = 6000 + i
           CALL HFILL(k6000,v,p2,1.)
 9811      CALL HFILL(6000,v,p2,1.)
           CALL HFILL(6100,v,p1,1.)
           if (i.ne.itmp) goto 9812
           k6200 = 6200 + i
           CALL HFILL(k6200,v,p3,1.)
 9812      CALL HFILL(6200,v,p3,1.)
         enddo
         if (i.eq.itmp) itmp = itmp + 3
      enddo
c---------------------------------------------------------------------------
c                2. QUALITY OF ENERGY TRANSFER SIMULATION
c---------------------------------------------------------------------------
c
c                            a) ABOVE EMIN
      write(0,*)'  '
      write(0,*)'Testing GETVPH (en. transfers simulation) with EMIN ...'
      write(0,*)'  '
        itr = 0
        do i=1,25
          k = int(ene(i))
          write(txt1,'(i10)') k
          txt2 = 'Phnu, MC En. transfers distribution for '//txt1//' Gev
     + muon(EMIN)'
          i1 = i + 5500
          CALL HBOOK1(i1,txt2,100,-11.,0.,0.)
        enddo
       open(11,file='cross_ph.dat',status='new',form='formatted')
       open(12,file='leng_ph.dat',status='new',form='formatted')
       open(13,file='loss_ph.dat',status='new',form='formatted')
       open(14,file='loss_ph1.dat',status='new',form='formatted')
       open(15,file='en_ph.dat',status='new',form='formatted')
       open(16,file='er_lo_ph.dat',status='new',form='formatted')
       open(17,file='efmc_ph.dat',status='new',form='formatted')
c
c  EN_PH.DAT - 25 values of muon energy for which all other values are computed.
c  CROSS_PH.DAT - 25 values of total cross-section for photonuclear interaction 
c                 with energy transfers > MAX(emin,emph) (Sq. cm).
c  LENG_PH.DAT - 25 values for averaged free path between two photonuclear
c                interactions with energy transfer > MAX(emin,emph) (cm). 
c  LOSS_PH.DAT - 25 values for energy losses due to P.N. interaction with energy
c                transfers > emin (MeV/cm) computed with splines (GETDEDPH 
c                routine).
c  LOSS_PH1.DAT - 25 values for energy losses for P.N. interaction with energy
c                 transfers > MAX(emin,emph) (MeV/cm) simulated by MC (GETVPH 
c                 routine) with 500000 interactions for each muon energy (lukum 
c                 = 0) or 9000000 interactions (lukum > 0).
c ER_LO_PH.DAT - 25 values of statistical error for values in LOSS_PH1.DAT on 
c                1-sigma level (MeV/cm).
c EFMC_PH.DAT - 25 values of MC efficiency = 
c                              (number of simulation / number of call getvph)
      do i=1,25
       write(0,*)'Energy Nb.',i,' of 25 is processed (',ene(i),' GeV ) ...'
       i1 = i + 5500
       elos = getdedph(ene(i))
       tcros = getctph(ene(i)) 
       rr = getlphnu(ene(i)) 
       write (13,234) elos
       write (11,234) tcros
       write (15,234) ene(i)
       write (12,234) rr
       esum = 0.d+0
       esumsq = 0.d+0
       mcn1 = 0
       mcn2 = 0
         do j=1,NA
           call getvph(ene(i),v,itr)
            v2 = alog10(v)
            eterm1 = ene(i) * v * 1.e+3
            eterm = dble(eterm1)
            esum = esum + eterm
            esumsq = esumsq + (eterm * eterm)
           CALL HFILL(i1,v2,0.,1.)
         enddo  
       efn = float(mcn2) / float (mcn1)
       mcn1 = 0
       mcn2 = 0
       write (17,234) efn
       esum1 = sngl(esum)
       esumsq1 = sngl(esumsq)
       RNA = float(NA)
       err=((RNA*esumsq1) - (esum1*esum1))/(RNA*RNA*(RNA - 1.))
       err = sqrt(err) / rr
       esum1 = (esum1 / rr) / RNA
       write (14,234) esum1
       write (16,234) err
       write(0,*)'En.los.=',esum1,' +-',err,' (should be ',elos,' ) MeV/cm'
          k = int(ene(i))
          write(txt1,'(i10)') k
          txt2 = 'Phnu, En. transfers distribution for '//txt1//' Gev mu
     +on'
          i1 = i + 5600
          enpa = ene(i)
          CALL HBFUN1(i1,txt2,100,-1.1e+1,0.e+0,filph1)
      enddo
       close (17)
       close (16)
       close (15)
       close (14)
       close (13)
       close (12)
       close (11)
c                            b) ABOVE VMIN
      write(0,*)'  '
      write(0,*)'Testing GETVPH (en. transfers simulation) with VMIN ...'
      write(0,*)'  '
        itr = 1
        do i=1,25
          k = int(ene(i))
          write(txt1,'(i10)') k
          txt2 = 'Phnu, MC En. transfers distribution for '//txt1//' Gev
     + muon(VMIN)'
          i1 = i + 5800
          CALL HBOOK1(i1,txt2,100,-11.,0.,0.)
        enddo
       open(11,file='cross_ph_v.dat',status='new',form='formatted')
       open(12,file='leng_ph_v.dat',status='new',form='formatted')
       open(13,file='loss_ph_v.dat',status='new',form='formatted')
       open(14,file='loss_ph1_v.dat',status='new',form='formatted')
       open(15,file='loss_ph_t.dat',status='new',form='formatted')
       open(16,file='er_lo_ph_v.dat',status='new',form='formatted')
       open(17,file='efmc_ph_v.dat',status='new',form='formatted')
c  CROSS_PH_V.DAT - 25 values of total cross-section for photonuclear 
c                   interaction with energy transfers > MAX(vmin,emph) 
c                   (Sq. cm).
c  LENG_PH_V.DAT - 25 values for averaged free path between two P.N.
c                  interactions with energy transfer > MAX(vmin,emph) (cm). 
c  LOSS_PH_V.DAT - 25 values for energy losses due to P.N. interaction with
c                  energy transfers > emin (MeV/cm) computed with splines 
c                  (GDEDPHV routine).
c  LOSS_PH_V.DAT - 25 values for total energy losses due to P.N. interaction
c                  (MeV/cm) computed with splines (GDEDPHT routine).
c  LOSS_PH1_V.DAT - 25 values for energy losses for P.N. interaction with 
c                   energy transfers > MAX(vmin,emph) (MeV/cm) simulated by 
c                   MC (GETVPH routine) with 500000 interactions for each 
c                   muon energy (lukum = 0) or 9000000 interactions 
c                  (lukum > 0).
c  ER_LO_PH_V.DAT - 25 values of statistical error for values in 
c                   LOSS_PH1_V.DAT on 1-sigma level (MeV/cm).
c  EFMC_PH_V.DAT - 25 values of MC efficiency = 
c                            (number of simulation / number of call getvph)
      do i=1,25
       write(0,*)'Energy Nb.',i,' of 25 is processed (',ene(i),' GeV ) ...'
       i1 = i + 5800
       elos = gdedphv(ene(i))
       elost = gdedpht(ene(i))
       tcros = gctphv(ene(i)) 
       rr = glphnuv(ene(i)) 
       write (13,234) elos
       write (11,234) tcros
       write (15,234) elost
       write (12,234) rr
       esum = 0.d+0
       esumsq = 0.d+0
       mcn1 = 0
       mcn2 = 0
         do j=1,NA
           call getvph(ene(i),v,itr)
            v2 = alog10(v)
            eterm1 = ene(i) * v * 1.e+3
            eterm = dble(eterm1)
            esum = esum + eterm
            esumsq = esumsq + (eterm * eterm)
           CALL HFILL(i1,v2,0.,1.)
         enddo  
       efn = float(mcn2) / float (mcn1)
       mcn1 = 0
       mcn2 = 0
       write (17,234) efn
       esum1 = sngl(esum)
       esumsq1 = sngl(esumsq)
       RNA = float(NA)
       err=((RNA*esumsq1) - (esum1*esum1))/(RNA*RNA*(RNA - 1.))
       err = sqrt(err) / rr
       esum1 = (esum1 / rr) / RNA
       write (14,234) esum1
       write (16,234) err
       write(0,*)'En.los.=',esum1,' +-',err,' (should be ',elos,' ) MeV/cm'
          k = int(ene(i))
          write(txt1,'(i10)') k
          txt2 = 'Phnu, En. transfers distribution for '//txt1//' Gev mu
     +on'
          i1 = i + 5900
          enpa = ene(i)
          CALL HBFUN1(i1,txt2,100,-1.1e+1,0.e+0,filph1a)
      enddo
       close (17)
       close (16)
       close (15)
       close (14)
       close (13)
       close (12)
       close (11)
c---------------------------------------------------------------------------
c   3. Files with differential cros-sections for photonuclear interaction
c---------------------------------------------------------------------------
c 9 files (ph_cr_1.dat, ... , ph_cr_9.dat) are produced with 1101 values in
c each. These values are equal to differential cross-section of photonuclear 
c     interaction for log10(v) = -11.00, -10.99, -10.98, ... , -0.01, 0.
c Figure "X" in file name pa_cr_X.dat corresponds to muon energy for which 
c            cross-section is computed (E_mu = 10.**X GeV).
c
       write(0,*)'  '
       write(0,*)'Computing dif. crossect. for a set of muon energies ...'
       write(0,*)'  '
       ilk = 0
       do i=1,25,3
       ilk = ilk + 1
       write(txt3,'(i1)') ilk
       txt4='ph_cr_'//txt3//'.dat'
       open(11,file=txt4,status='new',form='formatted')
          do j=1100,0,-1
             v1 = -float(j) * 1.e-2
             v1 = 1.e+1 ** v1
             some = phnu_tot(ene(i),v1)
             write (11,234) some
          enddo
      close (11)
      enddo 
c---------------------------------------------------------------------------
c  4. DIFFERENCES BETWEEN GETLPHNU, GETDEDPH, GLPHNUV, GDEDPHV, GDEDPHT and 
c     REAL VALUES OF AVERAGED FREE PATH AND ENERGY LOSSES (PRECISION OF 
c     SPLINES)
c---------------------------------------------------------------------------
       write(0,*)'Testing quality of 1-dim. splines ...'
       write(0,*)'  '
      txt2 = 'Phnu, rel. err. GETLPHNU - mean free path (percents)'
          CALL HBFUN1(5701,txt2,97,.9583334,9.04165,filph2)
      txt2 = 'Phnu, rel. err. GETDEDPH - energy losses (percents)'
          CALL HBFUN1(5702,txt2,97,.9583334,9.04165,filph3)
      txt2 = 'Phnu, rel. err. GLPHNUV - mean free path (percents)'
          CALL HBFUN1(5703,txt2,97,.9583334,9.04165,filph4)
      txt2 = 'Phnu, rel. err. GDEDPHV - energy losses (percents)'
          CALL HBFUN1(5704,txt2,97,.9583334,9.04165,filph5)
      txt2 = 'Phnu, rel. err. GDEDPHT - energy losses (percents)'
          CALL HBFUN1(5705,txt2,97,.9583334,9.04165,filph6)
c---------------------------------------------------------------------------
  234 format (e13.7) 
      return
      end
c===========================================================================
c N.16a
c                AN AUXILIARY FUNCTION FOR TESTPHNU SUBROUTINE
c
      function filph1(x)
      external getctph,phnu_tot,simps
      common /ph1/ NA,enpa
      common /general/ emin,vmin,emph
      dimension aux(0:20)
      lim = 20
      vtr=amax1((emin/enpa),(emph/enpa))
        op1 = x - .055
        op2 = x + .055
        ope1 = 2.302585093 * op1
        ope2 = 2.302585093 * op2
        op2_0 = 1.0e+1**(op2)
          if (op2_0.le.vtr) then
          filph1 = 0.
          goto 422
          else
            do j=0,lim
              rnu = 1.e+1**(op1 + ( (.11/float(lim)) * (float(j))))
              aux(j) = phnu_tot(enpa,rnu) * rnu
            enddo
          filph1 = simps(aux,ope1,ope2,lim) 
          filph1 = (filph1 / getctph(enpa)) * float(NA)
          goto 422
          endif
 422  return
      end
c===========================================================================
c N.16b
c                AN AUXILIARY FUNCTION FOR TESTPHNU SUBROUTINE
c
      function filph1a(x)
      external gctphv,phnu_tot,simps
      common /ph1/ NA,enpa
      common /general/ emin,vmin,emph
      dimension aux(0:20)
      lim = 20
      vtr=amax1(vmin,(emph/enpa))
        op1 = x - .055
        op2 = x + .055
        ope1 = 2.302585093 * op1
        ope2 = 2.302585093 * op2
        op2_0 = 1.0e+1**(op2)
          if (op2_0.le.vtr) then
          filph1 = 0.
          goto 422
          else
            do j=0,lim
              rnu = 1.e+1**(op1 + ( (.11/float(lim)) * (float(j))))
              aux(j) = phnu_tot(enpa,rnu) * rnu
            enddo
          filph1a = simps(aux,ope1,ope2,lim) 
          filph1a = (filph1a / gctphv(enpa)) * float(NA)
          goto 422
          endif
 422  return
      end
c===========================================================================
c N.16c
c                AN AUXILIARY FUNCTION FOR TESTPHNU SUBROUTINE
c
      function filph2(x)
      external getlphnu,phnu_tot,simps
      real*8 z1(10),w(10),aw(10),a_ef,ro,alfa
      real*8 rm_e,rm_mu,r_e,avog
      integer nsub
      common /general/ emin,vmin,emph
      common /media/ z1,w,aw,a_ef,ro,nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      dimension aux(0:2000)
      x1 = 1.e+1**x
      lim=2000
      vm1 = alog(emin/x1)
      vm2 = alog(emph/x1)
      vmi = amax1(vm1,vm2)  !------------> The lower limit for integration
      vma=0.e+0  !----------------> The upper limit for integration
      vst = (vma - vmi) / float(lim) !-> step for a rel. en. transfers grid
        do j=0,lim
        rnu = exp(vmi + vst*(float(j)))
        aux(j) = phnu_tot(x1,rnu) * rnu       !-> array to be integrated 
c                                             !   for getting free path
        enddo     
      filph2 = simps(aux,vmi,vma,lim)
      filph2 = sngl(a_ef / (avog * ro)) / filph2
      dummy = getlphnu(x1)
      filph2 = 1.e+2 * ( (dummy - filph2) / filph2 ) !-> rel.error (percents)
      return
      end
c===========================================================================
c N.16d
c                AN AUXILIARY FUNCTION FOR TESTPHNU SUBROUTINE
c
      function filph3(x)
      external getdedph,phnu_tot,simps
      real*8 z1(10),w(10),aw(10),a_ef,ro,alfa
      real*8 rm_e,rm_mu,r_e,avog
      integer nsub
      common /general/ emin,vmin,emph
      common /media/ z1,w,aw,a_ef,ro,nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      dimension aux(0:2000)
      x1 = 1.e+1**x
      lim=2000
      vm1 = alog(emin/x1)
      vm2 = alog(emph/x1)
      vmi = amax1(vm1,vm2)  !------------> The lower limit for integration
      vma=0.e+0     !----------------> The upper limit for integration
      vst = (vma - vmi) / float(lim) !-> step for a rel. en. transfers grid
       do j=0,lim
       rnu = exp(vmi + vst*(float(j)))
       aux(j) = phnu_tot(x1,rnu) * rnu * rnu    !-> array to be integrated 
c                                               !  for getting energy losses
       enddo     
      filph3 = simps(aux,vmi,vma,lim) * sngl(avog*ro/a_ef) * x1 * 1.e+3
      dummy = getdedph(x1)
      filph3 = 1.e+2 * ( (dummy - filph3) / filph3 ) !--> rel.error (percents)
      return
      end
c===========================================================================
c N.16e
c                AN AUXILIARY FUNCTION FOR TESTPHNU SUBROUTINE
c
      function filph4(x)
      external glphnuv,phnu_tot,simps
      real*8 z1(10),w(10),aw(10),a_ef,ro,alfa
      real*8 rm_e,rm_mu,r_e,avog
      integer nsub
      common /general/ emin,vmin,emph
      common /media/ z1,w,aw,a_ef,ro,nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      dimension aux(0:2000)
      x1 = 1.e+1**x
      lim=2000
      vm1 = alog(vmin)
      vm2 = alog(emph/x1)
      vmi = amax1(vm1,vm2)  !------------> The lower limit for integration
      vma=0.e+0  !----------------> The upper limit for integration
      vst = (vma - vmi) / float(lim) !-> step for a rel. en. transfers grid
        do j=0,lim
        rnu = exp(vmi + vst*(float(j)))
        aux(j) = phnu_tot(x1,rnu) * rnu       !-> array to be integrated 
c                                             !   for getting free path
        enddo     
      filph4 = simps(aux,vmi,vma,lim)
      filph4 = sngl(a_ef / (avog * ro)) / filph4
      dummy = glphnuv(x1)
      filph4 = 1.e+2 * ( (dummy - filph4) / filph4 ) !-> rel.error (percents)
      return
      end
c===========================================================================
c N.16f
c                AN AUXILIARY FUNCTION FOR TESTPHNU SUBROUTINE
c
      function filph5(x)
      external gdedphv,phnu_tot,simps
      real*8 z1(10),w(10),aw(10),a_ef,ro,alfa
      real*8 rm_e,rm_mu,r_e,avog
      integer nsub
      common /general/ emin,vmin,emph
      common /media/ z1,w,aw,a_ef,ro,nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      dimension aux(0:2000)
      x1 = 1.e+1**x
      lim=2000
      vm1 = alog(vmin)
      vm2 = alog(emph/x1)
      vmi = amax1(vm1,vm2)  !------------> The lower limit for integration
      vma=0.e+0     !----------------> The upper limit for integration
      vst = (vma - vmi) / float(lim) !-> step for a rel. en. transfers grid
       do j=0,lim
       rnu = exp(vmi + vst*(float(j)))
       aux(j) = phnu_tot(x1,rnu) * rnu * rnu    !-> array to be integrated 
c                                               !  for getting energy losses
       enddo     
      filph5 = simps(aux,vmi,vma,lim) * sngl(avog*ro/a_ef) * x1 * 1.e+3
      dummy = gdedphv(x1)
      filph5 = 1.e+2 * ( (dummy - filph5) / filph5) !--> rel.error (percents)
      return
      end
c===========================================================================
c N.16g
c                AN AUXILIARY FUNCTION FOR TESTPHNU SUBROUTINE
c
      function filph6(x)
      external gdedpht,phnu_tot,simps
      real*8 z1(10),w(10),aw(10),a_ef,ro,alfa
      real*8 rm_e,rm_mu,r_e,avog
      integer nsub
      common /general/ emin,vmin,emph
      common /media/ z1,w,aw,a_ef,ro,nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      dimension aux(0:2000)
      x1 = 1.e+1**x
      lim=2000
      vmi = alog(emph/x1)  !------------> The lower limit for integration
      vma=0.e+0     !----------------> The upper limit for integration
      vst = (vma - vmi) / float(lim) !-> step for a rel. en. transfers grid
       do j=0,lim
       rnu = exp(vmi + vst*(float(j)))
       aux(j) = phnu_tot(x1,rnu) * rnu * rnu    !-> array to be integrated 
c                                               !  for getting energy losses
       enddo     
      filph6 = simps(aux,vmi,vma,lim) * sngl(avog*ro/a_ef) * x1 * 1.e+3
      dummy = gdedpht(x1)
      filph6 = 1.e+2 * ( (dummy - filph6) / filph6) !--> rel.error (percents)
      return
      end
****************************************************************************
*********************** DELTA-ELECTRONS SUBROUTINES : **********************
****************************************************************************
* E.1
*
*     This routine was kindly provided by R.P.Kokoulin in November,1999.
*    It has been essentially (but carefully!) changed to adopt it for MUM
*  purposes. All changes were tested, they do not result in some changes in
*                                     output.
*
*  In current version it computes a differential cross-section for knock-on
*    electrons d_sigma/d_v (sq. cm) (including bremhsstrahlung e-diagram)
*    for an "averaged" atom with Z = sum( (Z_i)/N_tot ), where N_tot is the 
*  total numbers of atoms in molecula of medium, Z_i are an electric charge 
*    of atom nb. "i". Z is prepared in advance in subroutine MED_CONS and 
*                            passed here via common
*                                     /zav/.
*
*                                    INPUT:
*                                    =====
*
*            real*4 e : Muon energy (GeV)
*            real*4 v : Relative energy transfer for knock-on electron
*
*                  BELOW IS CHANGED CODE FROM R.P.KOKOULIN:
****************************************************************************
C***	Cross section for knock-on electron production by fast muons
C***	(including bremsstrahlung e-diagrams and rad. correction).
C***	Units: cm^2/(g*GeV); Tkin, ep - GeV.
C***	By R.P.Kokoulin, October 1998
C***	Formulae from Kelner,Kokoulin,Petrukhin, Phys.Atom.Nuclei, 1997
C***	(a bit simplified Kelner's version of Eq.30 - with 2 logarithms).
C***
	function getcde(e,v)
        common /zav/ z
	parameter (ame=0.51099907e-3) !	GeV
	parameter (amu=0.105658389)   !	GeV
	parameter (re=2.81794092e-13) !	cm
	parameter (alpha=1./137.036)
	parameter (pi=3.141592654)
	parameter (bmu=amu**2/(2.*ame))
	parameter (coeff0=2.*pi*ame*re**2)
	parameter (coeff1=alpha/(2.*pi))
        parameter (sok1=2./ame)
        parameter (sok2=5.88138263)
        common /exer1/ fa
	vpmax=e/(e+bmu)
   	ep=v*e
           if (v.ge.vpmax) then
           getcde=0.
           return
	   endif
	sigma0=coeff0*z*(1.-v/vpmax+.5*v*v)/(v*ep)
	a1=alog(1.+sok1*ep)
	a3=sok2+alog(e*(e-ep))
	getcde=sigma0*(1.+coeff1*a1*(a3-a1))*fa
	return
	end
****************************************************************************
* E.2
*     This routine was written on the base on a routine from R.P.Kokoulin.
*   It computes energy losses for muon due to knock-on electron production
*                   by bremsstrahlung with e-diagrams.
*
*       Formulae from Kelner,Kokoulin,Petrukhin, Phys.Atom.Nuclei, 1997
*     (a bit simplified Kelner's version of Eq.30 - with 2 logarithms).
*
*                 INPUT:   real*4 e : Muon energy (GeV)
*                 OUTPUT:  real*4 edbrt: Energy losses (MeV/cm)
*..........................................................................
      function edbrt(e)
      external simps
      real*8 alfa,rm_e,rm_mu,r_e,avog,z1(10),w(10),aw(10),a_ef,ro
      integer nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog ! from initial subroutine
      common /media/ z1,w,aw,a_ef,ro,nsub     !       MED_CONS
      common /zav/ z
      common /exer1/ fa
      parameter (ame=0.51099907e-3) !	GeV
      parameter (amu=0.105658389)   !	GeV
      parameter (re=2.81794092e-13) !	cm
      parameter (alpha=1./137.036)
      parameter (pi=3.141592654)
      parameter (bmu=amu**2/(2.*ame))
      parameter (coeff0=2.*pi*ame*re**2)
      parameter (coeff1=alpha/(2.*pi))
      parameter (sok1=2./ame)
      parameter (sok2=5.88138263)
      parameter (lim=2000)
      dimension aux(0:2000)
      vpmin = alog(7.5e-8 / e)
      vpmax = e / (e + bmu)
      vpmax1 = alog(vpmax)
      st = (vpmax1 - vpmin) * 5.e-4
      do i=0,lim
        v = exp(vpmin + st * float(i))
        ep = v * e
        if (v.ge.vpmax) then
         aux(i) = 0.
        else 
         sigma0 = coeff0 * z * v * (1. - v/vpmax + .5 * v * v) / ep
         a1 = alog(1. + sok1 * ep)
         a3 = sok2 + alog(e * (e - ep))
         aux(i) = sigma0 * coeff1 * a1 * (a3 - a1)
        endif
      enddo
      edbrt = simps(aux,vpmin,vpmax1,lim) * e * 1.e+3
      edbrt = fa * edbrt * sngl(avog * ro / a_ef)
      return
      end
****************************************************************************
* E.3
        function bebl(ene)
*
*  The routine calculates the ionization Losses of muons using Bethe-Bloch
*        formula taken from /E.V.Bugaev et al., hep-ph/9803488/.
*
*           Input  :  real*4 ene  -------> Muon Enegrgy (GeV) 
*           Output :  real*4 bebl -------> energy losses (MeV/cm)
*...........................................................................
      real*8 c_0,z_a,ri_z,x_0,x_1,a,rm,con1,con2,hnu,c,e
      real*8 beta,p,w_max,x,theta1,theta2,delta,difx1_x,e_loss
      real*8 alfa,rm_e,rm_mu,r_e,avog,z1(10),w(10),aw(10),a_ef,ro
      integer nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog ! from initial subroutine
      common /media/ z1,w,aw,a_ef,ro,nsub     !       MED_CONS
      common /med_ion/ z_a,ri_z,x_0,x_1,a,rm  !
      common /exer1/ fa
      parameter (c_0=1.535d-1)
c
      e = dble(ene) * 1.d+3
      con1 = c_0 * z_a * ro
      con2 = (2.d+0 * rm_e)/(rm_mu * rm_mu * ri_z * ri_z) 
      hnu = 2.8816d+1 * dsqrt(ro*z_a)/1.d+6 
      c = (2.d+0 * dlog(ri_z/hnu)) + 1.d+0 
c
      beta = dsqrt(1.d+0 - ((rm_mu * rm_mu)/(e * e)))
      p = beta * e
      w_max=(2.d+0*rm_e*p*p)/((rm_mu*rm_mu)+(rm_e*rm_e)+(2.d+0*rm_e*e))
      x = dlog10(p/rm_mu)
             if(x.gt.x_0) then
             theta1 = 1.d+0
             else
             theta1 = 0.d+0
             delta = 0.d+0
             goto 1
             endif
                if(x_1.gt.x) then
                theta2 = 1.d+0
                else
                theta2 = 0.d+0
                difx1_x=0.d+0
                goto 2
                endif
       difx1_x = (x_1 - x)**rm
 2     delta = theta1 * ((4.6052d+0 * x) + (a * theta2 * difx1_x) - c)
 1     e_loss=(con1/(beta*beta))*((dlog(con2*p*p*w_max))+((w_max*w_max)/
     +(4.d+0*e*e))-(2.d+0*beta*beta)-delta)
       bebl=sngl(e_loss) * fa
       return
       end
****************************************************************************
* E.4
       subroutine elec1
*
*                      The subroutine calculates:
*
*   Energy losses and averaged free path for knock-on-electrons production 
*                (including e-diagram for bremsstrahlung) by 
*     muon which passes a media with energy transfers which exceed EMIN -
*   arrays elo_el1(17) and crt_el1(17), correspondingly, or VMIN - arrays
*  elo_el2(17) and crt_el2(17). Total energy losses for knock-on electrons
*    production are calculated, as well by ordinary Bethe-Bloch formula 
*   (array elel_bb(17) and by Bethe-Bloch + e-diagram for bremsstrahlung 
*       (elel_bbb(17)). These arrays are computed for 17 values of 
*  muon energy starting with E_mu = 10 GeV and finishing with E_mu = 1 EeV
*  with logarithmically equidistant grid and passed to subroutine SPL1 via 
*  common blocks /elel_in1/ and /ctel_in1/ where they are used to prepare 
*              spline coefficients for further computing.
*  Also an array ELEL_TOT(101) is computed for total muon energy losses
* below muon energy 10 GeV. It is calculated as Bethe-Bloch + e-diagrams 
*  for bremsstrahlung for 101 muuon energy from 0.14 GeV to 14 GeV with
*   logarithmicallky equidestant grid and passed to SPL1 to cook spline 
*                               coefficients.
*  .....................................................................
       external getcde,dsimps
       real*8 um
       real*8 aux1(0:2000),aux2(0:2000),h1,h2,dsimps
       real*8 alfa,rm_e,rm_mu,r_e,avog,z1(10),w(10),aw(10),a_ef,ro
       integer nsub
       common /const/ alfa,rm_e,rm_mu,r_e,avog ! from initial subroutine
       common /media/ z1,w,aw,a_ef,ro,nsub     !       MED_CONS
       common /general/ emin,vmin,emph         !
       common /ctel_in1/ crt_el1(17),crt_el2(17) ! To SPL1 and ENLOS
       common /elel_in1/ elo_el1(17),elo_el2(17) ! To SPL1 and ENLOS
       common /elel_in2/ elel_bb(17),elel_bbb(17)  ! To SPL1 and ENLOS
       common /elel_in3/ elel_tot(101) ! To SPL1
       common /help_1/ aux1,aux2
       common /exer2/ noca !-> if noca=0 there are no catastrophic losses
c                              for knock-on electrons
c ....................................................................
c     PREPARATION OF ARRAYS WITH ENERGY LOSSES AND AVERAGED FREE PATH
c                        FOR KNOCK-ON-ELECTRONS:
c
c                     a) energy transfers > EMIN
c
      lim = 2000
      do i=1,17
      en = float(i)
      en = (1.e+1)**(5.e-1 + (5.e-1 * en))
      vmi = alog(emin / en) !--------------> The lower limit for integration
      um = (rm_mu * rm_mu * 1.d-3) / (2.d+0 * rm_e) 
      vma = sngl(um)
      vma = 1. / ( 1.e+0 + (vma / en) ) 
      vma = alog(vma)       !--------------> The upper limit for integration
      vst = (vma - vmi) / float(lim) !--> step for a rel. en. transfers grid
c
      do j=0,lim
      rnu = vmi + vst*(float(j))
      rnu = exp(rnu)
      aux1(j) = dble(getcde(en,rnu) * rnu)        !-> array to be integrated 
c                                                 !   to get mean free path
      aux2(j) = aux1(j) * rnu                     !-> array to be integrated
      enddo                                       !   for getting en. losses
c
      h1 = dble(vmi)
      h2 = dble(vma)
      crt_el1(i) = dsimps(aux1,h1,h2,lim) 
      crt_el1(i) = (sngl(a_ef/(avog*ro)))/crt_el1(i) !-> array with 17 values 
c                                                    !   of free path
      elo_el1(i) = sngl( dsimps(aux2,h1,h2,lim) * avog * ro / a_ef )
      elo_el1(i) = elo_el1(i) * en * 1.e+3 
      elo_el1(i) = alog10(elo_el1(i)) !-> array with 17 values of en. losses
      enddo
c                     b) energy transfers > VMIN
      do i=1,17
      en = float(i)
      en = (1.e+1)**(5.e-1 + (5.e-1 * en))
      vmi = alog(vmin)      !--------------> The lower limit for integration
      um = (rm_mu * rm_mu * 1.d-3) / (2.d+0 * rm_e) 
      vma = sngl(um)
      vma = 1. / ( 1.e+0 + (vma / en) ) 
      vma = alog(vma)       !--------------> The upper limit for integration
      vst = (vma - vmi) / float(lim) !--> step for a rel. en. transfers grid
c
      do j=0,lim
      rnu = vmi + vst*(float(j))
      rnu = exp(rnu)
      aux1(j) = dble(getcde(en,rnu) * rnu)        !-> array to be integrated 
c                                                 !   to get mean free path
      aux2(j) = aux1(j) * rnu                     !-> array to be integrated
      enddo                                       !   for getting en. losses
c
      h1 = dble(vmi)
      h2 = dble(vma)
      crt_el2(i) = dsimps(aux1,h1,h2,lim) 
      crt_el2(i) = (sngl(a_ef/(avog*ro)))/crt_el2(i) !-> array with 17 values 
      crt_el2(i) = alog(crt_el2(i))                  !   of free path
      if (noca.eq.0) crt_el2(i)=45. !-> no cat. losses for knock-on electrons
c   
      elo_el2(i) = sngl( dsimps(aux2,h1,h2,lim) * avog * ro / a_ef )
      elo_el2(i) = elo_el2(i) * en * 1.e+3 
      if (noca.eq.0) elo_el2(i)=1.e-30 !-> no cat. losses for knock-on electrons
      elo_el2(i) = alog10(elo_el2(i)) !-> array with 17 values of en. losses
      enddo
c                     c) energy transfers > 0 
      do i=1,17
      en = float(i)
      en = (1.e+1)**(5.e-1 + (5.e-1 * en))
      elel_bb(i) = bebl(en)                 !--> Ion. en. losses (Bethe-Bloch) 
      elel_bbb(i) = edbrt(en) + elel_bb(i)  !--> BB + bremsstahlung e-diagram
      enddo
c                     d) total energy losses below 10 GeV
c                 (Bethe-Bloch + e-diagrams for bremsstrahlung) 
      do i=1,101
      en = float(i)
      en = (1.e+1)**(alog10(0.14) - .02 + (.02 * en))
      elel_tot(i) = edbrt(en) + bebl(en)
      enddo
c
      return
      end
****************************************************************************
* E.5
           FUNCTION getlelec(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*     The subroutine calculates the value for mean free path for knock-on
*       electron production with energy transfers > EMIN using spline 
*                 coefficients prepared by spl1 subroutine.
*
*          X is a value for muon energy expressed in GeV; XMIN,
*   XMAX and STEP are assigned with some values in subroutine SPL1 (they
*              are passed here via /sok1/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                             common /sok1_e/
*  ......................................................................
*  ***   ATTENTION: X must be within a range of 10 GeV -- 1000 PeV   ***
*  ......................................................................
      COMMON /sok1/ XMIN,STEP,XMAX
      common /sok1_e/ C(19)
      X1 = alog10(X)
       if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
       write(0,*)'ERROR IN FUNCTION GETLELEC: MUON ENERGY IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      getlelec = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
****************************************************************************
* E.6
           FUNCTION glelecv(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*     The subroutine calculates the value for mean free path for knock-on
*       electron production with energy transfers > VMIN using spline 
*                 coefficients prepared by spl1 subroutine.
*
*          X is a value for muon energy expressed in GeV; XMIN,
*   XMAX and STEP are assigned with some values in subroutine SPL1 (they
*              are passed here via /sok1/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                             common /sok1_e2/
*  ......................................................................
*  ***   ATTENTION: X must be within a range of 10 GeV -- 1000 PeV   ***
*  ......................................................................
      COMMON /sok1/ XMIN,STEP,XMAX
      common /sok1_e2/ C(19)
      common /exer2/ noca
      if (noca.eq.0) then      
      glelecv = 1.e+34
      return
      endif
      X1 = alog10(X)
       if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
       write(0,*)'ERROR IN FUNCTION GLELECV: MUON ENERGY IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      glelecv = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      glelecv = exp(glelecv)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
****************************************************************************
* E.7
           function getctel(u)
*
* Function GETCTEL calculates the total cross-section for knock-on electrons 
*      production with energy transfers > EMIN using function GETLELEC.
*   The resulting cross-section is averaged over all atoms of given medium:
*
*                      sigma(E_mu) = SUM (w_i * sigma_i(E_mu)), 
*
*       where w_i = N_i/N_tot, N_i - number of type i atoms in molecula, 
* N_tot - total number of atoms in molecula, sigma_i(E_mu) is a cross-section
*                               for atom number i.
*      So, it is calculated for an "effective atoms" with atomic weight 
*   A_eff = SUM(N_i*A_i) / N_tot and electric charge Z_eff = SUM(Z_i)/N_tot
*   It also uses some media parameters prepared by subroutine MED_CONS
*                      (commons /const/ and /media/)
*                   Input: u = muon energy (GeV) 
*                   Output: getlelec = free length (cm)
* ..........................................................................
*        U MUST BE WITHIN THE RANGE OF 10 -- 1000,000,000 GeV
* ..........................................................................
*
      external getlelec
      real*8 z1(10),w(10),aw(10),a_ef,alfa,rm_e,rm_mu,r_e,avog,ro
      integer nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      common /media/ z1,w,aw,a_ef,ro,nsub
      if ((u.gt.1.0001e+9).or.(u.lt.9.9999e+0)) then   
      write(0,*)'ERROR IN FUNCTION GETCTEL: MUON ENERGY IS OUT OF RANGE'
      endif
      aef = sngl(a_ef / (avog * ro))
      getctel = aef / getlelec(u)
      return
      end
****************************************************************************
* E.8
           function gctelv(u)
*
*    Function calculates the total cross-section for knock-on electrons 
*      production with energy transfers > VMIN using function GLELECV.
*   The resulting cross-section is averaged over all atoms of given medium:
*
*                      sigma(E_mu) = SUM (w_i * sigma_i(E_mu)), 
*
*       where w_i = N_i/N_tot, N_i - number of type i atoms in molecula, 
* N_tot - total number of atoms in molecula, sigma_i(E_mu) is a cross-section
*                               for atom number i.
*      So, it is calculated for an "effective atoms" with atomic weight 
*   A_eff = SUM(N_i*A_i) / N_tot and electric charge Z_eff = SUM(Z_i)/N_tot
*   It also uses some media parameters prepared by subroutine MED_CONS
*                      (commons /const/ and /media/)
*                   Input: u = muon energy (GeV) 
*                   Output: getlelec = free length (cm)
* ..........................................................................
*        U MUST BE WITHIN THE RANGE OF 10 -- 1000,000,000 GeV
* ..........................................................................
*
      external glelecv
      real*8 z1(10),w(10),aw(10),a_ef,alfa,rm_e,rm_mu,r_e,avog,ro
      integer nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      common /media/ z1,w,aw,a_ef,ro,nsub
      common /exer2/ noca
      if (noca.eq.0) then      
      gctelv = 1.e-37
      return
      endif
      if ((u.gt.1.0001e+9).or.(u.lt.9.9999e+0)) then   
      write(0,*)'ERROR IN FUNCTION GCTELV: MUON ENERGY IS OUT OF RANGE'
      endif
      aef = sngl(a_ef / (avog * ro))
      gctelv = aef / glelecv(u)
      return
      end
************************************************************************
* E.9
             FUNCTION getdedel(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*       The subroutine calculates the value for muon energy losses for 
* knock-on electrons production with energy transfers above EMIN using spline 
*                coefficients prepared by spl1 subroutine.
*
*          X is a value for muon energy expressed in GeV; XMIN,
*  XMAX and STEP are assigned with some values in subroutine SPL1 (they
*               are passed here via /sok5/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                              common /sok5_e/
*  ......................................................................
*  ***   ATTENTION: X must be within a range of 10 GeV --- 1000 PeV   ***
*  ......................................................................
      COMMON /sok5/ XMIN,STEP,XMAX
      common /sok5_e/ C(19)
      X1 = alog10(X)
       if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
       write(0,*)'ERROR IN FUNCTION GETDEDEL: MUON ENERGY IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      getdedel = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      getdedel = (1.e+1)**(getdedel)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
************************************************************************
* E.10
             FUNCTION gdedelv(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*       The subroutine calculates the value for muon energy losses for 
* knock-on electrons production with energy transfers above VMIN using spline 
*                coefficients prepared by spl1 subroutine.
*
*          X is a value for muon energy expressed in GeV; XMIN,
*  XMAX and STEP are assigned with some values in subroutine SPL1 (they
*               are passed here via /sok5/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                              common /sok5_e2/
*  ......................................................................
*  ***   ATTENTION: X must be within a range of 10 GeV --- 1000 PeV   ***
*  ......................................................................
      COMMON /sok5/ XMIN,STEP,XMAX
      common /sok5_e2/ C(19)
      common /exer2/ noca
      if (noca.eq.0) then      
      gdedelv = 1.e-36
      return
      endif
      X1 = alog10(X)
       if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
       write(0,*)'ERROR IN FUNCTION GDEDELV: MUON ENERGY IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      gdedelv = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      gdedelv = (1.e+1)**(gdedelv)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
****************************************************************************
* E.11
             FUNCTION gdedelt1(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*    The subroutine calculates the value for total muon energy losses for 
*    ionization (Berthe-Bloch + e-diagram for bremsstrahlung) using spline 
*                 coefficients prepared by spl1 subroutine.
*
*          X is a value for muon energy expressed in GeV; XMIN,
*  XMAX and STEP are assigned with some values in subroutine SPL1 (they
*               are passed here via /sok5/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                              common /sok5_e4/
*  ......................................................................
*  ***   ATTENTION: X must be within a range of 10 GeV --- 1000 PeV   ***
*  ......................................................................
      COMMON /sok5/ XMIN,STEP,XMAX
      common /sok5_e4/ C(19)
      X1 = alog10(X)
       if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
       write(0,*)'ERROR IN FUNCTION GDEDELT1: MUON ENERGY IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      gdedelt1 = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
****************************************************************************
* E.12
             FUNCTION gdedelt2(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*    The subroutine calculates the value for total muon energy losses for 
*    ionization (Berthe-Bloch + e-diagram for bremsstrahlung) for low muon
* energies ( < 10 GeV) using spline coefficients prepared by spl1 subroutine.
*
*          X is a value for muon energy expressed in GeV; XMIN,
*  XMAX and STEP are assigned with some values in subroutine SPL1 (they
*               are passed here via /sok5et/ common block).
*   Array with spalin coefficients C(103) is passed here from SPL1 via
*                              common /sok5_e5/
*  ......................................................................
*  ***   ATTENTION: X must be within a range of 0.16 GeV --- 12 GeV   ***
*  ......................................................................
      COMMON /sok5et/ XMIN,STEP,XMAX
      common /sok5_e5/ C(103)
      X1 = alog10(X)
       if ((X1.lt.-.8e+0).or.(X1.gt.1.08e+0)) then
       write(0,*)'ERROR IN FUNCTION GDEDELT2: MUON ENERGY IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      gdedelt2 = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
****************************************************************************
* E.13
             FUNCTION gdeion(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*    The subroutine calculates the value for total muon energy losses for 
*    ionization (Berthe-Bloch + e-diagram for bremsstrahlung) for all muon
* energies ( 0.16 GeV < E_Mu < 10^9 GeV) using either gdedlt1 or gdedlt2
*                      depending on muon energy
*
*            X is a value for muon energy expressed in GeV.
*  ......................................................................
*  ***   ATTENTION: X must be within a range of 0.16 GeV --- 1 EeV   ***
*  ......................................................................
      external gdedelt1,gdedelt2
       if ((X.lt..159).or.(X.gt.1.0001e+9)) then
       write(0,*)'ERROR IN FUNCTION GDEDION: MUON ENERGY IS OUT OF RANGE'
       endif
      if(x.le.10.) then
      gdeion = gdedelt2(x)
      else
      gdeion = gdedelt1(x)
      endif
      return
      end
****************************************************************************
* E.14
             FUNCTION gdedelbb(X)
*
*  THE ROUTINE USES A PART OF splain.f CODE (~/flib/splain.f) DEVELOPED AT
*                         IRKUTSK STATE UNIVERSITY
*
*   The subroutine calculates the value for total muon energy losses for 
*        ionization computed by Berthe-Bloch formula using spline 
*                coefficients prepared by spl1 subroutine.
*
*          X is a value for muon energy expressed in GeV; XMIN,
*  XMAX and STEP are assigned with some values in subroutine SPL1 (they
*               are passed here via /sok5/ common block).
*   Array with spalin coefficients C(19) is passed here from SPL1 via
*                              common /sok5_e3/
*  ......................................................................
*  ***   ATTENTION: X must be within a range of 10 GeV --- 1000 PeV   ***
*  ......................................................................
      COMMON /sok5/ XMIN,STEP,XMAX
      common /sok5_e3/ C(19)
      X1 = alog10(X)
       if ((X1.lt..99999e+0).or.(X1.gt.9.0001e+0)) then
       write(0,*)'ERROR IN FUNCTION GDEDELBB: MUON ENERGY IS OUT OF RANGE'
       endif
      Y = (X1-XMIN)/STEP
      I = INT(Y+0.5)
ccc      IF(I.LT.0.OR.X1.GT.XMAX) PRINT 1,X1,XMIN,XMAX
      Y = Y-I
      Z = Y*Y + 0.25
      gdedelbb = (Z-Y)*C(I+1) + (2.-(Z+Z))*C(I+2) + (Z+Y)*C(I+3)
      RETURN
ccc1     FORMAT('*MISTAKE*X1=',D23.16,'XMIN=',D23.16,'XMAX=',D23.16)
      END
****************************************************************************
* E.15
         SUBROUTINE getvel(emw,vbr1,itr)
*
*      It simulates the relative enegy transfers for knock-on electrons 
*  (including bremsstrahlung e-diagram) with energy transfers > EMIN (itr=0) 
*                          or > VMIN (otherwise). 
*                 Input: emw = muon energy expressed in GeV, 
*                 Output: vbr = relative energy transfer. 
*
*   The simulation is done by the "rejection method" (see W.H.Press et al.,
*  NUMERICAL RECEIPES (THE ART OF SCIENTIFIC COMPUTING), Cambridge University
*            press, Chapter 7, pp. 200-204) with comparison function 
*             f(v)=2*pi*r_e*r_e*z*(m_e/E_mu)(1+0.032*lnE_mu)*(1/v^2)
* ..........................................................................
*            EMW MUST BE WITHIN THE RANGE OF 10 -- 1000,000,000 GeV
* ..........................................................................
      external getcde
      real*8 vbr,rmd,vtr,vtr1,rvec(1)
      parameter (c2=2.549551e-28)
      parameter (len=1)
      common /r48/ rvec
      common /zav/ zm
      common /general/ emin,vmin,emph
      common /mcef/ mcb1,mcb2,mcp1,mcp2,mcn1,mcn2,mce1,mce2
      mce1 = mce1 + 1
      emind = dble(emin)
      vmind = dble(vmin)
      emwd = dble(emw)
        if(itr.eq.0) then
         vtr = dble(emin/emw)
        else
         vtr = dble(vmin)
        endif
      vtr1 = 1.d+0 - vtr
      coef2 = c2 * zm *  ( 1. + ( 3.2e-2 * alog(emw) ) )
  155 call rm48(rvec,len)
      rmd=rvec(1)
      mce2 = mce2 + 1
      vbr = vtr / (vtr + (vtr1 * rmd))
      if(vbr.ge.1.d+0) vbr=.999999999999d+0
      vbr1=sngl(vbr)
      y1 = getcde(emw,vbr1)
      y2 = coef2 / ( emw * vbr1 * vbr1)
      yc = y2 * rndm(8)
           if(y2.lt.y1) then
           write(0,*)'*** ERROR IN GETVEL: COMP.FUNCTION IS TOO SMALL ***'
           endif
                if (y1.ge.yc) then
                  goto 154
                else
                  goto 155
                endif
  154 return
      end
****************************************************************************
* E.16
        subroutine testelec(lukum)
*
*   Performes some test for all routines which relate knock-on electrons 
*              with filling number of histograms and .dat files.
*  The test is short if lukum=0. Otherwise it takes more time but gives 
*                         more statistics. 
*
      external getcde,getdedel,getctel,getlelec,gdedelv,gctelv,glelecv
      external filel1,filel1a,filel2,filel3,filel4,filel5,filel6,filel7
      external filel8,gdedelt1
      dimension ene(25)
      real*8 esum,esumsq,eterm
      character *10 txt1
      character *100 txt2
      character *1 txt3 
      character *11 txt4 
      common /general/ emin,vmin,emph
      common /el1/ NA,enpa
      common /mcef/ mcb1,mcb2,mcp1,mcp2,mcn1,mcn2,mce1,mce2
      common /exer2/ noca
      data ene / 1.e+1,2.e+1,5.e+1,1.e+2,2.e+2,5.e+2,1.e+3,2.e+3,5.e+3,
     +           1.e+4,2.e+4,5.e+4,1.e+5,2.e+5,5.e+5,1.e+6,2.e+6,5.e+6,
     +           1.e+7,2.e+7,5.e+7,1.e+8,2.e+8,5.e+8,1.e+9             /
      write(0,*)' '
      write(0,*)'      TESTING THE PART FOR DELTA ELECTRONS:'
      write(0,*)' '
         if (lukum.eq.0) then
           NA = 1400000
         else
           NA = 50000000
         endif
c---------------------------------------------------------------------------
c                1. QUALITY OF ENERGY TRANSFER SIMULATION
c---------------------------------------------------------------------------
c
c                            a) above EMIN
c
      write(0,*)'Testing GETVEL (en. transfers simulation) with EMIN...'
      write(0,*)'  '
        do i=1,25
          k = int(ene(i))
          write(txt1,'(i10)') k
          txt2 = 'Elec, MC En. transfers distribution for '//txt1//' Gev
     + muon (EMIN)'
          i1 = i + 7500
          CALL HBOOK1(i1,txt2,100,-11.,0.,0.)
        enddo
       open(11,file='cross_el.dat',status='new',form='formatted')
       open(12,file='leng_el.dat',status='new',form='formatted')
       open(13,file='loss_el.dat',status='new',form='formatted')
       open(14,file='loss_el1.dat',status='new',form='formatted')
       open(15,file='en_el.dat',status='new',form='formatted')
       open(16,file='er_lo_el.dat',status='new',form='formatted')
       open(17,file='efmc_el.dat',status='new',form='formatted')
c  EN_EL.DAT - 25 values of muon energy for which all other values are computed.
c  CROSS_EL.DAT - 25 values of total cross-section for delta electrons production
c                 with energy transfers > EMIN (Sq. cm).
c  LENG_EL.DAT - 25 values for averaged free path between two delta electrons
c                produced with energy transfer > emin (cm). 
c  LOSS_EL.DAT - 25 values for energy losses due to delta electrons with energy
c                transfers > emin (MeV/cm) computed with splines (GETDEDEL 
c                routine).
c  LOSS_EL1.DAT - 25 values for energy losses for delta electrons with energy
c                 transfers > emin (MeV/cm) simulated by MC (GETVEL routine) 
c                 with 1400000 interactions for each muon energy (lukum = 0) or 
c                 30000000 interactions (lukum > 0).
c ER_LO_EL.DAT - 25 values of statistical error for values in LOSS_EL1.DAT on 
c                1-sigma level (MeV/cm).
c EFMC_EL.DAT - 25 values of MC efficiency = 
c                              (number of simulation / number of call getvel)
      itr = 0
      do i=1,25
       write(0,*)'Energy Nb.',i,' of 25 is processed (',ene(i),' GeV ) ...'
       i1 = i + 7500
       elos = getdedel(ene(i))
       tcros = getctel(ene(i)) 
       rr = getlelec(ene(i)) 
       write (13,234) elos
       write (11,234) tcros
       write (15,234) ene(i)
       write (12,234) rr
       esum = 0.d+0
       esumsq = 0.d+0
       mce1 = 0
       mce2 = 0
         do j=1,NA
           call getvel(ene(i),v,itr)
            v2 = alog10(v)
            eterm1 = ene(i) * v * 1.e+3
            eterm = dble(eterm1)
            esum = esum + eterm
            esumsq = esumsq + (eterm * eterm)
           CALL HFILL(i1,v2,0.,1.)
         enddo  
       efn = float(mce2) / float (mce1)
       mce1 = 0
       mce2 = 0
       write (17,234) efn
       esum1 = sngl(esum)
       esumsq1 = sngl(esumsq)
       RNA = float(NA)
       err=((RNA*esumsq1) - (esum1*esum1))/(RNA*RNA*(RNA - 1.))
       err = sqrt(err) / rr
       esum1 = (esum1 / rr) / RNA
       write (14,234) esum1
       write (16,234) err
       write(0,*)'En.los.=',esum1,' +-',err,' (should be ',elos,' ) MeV/cm'
          k = int(ene(i))
          write(txt1,'(i10)') k
          txt2 = 'Elec, En. transfers distribution for '//txt1//' Gev mu
     +on (EMIN)'
          i1 = i + 7600
          enpa = ene(i)
          CALL HBFUN1(i1,txt2,100,-1.1e+1,0.e+0,filel1)
      enddo
      close (17)
      close (16)
      close (15)
      close (14)
      close (13)
      close (12)
      close (11)

c                            b) above VMIN
      write(0,*)'  '
      write(0,*)'Testing GETVEL (en. transfers simulation) with VMIN...'
      write(0,*)'  '
        do i=1,25
          k = int(ene(i))
          write(txt1,'(i10)') k
          txt2 = 'Elec, MC En. transfers distribution for '//txt1//' Gev
     + muon (VMIN)'
          i1 = i + 7800
          CALL HBOOK1(i1,txt2,100,-11.,0.,0.)
        enddo
       open(11,file='cross_el_v.dat',status='new',form='formatted')
       open(12,file='leng_el_v.dat',status='new',form='formatted')
       open(13,file='loss_el_v.dat',status='new',form='formatted')
       open(15,file='loss_el_t.dat',status='new',form='formatted')
       open(14,file='loss_el1_v.dat',status='new',form='formatted')
       open(16,file='er_lo_el_v.dat',status='new',form='formatted')
       open(17,file='efmc_el_v.dat',status='new',form='formatted')
c  CROSS_EL_V.DAT - 25 values of total cross-section for delta electrons 
c                   production with energy transfers > VMIN (Sq. cm).
c  LENG_EL_V.DAT - 25 values for averaged free path between two delta 
c                  electrons produced with energy transfer > VMIN (cm). 
c  LOSS_EL_V.DAT - 25 values for energy losses due to delta electrons with 
c                  energy transfers > VMIN (MeV/cm) computed with splines 
c                  (GDEDELV routine).
c  LOSS_EL_T.DAT - 25 values for total energy losses due to ionization 
c                  (Bethe-Bloch + e-diagrams for bremsstrahlung) computed 
c                  with splines (GDEDELT1 routine).
c  LOSS_EL1_V.DAT - 25 values for energy losses for delta electrons with 
c                   energy transfers > VMIN (MeV/cm) simulated by MC 
c                   (GETVEL routine) with 1400000 interactions for each 
c                   muon energy (lukum = 0) or 50000000 interactions 
c                   (lukum > 0).
c ER_LO_EL_V.DAT - 25 values of statistical error for values in 
c                  LOSS_EL1_V.DAT on 1-sigma level (MeV/cm).
c EFMC_EL_V.DAT - 25 values of MC efficiency = 
c                           (number of simulation / number of call getvel)
      itr = 1
      do i=1,25
       write(0,*)'Energy Nb.',i,' of 25 is processed (',ene(i),' GeV ) ...'
       i1 = i + 7800
       elos = gdedelv(ene(i))
       tcros = gctelv(ene(i)) 
       rr = glelecv(ene(i)) 
       ttlos = gdedelt1(ene(i))
       write (15,234) ttlos
       write (13,234) elos
       write (11,234) tcros
       write (12,234) rr
       if (noca.eq.0) goto 9753
       esum = 0.d+0
       esumsq = 0.d+0
       mce1 = 0
       mce2 = 0
         do j=1,NA
           call getvel(ene(i),v,itr)
            v2 = alog10(v)
            eterm1 = ene(i) * v * 1.e+3
            eterm = dble(eterm1)
            esum = esum + eterm
            esumsq = esumsq + (eterm * eterm)
           CALL HFILL(i1,v2,0.,1.)
         enddo  
       efn = float(mce2) / float (mce1)
       mce1 = 0
       mce2 = 0
       write (17,234) efn
       esum1 = sngl(esum)
       esumsq1 = sngl(esumsq)
       RNA = float(NA)
       err=((RNA*esumsq1) - (esum1*esum1))/(RNA*RNA*(RNA - 1.))
       err = sqrt(err) / rr
       esum1 = (esum1 / rr) / RNA
       write (14,234) esum1
       write (16,234) err
       write(0,*)'En.los.=',esum1,' +-',err,' (should be ',elos,' ) MeV/cm'
          k = int(ene(i))
          write(txt1,'(i10)') k
          txt2 = 'Elec, En. transfers distribution for '//txt1//' Gev mu
     +on (VMIN)'
          i1 = i + 7900
          enpa = ene(i)
          CALL HBFUN1(i1,txt2,100,-1.1e+1,0.e+0,filel1a)
 9753 continue
      enddo
      close (17)
      close (16)
      close (15)
      close (14)
      close (13)
      close (12)
      close (11)
c---------------------------------------------------------------------------
c   2. Files with differential cros-sections for delta electrons production
c---------------------------------------------------------------------------
c 9 files (el_cr_1.dat, ... , el_cr_9.dat) are produced with 1101 values in
c each. These values are equal to differential cross-section of D-electrons 
c     production for log10(v) = -11.00, -10.99, -10.98, ... , -0.01, 0.
c Figure "X" in file name pa_cr_X.dat corresponds to muon energy for which 
c            cross-section is computed (E_mu = 10.**X GeV).
       write(0,*)'  '
       write(0,*)'Computing dif. crossect. for a set of muon energies ...'
       write(0,*)'  '
       ilk = 0
       do i=1,25,3
       ilk = ilk + 1
       write(txt3,'(i1)') ilk
       txt4='el_cr_'//txt3//'.dat'
       open(11,file=txt4,status='new',form='formatted')
          do j=1100,0,-1
             v1 = -float(j) * 1.e-2
             v1 = 1.e+1 ** v1
             some = getcde(ene(i),v1)
             write (11,234) some
          enddo
      close (11)
      enddo 
c---------------------------------------------------------------------------
c  3.   DIFFERENCES BETWEEN GETLELEC, GETDEDEL, GLELECV, GDEDELV and REAL 
c     VALUES OF AVERAGED FREE PATH AND ENERGY LOSSES (PRECISION OF SPLINES)
c---------------------------------------------------------------------------
       write(0,*)'Testing quality of 1-dim. splines ...'
       write(0,*)'  '
      txt2 = 'Elec, rel. err. GETLELEC - mean free path (percents)'
          CALL HBFUN1(7701,txt2,97,.9583334,9.04165,filel2)
      txt2 = 'Elec, rel. err. GETDEDEL - energy losses (percents)'
          CALL HBFUN1(7702,txt2,97,.9583334,9.04165,filel3)
      if (noca.eq.0) goto 9754
      txt2 = 'Elec, rel. err. GLELECV - mean free path (percents)'
          CALL HBFUN1(7703,txt2,97,.9583334,9.04165,filel4)
      txt2 = 'Elec, rel. err. GDEDELV - energy losses (percents)'
          CALL HBFUN1(7704,txt2,97,.9583334,9.04165,filel5)
 9754 txt2 = 'Elec, rel. err. GDEDELT1 - energy losses (percents)'
          CALL HBFUN1(7705,txt2,97,.9583334,9.04165,filel6)
      txt2 = 'Elec, rel. err. GDEION - energy losses (percents)'
          CALL HBFUN1(7707,txt2,98,-.8,9.,filel8)
      txt2 = 'Elec, rel. err. GDEDELBB - energy losses (percents)'
          CALL HBFUN1(7706,txt2,97,.9583334,9.04165,filel7)
c---------------------------------------------------------------------------
  234 format (e13.7) 
  333 return
      end
c===========================================================================
c E.16a
c                AN AUXILIARY FUNCTION FOR TESTELEC SUBROUTINE
c
      function filel1(x)
      external getctel,getcde,simps
      common /el1/ NA,enpa
      common /general/ emin,vmin,emph
      dimension aux(0:20)
      lim = 20
      vtr = emin / enpa
        op1 = x - .055
        op2 = x + .055
        ope1 = 2.302585093 * op1
        ope2 = 2.302585093 * op2
        op2_0 = 1.0e+1**(op2)
          if (op2_0.le.vtr) then
            filel1 = 0.
            goto 422
          else
            do j=0,lim
               rnu0 = 1.e+1**(op1 + ( (.11/float(lim)) * (float(j)))) 
               aux(j) = getcde(enpa,rnu0) * rnu0
            enddo
            filel1 = simps(aux,ope1,ope2,lim) 
            filel1 = (filel1 / getctel(enpa)) * float(NA)
            goto 422
          endif
 422  return
      end
c===========================================================================
c E.16b
c                AN AUXILIARY FUNCTION FOR TESTELEC SUBROUTINE
c
      function filel1a(x)
      external gctelv,getcde,simps
      common /el1/ NA,enpa
      common /general/ emin,vmin,emph
      dimension aux(0:20)
      lim = 20
      vtr = vmin
        op1 = x - .055
        op2 = x + .055
        ope1 = 2.302585093 * op1
        ope2 = 2.302585093 * op2
        op2_0 = 1.0e+1**(op2)
          if (op2_0.le.vtr) then
            filel1a = 0.
            goto 422
          else
            do j=0,lim
               rnu0 = 1.e+1**(op1 + ( (.11/float(lim)) * (float(j)))) 
               aux(j) = getcde(enpa,rnu0) * rnu0
            enddo
            filel1a = simps(aux,ope1,ope2,lim) 
            filel1a = (filel1a / gctelv(enpa)) * float(NA)
            goto 422
          endif
 422  return
      end
c===========================================================================
c E.16c
c                AN AUXILIARY FUNCTION FOR TESTELEC SUBROUTINE
c
      function filel2(x)
      external getlelec,getcde,simps
      real*8 z1(10),w(10),aw(10),a_ef,ro,alfa
      real*8 rm_e,rm_mu,r_e,avog,um
      integer nsub
      common /general/ emin,vmin,emph
      common /media/ z1,w,aw,a_ef,ro,nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog      
      dimension aux(0:2000)
      x1 = 1.e+1**x
      lim = 2000
      vmi = alog(emin/x1)  !------------------> The lower limit for integration
      um = (rm_mu * rm_mu * 1.d-3) / (2.d+0 * rm_e) 
      vma = sngl(um)
      vma = alog(1. / (1.e+0 + (vma / x1))) !-> The upper limit for integration
      vst = (vma - vmi) / float(lim)    !--> step for a rel. en. transfers grid
          do j=0,lim
          rnu = exp(vmi + vst*(float(j))) !-> array to be integrated
          aux(j) = getcde(x1,rnu) * rnu   !   for getting free path     
          enddo     
      filel2 = simps(aux,vmi,vma,lim)
      filel2 = sngl(a_ef / (avog * ro)) / filel2
      dummy = getlelec(x1)
      filel2 = 1.e+2 * ( (dummy - filel2) / filel2 ) !---> rel.error (percents)
      return
      end
c===========================================================================
c E.16d
c                AN AUXILIARY FUNCTION FOR TESTELEC SUBROUTINE
c
      function filel3(x)
      external getdedel,getcde,simps
      real*8 z1(10),w(10),aw(10),a_ef,ro,alfa
      real*8 rm_e,rm_mu,r_e,avog,um
      integer nsub
      common /general/ emin,vmin,emph
      common /media/ z1,w,aw,a_ef,ro,nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      dimension aux(0:2000)
      x1 = 1.e+1**x
      lim = 2000
      vmi = alog(emin/x1)  !-------------------> The lower limit for integration
      um = (rm_mu * rm_mu * 1.d-3) / (2.d+0 * rm_e) 
      vma = sngl(um)
      vma = alog(1. / (1.e+0 + (vma / x1) )) !-> The upper limit for integration
      vst = (vma - vmi) / float(lim) !------> step for a rel. en. transfers grid
         do j=0,lim
         rnu = exp(vmi + vst*(float(j)))       !-> array to be integrated 
         aux(j) = getcde(x1,rnu) * rnu * rnu   !  for getting energy losses 
         enddo     
      filel3 = simps(aux,vmi,vma,lim) * sngl(avog*ro/a_ef) * x1 * 1.e+3
      dummy = getdedel(x1)
      filel3 = 1.e+2 * ( (dummy - filel3) / filel3 ) !---> rel.error (percents)
      return
      end
c===========================================================================
c E.16e
c                AN AUXILIARY FUNCTION FOR TESTELEC SUBROUTINE
c
      function filel4(x)
      external glelecv,getcde,simps
      real*8 z1(10),w(10),aw(10),a_ef,ro,alfa
      real*8 rm_e,rm_mu,r_e,avog,um
      integer nsub
      common /general/ emin,vmin,emph
      common /media/ z1,w,aw,a_ef,ro,nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      dimension aux(0:2000)
      x1 = 1.e+1**x
      lim = 2000
      vmi = alog(vmin)     !------------------> The lower limit for integration
      um = (rm_mu * rm_mu * 1.d-3) / (2.d+0 * rm_e) 
      vma = sngl(um)
      vma = alog(1. / (1.e+0 + (vma / x1))) !-> The upper limit for integration
      vst = (vma - vmi) / float(lim)    !--> step for a rel. en. transfers grid
          do j=0,lim
          rnu = exp(vmi + vst*(float(j))) !-> array to be integrated
          aux(j) = getcde(x1,rnu) * rnu   !   for getting free path     
          enddo     
      filel4 = simps(aux,vmi,vma,lim)
      filel4 = sngl(a_ef / (avog * ro)) / filel4
      dummy = glelecv(x1)
      filel4 = 1.e+2 * ( (dummy - filel4) / filel4 ) !---> rel.error (percents)
      return
      end
c===========================================================================
c E.16f
c                AN AUXILIARY FUNCTION FOR TESTELEC SUBROUTINE
c
      function filel5(x)
      external gdedelv,getcde,simps
      real*8 z1(10),w(10),aw(10),a_ef,ro,alfa
      real*8 rm_e,rm_mu,r_e,avog,um
      integer nsub
      common /general/ emin,vmin,emph
      common /media/ z1,w,aw,a_ef,ro,nsub
      common /const/ alfa,rm_e,rm_mu,r_e,avog
      dimension aux(0:2000)
      x1 = 1.e+1**x
      lim = 2000
      vmi = alog(vmin)  !-------------------> The lower limit for integration
      um = (rm_mu * rm_mu * 1.d-3) / (2.d+0 * rm_e) 
      vma = sngl(um)
      vma = alog(1. / (1.e+0 + (vma / x1) )) !-> The upper limit for integration
      vst = (vma - vmi) / float(lim) !------> step for a rel. en. transfers grid
         do j=0,lim
         rnu = exp(vmi + vst*(float(j)))       !-> array to be integrated 
         aux(j) = getcde(x1,rnu) * rnu * rnu   !  for getting energy losses 
         enddo     
      filel5 = simps(aux,vmi,vma,lim) * sngl(avog*ro/a_ef) * x1 * 1.e+3
      dummy = gdedelv(x1)
      filel5 = 1.e+2 * ( (dummy - filel5) / filel5 ) !---> rel.error (percents)
      return
      end
c===========================================================================
c E.16g
c                AN AUXILIARY FUNCTION FOR TESTELEC SUBROUTINE
c
      function filel6(x)
      external gdedelt1,edbrt,bebl
      x1 = 1.e+1**x
      filel6 = edbrt(x1) + bebl(x1)
      dummy = gdedelt1(x1)
      filel6 = 1.e+2 * ( (dummy - filel6) / filel6 ) !---> rel.error (percents)
      return
      end
c===========================================================================
c E.16h
c                AN AUXILIARY FUNCTION FOR TESTELEC SUBROUTINE
c
      function filel8(x)
      external gdeion,edbrt,bebl
      x1 = 1.e+1**x
      filel8 = edbrt(x1) + bebl(x1)
      dummy = gdeion(x1)
      filel8 = 1.e+2 * ( (dummy - filel8) / filel8 ) !---> rel.error (percents)
      return
      end
c===========================================================================
c E.16i
c                AN AUXILIARY FUNCTION FOR TESTELEC SUBROUTINE
c
      function filel7(x)
      external gdedelbb,bebl
      x1 = 1.e+1**x
      filel7 = bebl(x1)
      dummy = gdedelbb(x1)
      filel7 = 1.e+2 * ( (dummy - filel7) / filel7 ) !---> rel.error (percents)
      return
      end
****************************************************************************
*                            T H E   E N D                                 *
****************************************************************************
      subroutine oppaw      
      PARAMETER (NWPAWC=6000000)
      COMMON /PAWC/ H(NWPAWC)
      call hlimit(NWPAWC)
      call htitle('MUM OUTPUT')
      return
      end
****************************************************************************
      subroutine clpaw
      CALL HRPUT(0,'mum.out','N')
      return
      end
****************************************************************************
